%\documentstyle[comments,mpicover]{cweb} 

\documentstyle[comments]{cweb} 

%\makempicover geht nicht mit latex209!

%\documentclass{cweb}
%\usepackage{cweb}

\newcommand{\straightletter}[1]{{\mathord{\mathrm I\mkern-3.6mu\mathrm #1}}}
\newcommand{\R}{\straightletter{R}}
\newcommand{\IR}{{\rm\hbox{I\kern-1.15em R}}}

\def\abs{|}
\def\labs{\left|}
\def\rabs{\right|}

\title{The LEDA class {\bf real} number}
\author{Christoph Burnikel \qquad Kurt Mehlhorn \qquad Stefan Schirra}

%\begin{acknowledgements}
%\phantom{x}
%This work was supported in part by the ESPRIT Basic Research Actions
%Program of the EC under contract No.\ 7141(ALCOM II) and the BMFT
%(F\"orderungskennzeichen ITS 9103).
%\end{acknowledgements}

\begin{document}

\phantom{x}

\begin{abstract}
We describe the implementation of the LEDA
\cite{Me-Naeher:LEDA,Naeher:manual,Me-Naeher:IFIP94} data
type {\bf real}. 
Every integer is a real and the reals are closed
under the operations addition, subtraction, multiplication, division and
squareroot.
The main features of the data type real are
\begin{enumerate}
\item The user--interface is similar to that of the built--in data type double.
\item All comparison operators $\{>, \geq, <, \leq, =\}$ are {\em exact}. 
In order to determine the sign of a real number $x$ the data type first 
computes a rational number $q$ such that $|x| \leq q$ implies $x = 0$  and then
computes an approximation of $x$ of sufficient precision to decide the 
sign of $x$.
The user may assist the data type by providing a separation bound $q$.
\item The data type also allows to evaluate real expressions with
arbitrary precision. 
One may either set the mantissae length of the underlying floating point
system and then evaluate the expression with that mantissa length or one
may specify an error bound $q$. The data type then computes an approximation 
with absolute error at most $q$.
\end{enumerate}
The implementation of the data type real is based on the LEDA data types
{\bf integer} and {\bf bigfloat} which are the types of
arbitrary precision integers and floating point numbers respectively.
The implementation takes various shortcuts for increased efficiency, e.g.,
a {\bf double} approximation of any real number together with an error
bound is maintained and tests are first performed on these approximations.
A high precision computation is only started when the test on the {\bf double}
approximation is inconclusive. 
\end{abstract}  

%\makempicover

\nopagebreak[4]
\tableofcontents
\phantom{x}

\pagestyle{headings}
\pagenumbering{arabic}

@s double double
@s bigfloat double
@s bigrational int
@s bigint int
@s integer int
@s rational int
@s long int
@s bool int
@s app_bf int
@s real int
@s real_rep int
@s ostream int
@s istream int

\newpage

@* Introduction.

The need for exact real arithmetic or at least higher precision arithmetic than
standard {\bf double} precision arithmetic arises frequently.
One prime example is computational geometry. Other examples are computational
metrology and constrained logic programming; we refer the reader to
\cite{DY_basisprec} for an extensive 
discussion.
Almost all algorithms in computational geometry are formulated under the
assumption of exact real arithmetic.
When implemented naively using floating point arithmetic these algorithms
tend to be unreliable, i.e., to either crash or produce wrong results at least 
on some inputs.
Two approaches have been taken to remedy this situation.
The first approach sticks with floating point arithmetic and reformulates the
algorithms.
This approach has been successful for a small number of problems only,
for example, two--dimensional convex hulls (\cite{LM_prec92}), line arrangements
(\cite{FM91_prec}) and Delaunay triangulations (\cite{Fortune:CG92}).
The other approach uses exact real arithmetic.
Although it has been known for a long time (\cite{Mignotte:book,Yap:Buch})
that exact computation with algebraic numbers%
\footnote{A real number is algebraic if it is the root of a polynomial with
integer coefficients.}
is possible, it has also been felt that the overhead would be prohibitive.
Serious experimental studies appeared only recently.
In \cite{KLN,Fortune-vWijk,BJMM_prec,Me-Naeher:IFIP94} it is shown
that the overhead for exact rational arithmetic is small in the context of
geometric computations.
In particular, floating point filters as introduced in \cite{Fortune-vWijk}
are very effective.
A floating point filter first approximates a computation using {\bf doubles}.
Only if a test cannot be performed conclusively on the basis of the
{\bf double} approximation, exact arithmetic is used as a back--up. 
In \cite{BMS:ESA} the same idea is generalized to a computation involving
non--rational algebraic numbers, and in \cite{Yap_exact_CCCG,DY_basisprec} it
is argued that it can be applied to all numerical
computations arising in computational geometry.
In this paper we define and implement the LEDA data type |real|.
Any integer is a |real| and the class of |real|s is closed under addition,
subtraction, multiplication, division and squareroot.
The full repertoire of relational operators ($=, \neq, <, \leq, >, \geq,$)
is available on |real|s. The outcome is {\em exact}.
In addition, arbitrary floating point approximations of |real| numbers can be
computed.
We define the class |real| in section \ref{Manual} and also give two examples 
of the use of |real| numbers: 
an incircle test arising in the computation of Voronoi diagrams of line segments
and the computation of line segment intersection.
Starting in section \ref{Implementation} we give the implementation of |real|s.
Our implementation is similar to the proposal by Yap and Dube 
\cite{DY_basisprec}.
In section \ref{Floats} we review some facts about floating point computations.

@* The manual pages of data type real.
 \label{Manual}
\def\environment{}
\begin{environment}
\def\name{real}
\def\type{real}
\input /LEDA/SRC/man/MANUAL.mac

@i real.man

\end{environment}

@* A Short Review of Floating Point Arithmetic.

\label{Floats}
We review some facts about double precision floating point arithmetic according
to the IEEE standard \cite{IEEE754}.
A |double| is specified by a sign $s \in \{0,1\}$, an exponent 
$e \in [0 \ldots 2047]$ and a binary fraction $f = f_1 f_2 \ldots f_{52}$
with $f_i \in \{0,1\}$ 
for all $i, 1 \leq i \leq 52$.
We also use $f$ to denote the number $\sum_{i}f_i 2^{-i}$.
The number $v$ represented by a triple $(s,e,f)$ is defined as follows:
\begin{itemize}
\item if $e = 2047$ and $f \neq 0$ then no number is represented ($v$ is a
      $NaN$)
\item if $e = 2047$ and $f = 0$ then $v = (-1)^{s} \cdot \infty$
\item if $0 < e < 2047$ then $v = (-1)^{s} \cdot (1 + f) \cdot 2^{e - 1023}$
\item if $e = 0$ and $f \neq 0$ then $v = (-1)^{s} \cdot f \cdot 2^{-1022}$ 
      and $v$ is called a denormalized number
\item if $e = 0$ and $f = 0$ then $v = (-1)^{s} \cdot 0$  ( = signed zero)
\end{itemize}
Let $MinDbl = 2^{-1022}$ and $MaxDbl = (2 - 2^{-52}) \cdot 2^{1023}$.
We call a real number {\em approximable} if either $a = 0$ or 
$MinDbl \leq \abs a \abs < MaxDbl + 2^{-53} \cdot 2^{1023}$.
For any approximable number $a$ let $round(a)$ be the floating point number
nearest to $a$.
If there are two floating point numbers equally near to $a$ then
$round(a)$ is the one where the least significant bit is zero.
For any of the arithmetic operations $+, -, *, /, \sqrt{}$,
the IEEE standard guarantees the following.
Let $z$ be the {\em exact} result of an operation applied to floating point
operands.
\begin{itemize}
\item if $z$ is approximable then $round(z)$ is delivered,
\item if $\abs z \abs \geq |MaxDbl| + 2^{-53} \cdot 2^{1023}$ then an
      overflow exception is signaled and $\pm \infty$ is delivered where
      the sign is determined by $z$,
\item if $0 < \abs z \abs < MinDbl$ then the result may be zero, a denormalized
      number or $\pm MinDbl$ with the restriction that $z$ and the delivered
      result do not have opposite signs%
\footnote{The standard is actually more specific but this is of no 
                importance for this paper};
      in some cases the underflow is signaled,
\item if any argument of an operation is infinity or not a number, then the
      result is guaranteed to be infinity or not a number.
\end{itemize}
We need a relation between an approximable number $a$ and its floating point
approximation $round(a)$.
Assume $a \neq 0$ and let $e$ be such that $2^{e} \leq \abs a \abs \leq
2^{e+1}$.
Then $round(a) = a + \rho (a)$ where 
$\abs \rho (a) \abs \leq  2^{-53} \cdot 2^e$ and 
$2^{e} \leq \abs round(a) \abs < 2^{e+1}$.
Thus 
\begin{eqnarray*}
\frac{\abs a-round(a)\abs}{\abs a\abs} 
      \leq \frac{2^{-53} \cdot 2^e}{2^e} = 2^{-53}
\end{eqnarray*}
and
\begin{eqnarray*}
\frac{\abs a-round(a)\abs}{\abs round(a)\abs} 
      \leq \frac{2^{-53} \cdot 2^e}{2^e} = 2^{-53}.
\end{eqnarray*}


The quantity $2^{-53}$ is so important that it deserves a special name.
We call it the {\em precision} of floating point arithmetic and use |eps| 
to denote it.
By the two bounds derived above we have 
\begin{center}
$round(a) \in [ (1 - eps) \cdot a, (1 + eps) \cdot a ]  \subset 
              [ a / (1 + eps), a \cdot (1 + eps) ]
$
\end{center}
and
\begin{center}
$a \in [ (1 - eps) \cdot round(a), (1 + eps) \cdot round(a) ]$
\end{center}
for any non--zero approximable number $a$.

We will frequently derive expressions for error bounds.
These expressions involve the operators $+, *, /$ and have non--negative 
operands.
When such an expression is evaluated using floating point arithmetic,
further error incurs which must be compensated by post--multiplying 
with a correction factor.
We now show how to compute this factor.
We first determine for any expression $E$ a number $ind(E)$ such that
\begin{eqnarray*}
\hat{E} \in [ E / (1 + eps)^{ind(E)} , E \cdot (1 + eps)^{ind(E)} ]
\end{eqnarray*}
where $\hat{E}$ is the value of $E$ computed in floating point arithmetic.
We assume first that no underflow or overflow occurs in the computation of
$\hat{E}$. 
This implies that all intermediate results are approximable.
If $E$ is an operand then $ind(E) = 0$, if $E = E_1 + E_2$ then
($\oplus$ denotes floating point addition)
\begin{eqnarray*}
\hat{E} = \hat{E_1} \oplus \hat{E_2} \geq 
      \frac{\hat{E_1} + \hat{E_2}}{1 + eps} \geq
      \frac{ E_1 / (1 + eps)^{ind(E_1)} + E_2 / (1 + eps)^{ind(E_2)} }
           {1 + eps}
\end{eqnarray*}
and hence $ind(E_1 + E_2) = 1 + max(ind(E_1), ind(E_2))$,
and if $E = E_1 \cdot E_2$ or $E = E_1 / E_2$ then
\begin{eqnarray*}
\hat{E} = \hat{E_1} \odot \hat{E_2} \geq 
          \frac{\hat{E_1} \cdot \hat{E_2}}{1 + eps} \geq 
          \frac{E_1 \cdot E_2}{(1 + eps)^{1 + ind(E_1) + ind(E_2)} }
\end{eqnarray*}
and hence $ind(E_1 \cdot E_2) = 1 + ind(E_1) + ind(E_2)$ and analoguously
$ind(E_1/E_2) = 1 + ind(E_1) + ind(E_2)$.
The upper bound for $\hat{E}$ can be shown by similar reasoning.
We now choose our correction factor as $1 + k \cdot eps$ where $k$ is 
a power of $2$ and $(1 + eps)^{ind(E) + 1} \leq 1 + k \cdot eps$.
Then 
\begin{eqnarray*}
E \leq (1 + eps)^{ind(E)} \cdot \hat{E} \leq 
    \frac{(1 + k \cdot eps) \cdot \hat{E} }{1 + eps} \leq
    (1 \oplus k \odot eps) \odot \hat{E},
\end{eqnarray*}
where the last inequality follows from the fact that 
$1 \oplus k \odot eps = 1 + k \cdot eps$ since $k$ is a power of two.
For small values of $ind(E)$ one can choose $k$ to be the smallest power of two
with $ind(E) + 2 \leq k$.

What happens when there is overflow or underflow in the evaluation of $E$?
We detect overflow by inspecting the final result; it must be infinity
or not a number.
So assume that the evaluation of some subexpression $E_s$ underflows.
Let us also assume that there is no division operator on the path from the
subexpression $E_s$ to the root of the expression $E$ (in our error bounds
this will be the case for any subexpression that can underflow).
Then we may conceptually write $E$ as $E = E_{a(s)} \cdot E_s + E_{b(s)}$
since any expression over $+$ and $*$ is a linear function of each of its subexpressions.
Let $M_{a(s)}$ be a bound on $E_{a(s)}$.
The value of $\widehat{E_s}$ is less than $MinDbl$ and due to underflow it
may be set to zero.
The underflow of $E_s$ can therefore decrease the value of $E$ by no more
than $MinDbl \cdot M_{a(s)}$.
Summing over all subexpressions we conclude that underflow can decrease the
value of $E$ by no more than $MinDbl \cdot \sum_s M_{a(s)}$  where $s$ ranges
over all subexpressions of $E$.
This implies that we have $E \leq (1 + eps)^{ind(E)} \cdot \hat{E} +
MinDbl \cdot \sum_{s} M_{a(s)}$ where $\hat{E}$ is the computed value
(now with the possibility of underflow).
In our applications, we will always have $\hat{E} \geq eps$.
In that case, if we guarantee that $MinDbl \cdot (\sum_s M_{a(s)} / eps) 
\leq eps/2$ and choose $k$ such that
$(1 + eps)^{ind(E)+1} + (1 + eps) \cdot eps/2 \leq 1 + k \cdot eps$ then
\begin{eqnarray*} 
E &\leq& (1 + eps)^{ind(E)} \cdot \hat{E} + MinDbl \cdot \sum_s M_{a(s)} \\
  &\leq& ((1 + eps)^{ind(E)} + (MinDbl \cdot \sum_s M_{a(s)}) / eps)
            \cdot \hat{E} \\
  &\leq& ((1 + eps)^{ind(E)} + eps/2) \cdot \hat{E} \\
  &\leq& (1 + k \cdot eps) \cdot \hat{E} / (1 + eps) \leq 
           (1 \oplus k \odot eps) \odot \hat{E} \\
\end{eqnarray*}
We close with an example.
Let $E = eps + \abs x / z \abs \cdot \epsilon_x 
             + \abs y / z \abs \cdot \epsilon_y$.
The expression $E$ has index $4$ and the two multiplications and the two
divisions may underflow.
The sum of the multipliers of the ``dangerous'' subexpressions is 
$M := \epsilon_x + \epsilon_y + 1 + 1$. If $\epsilon_x$ and $\epsilon_y$
are both bounded by $2^{915}$ then clearly $MinDbl \cdot M \leq eps^2 /2$
and hence we may use the correction factor $1 + 8 \cdot eps$.
In fact, the factor $1 + 8 \cdot eps$ suffices for all bounds in this
paper provided that all incoming error bounds are bounded by $2^{915}$ 
and hence we define both quantities as global constants.

@<Global...@>=
double pow2(long);
const double eps = pow2(-53); 
const double correction = 1 + 8 * eps;
const double MaxError = pow2(915);
const double MinDbl = pow2(-1022); 
const double twoMinDbl = pow2(-1021);
const double MaxDbl = (2-pow2(-52))*pow2(1023);
const double Infinity = pow2(1024);
      
@ Bigfloats.

\label{Bigfloats}
LEDA's |bigfloat|s are a generalization of |double|s.
The differences are:
\begin{itemize}
\item The exponent range is the set of all integers (type |integer|) and hence
there is no underflow and overflow.
This implies that all numbers are approximable and that $round(a) = 0$ implies 
$a = 0$.
\item The mantissa length can be prescribed by the user. 
There are two ways to do this.
One way is to call the function |set_precision(int p)| to set the mantissa
length to $p$. 
The other way is to call |add(x,y,p)| for the addition of |bigfloat|s $x$ and
$y$ with mantissa length $p$, and analogously |sub|, |mul|, |div| for
subtraction, multiplication and division, respectively%
\footnote{The motivation for this syntax is to be able to evaluate different
operations in a large expression with different precisions.}.
\end{itemize}


@ We need functions to decide whether |integer|s and |bigfloat|s are 
exactly representable by |double|s.

@<Global...@>+=
  int isdouble(const integer& x)
  { @+ return ( x == bigfloat(bigfloat(x).todouble()) ); @+ }
  int isdouble(const bigfloat& x) 
  { @+ return (x == bigfloat(x.todouble())); @+ }

@ Also we use constants for zero as a |bigfloat| and as an |integer|.

@<Global...@>+=
  const bigfloat zero_bf = pZero;
  const integer zero = 0;

@* The implementation.

\label{Implementation}
The implementation defines the files real.h and real.c.
The first file contains the interface of class |real| and must be included 
into any program using |real|s.
The latter file can be compiled into real.o.
Any application of |real|s must specify real.o in the linking phase.

The public functions of class |real| are exactly the ones defined in
the manual. 

@(real.h@>=

/*{\Manpage {real} {} {The data type real} }*/

#ifndef REAL_H
#define REAL_H

/*{\Mdefinition

An instance $x$ of the data type |real| is an algebraic real number. 
There are many ways to construct a |real|: 
either by conversion from |double|, |bigfloat|, |integer| or |rational|
or by applying one of the arithmetic operators $+, -, *, /$ or $\sqrt{} $ to 
|real| numbers.
One may test the sign of a |real| number or compare two |real| numbers by
any of the comparison relations $=, \neq, < , \leq, >$ and $\geq$.
The outcome of such a test is {\em exact}.
There is also a non--standard version of the sign function: the call
|x.sign(integer q)| computes the sign of |x| under the precondition
that $\abs x\abs \leq 2^{-q}$ implies $x = 0$.
This version of the sign function allows the user to assist the data type in the
computation of the sign of $x$, see the example below.

There are also several functions to compute approximations of |real|s.
The calls |x.tobigfloat()| and |x.get_bigfloat_error()| return |bigfloat|s
$xnum$ and $xerr$ such that $\abs xnum - x \abs \leq xerr$. 
The user may set a bound on $xerr$.
More precisely, after the call |x.reduce_error_in_approximation_to(integer q)| the
data type guarantees $xerr \leq 2^{-q}$. 
One can also ask for |double| approximations of a real number $x$.
The calls |x.todouble()| and |x.get_double_error()| return |double|s $xnum$ and
$xerr$ such that $\abs xnum - x \abs \leq  \abs xnum \abs * xerr$. 
Note that $xerr = \infty$ is possible. 
}*/

#include <LEDA/bigfloat.h>
#include <LEDA/rational.h>
class real_rep; @#
class real @#
{ @#
  @<private data and functions of type real@>@;
public:
@#

/*{\Mcreation x
}*/

/*{\Mtext |real|s may be constructed from data types |double|, |bigfloat|,
 |long|, |int| and |integer|.
 The default constructor |real()| initializes the |real| to zero.
}*/

  @<real constructors@>@;

  ~real();

  real& operator=(const real&);

/*{\Moperations 1.5 3.5
}*/

  double todouble() const; 
/*{\Mop returns the current double approximation of \var. 
}*/

  bigfloat tobigfloat() const; 
/*{\Mop returns the current bigfloat approximation of \var. 
}*/

  double get_double_error() const;
/*{\Mop returns the relative error of the current double approximation of \var, 
    i.e., $\abs\mbox{|x|} - \mbox{|x.todouble()|}\abs \leq 
    \mbox{|x.get_double_error()|} \cdot \abs \mbox{|x.todouble()|} \abs$.
}*/

  bigfloat get_bigfloat_error() const;
/*{\Mop returns the absolute error of the current bigfloat approximation of
   |x|, i.e., $\abs\mbox{|x|} - \mbox{|x.tobigfloat()|} \abs \leq
    \mbox{|x.get_bigfloat_error()|}$.
}*/

  int sign() const;
/*{\Mop returns $-1$ if (the exact value of) \var\ is $< 0$, $1$ if \var\ 
   $> 0$, $0$ if \var\ is $= 0$. 
}*/

  int sign(const integer& q) const; 
/*{\Mop as above. {\em Precondition:} if $\abs \mbox{|x|} \abs  \leq  2^{-q}$ 
   then $\mbox{|x|} = 0$.
}*/

  int sign(long) const;
 
  void reduce_error_in_approximation_to(const integer& q) const;
/*{\Mopl (re-)computes the approximation of \var\ such that
  $\mbox{|x.get_bigfloat_error()|} \leq 2^{-q}$ after the call
  |x.reduce_error_in_approximation_to(q)|.
}*/

  void compute_with_mantissa_length(long k) const;
/*{\Mopl (re-)computes the approximation of \var; each numerical operation 
   is carried out with $k$ binary places. 
}*/

  void guarantee_relative_error_two_to_minus(long k) const;
/*{\Mopl (re-)computes an approximation of \var\ such that the relative error 
    of the approximation is less than $2^{-k}$. 
}*/

  friend ostream& operator<<(ostream& O, const real& x);
/*{\Mbinopfunc writes the closest interval that is known to contain $x$
    to the output stream $O$
}*/
  friend istream& operator>>(istream& I, real& x); 
/*{\Mbinopfunc reads |real| number |x| from the output stream $I$
   (in |double| format)
}*/


  @<Comparison operators@>@;
  @<Arithmetic operators@>@;
};
  @<LEDA functions@>@;
  @<Assignment operators@>@;
  @<Declaration of mathematical functions in real.h@>@;

#endif

@ The following constructors are available.

@<real constructors@>=
  real();
  real(double);
  real(int);
  real(const integer&);
  real(const bigfloat&);
  real(const rational&);
  real(const real&);
  real(real_rep&);

@ We have the full list of comparison operators.

@<Comparison operators@>=
 friend int operator<(const real&,const real&);
 friend int operator<=(const real&,const real&);
 friend int operator>(const real&,const real&);
 friend int operator>=(const real&,const real&);
 friend int operator==(const real&,const real&);
 friend int operator!=(const real&,const real&);

@ All arithmetic operators are friends of classes |real| and |real_rep|.

@<Arithmetic operators@>=
  friend real operator+(const real&,const real&);
  friend real operator-(const real&,const real&);
  friend real operator*(const real&,const real&);
  friend real operator/(const real&,const real&);
  friend real operator-(const real&);
  friend real sqrt(const real&);

@ We also have some fancy assignment operators.

@<Assignment operators@>+=
 inline real operator +=(real& x,const real& y) 
 { @+ x = x+y; @+ return x; @+ }
 inline real operator -=(real& x,const real& y) 
 { @+ x = x-y; @+ return x; @+ }  
 inline real operator *=(real& x,const real& y) 
 { @+ x = x*y; @+ return x; @+ }

@ The following functions have to be defined for every LEDA type.

@<LEDA functions@>=
  inline void Print(const real& x, ostream& out) @+ { @+ out << x; @+ }
  inline void Read(real& x, istream& in) @+ { @+ in >> x; @+ }
  inline int compare(const real&x, const real& y) @+ 
  { @+ return (x-y).sign(); @+}
  inline char* Type_Name(const real*) @+ { @+ return "real"; @+ }

@ Mathematical functions. This list is still small.

@<Declaration of mathematical functions...@>=
   real fabs(const real& x);
/*{\Mfunc absolute value of |x|
}*/
   real sq(const real& x); 
/*{\Mfunc square of |x| 
}*/
   real dist(const real& x,const real& y); 
/*{\Mfunc euclidean distance of vector (x,y) to the origin
}*/ 
   real powi(const real& x, int n); 
/*{\Mfunc |n|.th power of |x| 
}*/

/*{\Mimplementation

A \type\ is represented by the expression which defines it and a |double|
approximation $\hat{x}$ together with a relative error bound
$\epsilon_{x}$.
The arithmetic operators $+, -, *, \, \sqrt{}$ take constant time.
When the sign of a \type\ number needs to be determined, the data type first
computes a number $q$, if not already given as an argument to |sign|, such that 
$\abs \mbox{\var} \abs \leq 2^{-q}$ implies $x = 0$.
The bound $q$ is computed as described in \cite{Mignotte:book}.
The data type then computes a bigfloat approximation $xnum$ for \var\ with
error bound $xerr \leq 2^{-q}$. 
The sign of $xnum$ is then returned as the sign of \var.

Two shortcuts are used to speed up the computation of the sign. 
Firstly, if the |double| approximation already suffices to determine the sign,
then no |bigfloat| approximation is computed at all.
Secondly, the |bigfloat| approximation is first computed only with small precision.
The precision is then doubled until either the sign can be decided (i.e.,
if $xerr < \abs xnum \abs$) or the full precision $2^{-q}$ is reached.
}*/

/*{\Mexample
We give two examples of the use of the data type \type. 
The examples deal with the Voronoi diagram of line segments and the intersection
of line segments, respectively. 

The following incircle test arises in the computation of Voronoi diagrams of
line segments \cite{BMS:ESA}.
For $i$, $1 \leq i \leq 3$, let $l_i : a_i x + b_i y + c_i = 0$ be a line
in two--dimensional space and let $p = (0,0)$ be the origin.
In general, there are two circles passing through $p$ and touching $l_1$ and
$l_2$.
The centers of these circles have coordinates
\begin{eqnarray*}
x_v = \frac{a_1 c_2 +a_2 c_1 \pm  \sqrt{2 c_1 c_2 (\sqrt{N} + C)}}
           {\sqrt{N} - a_1 a_2 - b_1 b_2}  , 
y_v = \frac{b_1 c_2 +b_2 c_1 \pm  \sqrt{2 c_1 c_2 (\sqrt{N} - C)}}
           {\sqrt{N} - a_1 a_2 - b_1 b_2}
\end{eqnarray*}
where $N = (a_1^{2} + b_1^{2}) (a_2^{2} + b_2^{2})$ and $C = a_1 a_2 - b_1 b_2$.
Let us concentrate on one of these (say, we take the plus sign in both cases).
The test whether $l_3$ intersects, touches or misses the circle amounts to
determining the sign of
\begin{eqnarray*}
D := dist^{2}(v,l_3) - dist^{2}(v,p) = 
     \frac{(a_3 x_v + b_3 y_v + c_3)^2}{a_3^2 + b_3^2} - (x_v^2 + y_v^2)
\end{eqnarray*}
We show how to compute the sign of $\tilde{D} := (a_3^2 + b_3^2) \cdot D$ using
our data type \type. 
We assume that all coefficients $a_i, b_i$ and $c_i$, $1 \leq i \leq 3$,
are $k$ bit integers, i.e., integers whose absolute value is bounded 
by $2^k -1$.
In \cite{BMS:ESA} we showed:
\begin{itemize}
\item if $\tilde{D} \neq 0$ then $\abs \tilde{D} \abs \geq 2^{-24 k - 50}$,
\item if $N$ is a square and $\tilde{D} \neq 0$ then $\tilde{D} \geq 
          2^{-8 k - 18}
         $
\end{itemize}
where $k_1$ and $k_2$ are integer constants.
These observations suggest the following program.
\begin{quote}
|int incircle(real k, a1, b1, c1, a2, b2, c2, a3, b3, c3)| \\
|{ | \\
| real N = (a1*a1 + b1*b1) * (a2*a2 + b2*b2); | \\
| real C = a1 * a2 - b1 * b2; | \\
| real S = - (a1 * b2 + a2 * b1); | \\
| real D = sqrt(N) - (a1 * a2 + b1 * b2); | \\
| real x_v = (a1 * c2 + a2 * c1 + sqrt(2*c1*c2*(sqrt(N) + C)) / D; | \\
| real y_v = (b1 * c2 + b2 * c1 + S.sign() * sqrt(2*c1*c2*(sqrt(N) - C)) / D; |
 \\
| real E = (a3 * x_v + b3 * y_v + c3)*(a3 * x_v + b3 * y_v + c3)
           - (a3*a3 + b3*b3)*(x_v*x_v + y_v*y_v); | \\
| int max_prec = (is_square(N) ? 8*k + 18 : 24*k + 50); |\\
| return E.sign(max_prec); | \\
| } | \\
\end{quote}

We turn to the line segment intersection problem next. 
Assume that all endpoints have $k$--bit integer homogeneous coordinates.
This implies that the intersection points have homogeneous coordinates 
$(X,Y,W)$ where $X,Y$ and $W$ are (4 k + 3) - bit integers.
The Bentley--Ottmann plane sweep algorithm for segment intersection
\cite{Me-Naeher:sweep} needs to sort points by their $x$--coordinates, i.e.,
to compare fractions $X_1 / W_1$ and $X_2 / W_2$ where $X_1, X_2, W_1, W_2$
are as above.
This is tantamount to determining the sign of the $8 k + 7$ bit integer 
$X_1 * W_2 - X_2 * W_1$.
If all variables $X_i, W_i$ are declared {\bf real} then their sign
test will be performed quite efficiently.
First, a |double| approximation is computed and then, if necessary,
|bigfloat| approximations of increasing precision.
In many cases, the |double| approximation already determines the sign.
In this way, the user of the data type |real| gets the efficiency of a 
floating point filter \cite{Fortune-vWijk,Me-Naeher:IFIP94} without
any work on his side.
This is in marked contrast to \cite{Fortune-vWijk,Me-Naeher:IFIP94} and
will be incorporated into \cite{Me-Naeher:sweep}.

}*/


   

@ The file real.c has a similar structure as the file real.h.
For each group of functions we have a section containing the implementation.
In addition, we define some constants and the class |real_rep|.

\label{real.c}
@(real.c@>=
#include <LEDA/real.h>
#include <math.h>
@<Basic declarations@>@;
@<Global functions and constants@>@;
@<Declaration of class |real_rep|@>@;
@<Constructors and destructors@>@;
@<Basic functions@>@;
@<Operators@>@;
@<Implementation of improve...@>@;
@<Sign functions@>@;
@<Implementation of mathematical functions in real.c@>@;

@ Realisation of class real.

A |real| may be exactly representable by a |double|.
In this case, we call it a {\bf basic} |real|.
Otherwise, if a |real| is not (known to be) a |double|, it is defined
by an expression tree whose leaves are given by basic |real|s.
In that case we maintain the expression tree and the real only needs to know
a pointer to the root of the tree.
The nodes of expression trees are realized by the class |real_rep| which is 
the work horse of our implementation.
The member functions and all operations, except the arithmetic operators, 
are first implemented on |real_rep|s and then lifted to |real|s.
The design to separate an object of type |real| from its tree representation
has the following advantages:
\begin{itemize}
\item Copying a |real| takes constant time since it basically amounts to 
copying a pointer to a |real_rep|.
\item Since reals may share subtrees, we have to provide our own
 memory management. 
The part of the information that can be automatically allocated is stored 
in class |real| itself, and the dynamically allocated part is stored in
class |real_rep|.
\end{itemize}

@ The private data of class |real| is organized as follows.
\begin{itemize}
\item If |is_double| is true then the |real| is basic and the value is stored
      in the member |value|.
      If the |real| is part of an expression tree then |PTR| is defined and
      points to a |real_rep| also containing the value.
      Storing the value twice simplifies programming; of course, it also wastes
      space.
\item If |is_double| is |false|, |PTR| points to the root of an expression tree
      defining the |real|; the content of |value| is meaningless in this case.
\end{itemize}

@<private data...@>=
bool is_double;
double value;
real_rep* PTR;

@ The class |real_rep|.

A |real_rep| object is either a leaf, i.e., represents a |double| number or a
|bigfloat| number, or it is constructed by an arithmetic operation from
two other |real_rep| objects.
We define the enumeration type |constructortype| to distinguish these cases.  

A |real_rep| may contain a |bigfloat| approximation to the exact value of the
represented number.
Since this approximation is space--consuming and not always needed we
encapsulate it in a structure called |bf_app|.
The structure is organized as follows.
\begin{itemize}
\item |NUM_BF| is the |bigfloat| approximation,
\item |ERROR_BF| bounds the error of the approximation |NUM_BF|, i.e.,
      $\abs$|NUM_BF - NUM_EXACT|$\abs \leq |ERROR_BF|$.
\end{itemize}

@<Basic declarations@>+=
enum constructortype { DOUBLE = 0, BIGFLOAT = 1, NEGATION = 2,
SQUAREROOT = 3, ADDITION = 4, SUBTRACTION = 5,  MULTIPLICATION = 6, 
DIVISION = 7 };

struct app_bf  
{ bigfloat NUM_BF;
  bigfloat ERROR_BF; 
  app_bf() @+ {}
  app_bf(const bigfloat& x, const bigfloat& error_x = pZero)
  { @+ NUM_BF = x; @+ ERROR_BF = error_x; @+ }
};

@ We can now define the class |real_rep|.
Any object of class |real_rep| contains a |double| approximation |NUM|,
a |double| |ERROR|, a pointer |APP_BF_PTR| to a |bigfloat| approximation
(of type |app_bf|), the |constructortype| |CON|, pointers |OP1| and |OP2|
to the two operands, and a {\em count} of the number of pointers to the object.
Furthermore there is a long member |status| that we use for efficient
searching through the expression trees and a pointer |sep_bound_ptr|
to a structure of type |sep_bound| that we define in the section about 
sign computation.

We will always use |NUM_EXACT| to denote the exact value of a |real_rep|.
The quantities |NUM|, |ERROR|, |NUM_BF| and |ERROR_BF| satisfy the following
inequalities: 
Either |ERROR =| $\infty$ or |ERROR <= MaxError|, |NUM = 0| or 
|MinDbl <= NUM <= MaxDbl| (i.e., |NUM| is not a denormalized number) and
\begin{eqnarray*}
\abs |NUM_EXACT| - |NUM| \abs  \;\leq\; \abs |NUM| \abs \cdot |ERROR|.
\end{eqnarray*}
If |APP_BF_PTR != nil| then 
\begin{eqnarray*} 
\abs |NUM_EXACT| - |NUM_BF| \abs \;\leq\; |ERROR_BF| < \infty. 
\end{eqnarray*}
Note that |ERROR| is the {\em relative} error of the |double| approximation
and |ERROR_BF| is the {\em absolute} error of the |bigfloat| approximation.
Also note that the |bigfloat| approximation may not exist 
(if |APP_BF_PTR == nil|) and that some numbers have no |double| approximation
 (if |ERROR ==| $\infty$).
However, if the |bigfloat| approximation of a |real_rep| is defined, then it is 
also defined for its children.

If a |real| has a |double| and a |bigfloat| approximation then the |double|
approximation is {\em essentially} the conversion of the |bigfloat| value to
|double|.
Since our algorithms do not exploit any connection between the |double| and the
|real| approximation we formalize no connection.

The member functions of class |real_rep| do the following:
\begin{itemize}
\item |init_app_bf()| initializes the |bigfloat| approximation,
\item |adjust_dbl()| adjusts the |double| approximation to an improved
      |bigfloat| approximation,
\item |compute_op(int p)| computes the last operation (given by member |CON|) in
      precision $p$, using bigfloat arithmetic,
\item |compute_approximation| implements the function
      |real::compute_with_mantissa_length|,
\item |sign_with_separation_bound(int k, bool supported)| returns the sign of
      the represented number $x$;
      if the boolean parameter |supported| is true, the sign computation is
      supported by the information that $\abs x \abs \leq 2^k$ implies
      $x = 0$,
\item |guarantee_bound_two_to(integer e)| improves the |bigfloat| approximation
      of the number such that |ERROR_BF| is bounded by $2^e$,
\item |num_bf()| and |error_bf()| return references to the values
      |APP_BF_PTR->NUM_BF| and |APP_BF_PTR->ERROR_BF|, 
\item |num_exp()| and |err_exp()| compute the binary logarithms of nonzero
      |NUM_BF| and |ERROR_BF|, rounded up towards the next integer,
\item |exact()| tells whether |ERROR_BF| is zero.
\end{itemize}

@<Declaration of class |real_rep|@>=
struct sep_bound;@;
class real_rep 
{ @#
  friend class real;
  @#
  double NUM;
  double ERROR; 
  app_bf* APP_BF_PTR; 
  constructortype CON;
  real_rep *OP1, *OP2; 
  int count;
  long status;
  sep_bound* sep_bound_ptr;
@#
  real_rep();
  real_rep(double); 
  real_rep(const bigfloat&);

  ~real_rep(); @/

  LEDA_MEMORY(real_rep)@;@#
  
  void init_app_bf();@;
  void adjust_dbl();@;
  void compute_op(long); 
  void compute_approximation(long);@; 
  int sign_with_separation_bound(const integer&,bool=false);@;
  void guarantee_bound_two_to(const integer&);@;
  inline bigfloat& num_bf() @+ { @+ return APP_BF_PTR->NUM_BF; @+ }
  inline bigfloat& error_bf() @+ { @+ return APP_BF_PTR->ERROR_BF; @+ }
  inline bool exact() @+ { @+ return error_bf().isZero(); @+ }
  inline integer num_exp() @+ { @+ return log2(num_bf()); @+ } 
  inline integer err_exp() @+ { @+ return log2(error_bf()); @+ } 

  @<Local functions of class |real_rep|@>@;

  friend ostream& operator<<(ostream&,const real&);
  @<Arithmetic operators@>@;
  @<Comparison operators@>@;

};


@* Constructors and destructors.

We start with the constructors for |class real|.
The first three construct basic |real|s from an empty argument list, from a
|double|, and from an |int|.
They are easy to implement.
The next two -- for |integer| and |bigfloat| -- construct |real|s that in general
are not representable by |double|s but are defined by a single |real_rep| node.
Rationals are constructed differently.
We first construct |real|s for numerator and denominator, respectively, and 
then return their |real| division.
If the rational is not a |double|, the result is a pointer to a tree with three
nodes.
The constructor who takes an |real_rep| argument simply assigns the adress
of the |real_rep| to PTR and sets |is_double == false|.

@<Constructors...@>=
  real::real() @+ { @+ is_double = true; @+ value = 0; @+  PTR = nil;  @+ }

  real::real(int x) @+ { @+ is_double = true; @+ value = x; @+ PTR = nil; @+ }

  real::real(double x) { @+ is_double = true; @+ value = x; @+ PTR = nil; @+ }
  
  real::real(const bigfloat& x) 
  { 
    if (is_double = isdouble(x)) 
    {  
       value = x.todouble();
       PTR = nil;
    }
    else
       PTR = new real_rep(x);
  }

  real::real(const integer& x) 
  { 
    if (is_double = isdouble(x)) 
    {  
       value = x.todouble();
       PTR = nil;
    }
    else
       PTR = new real_rep(bigfloat(x));
  }

  real::real(const rational& x)  
  {  
    real OP1 = *new real_rep(x.numerator());
    real OP2 = *new real_rep(x.denominator());
    *this = OP1 / OP2;
  }
  
  real::real(real_rep& x_rep) @+ { @+ is_double = false; @+ PTR = &x_rep; @+ }

@ Copy constructor and assignment operator are treated in direct succession
because they are closely related. 
Both functions distinguish the two possible cases of |is_double|.
If the argument is not basic,
the copy constructor simply copies the pointer to the |real_rep| of its
arguments and records the fact that the number of |real|s sharing the
representation has increased by one.
The assignment operator does basically the same, but is difficult to
implement correctly.
Notice that we must take care that
\begin{enumerate}
\item an existing pointer |x.PTR != nil| is {\em not} copied for 
      |x.is_double == true|
\item an existing reference |PTR| is correctly deleted
\item everything remains correct even for |x.PTR == PTR| or even |*this = x|
\end{enumerate}
In addition, if |PTR| pointed to some |real_rep| node, it decreases the count of
this node by one and deletes the node if the count goes to zero. 

@<Constructors...@>=
  real::real(const real& x) 
  {  
     if (is_double = x.is_double) @+ { @+ value = x.value; @+ PTR = nil; @+ }
     else @+ { @+ x.PTR->count++; @+ PTR = x.PTR; }
  }

  real& real::operator=(const real& x)
  { 
     @#
     /* we avoid problems by ruling out the assignment |x = x;| */
     if (this == &x)
        return (*this);

     /* first the actions that must anyway be performed, in the respective cases
        of |is_double| */
     if (is_double = x.is_double)
        value = x.value;
     else
        x.PTR->count++;

     /* second, an existing reference in PTR must always be deleted;
        if |x.PTR == PTR != nil| and |is_double == false|, this leaves the
        number of references to |*PTR| unchanged, because we increased
        |x.PTR| before, and for |is_double == true| we in fact lose one 
        reference because |PTR| is set to |nil| at the end 
     */
     if (PTR != nil) @+ if (--PTR->count == 0) 
       delete PTR;

     /* last, we set |this->PTR| */
     if (is_double) @+ PTR = nil; @+ else @+ PTR = x.PTR;

     return (*this);
  }

@ Beside the constructors for class |real| we need three constructors for
class |real_rep|.
The first one is the straightforward empty constructor.
The second one takes |double|s and is only needed when one of the |real|
operators needs to initialize a leaf representing a basic |real|.
The third constructor takes |bigfloat|s and is used by the corresponding
|real::| constructor.
Note that even in this case we have to initialize the |double| approximation
by calling the function |adjust_dbl()|.
It is defined in section \ref{basic functions}.

@<Constructors...@>=
  real_rep::real_rep() 
  { @+ APP_BF_PTR = nil; @+ sep_bound_ptr = nil;
    @+ count = 1; @+ status = 0; @+ 
  }
  real_rep::real_rep(double x) 
  { @+ APP_BF_PTR = nil; @+ sep_bound_ptr = nil;
    OP1 = OP2 = nil; @+ CON = DOUBLE;
    NUM = x; @+ ERROR = 0;  
    count = 1; @+ status = 0;
  }
  real_rep::real_rep(const bigfloat& x) 
  { 
    APP_BF_PTR = new app_bf(x,0); @+ sep_bound_ptr = nil;
    OP1 = OP2 = nil; @+ CON = BIGFLOAT; 
    adjust_dbl();
    count = 1; @+ status = 0;
  }

@ We discuss the destructors of classes |real| and |real_rep|.
When a |real| is destroyed we simply decrease the counter of the corresponding
|real_rep| (if it exists).
When the counter reaches zero we also delete the |real_rep|.
In order to delete a |real_rep| we free the storage allocated for the 
|real_rep| (this is done automatically except for the storage allocated for
the optional |bigfloat| approximation) and decrease the counters of the 
operands (if any) defining the |real_rep|.
If any counter reaches zero we destroy the operands recursively.

@<Constructors...@>+=
  real::~real() 
  {  @+ if (PTR != nil) @+ if (--PTR->count == 0) @+ delete PTR; 
  } @#

  real_rep::~real_rep() 
  { 
    if (APP_BF_PTR != nil) 
      delete APP_BF_PTR;
    if (sep_bound_ptr != nil)
      delete sep_bound_ptr;
    if ( (OP1 != nil) && (--OP1->count == 0)) 
      delete OP1;
    if ( (OP2 != nil) && (--OP2->count == 0)) 
      delete OP2;
#ifdef DEBUG_REAL
      APP_BF_PTR = nil;
      OP1 = OP2 = nil;
#endif
  }


@* Basic functions.

Class |real_rep| has the basic functions |init_app_bf|, |adjust_dbl|, |num_exp|
and |err_exp|, and the basic functions of class |real| are |todouble|,
|get_dbl_error|, |tobigfloat|, |get_precision|, and the stream operators |<<|
and |>>|.
We start with the basic functions of class |real_rep|.

@ The function |init_app_bf()| initializes the |bigfloat| approximation
of a |real_rep| (if not defined already).
If the |double| approximation is meaningful then we simply convert it
to |bigfloat| format.
Otherwise the functions calls itself recursively for the operands and 
computes the operation given by |CON| in precision $32$.

@<Basic functions@>+=
 void real_rep::init_app_bf()
  {  
    if (APP_BF_PTR != nil) return;
    if (OP1 != nil) OP1->init_app_bf();
    if (OP2 != nil) OP2->init_app_bf();
    if (ERROR < Infinity)
    {   
        APP_BF_PTR = 
          new app_bf(bigfloat(NUM),fabs(bigfloat(ERROR*NUM*correction)));
        return;
    }
    APP_BF_PTR = new app_bf();
    compute_op(53);
  }

@ The function |adjust_dbl()| adjusts the |double| approximation
$(|NUM|, |ERROR|)$ to the current |bigfloat| approximation (which is guaranteed
to exist when |adjust_dbl| is called).
The adjustment is non--trivial since the |bigfloat| error bound is absolute 
and the |double| error bound is relative and since we have to take care of
underflow.
We need to distinguish several cases, two of which are simple.
If $\abs |NUM_BF| \abs  > |MaxDbl|$ or |ERROR_BF > MaxDbl| then we simply set
|ERROR| to infinity and if |NUM_BF = ERROR_BF = 0| then we set 
|NUM = ERROR = 0|.
So assume that neither is the case.
Let $n = |NUM_EXACT|$, $nbf$ = |NUM_BF|, $ebf$ = |ERROR_BF|, $\hat{e}$ = |double(ERROR_BF)|, and
$\hat{n}$ = |double(NUM_BF)|.
Assume first that $\hat{n} \neq 0$.
Then
\begin{eqnarray*}
  \abs \hat{n} - n \abs 
     & \leq & \abs \hat{n} - nbf \abs + \abs nbf - n \abs \\
     & \leq & eps \cdot \abs \hat{n} \abs +  ebf \\
     & \leq & (eps + ebf/\abs \hat{n}\abs)\cdot\abs \hat{n}\abs \\
     & \leq & (eps + Max(MinDbl, (1 + eps) \cdot \hat{e})/\abs \hat{n}\abs)
              \cdot \abs \hat{n}\abs,
\end{eqnarray*}
since $ebf \leq Max(MinDbl, (1+eps) \hat{e})$.
The formula above has index 4 (cf. section \ref{Floats}) and hence
$1 + 8 eps$ is a suitable correction factor for its floating point evaluation
(this correction factor even allows to drop the multiplication of $\hat{e}$
by $(1 + eps)$).
Assume next that $\hat{n} = 0$.
Then $\abs n\abs \leq ebf + MinDbl \leq (1 + eps) \cdot \hat{e}+2  MinDbl$.
We set $|NUM| = \hat{e} \oplus 2  MinDbl$.
Then 
\begin{eqnarray*}
\abs n - |NUM|\abs \leq 2 (1 + eps)^2 |NUM| \leq 2 \odot (1 + 8 eps) \odot
 |NUM|.
\end{eqnarray*}
In either case, if |ERROR| exceeds |MaxError| (this will also be the case when
the division $\abs \hat{e}\abs / \abs \hat{n}\abs$ overflows) we invalidate
our computation and set |ERROR| to infinity.
\label{basic functions}

@<Basic functions@>+=
  void real_rep::adjust_dbl()
  {
   double n_head =   num_bf().todouble();
   double e_head = error_bf().todouble();
   if (n_head != 0)
   { NUM = n_head;
     ERROR = eps + Max(MinDbl,e_head)/fabs(n_head); 
   }
   else
   { NUM = e_head + 2 * MinDbl;
     ERROR = 2;
   }
   ERROR = ERROR * correction;
   if ((ERROR >= MaxError) || (fabs(n_head) == Infinity))
   { NUM = 1; @+ ERROR = Infinity;
   }
  }

@ We declare a private function |real::adjust| to adjust the state of a real.
That is, we check whether a real is exactly a |double| and if so, we turn it
into a basic |real|.
In any case we adjust the |double| approximation.
Not that we have to cast away the constancy of the |real| before changing its
private data.

@<private data...@>+=
void adjust() const;

@
@<Basic functions@>+=
  void real::adjust() const
  {
    if (is_double) return;
    if (PTR->exact() && isdouble(PTR->num_bf()))
    {
      ((real&) *this).is_double = true;
      ((real&) *this).value = PTR->num_bf().todouble();
      return;
    }
    PTR->adjust_dbl();
  }


@ We give out a |real| as an interval containing its exact value.
The size of the interval depends on the quality of the |bigfloat| approximation.

We assume that |real| input reads from a |double| variable.
 
@<Basic functions@>+=
  ostream& operator<<(ostream& out, const real& x)
  {   bigfloat low_bound, upp_bound;
      long prec;
      if (x.is_double) low_bound = upp_bound = x.value;
      else
      { x.PTR->init_app_bf();
        prec = (log2(x.PTR->num_bf())-log2(x.PTR->error_bf())).tolong();
        if (prec < 53) prec = 53;
        low_bound =
      sub(x.tobigfloat(),x.PTR->error_bf(),prec,TO_N_INF);
        upp_bound = 
      add(x.tobigfloat(),x.PTR->error_bf(),prec,TO_P_INF);
      }
      out << "[" << low_bound <<",";
      out <<  upp_bound << "]";
      return out;
  }
  istream& operator>>(istream& in, real& x)
  { @+
    double x_num; @+ in >> x_num; 
    x = real(x_num); @+ return in;
  } 

@ The other basic functions of class |real| do the obvious things.

@<Basic functions@>+=
  double real::todouble() const
  {  if (is_double)
        return value;
     if (PTR->APP_BF_PTR != nil)
       PTR->adjust_dbl();
     return PTR->NUM;
  }
  double real::get_double_error() const
  {  if (is_double)
        return 0;
     if (PTR->APP_BF_PTR != nil)
       PTR->adjust_dbl();
     return PTR->ERROR;
  }
  bigfloat real::tobigfloat() const
  {  if (is_double)
        return value;
     else
     {  PTR->init_app_bf(); 
        return PTR->num_bf(); 
     }
  }
  bigfloat real::get_bigfloat_error() const
  {  if (is_double)
        return 0;
     else
     {  PTR->init_app_bf(); 
        return PTR->error_bf(); 
     }
  }

  

@ Now we list several auxiliary functions that we use in later sections.
We need to compute 
\begin{itemize}
\item |ldexp_bf(x,k)| $= x \cdot 2^k$ for |bigfloat|s |x| and integers |k|,
\item the maximum of two integers,
\item the sign of a |double|.
\end{itemize}

@<Global...@>+=
inline bigfloat ldexp_bf(const bigfloat& x, const integer& k)
{ @+ return bigfloat(x.get_significant(),x.get_exponent() + k); @+ }
integer Maximum(integer x, integer y) @+ { @+ return (x > y ? x : y); @+ }
int sign(double x) 
{ @+ if (x == 0) return 0; @+ else return (x > 0 ? 1 : -1); @+ }


@* The arithmetic operations.

We give the implementation of the arithmetic operators $+, -, *, /$ and 
$\sqrt{}$.
The implementations have a common structure.
If both operands are basic, we check if the |double| operation already 
gives an {\em exact} result which may then be returned immediately.
We believe that the time for checking this is well invested because,
if the check succeeds
\begin{enumerate}
\item it saves space,
\item it saves the time for computing error bounds and doing dynamic memory
      allocation,
\item it may detect ``degenerate'' values without expensive |bigfloat|
arithmetic.
\end{enumerate}
If one of the operands is not a |double|, or if the result is not exactly 
representable in |double| format, we create |real_rep| nodes for the
operands, if necessary, and also a we create new node for the result $z$.
There we record the type of the operation in the |CON| field, store pointers to
the operands, increase the counters of the operands, and then compute the
|double| approximation and the error bound for $z$.

Clearly, if either operand is not approximable (or no approximation is known),
we cannot compute an approximation of $z$.
So in our analysis we tacitly assume that both operands are approximable.

@ We start with the binary operations addition, subtraction,
and multiplication. 

@<Operators@>+=
 real operator+(const real& x,const real& y)
   { 
     @<if addition is exact return double result@>@;
     @<create nodes for two operands@>@;
     real_rep& z_rep = *new real_rep(); 
     @<initialize |z_rep| as result of a binary operation@>@;
     z_rep.CON = ADDITION;
     z_rep.NUM = x.PTR->NUM + y.PTR->NUM;
     @<compute error for addition or subtraction@>@;
     @<if approximation is not valid set error to Infinity@>@; 
     return z_rep;
   }
   real operator-(const real& x,const real& y)
   { 
     @<if subtraction is exact return double result@>@;
     @<create nodes for two operands@>@;
     real_rep& z_rep = *new real_rep(); 
     @<initialize |z_rep| as result of a binary operation@>@;
     z_rep.CON = SUBTRACTION;
     z_rep.NUM = x.PTR->NUM - y.PTR->NUM;
     @<compute error for addition or subtraction@>@;
     @<if approximation is not valid set error to Infinity@>@;
     return z_rep;
   }
   real operator*(const real& x,const real& y)
   { 
     @<if multiplication is exact return double result@>@;
     @<create nodes for two operands@>@;
     real_rep& z_rep = *new real_rep(); 
     @<initialize |z_rep| as result of a binary operation@>@;
     z_rep.CON = MULTIPLICATION;
     z_rep.NUM = x.PTR->NUM * y.PTR->NUM;
     @<compute error for multiplication@>@;
     @<if approximation is not valid set error to Infinity@>@;
     return z_rep;
   }

@ We give the common parts of the implementations first. 
First we create nodes for the operands |x| and |y|, if not already existing.
We simply have to use the |real_rep| constructor.
Note that we have to cast away the constancy of |x| and |y| away before
manipulating their data.

@<create nodes for two operands@>=
    if (x.PTR == nil) ((real&) x).PTR = new real_rep(x.value);
    if (y.PTR == nil) ((real&) y).PTR = new real_rep(y.value);
    double& x_num = x.PTR->NUM; 
    double& y_num = y.PTR->NUM;

@ We initialize the result |z_rep|.

@<initialize |z_rep|...@>=
    z_rep.OP1 = x.PTR; @+ z_rep.OP2 = y.PTR;
    z_rep.OP1->count++; @+ z_rep.OP2->count++;
    double& NUM = z_rep.NUM;

@ Checking the validity of the |double| approximation is as follows.
If one of the incoming error bounds is |Infinity|, our new error bound cannot
be valid.
The same holds if the computed approximation |z_rep.NUM| is |NaN| or 
$\pm\infty$, or if the computed error bound |z_rep.NUM| is larger than
|MaxError| or |NaN|.
Note that we do not need to check for the |NaN| cases separately because
a comparison with a |NaN| always gives the answer ``no''.

@<if approximation is not valid...@>=
   if (! (    (fabs(NUM) + x.PTR->ERROR + y.PTR->ERROR < Infinity) 
            && (z_rep.ERROR < MaxError) 
          )
       )
    {
      z_rep.ERROR = Infinity;
      NUM = 1;
    }

@ Now we skip to the addition--specific parts.
It is possible to test efficiently whether a |double| addition
$s = a + b$ is exact.
In fact, one can show that -- independent of which rounding mode is active
or whether the operands are normalized or denormalized --,
one of the two subtractions $(a \oplus b) \ominus b$ or 
$(a \oplus b) \ominus a$ is exact.
This implies immediately that the addition is exact if and only if both of the 
following equalities hold (which is easy to test):
$$ (a \oplus b) \ominus b = a \wedge (a \oplus b) \ominus a = b.$$

@<if addition is exact return double result@>=
  double NUM_result;
  if (x.is_double && y.is_double)
  {  NUM_result = x.value + y.value;
     if (     (NUM_result - x.value == y.value ) 
           && ( (NUM_result - y.value) == x.value) 
        )
        return NUM_result;
  }

@ The same test procedure applies to subtraction.

@<if subtraction is exact return double result@>=
  double NUM_result;
  if (x.is_double && y.is_double)
  {  NUM_result = x.value - y.value;
     if (    (NUM_result + y.value == x.value)
          && (x.value - NUM_result == y.value)
        )
        return NUM_result;
  }


@ Now we give the error analysis for addition and subtraction.
Because they are essentially the same, we only deal with addition.

Here and in the analysis of the operations to follow we use $x$ and $y$ to
denote the operands, $\hat{x}$ and $\hat{y}$ to denote the |double|
approximations of $x$ and $y$ and $\epsilon_{x}$ and
$\epsilon_{y}$ to denote the relative error of $\hat{x}$ and $\hat{y}$.
Thus $\abs x - \hat{x} \abs \leq  \epsilon_{x} \cdot \abs \hat{x} \abs$
and $\abs y - \hat{y} \abs \leq  \epsilon_{y} \cdot \abs \hat{y} \abs$.
Let $\hat{z}$ denote the floating point approximation of the result of the
floating point addition of $\hat{x}$ and $\hat{y}$.
Assume first that $\abs \hat{z} \abs \geq |MinDbl|$.
Then
\begin{eqnarray*} \label{Addition_abs}
\abs \hat{z} - (x+y) \abs   &\leq&  
\abs \hat{z} - (\hat{x} + \hat{y}) \abs  +  
\abs \hat{x}-x \abs + \abs \hat{y} - y \abs  \\
& \leq & \abs \hat{z} \abs \cdot  eps +
 \abs \hat{x} \abs \cdot \epsilon_{x} +
 \abs \hat{y} \abs \cdot \epsilon_{y} \\
& \leq &  \abs \hat{z} \abs \cdot  (eps  + 
\abs \frac{\hat{x}}{\hat{z}} \abs \cdot \epsilon_{x} +
\abs \frac{\hat{y}}{\hat{z}} \abs \cdot \epsilon_{y} ) 
\end{eqnarray*}
Dividing both sides of the inequality by $\abs\hat{z}\abs$ we obtain
\begin{equation} \label{Addition}
\epsilon_{z} \leq
eps  + 
\abs \frac{\hat{x}}{\hat{z}} \abs \cdot \epsilon_{x} +
\abs \frac{\hat{y}}{\hat{z}} \abs \cdot \epsilon_{y}
\end{equation}
This formula has index 4.
According to section \ref{Floats} we may therefore use the correction factor
$1 + 8 \cdot eps$ provided that $\epsilon_x$ and  $\epsilon_y$ are both
bounded by $2^{900}$.


Assume next that $\abs \hat{z} \abs  < |MinDbl|$. 
Then $\abs \hat{x} + \hat{y} \abs < |MinDbl|$ and hence
\begin{eqnarray*}
\abs z \abs & \leq & 
\abs x - \hat{x} \abs + \abs y - \hat{y} \abs + \abs \hat{x} + \hat{y} \abs \\ 
& \leq & \abs \hat{x}\abs \cdot \epsilon_x
                   + \abs \hat{y}\abs \cdot \epsilon_y + MinDbl \\
& \leq & (1 + eps)^2 \cdot (       \abs \hat{x}\abs \odot \epsilon_x
                            \oplus \abs \hat{y}\abs \odot \epsilon_y)
         + 3 MinDbl \\
& \leq & (1 + eps)^3 \cdot (       \abs \hat{x}\abs \odot \epsilon_x
                            \oplus \abs \hat{y}\abs \odot \epsilon_y
                            \oplus 4 MinDbl 
                           )
\end{eqnarray*}
where the term $2 MinDbl$ accounts for the possible underflow in the
multiplications.
With |NUM =| $(\abs \hat{x}\abs \odot \epsilon_x \oplus \abs \hat{y}\abs \odot
              \epsilon_y \oplus 4 |MinDbl| $
we have $\abs x + y - |NUM| \abs \leq  2 \cdot (1 + eps)^3 \abs |NUM| \abs$.
We may therefore set |ERROR = 2 * correction|.

@<compute error for addition or subtraction@>=
    if (fabs(NUM) >= MinDbl)
    { 
      z_rep.ERROR =   fabs(x_num/NUM)*x.PTR->ERROR
                    + fabs(y_num/NUM)*y.PTR->ERROR 
                    + eps;
    }
    else
    {  
       @+
       NUM =  fabs(x_num)*x.PTR->ERROR + 
                    fabs(y_num)*y.PTR->ERROR + 4*MinDbl;
       z_rep.ERROR = 2;
    }
    z_rep.ERROR *= correction;

@ We turn to multiplication--specific parts.
Checking exactness of |double| multiplication seems more difficult than for
addition.
However, if both operands are ``single--precision'', that is, if the last
$26$ bits are zero, we know that the exact value of the multiplication is
representable in |double| format.
Hence in this case the |double| multiplication is guaranteed to compute exactly.

Note that the position of the |double|'s low word is machine--dependent.
For example, for SUN SPARC's the |BIG_ENDIAN| byte ordering is active,
an INTEL PC's use |LITTLE_ENDIAN| byte ordering.

@<if multiplication is exact...@>=
  double NUM_result;
  long *x_ptr, *y_ptr;
  if (x.is_double && y.is_double)
  {  NUM_result = x.value * y.value;
     x_ptr = (long*) &(x.value); y_ptr = (long*) &(y.value);
#ifndef LITTLE_ENDIAN
     x_ptr++; @+ y_ptr++;
#endif
     if ( !(*x_ptr & 0x03ffffff) && !(*y_ptr& 0x03ffffff) )
        return NUM_result;
  }


@ Next we analyze multiplication.
If either $\hat{x} = 0$ or $\hat{y} = 0$ then $z = 0$.
If $\abs \hat{z} \abs < MinDbl$ then 
$\abs \hat{x} \cdot \hat{y} \abs \leq MinDbl$
and hence
\begin{eqnarray*} 
   \abs x \cdot y \abs 
 & \leq &  
       \abs \hat{x} \abs \cdot (1 + \epsilon_{x})
 \cdot \abs \hat{y} \abs \cdot (1 + \epsilon_{y}) \\
 & \leq &  
   MinDbl \cdot (1 + \epsilon_{x}) \cdot (1 + \epsilon_{y})
\end{eqnarray*}
We set |NUM| to $MinDbl \cdot (1 + \epsilon_{x}) \cdot (1 + \epsilon_{y})$
and |ERROR| to |2 * correction|.
If $\abs \hat{z} \abs \geq MinDbl$ and $\abs \hat{z}\abs \neq \infty$ then
\begin{eqnarray*}
\abs \hat{z} - (x \cdot y) \abs 
   &\leq&  
\abs \hat{z} - \hat{x} \cdot \hat{y} \abs  +  
\abs \hat{x} \cdot (\hat{y} - y) \abs  + 
\abs y \cdot (\hat{x} - x) \abs  \\
  & \leq & 
\abs \hat{z} \abs \cdot eps +
 \abs \hat{x}\cdot\hat{y} \abs \cdot \epsilon_{y} + 
 (\abs \hat{y} \abs  +  \abs y - \hat{y} \abs) \cdot \abs \hat{x} \abs \cdot
  \epsilon_{x} \\
  & \leq &
 \abs \hat{z} \abs \cdot eps +
 \abs \hat{x}\cdot\hat{y} \abs \cdot \epsilon_{y} + 
 \abs \hat{x}\cdot\hat{y} \abs \cdot  (1 + \epsilon_{y}) \cdot \epsilon_{x}
 \\
  & = & 
 \abs \hat{z} \abs \cdot eps +
 \abs \hat{x}\cdot\hat{y} \abs \cdot 
 (\epsilon_{x} + \epsilon_{y} + \epsilon_{x} \cdot \epsilon_{y})
\end{eqnarray*}
Because of 
$ \abs \hat{x}\cdot\hat{y} \abs   \leq 
  (1 + eps)  \cdot  \abs \hat{z} \abs
$,
we finally get after dividing both sides by the term $\abs \hat{z} \abs$ 
the estimate
\begin{equation} \label{Multiplication}
\epsilon_{x\cdot y}
\leq ( \epsilon_{x} + \epsilon_{y} + \epsilon_{x} \cdot \epsilon_{y} ) \cdot 
  (1 + eps) + eps
\end{equation}
Since we post-multiply the error bound by the correction factor 
$1 + 8 \cdot eps$ we may drop the factor $1 + eps$.

@<compute error for multiplication@>=
    double& qx = x.PTR->ERROR;
    double& qy = y.PTR->ERROR;
    if (fabs(NUM) >= MinDbl) 
    { 
      z_rep.ERROR = qx + qy + qx*qy + eps;
    }
    else
    { 
      if ((x_num != 0) && (y_num != 0))
      { 
        NUM = MinDbl*(1 + qx)*(1 + qy);
        z_rep.ERROR = 2;
      }
      else
        return 0;
    }
    z_rep.ERROR *= correction;

@  Now we come to divisions. 
The only part where division globally differs from the previous operations 
is the treatment of the zero--exception.
Also, we must set the error to Infinity if the sign of the divisor is
unsure.

@<Operators@>+=
 real operator/(const real& x,const real& y)
   { 
     @<check for division by zero@>@;
     @<if division is exact return double result@>@;
     @<create nodes for two operands@>@;
     real_rep& z_rep = *new real_rep(); 
     @<initialize |z_rep| as result of a binary operation@>@;
     z_rep.CON = DIVISION;
     NUM = x_num/ y_num;
     @<compute error for division@>@;
     @<if approximation is not valid set error to Infinity@>@;
     return z_rep;
   }

@ We have to check for division by zero. 
This is easy to do if the divisor is an exact |double|.
(If the sign of the divisor is unsure, we later set the error to infinity).

@<check for division by zero@>=
   if (y.is_double) 
   { 
     if (y.value == 0)
       error_handler(1,"real::operator/:Division by zero");
   }
   else
     if ((y.PTR->NUM == 0) && (y.PTR->ERROR == Infinity) )
        error_handler(1,"real::operator/:Division by zero");

   
@ One obvious sufficient condition for the exactness of |double| division
is the following.
Suppose that both the computed division result |NUM_result| and the dividend
|y.NUM| are single--precision.
Then the multiplication |NUM_result * y.NUM| is exact and we only have to
compare with the dividend |x.NUM| to see if |NUM_result| was exact.

@<if division is exact return double result@>=
  double NUM_result;
  long *num_ptr, *y_ptr;
  if (x.is_double && y.is_double)
  {  NUM_result = x.value * y.value;
     num_ptr = (long* ) &NUM_result; 
     y_ptr = (long* ) &y.value; 
#ifndef LITTLE_ENDIAN
     num_ptr++; @+ y_ptr++;
#endif
     if ( !(*num_ptr & 0x03ffffff) && !(*y_ptr& 0x03ffffff) )
       if (NUM_result * y.value == x.value)
        return NUM_result;
  }

@ We analyze the error in |double| division.
If $\hat{x} = 0$ then $x = 0$ and hence $z = 0$.
If $\hat{y} = 0$ then $y = 0$ and the operation is illegal.
If $\epsilon_y \geq 1$ then $y$ may be zero and we can give no approximation for the result.
So let us assume $\epsilon_y < 1$, $\hat{x} \neq 0$ and $\hat{y} \neq 0$.
If $\abs \hat{x} / \hat{y} \abs < MinDbl$ then 
$\abs \hat{x} \abs \oslash \abs \hat{y} \leq MinDbl$
and hence 
\begin{eqnarray*}
\abs x/y \abs \leq 
\frac{\abs\hat{x}\abs (1 + \epsilon_x)}{\abs\hat{y}\abs (1 - \epsilon_y)} \leq
\frac{MinDbl (1 + \epsilon_x)}{1 - \epsilon_y}.
\end{eqnarray*}
We use $NUM = MinDbl (1 + \epsilon_x) / (1 - \epsilon_y)$ and
$ERROR = 2 * correction$.
Finally, if $\abs \hat{z} \abs \geq MinDbl$ then
\begin{eqnarray*}
\abs \widehat{x/y} - x/y \abs 
   & \leq &   
\abs \widehat{x/y} - \hat{x} / \hat{y} \abs +
  \abs \hat{x} / \hat{y} - x/y \abs \\
  & \leq &
\abs \widehat{x/y} \abs \cdot  eps +
  \abs \frac{ \hat{x}\cdot y - x\cdot \hat{y}}{y \cdot \hat{y} } \abs \\
  & \leq &
\abs \widehat{x/y} \abs \cdot  eps +
  \frac{ \abs \hat{x} \cdot (\hat{y}-y) \abs + 
         \abs \hat{y} \cdot (\hat{x}-x) \abs       }{\abs y \cdot \hat{y} \abs}
\\ & \leq & 
\abs \widehat{x/y} \abs \cdot  eps +
  \abs \frac{\hat{x}\cdot \hat{y}}{y \cdot \hat{y}} \abs \cdot 
   (\epsilon_{x} + \epsilon_{y})
\end{eqnarray*}
Dividing by $\abs \widehat{x/y} \abs$ yields 
\begin{eqnarray*} \label{Division}
\epsilon_{x/y}
  & \leq & 
    eps  + \abs \hat{y} / y \abs \cdot 
    (\epsilon_{x} + \epsilon_{y}) \cdot (1 + eps) \\
  & \leq &
    eps + \frac{\abs \hat{y} \abs \cdot  (1 + eps)}
                    {\abs \hat{y} \abs - \abs \hat{y} - y\abs} 
               \cdot (\epsilon_{x} + \epsilon_{y}) \\  
 & \leq &
 eps + \frac{\epsilon_{x} + \epsilon_{y}}{1 - \epsilon_{y}}  \cdot (1 + eps )
\end{eqnarray*}
Again, we may absorb the factor $1 + eps$ into the correction factor.
\label{real_division}

@<compute error for division@>=
    if (y.PTR->ERROR >= 1)
    {
      z_rep.ERROR = Infinity; @+ NUM = 1;
      return z_rep;
    }
    if (fabs(NUM) >= MinDbl)
       z_rep.ERROR =   (x.PTR->ERROR + y.PTR->ERROR) / (1 - y.PTR->ERROR)
                     + eps;
    else
    { 
      if (x_num != 0)
      { 
        NUM = MinDbl*(1 + x.PTR->ERROR)/(1 - y.PTR->ERROR);
        z_rep.ERROR = 2;
      }
      else 
        return 0;
    }
    z_rep.ERROR *= correction; @#


@ We come to the squareroot operation.
Here we have to take care if the argument is negative or if the sign of
the argument is unknown.
Otherwise we proceed exactly as before, only we have just one argument.

@<Operators@>+=
  real sqrt(const real& x)
  {
     @<check for negative argument@>@;
     @<if squareroot is exact return double result@>@;
     if (x.PTR == nil) ((real&) x).PTR = new real_rep(x.value);
     real_rep& z_rep = *new real_rep(); 
     z_rep.OP1 = x.PTR; @+ z_rep.OP1->count++;
     z_rep.OP2 = nil;
     z_rep.CON = SQUAREROOT;
     if (x.PTR->ERROR >= 1)
     { z_rep.ERROR = Infinity; @+ z_rep.NUM = 1;
       return real(z_rep);
     }
     if (x.PTR->NUM < 0)
       error_handler(1,"real::operator sqrt:negative argument");
     z_rep.NUM = sqrt(x.PTR->NUM);
     @<compute error for squareroot@>@;
     return z_rep;
}
  
@ If the argument of squareroot is negative, we exit with an error message.
@<check for negative argument@>=
  if ( (x.is_double) && (x.value < 0) )
      error_handler(1,"real::operator sqrt:negative argument");
 
 
@ We give a simple test whether the squareroot is exact.
If the squareroot approximation is only ``single--precision'' and if squaring
gives back the original value, then it must be exact. 
Vice versa, if the approximation has a one after the $26^{th}$ most 
significant digit, then it can never give back the original result by
squaring.

@<if squareroot is exact...@>=
  double NUM_result;
  long *num_ptr;
  if (x.is_double)
  {  NUM_result = sqrt(x.value);
     num_ptr = (long* ) &NUM_result; 
#ifndef LITTLE_ENDIAN
     num_ptr++;
#endif
     if ( !(*num_ptr & 0x03ffffff))
        return NUM_result;
  }

   
@ It remains to analyze the error in squareroot.

If $\hat{x} = 0$ then $x = z = 0$.
If $\hat{x} \neq 0$ and $\epsilon_x > 1$ or $\hat{x} < 0$ then
$x$ may be negative and we can give no meaningful approximation.
So let us finally assume that $\hat{x} > 0$ and $\epsilon_x \leq 1$. 
Then $\hat{x} \geq MinDbl$ and hence $\hat{z} \geq MinDbl$.
We have
\begin{eqnarray*}
   \abs \widehat{\sqrt{x}} - \sqrt{x} \abs 
 & \leq & 
   \abs \widehat{\sqrt{x}} - \sqrt{\hat{x}} \abs +
   \abs \sqrt{\hat{x}} - \sqrt{x} \abs \\
 & \leq & 
   \widehat{\sqrt{x}} \cdot eps +
   \frac{\abs (\sqrt{\hat{x}}-\sqrt{x}) \cdot (\sqrt{\hat{x}}+\sqrt{x}) \abs}
       {\sqrt{\hat{x}} + \sqrt{x}} \\
 & = &
 \widehat{\sqrt{x}} \cdot eps +
   \frac{\abs \hat{x} - x \abs}{\sqrt{\hat{x}} + \sqrt{x}} \\
 & \leq &
 \widehat{\sqrt{x}} \cdot eps +
   \frac{\hat{x} \cdot \epsilon_{x}}{\sqrt{\hat{x}}}
\end{eqnarray*}
Dividing by $\widehat{\sqrt{x}}$ yields
\begin{equation}
  \epsilon_{\sqrt{x}} \leq eps + \epsilon_{x} \cdot (1 + eps)
\end{equation}

@<compute error for squareroot@>=
    z_rep.ERROR = x.PTR->ERROR + eps;
    z_rep.ERROR *= correction;

@ Finally we implement the unary ``$-$'' operator.
Because there is no error in this calculation, we only have to copy
the operand with its approximation negated. 

@<Operators@>+=
  real operator-(const real& x)
  { if (x.is_double) return -x.value;
    real_rep& z_rep = *new real_rep();
    z_rep.CON = NEGATION;
    z_rep.OP1 = x.PTR; @+ z_rep.OP1->count++; @+ z_rep.OP2 = nil;
    z_rep.NUM = -x.PTR->NUM; @+ z_rep.ERROR = x.PTR->ERROR;
    return z_rep;
  }


  
@* Sign determination.

We discuss the different versions of the function |real::sign|.
They all compute the sign of the |real| and their implementation rests on
the function \newline |real_rep::sign_with_separation_bound|.

@<Sign...@>+=
int real::sign() const
{  if (is_double)
     return ::sign(value);
   else
     return PTR->sign_with_separation_bound(zero); 
}
int real::sign(const integer& q) const
{  if (is_double)
     return ::sign(value);
   else
     return PTR->sign_with_separation_bound(q,true); 
}
int real::sign(long p) const
{  if (is_double)
     return ::sign(value);
   else
     return PTR->sign_with_separation_bound(p,true); 
}

@ For the implementation of the function |sign_with_separation_bound| we need
the concept of a {\em separation bound}.
A separation bound for a real number $x$ is a number $q$ such that $x \leq q$
implies $x = 0$.
We allow the user to provide a separation bound.
Following Mignotte \cite{Mignotte:book} we compute for each |real| $x$ 
quantities $M$ and $deg$ such that $2^{-M}$ is a separation bound for $x$.
The quantities $M$ and $deg$ are defined inductively according to the following
rules.

If $x$ is an integer then 
$$M(x) = \lceil \log_2(\abs x\abs)\rceil , deg(x) = 1, $$
if $x$ is a |bigfloat|, $x = significant \cdot 2^{exponent}$, then we have
$$M = 1 + Max(significant.length()+exponent,-exponent),$$
if $x = y + z$ or $x = y - z$ then
\begin{eqnarray*}
M(x) &=& deg(y)*deg(z) + deg(z)*M(y) + deg(y)*M(z), \\
deg(x) &=& deg(y)*deg(z), 
\end{eqnarray*}
if $x = y*z$ or $x = y/z$ then
\begin{eqnarray*}
M(x) &=& deg(z)*M(y) + deg(y)*M(z), \\
deg(x) &=& deg(y)*deg(z)
\end{eqnarray*}
and if $x = \sqrt{y}$ then
\begin{eqnarray*}
M(x) = M(y) , 
deg(x) = 2 * deg(y). 
\end{eqnarray*}

@ We store the quantities $M$ and $deg$ in a structure.

@<Basic declarations@>+=
struct sep_bound
{
  integer M, deg;
  sep_bound() @+ {}
};

@ The following function computes the separation bound of a node.
It asssumes that the separation bounds of the children already exist.

@<Local functions of class |real_rep|@>=
void compute_separation_bound();

@

@<Sign...@>+=
void real_rep::compute_separation_bound() @#
{
  sep_bound* ptr = sep_bound_ptr;
  
  switch(CON)
  { case DOUBLE:  case BIGFLOAT: 
        {
          bigfloat& num = APP_BF_PTR->NUM_BF;@;
          ptr->deg = 1;
          if (num.isZero()) ptr->M = zero; 
          else 
          { if (sign(num.get_exponent()) >= 0)
              ptr->M = log2(num);
            else
              ptr->M = 1 + Maximum(num.get_precision(),-num.get_exponent());
          }
          break;
        }
    case NEGATION: 
          ptr->deg = OP1->sep_bound_ptr->deg;
          ptr->M   = OP1->sep_bound_ptr->M;
          break;
    case MULTIPLICATION: case DIVISION:
          ptr->M = OP1->sep_bound_ptr->M*OP2->sep_bound_ptr->deg 
                 + OP2->sep_bound_ptr->M*OP1->sep_bound_ptr->deg;
          ptr->deg = OP1->sep_bound_ptr->deg*OP2->sep_bound_ptr->deg;
          break;
    case ADDITION: case SUBTRACTION:
          ptr->deg = OP1->sep_bound_ptr->deg*OP2->sep_bound_ptr->deg;
          ptr->M = ptr->deg
                 + OP1->sep_bound_ptr->M*OP2->sep_bound_ptr->deg 
                 + OP2->sep_bound_ptr->M*OP1->sep_bound_ptr->deg;
          break;
    case SQUAREROOT:
          ptr->M = OP1->sep_bound_ptr->M; 
          ptr->deg = OP1->sep_bound_ptr->deg << 1;
          break;
  }
}

@ We also need a recursive function that initializes the separation bound for 
the whole expression tree of a |real_rep|.

@<Local functions of class |real_rep|@>=
void initialize_separation_bound();

@

@<Sign...@>=
void real_rep::initialize_separation_bound()
{
  if (sep_bound_ptr != nil) 
    return;
  if (OP1 != nil) OP1->initialize_separation_bound();
  if (OP2 != nil) OP2->initialize_separation_bound();
  sep_bound_ptr = new sep_bound();
  compute_separation_bound();
}

@ Now we come to the function |sign_with_separation_bound|, the core of this
section.
The basic structure of the function is as follows.
We first decide whether the current approximations already determine the sign.
If so, we return the sign immediately.
Otherwise, we first compute a separation bound for the |real_rep|.
In the main part of |sign|, we repeatedly improve the quality of the 
|bigfloat| approximation.
As long as the current precision is less than $q$ and less than the separation
bound and the current approximation does not determine the sign we square the
precision and repeat.
If after that loop the sign is still not clear from the current approximation, 
the sign must be zero and we can remove the operand pointers.

@<Sign...@>+=
int real_rep::sign_with_separation_bound(const integer& q, bool supported)
{ @# @;
  @<if current approximations determine the sign return it@>@;
  initialize_separation_bound();
  @<compute bigfloat approximations of increasing precision@>@;
  if @<sign is clear...@> 
    return num_bf().sign();
  NUM = ERROR = 0;
  num_bf() = error_bf() = pZero;
  @<remove operand pointers@>@;
  return 0;
}

@ When do the current approximations determine the sign?
The |double| approximations does when either |NUM == 0| or |ERROR < 1|.
 
@<if current...@>=
if ( (ERROR != Infinity) && ( (ERROR < 1) || (NUM == 0) ) ) return ::sign(NUM);
init_app_bf();
if @<sign is clear...@> return num_bf().sign();

@ The |bigfloat| approximation determines the sign when |ERROR_BF == 0| or
|ERROR_BF < fabs(NUM_BF)|.

@<sign is clear from the bigfloat approximation@>=
  ( exact() || (!num_bf().isZero() && (num_exp() > err_exp())) ) 


@ Our current approximations do not determine the sign and we have computed
a maximal precision bound $q$. 
Now we repeatedly improve the working precision until either precision $q$
or the bound |bound| is reached or the sign is determined.

@<compute bigfloat...@>=
bool user_bound_is_reached = false;
integer current_precision = 
                  (err_exp() <= zero ? (-err_exp()+53)/2 : integer(26));
do
{  
  current_precision = current_precision*2;
  guarantee_bound_two_to(-current_precision);
  user_bound_is_reached = supported && (current_precision >= q); 
}
while (!(user_bound_is_reached || @<sign is clear...@> ));

@ The comparison operators could have been directly derived from the sign
function in the obvious way.
But then in most cases we would have to allocate an obsolete new node
by calling the subtraction operator.
This can be avoided if we use a straightforward test that tells us if the
comparison is easy and can be done by comparing the |double|s.

@<check if comparison is easy@>=
double x_error, y_error;
bool is_easy;
if (x.is_double && y.is_double) is_easy = true;
else
{
  if (x.is_double) x_error = 0;
  else 
  { 
     ((real&) x).value = x.PTR->NUM; 
     x_error = fabs(x.value)*x.PTR->ERROR; 
  }
  if (y.is_double) y_error = 0;
  else 
  {  ((real&) y).value = y.PTR->NUM; 
     y_error = fabs(y.value)*y.PTR->ERROR; 
  }
  is_easy = 
    (fabs(x.value - y.value) > (x_error + y_error + twoMinDbl) * correction );
}

@ Now we can give the comparison operators.
If we know that the comparison is easy, then we compare the 
|double| values.
Otherwise we call the sign function. 

@<Sign functions@>+=
int operator==(const real& x,const real& y)
{ @<check if comparison...@>@; 
  if (is_easy) return (x.value == y.value);
  return ( (y-x).sign() == 0); @# 
} @# 

int operator!=(const real& x,const real& y)
{ @<check if comparison...@>;
  if (is_easy) return (x.value != y.value);
  return ( (y-x).sign() != 0); 
}

int operator<(const real& x,const real& y)
{ @<check if comparison...@>;
  if (is_easy) return (x.value < y.value);
  return ( (y-x).sign() > 0);
}

int operator>(const real& x,const real& y)
{ @<check if comparison...@>;
  if (is_easy) return (x.value > y.value);
  return ( (y-x).sign() < 0);
}

int operator<=(const real& x,const real& y)
{ @<check if comparison...@>;
  if (is_easy) return (x.value <= y.value);
  return ( (y-x).sign() >= 0);
}

int operator>=(const real& x,const real& y)
{ @<check if comparison...@>;
  if (is_easy) return (x.value >= y.value);
  return ( (y-x).sign() <= 0);
}


  
@* The functions improve and related functions.

Function |real::reduce_error_in_approximation_to| is realized by calling
function |real_rep::guarantee_bound_two_to| with the sign of the argument
inversed.
The reason to invert the sign is that this syntax makes it easier to understand
the numerical details of |guarantee_bound_two_to|.
 
@<Implementation of improve and compute@>=
void real::reduce_error_in_approximation_to(const integer& p) const
{ if (is_double) return;
  PTR->init_app_bf();
  PTR->guarantee_bound_two_to(-p); 
  adjust();
}

@ The function |guarantee_bound_two_to| is among the core of our data structure.
The call |guarantee_bound_two_to(integer e)| improves |error_bf()| to a value $\leq 2^e$.
For each of the operations addition, subtraction, multiplication, division
and squareroot, we have a function |improve_operation| to do this job.
In the case the |bigfloat| approximation is exact, we remove the obsolete
pointers |OP1| and |OP2|.
If the function changes the |bigfloat| approximation of the |real_rep| it also
updates its separation bound.
After doing so we check whether the new separation bound and the new
approximation together imply that the number is zero.

@<Implementation of improve...@>+=
  void real_rep::guarantee_bound_two_to(const integer& e)
  {  
    bool current_precision_is_sufficient =
      error_bf().isZero() || (err_exp() <= e);
    if (current_precision_is_sufficient) return;

    switch(CON)
    { 
      case DOUBLE: case BIGFLOAT: 
                   return;
      case NEGATION: OP1->guarantee_bound_two_to(e); 
                     num_bf() = -OP1->num_bf();
                     error_bf() = OP1->error_bf();
                     break;
      case ADDITION: @+ improve_add(e); @+ break;
      case SUBTRACTION: @+ improve_sub(e); @+ break;
      case MULTIPLICATION: @+ improve_mul(e); @+ break;
      case DIVISION: @+ improve_div(e); @+ break;
      case SQUAREROOT: @+ improve_sqrt(e); @+ break;
    }
    
    if (exact()) 
      @+ @<remove operand pointers@>@;

    if (sep_bound_ptr)
    {
      compute_separation_bound();
      if (error_bf() + fabs(num_bf()) <= pow2(-sep_bound_ptr->M) )
      {
        @<remove operand pointers@>@;
        num_bf() = error_bf() = zero_bf;
        sep_bound_ptr->M = zero; 
        sep_bound_ptr->deg = 1;
      }
    }

    adjust_dbl();
  }

@ At this moment we list only the prototypes of the functions |improve_op|.
The function bodies will be given later, when we present the functions 
|compute_op|.

@<Local functions...@>+=
void improve_add(const integer&);
void improve_sub(const integer&);
void improve_mul(const integer&);
void improve_div(const integer&);
void improve_sqrt(const integer&);

@ 
@<remove operand pointers@>=
{
  if (OP1 != nil) @+ if (--OP1->count == 0) @+ delete OP1;
  if (OP2 != nil) @+ if (--OP2->count == 0) @+ delete OP2;
  OP1 = OP2 = nil;
  CON = BIGFLOAT;
}

@ We implement the function |real_rep::compute_op| that we used in \newline
|real::init_app_bf|.
The basic structure of |compute_op| is as follows.
Suppose that the |real_rep| object has been constructed by the operation $op$
and has operands that represent numbers $x$ and $y$ with approximations
$\hat{x}$ and $\hat{y}$.
Again we distinguish the respective cases of $op$.
First we compute $\hat{x}$ $op$ $\hat{y}$ in precision $p$.
The error in this operation alone is called the operation--induced error.
It is given by $\abs\hat{x}$ $op$ $\hat{y}\abs \cdot 2^{-p}$.
Then we compute the error arising from the inaccuracies of $\hat{x}$ and
$\hat{y}$.
We call this error the operand--induced error.
For multiplication, division and squareroot we use special functions to 
compute the operand--induced error.
The overall error of the approximation finally is the sum of the
operand--induced and the operation--induced error. 
We remark that for division the sign of the dividend must be known to be 
nonzero and similarly the argument of squareroot must known to be nonnegative.

@<Implementation of improve...@>+=
void real_rep::compute_op(long p)
{
 bool isexact;
 bigfloat operation_error, operand_error;
 switch(CON) 
 {
    case DOUBLE: case BIGFLOAT: return;
    case NEGATION: 
          num_bf() = -OP1->num_bf();
          error_bf() = OP1->error_bf();
          break;
    case ADDITION:
          num_bf() = add(OP1->num_bf(),OP2->num_bf(),p,TO_NEAREST,isexact);
          @<compute operand error for addition and subtraction@>@;
          break;
    case SUBTRACTION: 
          num_bf() = sub(OP1->num_bf(),OP2->num_bf(),p,TO_NEAREST,isexact);
          @<compute operand error for addition and subtraction@>@;
          break;
     case MULTIPLICATION:
          num_bf() = mul(OP1->num_bf(),OP2->num_bf(),p,TO_NEAREST,isexact);
          compute_mul_error(operand_error);
          break;
    case DIVISION: 
          if (OP2->sign_with_separation_bound(zero) == 0)
          error_handler(1,"compute_op:division by zero");
          num_bf() = div(OP1->num_bf(),OP2->num_bf(),p,TO_NEAREST,isexact);
          compute_div_error(operand_error);
          break;
    case SQUAREROOT: 
          if (OP1->sign_with_separation_bound(zero) < 0)
             error_handler(1,"compute_op:sqrt of negative number");
          num_bf() = sqrt(OP1->num_bf(),p,TO_NEAREST,isexact);
          compute_sqrt_error(operand_error);
   }
  if (!isexact) operation_error = ldexp_bf(num_bf(),-p);
  error_bf() = operation_error + operand_error;
  if (exact()) @<remove operand pointers@>@;
}

@ We list the prototypes of the functions |compute_op_error|.

@<Local functions...@>+=
void compute_mul_error(bigfloat&);
void compute_div_error(bigfloat&);
void compute_sqrt_error(bigfloat&);


@ We now give the details of |compute_op_error| and |improve_op| for the various
cases of numerical operations |op|. 
In the context of |compute_op_error| we want to compute the error $q$ of 
an approximation $\hat{z}$ of some real number $z$.
For the function |improve|, we face the opposite problem.
There we are {\em given} a bound $q$ and we want to determine 
an approximation $\hat{z}$ with error at most $q$.
In both cases, the number $z$ is defined by two (one in the case of
squareroot) other numbers $x$ and $y$ by means of the operation $op$.

@ Let's deal with addition and subtraction first.
We have
\begin{eqnarray} 
\abs \hat{z} - z \abs
 & \leq &   
  \abs\hat{z}-(\hat{x}+\hat{y})\abs + \abs\hat{x}-x\abs + \abs\hat{y}-y\abs
  \nonumber \\
 & \leq &  
  \epsilon \cdot \abs \hat{z} \abs + qx + qy \label{AddError}
\end{eqnarray}
where $\epsilon$ is the relative precision of the |bigfloat| addition.
We use this formula to compute the error bounds for addition and subtraction
directly with |bigfloat|s.

@<compute operand error for addition...@>=
operand_error = add(OP1->error_bf(),OP2->error_bf(),16,TO_INF);


@ Let us discuss how to improve the operations addition and subtraction.
Here we need to determine two bounds $qx$ and $qy$ and a relative precision 
$\epsilon$ such that the following is true:
If $\hat{x}$ and $\hat{y}$ are approximations of $x$ and $y$ of error 
$qx$ and $qy$ respectively and $\hat{z}$ is computed from $\hat{x}$ and
$\hat{y}$ by performing operation $op$ with relative precision $\epsilon$
then $\hat{z}$ has maximal error $q$.
Looking again at inequality \ref{AddError}, we are done if we bound the first
term (the operation induced error) by $q/2$ and the second and third term
(the operand errors) by $q/4$.
That is, we need
$$\epsilon \leq \frac{q}{2 \abs\hat{z}\abs}, qx \leq q/4, qy \leq q/4.$$
The second two conditions are easy to satisfy.
Taking $e = 2^q$, we only need to call |x.guarantee_bound_two_to(e-2)| and
|y.guarantee_bound_two_to(e-2)|.
How can we guarantee the first condition?
The problem is that we do not know $\hat{z}$.
So we replace $\log(\hat{z})$ by the crude bound 
$Max(\log(\hat{x}),\log(\hat{y})) + 1$.

@<Implementation of improve...@>+=
void real_rep::improve_add(const integer& e)
{
  long p;
  @<improve operands and compute p for add...@>@;
  bool isexact;
  num_bf() = add(OP1->num_bf(),OP2->num_bf(),p,TO_NEAREST,isexact);
  @<set error $2^e$@>@;
}

void real_rep::improve_sub(const integer& e)
{
  long p;
  @<improve operands and compute p for add...@>@;
  bool isexact;
  num_bf() = sub(OP1->num_bf(),OP2->num_bf(),p,TO_NEAREST,isexact);
  @<set error $2^e$@>@;
}
  
@ 
@<improve operands and compute p for add and sub@>=
  OP1->guarantee_bound_two_to(e-2);
  OP2->guarantee_bound_two_to(e-2);
  integer log_epsilon = -1;
  if (!OP1->num_bf().isZero() && !OP2->num_bf().isZero())
    log_epsilon = e - Maximum(OP1->num_exp(),OP2->num_exp()) - 2;
  @<convert log\_epsilon to p@>@;
  
@
@<convert log\_epsilon to p@>=
   if (!log_epsilon.islong())
      error_handler(1,"sorry:improve:computation to expensive");
   else
       p = -log_epsilon.tolong();

@
@<set error $2^e$@>=
   if ( isexact && OP1->exact() && OP2->exact() )
     error_bf() = pZero;   
   else
     error_bf() = pow2(e);
  

@ We deal with multiplication next.
We have
\begin{eqnarray} 
\abs \widehat{x \cdot y} - (x \cdot y) \abs 
   &\leq&  
\abs \widehat{x \cdot y} - (\hat{x} \cdot \hat{y}) \abs  +  
\abs \hat{x} \cdot (\hat{y} - y) \abs  + 
\abs y \cdot (\hat{x} - x) \abs  \nonumber \\
  & \leq & 
\abs \widehat{x \cdot y} \abs \cdot \epsilon +
 \abs \hat{x} \abs \cdot qy  +  \abs y \abs \cdot qx
\label{MulError}
\end{eqnarray}
We first implement the part of computing errors.
Since multiplications are expensive for operands with large precision, we
first round them to $16$ bit and then do the multiplications in precision $32$.
By choosing rounding mode ``to $\infty$'' we avoid having to care for the 
inaccuracies.

@<Implementation of improve...@>+=
void real_rep::compute_mul_error(bigfloat& operand_error)
{
  bigfloat x = OP1->num_bf();
  bigfloat y = OP2->num_bf();
  bigfloat error_x = mul(fabs(x.round(16,TO_INF)),OP2->error_bf(),32,TO_INF);
  bigfloat error_y = mul(fabs(y.round(16,TO_INF)),OP1->error_bf(),32,TO_INF);
  operand_error = add(error_x,error_y,16,TO_INF);
}

@ Now we implement the improve function for multiplication.
Again, our goal is to bound the first term of \ref{MulError} by $q/2$ and the
two other terms by $q/4$, or equivalently
\begin{equation} \label{MulImprove}
  \epsilon \leq \frac{q}{2\abs\widehat{x \cdot y}\abs},
  qx \leq \frac{q}{4 \abs y\abs}, qy \leq \frac{q}{4 \abs\hat{x}\abs}
\end{equation}
Since we don't know $y$, we bound it by |y_high = fabs(y.num_bf()) +
y.error_bf()|.

@<Implementation of improve...@>+=
void real_rep::improve_mul(const integer& e)
{
  bigfloat y_high = add(fabs(OP2->num_bf()),OP2->error_bf(),32,TO_P_INF);
  integer ex = e - log2(y_high) - 2;
  OP1->guarantee_bound_two_to(ex);
  integer log_epsilon = -1;
  if (!OP1->num_bf().isZero())
  {
    integer ey = e - OP1->num_exp() - 2;
    OP2->guarantee_bound_two_to(ey);
    log_epsilon = e - (OP1->num_exp()+OP2->num_exp()) - 1;
  }
  bool isexact;
  long p;
  @<convert log\_epsilon to p@>@;
  num_bf() = mul(OP1->num_bf(),OP2->num_bf(),p,TO_NEAREST,isexact);
  @<set error $2^e$@>@;
}


@ Division is next.
Remember that when the functions |compute_div_error| and |improve_div| are
called the sign of the divisor $y$ is known to be nonzero, because this
exception was handled in the division operator or by |init_app_bf|.
Hence we can use |y_low = fabs(y.num_bf()) - y.error_bf() > 0| in our
calculation.
Let us consider the error estimate given for the division operator.
\begin{eqnarray} 
\abs \widehat{ \frac{x}{y} } - \frac{x}{y} \abs 
   & \leq & 
\abs \widehat{\frac{x}{y}} \abs \cdot  \epsilon +
  \frac{ \abs\hat{x}\abs \cdot qy}{\abs y \cdot \hat{y} \abs} +
  \frac{qx}{\abs y \abs} \label{DivError} \\
   & = &
\abs \widehat{\frac{x}{y}} \abs \cdot  \epsilon + \frac{1}{\abs y\abs} \cdot 
   (\frac{ \abs\hat{x}\abs \cdot qy}{\abs\hat{y}\abs} + qx) \nonumber
\end{eqnarray}
As before we avoid multiplication with operands having large precisons and 
also we avoid divisions completely.

@<Implementation of improve...@>+=
void real_rep::compute_div_error(bigfloat& operand_error)
{
  bigfloat y_low = sub(fabs(OP2->num_bf()),OP2->error_bf(),32,TO_N_INF);
  bigfloat x = OP1->num_bf();
  bigfloat z = mul(fabs(x.round(16)),OP2->error_bf(),32,TO_INF);
           z = ldexp_bf(z,1-OP2->num_exp());
  operand_error = add(z,fabs(OP1->error_bf()),16,TO_INF);
           operand_error = ldexp_bf(operand_error,1-log2(y_low));
}

@ In the div version of improve, we again want to bound the last two terms
of \ref{DivError} by $q/4$ and the first one by $q/2$. 
This means
\begin{equation}
\epsilon \leq \frac{q}{2 \abs\hat{z}\abs}, 
      qx \leq \frac{q \cdot y}{4},
      qy \leq \frac{q \cdot y \cdot \abs\hat{y}\abs}{4 \abs\hat{x}\abs}
\end{equation}
We proceed exactly as for the other cases of |improve|.

@<Implementation of improve...@>+=
void real_rep::improve_div(const integer& e)
{
  bigfloat y_low = sub(fabs(OP2->num_bf()),OP2->error_bf(),32,TO_N_INF);
  integer ex = e + log2(y_low) - 3;
  OP1->guarantee_bound_two_to(ex);
  integer log_epsilon = -1;
  if (!OP1->num_bf().isZero())
  {
    integer ey = ex  + OP2->num_exp() - OP1->num_exp() - 3;
    OP2->guarantee_bound_two_to(ey);
    log_epsilon = e + OP2->num_exp() - OP1->num_exp() - 2;
  }
  bool isexact;
  long p;
  @<convert log\_epsilon to p@>@;
  num_bf() = div(OP1->num_bf(),OP2->num_bf(),p,TO_NEAREST,isexact);
  @<set error $2^e$@>@;
}

@ We finally deal with the squareroot operation.
We have
\begin{equation} \label{SqrtError}
   \abs \widehat{\sqrt{x}} - \sqrt{x} \abs  
\leq  
 \widehat{\sqrt{x}} \cdot \epsilon +
   \frac{\abs \hat{x} - x \abs}{\sqrt{\hat{x}} + \sqrt{x}} 
\leq  
 \widehat{\sqrt{x}} \cdot \epsilon + \frac{qx}{\sqrt{\hat{x}}}  
\end{equation}  
We bound the logarithm of $\sqrt{\hat{x}}$ by |(x.num_exp()+1)/2|.

@<Implementation of improve...@>+=
void real_rep::compute_sqrt_error(bigfloat& operand_error)
{  
  integer log_sqrt = (num_exp()+1) >> 1;
  if (!OP1->exact())
    operand_error = ldexp_bf(OP1->error_bf(),-log_sqrt); 
}


@ Last we show how to improve the squareroot operation.
To bound the two terms on the right hand side of \ref{SqrtError} by
$q/2$, we set $qx = q \cdot \sqrt{x} / 2$ and 
$\epsilon = q / (2 \widehat{\sqrt{x}})$.
Since we don't know $\sqrt{x}$, we bound its binary logarithm by 
$\lfloor \log_2(\mbox{|x_low|})/2 \rfloor$ from below and by
$\lceil \log_2(\mbox{|x_high|})/2 \rceil$ from above.

@<Implementation of improve...@>+=
void real_rep::improve_sqrt(const integer& e)
{
  bool isexact;
  bigfloat x_low  = sub(OP1->num_bf(),OP1->error_bf(),32,TO_N_INF);
  integer ex = e + (log2(x_low)-1)/2 - 1;
  OP1->guarantee_bound_two_to(ex);
  bigfloat x_high = add(OP1->num_bf(),OP1->error_bf(),32,TO_P_INF);
  integer log_epsilon = e - (log2(x_high)+1)/2 - 1;
  long p;
  @<convert log\_epsilon to p@>@;
  num_bf() = sqrt(OP1->num_bf(),p,TO_NEAREST,isexact,num_bf());
  if ( isexact && OP1->exact() ) @+ error_bf() = pZero; 
  else @+ error_bf() = pow2(e);
}

@ It remains to implement the functions |x.guarantee_relative_error_two_to_minus| and
|x.compute_with_mantissa_length(p)|.
We begin with function \newline |guarantee_relative_error_two_to_minus|.
It first performs a sign test.
If the sign is nonzero, it calls |improve| with an appropriate 
error bound.

@<Implementation of improve and compute@>=
void real::guarantee_relative_error_two_to_minus(long p) const
{  if (is_double) return;
   if (PTR->sign_with_separation_bound(zero) == 0) return;
   PTR->init_app_bf();
   bigfloat num_low = fabs(PTR->num_bf()) - PTR->error_bf(); 
   PTR->guarantee_bound_two_to(log2(num_low) - p);
   adjust(); 
}

@ Function |compute_with_mantissa_length| uses the function
|compute_approximation| that in turn 
calls |compute_op| for every unvisited node in the expression tree.
To allow intelligent searching in the expression tree, we assign values of 
an enumeration type |statustype| to the member |status|.
The meaning of these states is the following.
\begin{itemize}
\item |EMPTY| is the initial state.
\item |VISITED| means that the computation for the node has been successfully 
finished.
\item |CLEARED| is used by the function |clear_visited_marks| that we define in
a moment.
\end{itemize}

@<Basic declarations@>+=
enum statustype @+ { @+ EMPTY = 0, CLEARED = 1, VISITED = 2 @+ }; 

@
@<Implementation of improve and compute@>+=
void real::compute_with_mantissa_length(long p) const
{  if (is_double) return;
   PTR->init_app_bf();
   PTR->clear_visited_marks();
   PTR->compute_approximation(p);
   adjust();
}


@ We need a function to clear the visited marks used by |compute_approximation|.

@<Local functions...@>+=
void clear_visited_marks();

@ This function is implemented as follows.
We maintain the invariant that if any node is in the state |CLEARED|, then all 
children of that node are |CLEARED| too.

@<Sign...@>+=
void real_rep::clear_visited_marks()
{
  if (status == CLEARED) return;
  if (OP1 != nil) OP1->clear_visited_marks();
  if (OP2 != nil) OP2->clear_visited_marks();
  status = CLEARED;
}

@ Now we consider the function |compute_approximation(long p)|.
When the function is called for the object |x_rep|, we compute every operation
in the expression tree of |x_rep| with |bigfloat|s in precision $p$.
The final result is stored as the new |bigfloat| approximation of |x_rep|.


@<Implementation of improve and compute@>+=
 void real_rep::compute_approximation(long p)
  {  
    if (status == VISITED) return;
    if (OP1 != nil) OP1->compute_approximation(p);
    if (OP2 != nil) OP2->compute_approximation(p);
    compute_op(p);
    status = VISITED;
  }



@* Mathematical functions.

We implement some of the most frequently used functions from ``math.h'' 
for type |real|.

@<Implementation of mathematical functions ...@>+=
  real fabs(const real& x) @+ { @+ return (x.sign() < 0 ? -x : x); @+ }
  real sq(const real& x) @+ { @+ return x*x; @+ } @;
  real dist(const real& x, const real& y) @+ { @+ return sqrt(x*x+y*y); @+ } @;
  real powi(const real& x, int n) 
  { real y = x, z = 1;
    int n_prefix = n;
     while (n_prefix > 0)
     {
       if (n_prefix % 2) z = z*y;
       n_prefix = n_prefix / 2;
       y = y*y;
     }
    return z;
   }

@ References.

\def\mybib{/KM/usr/burnikel/bib}
\bibliographystyle{alpha}
\bibliography{\mybib/ca,\mybib/cg,\mybib/ds,\mybib/my,\mybib/infi}
@
\end{document}
