#include <LEDA/window.h>
#include <LEDA/plane_alg.h>

static GRAPH<rat_point,rat_segment>* sub1_ptr;
static GRAPH<point,segment>* sub_ptr;

void report_intersection(const rat_segment& s1, const rat_segment& s2)
{ rat_point p;
  s1.intersection_of_lines(s2,p);
  sub1_ptr->new_node(p);
 }

void report_intersection(const segment& s1, const segment& s2)
{ point p;
  s1.intersection_of_lines(s2,p);
  sub_ptr->new_node(p);
}



int main()
{ 
  int N = 200;
  int gridlen = 5;
  bool repeat = false;
  int alg = 0;
  int num = 0;

  int h = int(0.8 * window::screen_height());
  if (h < 480) h = 480;
  int w = int(0.8 * h);

  window W(w,h);


  W.int_item("SEGMENTS", N,0,1000);
  W.int_item("GRID",gridlen,1,40);
  //W.choice_item("NUMBERS",num,"rational","float");
  W.choice_item("ALGORITHM",alg,"B/O-Sweep","Mulmuley","Balaban","Trivial");

  W.button("mouse",1);
  W.button("random",2);
  W.button("run",3);
  W.button("repeat",4);
  W.button("quit",0);

  W.set_node_width(2);
  W.init(-1000,1000,-1000);
  W.display();

  int but;

  list<rat_segment> seglist1;
  list<segment> seglist;

  for(;;)
  {
    if (!repeat || W.get_button() != NO_BUTTON)
      { repeat = false;
        but = W.read_mouse();
       }

    if (but == 4) 
    { repeat = true;
      but = 3;
     }

    if (but == 0) break;

  
    switch (but) {

       case 1: // mouse
         { seglist.clear();
           seglist1.clear();

           int grid_width = int((W.xmax() - W.xmin())/(gridlen+1));
           W.set_grid_mode(grid_width);
     
           segment s;
           while (W >> s)
           { seglist.append(s);
             int x1 = int(s.xcoord1());
             int x2 = int(s.xcoord2());
             int y1 = int(s.ycoord1());
             int y2 = int(s.ycoord2());
             seglist1.append(rat_segment(x1,x2,1,y1,y2,1));
            }
     
           break;
          }
     
   
       case 2: // random
        { seglist.clear();
          seglist1.clear();

          W.set_grid_mode(0);

          int xmin = int(W.xmin())+100;
          int xmax = int(W.xmax())-100;
          int ymin = int(W.ymin())+100;
          int ymax = int(W.ymax())-100;

          int dx = (xmax - xmin)/gridlen;
          int dy = (ymax - ymin)/gridlen;

          for(int i = 0; i < N; i++)
          { int x1 = xmin + rand_int(0,gridlen)*dx;
            int y1 = ymin + rand_int(0,gridlen)*dy;
            int x2 = xmin + rand_int(0,gridlen)*dx;
            int y2 = ymin + rand_int(0,gridlen)*dy;
            seglist.append(segment(x1,y1,x2,y2));
            seglist1.append(rat_segment(x1,y1,1,x2,y2,1));
            W.del_messages();
            W.message(string("%4d",i));
           }
           break;
        }
      }

    W.clear();

    int j = 0;
    segment s;
    forall(s,seglist) 
    { color c = color(1 + (j++ % 15));
      W.draw_segment(s,c);
    }

  
    if (but != 3) continue;
  
    GRAPH<point,segment>         SUB;
    GRAPH<rat_point,rat_segment> SUB1;

    sub1_ptr = &SUB1;
    sub_ptr  = &SUB;

    string name;
    float t;
    int n;
    int m;

    W.del_messages();

    float T = used_time();

    switch(3*num + alg) {
  
      case 0: name = "rat_sweep:    ";
              W.message(name);
              SWEEP_SEGMENTS(seglist1,SUB1);
              t = used_time(T);
              n = SUB1.number_of_nodes();
              m = SUB1.number_of_edges();
              break;

      case 1: name = "rat_mulmuley: ";
              W.message(name);
              MULMULEY_SEGMENTS(seglist1,SUB1);
              t = used_time(T);
              n = SUB1.number_of_nodes();
              m = SUB1.number_of_edges();
              break;

      case 2: { name = "rat_balaban: ";
                BALABAN_SEGMENTS(seglist1,report_intersection);
                rat_segment s;
                forall(s,seglist1)
                { SUB1.new_node(s.source());
                  SUB1.new_node(s.target());
                 }
                t = used_time(T);
                n = SUB1.number_of_nodes();
                m = SUB1.number_of_edges();
                break;
               }

      case 3: { name = "rat_trivial: ";
                TRIVIAL_SEGMENTS(seglist1,report_intersection);
                rat_segment s;
                forall(s,seglist1)
                { SUB1.new_node(s.source());
                  SUB1.new_node(s.target());
                 }
                t = used_time(T);
                n = SUB1.number_of_nodes();
                m = SUB1.number_of_edges();
                break;
              }
  
      case 4: name = "float_sweep:  ";
              W.message(name);
              SWEEP_SEGMENTS(seglist,SUB);
              t = used_time(T);
              n = SUB.number_of_nodes();
              m = SUB.number_of_edges();
              break;
  
      case 5: name = "float_mulmuley:  ";
              W.message(name);
              MULMULEY_SEGMENTS(seglist,SUB);
              t = used_time(T);
              n = SUB.number_of_nodes();
              m = SUB.number_of_edges();
              break;

      case 6: { name = "float_balaban: ";
                BALABAN_SEGMENTS(seglist,report_intersection);
                segment s;
                forall(s,seglist)
                { SUB.new_node(s.source());
                  SUB.new_node(s.target());
                 }
                t = used_time(T);
                n = SUB.number_of_nodes();
                m = SUB.number_of_edges();
                break;
               }

      case 7: { name = "float_trivial: ";
                TRIVIAL_SEGMENTS(seglist,report_intersection);
                segment s;
                forall(s,seglist)
                { SUB.new_node(s.source());
                  SUB.new_node(s.target());
                 }
                t = used_time(T);
                n = SUB.number_of_nodes();
                m = SUB.number_of_edges();
                break;
              }
      }

    W.del_messages();
    W.message(string("%s |V| = %d |E| = %d  t = %6.2f sec", ~name,n,m,t));

    node v;

    if (num == 0)
       forall_nodes(v,SUB1) 
       { rat_point p = SUB1[v];
         W.draw_filled_node(p.xcoordD(),p.ycoordD(),red);
        }
    else
       forall_nodes(v,SUB) W.draw_filled_node(SUB[v],red);
  
 } 

 return 0;

}

