#include <LEDA/graphwin.h>
#include <LEDA/graph_alg.h>


void highlight(GraphWin& GW, list<node> V, list<edge> E, node_array<int>& kind) {

  node v;
  edge e;

  forall(v,V) 
    switch (kind[v]) {
        
      case  2: GW.set_color(v,black);
               GW.set_width(v,4);
               break;

      case  4:
      case  3: GW.set_type(v,rectangle_node);
               GW.set_color(v,red);
               break;

      case -3: GW.set_type(v,rectangle_node);
               GW.set_color(v,blue2);
               break;
    }

    const graph& G=GW.get_graph();

    forall_edges(e,G) GW.set_style(e,dotted);
        
    forall(e,E) {
      GW.set_style(e,solid);
      GW.set_color(e,black);
      GW.set_width(e,2);
    }

    GW.redraw();
}


main() {

  graph G;

  GraphWin GW(G);

  GW.set_flush(false);
  GW.display();

  while (GW.edit()) {
    if (PLANAR(G)) { 
        
        list<edge> b_edges = Make_Biconnected(G);
        list<edge> r_edges = Make_Bidirected(G);
        node_array<double> xcoord(G);
        node_array<double> ycoord(G);
        PLANAR(G,true);
        STRAIGHT_LINE_EMBEDDING(G,xcoord,ycoord);
        node v;
        forall_nodes(v,G) {  
           xcoord[v] = 10 + 80*xcoord[v];
           ycoord[v] = 10 + 80*ycoord[v];
         }
        edge e;
        forall(e,b_edges) G.del_edge(e);
        forall(e,r_edges) G.del_edge(e);
        
        GW.set_flush(true);
        GW.set_position(xcoord,ycoord);
        GW.set_flush(false);
      
    }
    else 
      { list<node> V;
        list<edge> E;
        node_array<int> kind(G);
  
        KURATOWSKI(G,V,E,kind);
        highlight(GW,V,E,kind);

        GW.wait("Press continue!");  
        GW.reset();
        GW.redraw();
      }

  }

  return 0;
}
