/*******************************************************************************
+
+  LEDA 3.4
+
+  bigfloat.h
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

/* This file has been automatically generated from "_bigfloat.w"
   by CTANGLE (Version 3.1 [km2c]) */


/*{\Manpage {bigfloat} {} {The data type bigfloat}
}*/

#ifndef BIGFLOAT_H
#define BIGFLOAT_H

#if !defined(LEDA_ROOT_INCL_ID)
#define LEDA_ROOT_INCL_ID 340019
#include <LEDA/REDEFINE_NAMES.h>
#endif


#include <LEDA/integer.h>
#include <math.h>

/* we use the LEDA data type |integer| for the representation
of |bigfloat|s */

class bigfloat;



enum special_values {
  NOT_VALUE, PZERO_VALUE, NZERO_VALUE, PINF_VALUE, NINF_VALUE, NAN_VALUE
};



enum rounding_modes {
  TO_NEAREST, TO_ZERO, TO_P_INF, TO_N_INF, TO_INF, EXACT
};




extern long global_prec;
extern rounding_modes round_mode;
extern bool dbool;





enum output_modes {
  BIN_OUT, DEC_OUT
};

extern output_modes output_mode;




extern const bigfloat NaN;
extern const bigfloat pInf;
extern const bigfloat nInf;
extern const bigfloat pZero;
extern const bigfloat nZero;




class bigfloat {

/*{\Mdefinition
In general a |bigfloat| is given by two |integer|s $s$ and $e$ where $s$ is the
significant and $e$ is the exponent.
The tuple $(s,e)$ represents the real number $$s \cdot 2^e.$$
In addition, there are the special |bigfloat| values |NaN| ($=$ not a number),
|pZero|, |nZero| ($= +0, -0$), and |pInf|, |nInf| ($= +\infty, -\infty$).
These special values behave as defined by the IEEE floating point standard.
In particular,
$\frac{5}{+0}=\infty$,$\frac{-5}{+0}=-\infty$,$\infty+1=\infty$,
$\frac{5}{\infty}=+0$,$+\infty+(-\infty)=NaN$ and $0 \cdot \infty = NaN$.

Arithmetic on |bigfloat|s uses two parameters:
The precision |prec| of the result (in number of binary digits) and the rounding
mode |mode|.
Possible rounding modes are:
\begin{itemize}
\item $TO\_NEAREST$: round to the closest representable value
\item $TO\_ZERO$: round towards zero
\item $TO\_INF$: round towards $\infty$
\item $TO\_P\_INF$: round towards $+\infty$
\item $TO\_N\_INF$: round towards $-\infty$
\item $EXACT$: compute exactly ($/$ and $\sqrt{\hspace{1ex}}$ in $TO\_NEAREST$ mode)
\end{itemize}
Operations $+,-,*$ work as follows.
First, the exact result |z| is computed.
If the rounding mode is EXACT then |z| is the result of the operation.
Otherwise, let |s| be the significant of the result; |s| is rounded to |prec|
binary places as dictated by |mode|.
Operations $/$ and $\sqrt{\hspace{1ex}}$ work accordingly except that
EXACT is treated as TO\_NEAREST.

The parameters |prec| and |mode| are either set directly for a single
operation or they are set globally for every operation to follow.
The default values are 53 for |prec| and TO\_NEAREST for |mode|.
}*/



  integer significant, exponent;
  special_values special;
  long precision;




  long non_zero_sign(void) const;



  void print_contents(void) const;
/* soll spaeter raus */


  void normalize(void);

 public:

   ~bigfloat(void) {
  }


/*{\Mcreation x }*/

/*{\Mtext
|bigfloat|s may be constructed from data types |double|, |long|, |int| and
|integer|, without loss of accuracy.
In addition, |bigfloat|s can be created as follows.
}*/

   bigfloat();
  bigfloat(double);
  bigfloat(long);
  bigfloat(int);
  bigfloat(const integer &);

  bigfloat(const integer &, const integer &);
/*{\Mcreate bigfloat(s,e) introduces a variable |\Mvar| of type |bigfloat|
 and initializes it to the value given by the tuple $(s,e)$
 }*/

  bigfloat(special_values);

/* the default copy constructor and assignment operator are used
     (element-wise copy) and do the right operations */


  long sign() const;




/*{\Moperations 1.75 3.5
}*/

/*{\Mtext
The arithmetical operations $+, -, *, \, sqrt$, the comparison operators
$<, \leq, >, \geq, =, \neq$ and the stream operators are all available.

Addition, subtraction, multiplication, division and squareroot are implemented
by the functions |add|, |sub|, |mul|, |div| and |sqrt|, respectively.
For example, the call $$add(x,y,prec,mode,is\_exact)$$
computes the sum of |bigfloat|s |x| and |y| up to |prec| binary digits, in
rounding mode |mode|, and returns it.
The optional last parameter |is_exact| is a boolean variable that is set to
|true| if and only if the returned |bigfloat| exactly equals the sum of
|x| and |y|.
The parameters |prec| and |mode| are also optional and have the global default
values |global_prec| and |round_mode|.
That is, the three calls |add(x,y,glob_prec,round_mode)|, |add(x,y,glob_prec)|,
|add(x,y)| are all equivalent.
The syntax for functions |sub|, |mul|, |div| and |sqrt| is analoguous.

The standard operators $+, -, *, \ $ are also available and are implemented by
their counterparts among the functions |add|, |sub|, |mul|, |div| and |sqrt|.
For example, the call $x+y$
is equivalent to |add(x,y)|.
}*/


  friend bigfloat add(const bigfloat &, const bigfloat &, long = global_prec,
    rounding_modes = round_mode, bool &is_exact = dbool);

  friend bigfloat sub(const bigfloat &, const bigfloat &, long = global_prec,
    rounding_modes = round_mode, bool &is_exact = dbool);

  friend bigfloat mul(const bigfloat &, const bigfloat &, long = global_prec,
    rounding_modes = round_mode, bool &is_exact = dbool);

  friend bigfloat div(const bigfloat &, const bigfloat &, long = global_prec,
    rounding_modes = round_mode, bool &is_exact = dbool);

  friend bigfloat sqrt(const bigfloat &, const bigfloat & start, long = global_prec,
    rounding_modes = round_mode, bool &is_exact = dbool);



  friend bigfloat operator + (const bigfloat &, const bigfloat &);
  friend bigfloat operator - (const bigfloat &, const bigfloat &);
  friend bigfloat operator *(const bigfloat &, const bigfloat &);
  friend bigfloat operator / (const bigfloat &, const bigfloat &);
  friend bigfloat operator - (const bigfloat &);





  friend bool operator == (const bigfloat &, const bigfloat &);
  friend bool operator > (const bigfloat &, const bigfloat &);

  friend bool operator != (const bigfloat & a, const bigfloat & b) {
    return !(a == b);
  }
  friend bool operator >= (const bigfloat & a, const bigfloat & b) {
    return ((a > b) || (a == b));
  }
  friend bool operator < (const bigfloat & a, const bigfloat & b) {
    return (!(a >= b));
  }
  friend bool operator <= (const bigfloat & a, const bigfloat & b) {
    return (!(a > b));
  }




/*{\Mtext
There are several functions to check whether the outcome of an arithmetical
computation on |bigfloat|s is a special-valued |bigfloat|.
These functions are necessary because the equality operator fails.
Further operations include some of the most important mathematical
functions, functions to approximate |bigfloat|s by |double|s or |integer|s,
and functions to set global precision, rounding mode, and output mode.
}*/

  bool isNaN() const {
    return (special == NAN_VALUE);
  }
/*{\Mop returns true if and only if |\Mvar| is |NaN|.
}*/

  bool isnInf() const {
    return (special == NINF_VALUE);
  }
/*{\Mop returns true if and only if |\Mvar| is |nInf|.
}*/

  bool ispInf() const {
    return (special == PINF_VALUE);
  }
/*{\Mop returns true if and only if |\Mvar| is |pInf|.
}*/


  bool isnZero() const {
    return (special == NZERO_VALUE);
  }
/*{\Mop returns true if and only if |\Mvar| is |nZero|.
}*/

  bool ispZero() const {
    return (special == PZERO_VALUE);
  }
/*{\Mop returns true if and only if |\Mvar| is |pZero|.
}*/

  bool isZero() const {
    return ((special == PZERO_VALUE) || (special == NZERO_VALUE));
  }
/*{\Mop returns true if and only if |\Mvar| is |pZero| or |nZero|.
}*/

  bool isInf(void) const {
    return ((special == PINF_VALUE) || (special == NINF_VALUE));
  }
/*{\Mop returns true if and only if |\Mvar| is |pInf| or |nInf|.
}*/

  bool isSpecial(void) const {
    return (special != NOT_VALUE);
  }
/*{\Mop returns true if and only if |\Mvar| is $\pm0$, $\pm\infty$ or |NaN|.
}*/


  integer to_integer(rounding_modes = TO_ZERO) const;
/*{\Mop returns the |integer| value next to |\Mvar| (in the corresponding rounding mode).
}*/


  inline integer ceil(void) {
    return to_integer(TO_P_INF);
  }
/*{\Mop rounds |\Mvar| up to the next integer.
}*/

  inline integer floor(void) {
    return to_integer(TO_N_INF);
  }
/*{\Mop rounds |\Mvar| down to the next integer.
}*/



  double to_double() const;
/*{\Mop returns the |double| value next to |\Mvar| (in rounding mode |TO_NEAREST|).
}*/

  double todouble() const { return to_double(); }
/*{\Xop returns the |double| value next to |\Mvar| (in rounding mode |TO_NEAREST|).




  int sign(bigfloat b) const { return b.sign(); }
/*{\Mfunc returns the sign of $b$.
}*/

  inline friend bigfloat fabs(const bigfloat & b) {
    if (b < pZero)
      return -b;
    else
      return b;
  }

  inline friend bigfloat abs(const bigfloat & b) {
    return fabs(b);
  }
/*{\Mfunc the absolute value of $b$. 
}*/

  inline friend bigfloat pow2(const integer & p) {
    return bigfloat(1, p);
  }
/*{\Mfunc returns 2 to the power of $p$. 
}*/

  inline friend integer log2(const bigfloat & x) {
    return x.get_precision() + x.get_exponent();
  }
/*{\Mfunc $log2(x)$ returns the binary logarithm of $x$, rounded up to the
    next integer.\\
    \precond: $x != 0$
}*/


  inline friend bigfloat sqrt(const bigfloat & x, long prec = global_prec,
    rounding_modes r_mode = round_mode, bool &is_exact = dbool) {return sqrt(x, pZero, prec, r_mode, is_exact);
  }

  inline friend bigfloat sqrt(const bigfloat & x) {
    return sqrt(x, pZero);
  }
/*{\Mfunc returns the square root of $x$ with precison $global\_prec$.\\
   \precond $x >= 0$. }*/




  friend ostream& operator << (ostream &, const bigfloat &);
/*{\Mbinopfunc writes the |bigfoat| in decimal or binary mode.
}*/
  friend istream& operator >> (istream &, bigfloat &);
/*{\Mbinopfunc the input format is $dd \cdots d.dd \cdots d E d \ldots d$ where
  $d$ stands for any decimal digit.
}*/



  long get_precision(void) const {
    return precision;
  }

  integer get_exponent() const {
    return exponent;
  }
/*{\Mop returns the exponent of |\Mvar|.
}*/

  integer get_significant() const {
    return significant;
  }
/*{\Mop returns the significant of |\Mvar|.
}*/





/*{\Mtext
A bigfloat |\Mvar| can be rounded by the call
$$x.round(prec,mode,is\_exact,bias).$$
Here |bias| is an optional integer variable taking values in $\{-1,0,+1\}$,
with default value $0$.
If |bias| is zero, the function rounds |\Mvar| up to |prec| binary digits in
rounding mode |mode|.
The optional boolean variable |is_exact| is set to true if and only if the
rounding operation did not change the value of $x$.
If |bias| is nonzero, the result is the same as above, only the value to be
rounded is not |\Mvar| but $\Mvar + bias \cdot \epsilon$, where $\epsilon$ is a
positive infinitesimal.
}*/

  bigfloat round(long = 0, rounding_modes = round_mode, bool &is_exact = dbool, long = 0);

  bigfloat round_with_error(long = 0, rounding_modes = round_mode, bool
    &is_exact = dbool, long = 0);



};



inline void set_glob_prec(long p)
{
  global_prec = p;
}
/*{\Mfunc sets the global precision to |p|
}*/

inline void set_round_mode(rounding_modes m = TO_NEAREST)
{
  round_mode = m;
}
/*{\Mfunc sets the global rounding mode.
}*/

inline void set_output_mode(output_modes o_mode = DEC_OUT)
{
  output_mode = o_mode;
}
/*{\Mfunc sets the output mode (possible values are |BIN_OUT| and |DEC_OUT|).
}*/

bigfloat outofchar(char* rep, long prec = 0);


#if LEDA_ROOT_INCL_ID == 340019
#undef LEDA_ROOT_INCL_ID
#include <LEDA/UNDEFINE_NAMES.h>
#endif

#endif
