/*******************************************************************************
+
+  LEDA 3.4
+
+  integer_vector.h
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

/* This file has been automatically generated from "integer_vector.w"
   by CTANGLE (Version 3.1 [km2c]) */

#ifndef LEDA_INTEGER_VECTOR_H
#define LEDA_INTEGER_VECTOR_H

#if !defined(LEDA_ROOT_INCL_ID)
#define LEDA_ROOT_INCL_ID 340055
#include <LEDA/REDEFINE_NAMES.h>
#endif


#include <LEDA/basic.h>
#include <LEDA/integer.h>


/*{\Manpage {integer_vector} {} {Vectors with Integer Entries}}*/

class integer_vector
{

/*{\Mdefinition
An instance of the data type |integer_vector| is a vector of variables
of type integer.}*/

friend class integer_matrix;

integer*v;
int d;

void check_dimensions(const integer_vector&)const;
void allocate_small(integer*&v,int d);
void deallocate_small(integer*v,int d);

public:

/*{\Mcreation v 5}*/

integer_vector();
/*{\Mcreate creates an instance |v| of type |integer_vector|; |v| is
            initialized to the zero-dimensional vector.}*/

integer_vector(int d);
/*{\Mcreate creates an instance |v| of type |integer_vector|; |v| is
            initialized to the zero vector of dimension $d$.}*/

integer_vector(integer a,integer b);
/*{\Mcreate creates an instance |v| of type |integer_vector|; |v| is
            initialized to the two-dimensional vector $(a,b)$.}*/

integer_vector(integer a,integer b,integer c);
/*{\Mcreate creates an instance |v| of type |integer_vector|; |v| is
            initialized to the three-dimensional vector $(a,b,c)$.}*/

integer_vector(const integer_vector&);
~integer_vector();
integer_vector&operator= (const integer_vector&);


/*{\Moperations 3.3 4.5}*/

int dim()const{return d;}
/*{\Mop       returns the dimension of |v|.}*/


integer&operator[](int i)
/*{\Marrop     returns $i$-th component of |v|.\\
	       \precond $0\le i \le \mbox{|v.dim()-1|}$. This check can be
               turned off by the flag \texttt{LEDA\_CHECKING\_OFF}.}*/
{
#ifndef LEDA_CHECKING_OFF
if(i<0||i>=d)
error_handler(1,"integer_vector::operator[]: index out of range.");
#endif
return v[i];
}

integer operator[](int i)const
{
#ifndef LEDA_CHECKING_OFF
if(i<0||i>=d)
error_handler(1,"integer_vector::operator[]: index out of range.");
#endif
return v[i];
}


integer_vector&operator+= (const integer_vector&v1);
/*{\Mbinop     Addition plus assignment.\\
	       \precond $v.dim() = v1.dim()$.}*/


integer_vector&operator-= (const integer_vector&v1);
/*{\Mbinop     Subtraction plus assignment.\\
	       \precond $v.dim() = v1.dim()$.}*/


integer_vector operator+(const integer_vector&v1)const;
/*{\Mbinop     Addition.\\
	       \precond $v.dim() = v1.dim()$.}*/

integer_vector operator-(const integer_vector&v1)const;
/*{\Mbinop     Subtraction.\\
	       \precond $v.dim() = v1.dim()$.}*/

integer operator*(const integer_vector&v1)const;
/*{\Mbinop     Inner Product.\\
               \precond $v.dim() = v1.dim()$.}*/

integer_vector compmul(integer r)const;

friend integer_vector operator*(const integer& r,const integer_vector&v);
/*{\Mbinopfunc     Componentwise multiplication with number $r$.}*/

friend integer_vector operator*(const integer_vector&v, const integer& r);
/*{\Mbinopfunc     Componentwise multiplication with number $r$.}*/

integer_vector operator-()const;

bool operator==(const integer_vector&w)const;
/*{\Mbinop     Test for equality.}*/

bool operator!=(const integer_vector&w)const
{return!(*this==w);}
/*{\Mbinop     Test for inequality.}*/



friend ostream&operator<<(ostream&O,const integer_vector&v);
/*{\Mbinopfunc  writes |v| componentwise to the output stream $O$.}*/

friend istream&operator>>(istream&I,integer_vector&v);
/*{\Mbinopfunc  reads |v| componentwise from the input stream $I$.}*/

LEDA_MEMORY(integer_vector)

};


LEDA_TYPE_PARAMETER(integer_vector)

/*{\Mimplementation
Vectors are implemented by arrays of integers. All operations on a
vector |v| take time $O(\mbox{|v.dim()|})$, except for |dim| and $[\ ]$
which take constant time. The space requirement is $O(\mbox{|v.dim()|})$.
}*/

#if LEDA_ROOT_INCL_ID == 340055
#undef LEDA_ROOT_INCL_ID
#include <LEDA/UNDEFINE_NAMES.h>
#endif

#endif



