/*******************************************************************************
+
+  LEDA 3.4
+
+  iteration.h
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#ifndef LEDA_ITERATION_H
#define LEDA_ITERATION_H

#if !defined(LEDA_ROOT_INCL_ID)
#define LEDA_ROOT_INCL_ID 340057
#include <LEDA/REDEFINE_NAMES.h>
#endif



//------------------------------------------------------------------------------
//   forall_items(it,...)
//------------------------------------------------------------------------------

#define forall_items(x,S)\
for(x = (S).first_item(); x; x = (S).next_item(x) )


//------------------------------------------------------------------------------
//   forall(x,...)
//   forall_rev(x,...)   (Forall)
//   forall_defined(x,...)
//------------------------------------------------------------------------------

// As long as the new scoping rules are not supported we avoid multiple 
// declarations of the forall-loop variable by placing it in the else 
// part of a dummy if-statement. 


#if defined(__GNUC__) || defined(__ELSE_SCOPE_BUG__)
#define LEDA_FORALL_PREAMBLE
#define for_loop_cat(x,y) x ## y 
#define for_loop_var(y) for_loop_cat(for_loop_var,y)
#else
#define LEDA_FORALL_PREAMBLE  if (0); else
#define for_loop_var(y) for_loop_var
#endif


#define forall(x,S)\
LEDA_FORALL_PREAMBLE \
for(GenPtr for_loop_var(__LINE__) = (S).first_item();\
(S).forall_loop_test(for_loop_var(__LINE__),x);\
(S).loop_to_succ(for_loop_var(__LINE__)))

#define forall_rev(x,S)\
LEDA_FORALL_PREAMBLE \
for(GenPtr for_loop_var(__LINE__) = (S).last_item();\
(S).forall_loop_test(for_loop_var(__LINE__),x);\
(S).loop_to_pred(for_loop_var(__LINE__)))

#define forall_defined(x,S)\
LEDA_FORALL_PREAMBLE \
for(GenPtr for_loop_var(__LINE__) = (S).first_item();\
(S).forall_defined_test(for_loop_var(__LINE__),x);\
(S).loop_to_succ(for_loop_var(__LINE__)))


// historical
#define Forall forall_rev


#if LEDA_ROOT_INCL_ID == 340057
#undef LEDA_ROOT_INCL_ID
#include <LEDA/UNDEFINE_NAMES.h>
#endif

#endif
