/*******************************************************************************
+
+  LEDA 3.4
+
+  pq_tree.h
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#ifndef LEDA_PQ_TREE_H
#define LEDA_PQ_TREE_H

#define NDEBUG

#include <LEDA/list.h>

#ifndef NDEBUG
#include <stdlib.h>
#endif

class pq_node_struct;

typedef pq_node_struct* pq_node;   


class pq_node_struct {

friend class pq_tree;
friend class show_pq_tree;

 unsigned char 	label;
 int 		leaf_index;
 int 		child_count;		//P only
 int 		pert_child_count;	//P and Q
 int 		pert_leaf_count; 	//number of pertinent leaves 
					//under this node
 int		full_count;
 pq_node        parent;
 pq_node        link1,link2;
 pq_node        left_most,right_most;	//"left_most_child"  
 pq_node        part_child1,part_child2;


public:

 pq_node_struct(); 

 void special_reset();
 char check(char);
 void mark(char);

 LEDA_MEMORY(pq_node_struct);

};




class pq_tree 
{

list<pq_node>	Processed;
pq_node 	*leaves;	// a dynamic array of size "leaf_size"
int 		leaf_size;
int 		block_count;
int 		blocked_nodes;
char		off_the_top;
char		too_many_part;
  
	bool 	array_copy(); 		// fuer Konstruktor mit U und update(S)
	void	reset(); 		// Ruecksetzen der bei reduction() prozessierten Knoten
	void 	del_pert_subtree(); 	// fr update(S), loescht alles unter p_root
	bool	bubble(list<int>& S);
	void	bubble_reset(pq_node);
	bool	reduce(list<int>& S);
  	void	del_subtree(pq_node);
  	void 	sequence(list<int>&, pq_node);
inline 	void	skip_di(pq_node&, pq_node&);
/*
inline 	void*	pq_new(size_t bytes);
inline 	void*	get_from_list();
inline 	void*	get_from_mem();
inline 	void	pq_delete(void* p,size_t bytes);
	void	pq_mem_init();
  	void	pq_mem_off();
*/ 
  bool	template_P1(pq_node x);
  bool	template_P2(pq_node x);
  bool	template_P3(pq_node x);
  bool	template_P4(pq_node x);
  bool	template_P5(pq_node x);
  bool	template_P6(pq_node x);
  bool	template_Q1(pq_node x);
  bool	template_Q2(pq_node x);
  bool	template_Q3(pq_node x);
  
//  void	hang_down_fulls(pq_node,pq_node);
  inline  void	fix_part_child_direction(pq_node x,char what=0);



  #ifndef NDEBUG
  int   cons_pq_tree(pq_node, char *,char*);
  #endif  


protected:

  pq_node 	p_root;
  pq_node 	root,newroot; 
  bool		positiv;
  
public:
  
  pq_tree();
  pq_tree(list<int>&);
 ~pq_tree();
  
  bool reduction(list<int>& S);
  bool update(list<int>& S);  // clears S 
  void frontier(list<int> &F);
  void pert_sequence(list<int> &S);

};

#endif

