/*******************************************************************************
+
+  LEDA 3.4
+
+  system.h
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#ifndef LEDA_SYSTEM_H
#define LEDA_SYSTEM_H

#if !defined(LEDA_ROOT_INCL_ID)
#define LEDA_ROOT_INCL_ID 340110
#include <LEDA/REDEFINE_NAMES.h>
#endif


//----------------------------------------------------------------------------
// system/compiler dependent definitions 
//----------------------------------------------------------------------------

// In the first section of this file some flags are defined indicating
// that certain c++ features are supported or not by particular compilers. 
// If your compiler does not appear you may have to add it.
// 
// __BUILTIN_BOOL__         : built-in boolean
// __NEW_SCOPE_RULES__      : new scope rules 
// __ELSE_SCOPE_BUG__       : else-part does not begin a new scope
// __TEMPLATE_FUNCTIONS__   : function templates  
// __TEMPLATE_ARG_BASE__    : template arguments may be used as base class
// __EXPLICIT_DESTRUCTION__ : explicit destructor call
// __INPLACE_NEW__          : in-place new operator
// __USE_VALUES_H__         : <value.h> header file 


//----------------------------------------------------------------------------
// define __BUILTIN_BOOL__ for compilers supporting type bool  
//----------------------------------------------------------------------------

#if defined(_BOOL) || defined(__GNUC__) || (__BORLANDC__ >= 0x500)
#define __BUILTIN_BOOL__
#endif


//----------------------------------------------------------------------------
// define __NEW_SCOPE_RULES__ for compilers supporting  the new scope rules
// currently only g++ >= 2.7
//----------------------------------------------------------------------------

#if (__GNUC__ > 2 || __GNUC_MINOR__ >= 7)
#define __NEW_SCOPE_RULES__
#endif


//----------------------------------------------------------------------------
// define __ELSE_SCOPE_BUG__ for compilers not starting a new scope for the
// else-part of an if-statement  
//----------------------------------------------------------------------------

#if !defined(__GNUC__)

#if defined(sgi) && _COMPILER_VERSION < 602
#define __ELSE_SCOPE_BUG__
#endif

#if defined(hpux)
#define __ELSE_SCOPE_BUG__
#endif

#endif


//----------------------------------------------------------------------------
// define  __TEMPLATE_FUNCTIONS__ except for compilers  having problems
// with function templates  (bcc < 4.5, DEC C++)
//----------------------------------------------------------------------------

#define __TEMPLATE_FUNCTIONS__

#if defined(__BORLANDC__) && (__BORLANDC__ < 0x500)
#undef __TEMPLATE_FUNCTIONS__
#endif

#if defined(__DECCXX)
#undef __TEMPLATE_FUNCTIONS__
#endif

//----------------------------------------------------------------------------
// define  __TEMPLATE_ARG_BASE__ if template arguments can be used as
// base classes in derivations
//----------------------------------------------------------------------------

#define __TEMPLATE_ARG_BASE__

#if defined(__ATTCFRONT__)
#undef __TEMPLATE_ARG_BASE__
#endif

#if defined(__lucid)
#undef __TEMPLATE_ARG_BASE__
#endif

#if defined(hpux) &&  !defined(__GNUC__)
#undef __TEMPLATE_ARG_BASE__
#endif


//----------------------------------------------------------------------------
// define __EXPLICIT_DESTRUCTION__ except for compilers not supporting
// this feature (AT&T cfront, borland, mips, xlC)
//----------------------------------------------------------------------------

#define __EXPLICIT_DESTRUCTION__

#if defined(__ATTCFRONT__)
#undef __EXPLICIT_DESTRUCTION__
#endif

#if defined(__BORLANDC__) && (__BORLANDC__ < 0x500)
#undef __EXPLICIT_DESTRUCTION__
#endif

#if defined(mips) && _COMPILER_VERSION < 602 && !defined(__GNUC__)
#undef __EXPLICIT_DESTRUCTION__
#endif

#if defined(__xlC__)
#undef __EXPLICIT_DESTRUCTION__
#endif

#if defined(__IBMCPP__)
#undef __EXPLICIT_DESTRUCTION__
#endif

//----------------------------------------------------------------------------
// define  __INPLACE_NEW__ except for compilers not supporting this feature 
//----------------------------------------------------------------------------

#define __INPLACE_NEW__

#if defined(__BORLANDC__) && (__BORLANDC__ < 0x500)
#undef __INPLACE_NEW__
#endif



//----------------------------------------------------------------------------
// define  __USE_VALUES_H__ for compilers/systems having a <values.h> file 
//----------------------------------------------------------------------------

#if defined(unix) || defined(_AIX) || defined(__BORLANDC__)
#if !defined(__DECCXX)
#define __USE_VALUES_H__
#endif
#endif




//------------------------------------------------------------------------------
//  Other Flags
//------------------------------------------------------------------------------

#if !defined(__svr4__) && ( defined(__SVR4) || defined(__SOLARIS_23) || defined(__SOLARIS_24) )
#define __svr4__
#endif

#if !defined(unix) && defined(_AIX)
#define unix
#endif

#if defined(_MSC_VER)||defined(__NT__)||defined(_Windows) || __GNUC__==rsxnt-2
#define __WINDOWS__
#endif



#if defined(__STDC__)
#define STRINGIZE(x) #x
#else
#define STRINGIZE(x) "x"
#endif




//------------------------------------------------------------------------------
//  DLL definitions
//------------------------------------------------------------------------------

#if !defined(BUILD_DLL) && !defined(USE_DLL)
#define __exportC
#define __exportF
#define __exportD

#else

#if defined(BUILD_DLL)
#if defined(_MSC_VER)
#define __exportC __declspec(dllexport)
#define __exportF __declspec(dllexport)
#define __exportD __declspec(dllexport)
#else
#define __exportC __export
#define __exportF __export
#define __exportD __export
#endif
#endif

#if defined(USE_DLL)
#if defined(_MSC_VER)
#define __exportC __declspec(dllimport)
#define __exportF __declspec(dllimport)
#define __exportD __declspec(dllimport)
#else
#define __exportC __import
#define __exportF __import
#define __exportD __import
#endif
#endif

#endif


#if LEDA_ROOT_INCL_ID == 340110
#undef LEDA_ROOT_INCL_ID
#include <LEDA/UNDEFINE_NAMES.h>
#endif


#endif
