#include <LEDA/bigfloat.h>
#include <math.h>


/*----------------------------------------------------------------------
 This is our standard testfile for checking the LEDA data type bigfloat
------------------------------------------------------------------------*/

main()
{

//----------------------------------------------------------------------
// Input and output tested
//------------------------------------------------------------------------

char c;

char*    str_in = "7.612712412452375193581591359E6152";
bigfloat number = outofchar(str_in);

cout << "in : " << str_in << endl;
cout << "out: " << number << endl;

//----------------------------------------------------------------------
// double-bigfloat conversion tested
//----------------------------------------------------------------------

cout << "double-bigfloat conversion tested\n" << endl;

int i,j;
double powi = 1; 
double x;
double double_min = ldexp(1.0,-1022);

for (i=1;i<=53;i++)
{
  powi = powi*2;
  x = (powi-1)*double_min;
  for (j=1;j<=52+i;j++)
  { x = x/double(2);
    if (x != bigfloat(x).todouble())
      error_handler(1,"error in double-bigfloat conversion");
    if (-x != bigfloat(-x).todouble())
      error_handler(1,"error in double-bigfloat conversion");
  }
}

//-----------------------------------------------------------------------
// addition, subtraction, multiplication and division tested with precision 
// 53 and rounding mode "to nearest"
//-----------------------------------------------------------------------

 cout << "addition, subtraction, multiplication and division tested with";
 cout << "precision 53 and rounding mode \"to nearest\"" << endl;

 double y;
 int exp;
 bigfloat x_bf, y_bf;

 for (i=1;i<=53;i++)
 { 
   cout << "i = " << i << endl; 

   for (j=1; j<=i; j++)
   for (exp=0;exp<=53 + j - i;exp++)
   {
     x = (pow(2,i)-1)*pow(2,exp);
     y = pow(2,j)-1;

     // x = 11  ...  1 * 2^exp
     //     [i times ]

     // y = 11  ...  1
     //     [j times ]

     x_bf = x; y_bf = y;

     if ( x+y != add(x_bf,y_bf,53,TO_NEAREST).todouble() )
       error_handler(1,"error in addition");
     if ( y-x != sub(y_bf,x_bf,53,TO_NEAREST).todouble() )
       error_handler(1,"error in subtraction");
     if (exp == 0) // here exp not important
       if ( x*y != mul(x_bf,y_bf,53,TO_NEAREST).todouble() )
         error_handler(1,"error in multiplication");
     if (exp == 0) // here exp not important
       if ( bigfloat(x/y) != div(x_bf,y_bf,53,TO_NEAREST) )
        error_handler(1,"error in division");
   } // for j, exp

 } // for i

/*------------------------------------------------------------------------
  sqrt tested with different precisions and rounding mode "to_nearest"
  ------------------------------------------------------------------------*/

 cout << "sqrt tested with different precisions and rounding mode \
          \"to_nearest\"" << endl;

 int prec;
 bigfloat seven = 7, sqrt7;
 bigfloat sqrt_low,sqrt_high;
 bigfloat sum;
 integer exponent, sig;

 for (prec=2;prec<=10000;prec*=2)
 {
   cout << "prec = " << prec << endl;
   sqrt7 = sqrt(seven,prec,TO_NEAREST);
   exponent = sqrt7.get_exponent();
   sig = sqrt7.get_significant()-1;
   sqrt_low  = bigfloat(sig,exponent);
   sig = sqrt7.get_significant()+1;
   sqrt_high = bigfloat(sig,exponent);

   // first we test whether sqrt_low is less than sqrt(7) and whether
   // sqrt_high is more than sqrt(7)
 
   if (    (mul(sqrt_low ,sqrt_low ,53,EXACT) >= seven)
        || (mul(sqrt_high,sqrt_high,53,EXACT) <= seven)
      )
     error_handler(1,"error in sqrt");

   // now we test whether sqrt7 really is nearer to sqrt(7) than sqrt_low
   // and sqrt_high

   if  (mul(sqrt7,sqrt7,53,EXACT) >= seven)
   {
     // compare sqrt7 - sqrt(7) and sqrt(7) - sqrt_low

     sum = add(sqrt7,sqrt_low,1,EXACT);
     if ( mul(sum,sum,53,EXACT) > bigfloat(28) )
       error_handler(1,"sqrt not nearest value");
   }
   else
   {
     // compare sqrt(7) - sqrt7 and sqrt_high - sqrt(7)

     sum = add(sqrt7,sqrt_high,1,EXACT);
     if ( mul(sum,sum,53,EXACT) < bigfloat(28) )
       error_handler(1,"sqrt not nearest value");
   }

 } // for


/*--------------------------------------------------------------------------
  tests with various operations and all rounding modes of IEEE, in precision 53;
  we use the taylor series to approximate sin(x)
  NOTE: in bigfloat.h the debug mode DEBUG_BF must be defined
 ---------------------------------------------------------------------------*/

 cout << "  tests with various operations and all rounding modes of IEEE, in \
            precision 53;\n we use the taylor series to approximate sin(x)";
 cout << endl;

 set_glob_prec(53);

 bigfloat s, ds, z, x_sq, sign = -1;
 int k ;

for (j=1;j<=100;j++) 
{
 cout << j << " " << flush;

 x_sq = j*j;
 
 set_round_mode(TO_NEAREST);

 s = j; ds = j; sign = -1; 
 i = 1;
 while (ds >= ldexp(1.0,-60))
 {
   ds = ds * x_sq;
   k = 2*i*(2*i+1);
   ds = ds / k;
   s = s + sign*ds;
   sign = -sign;
   i++;
 }

 set_round_mode(TO_ZERO);

 s = j; ds = j; sign = -1; 
 i = 1;
 while (ds >= ldexp(1.0,-60))
 {
   ds = ds * x_sq;
   k = 2*i*(2*i+1);
   ds = ds / k;
   s = s + sign*ds;
   sign = -sign;
   i++;
 }

 set_round_mode(TO_P_INF);

 s = j; ds = j; sign = -1; 
 i = 1;
 while (ds >= ldexp(1.0,-60))
 {
   ds = ds * x_sq;
   k = 2*i*(2*i+1);
   ds = ds / k;
   s = s + sign*ds;
   sign = -sign;
   i++;
 }

 set_round_mode(TO_N_INF);

 s = j; ds = j; sign = -1; 
 i = 1;
 while (ds >= ldexp(1.0,-60))
 {
   ds = ds * x_sq;
   k = 2*i*(2*i+1);
   ds = ds / k;
   s = s + sign*ds;
   sign = -sign;
   i++;
 }

}

cout << endl << endl << "Alle Tests absolviert" << endl;


return 0;

} // main
