#include <LEDA/real.h>
#include <LEDA/array.h>
#include <math.h>

main()
{
  float time;
  float overall_time;

/*------------------------------------------------------------------------
teste real::sign an Beispiel aus Voronoidiagrammrechnung
--------------------------------------------------------------------------*/

cout << "Teste real::sign an Beispiel aus Voronoidiagrammberechnung\n";

real a1= 15,a2= 21,b1= -19, b2= -a2*b1/a1;
real F,G,SQR;

bool failed = false;
if ((a1*b2+a2*b1).sign() != 0) failed = true;

SQR= sqrt( (a1*a1+b1*b1)*(a2*a2+b2*b2) );
F= real(2)*(SQR-a1*a2+b1*b2);
G= real(2)*(SQR+a1*a2-b1*b2);

if (G.sign() != 1) failed = true;
if (F.sign() != 0) failed = true;

if (failed)
  error_handler(1,"Fehler in Voronoitest");

/*-------------------------------------------------------------------------
teste real::sign an Beispiel der Exponentialfunktion (berechne e)
teste au"serdem, ob Exaktheit erkannt wird.
---------------------------------------------------------------------------*/

cout << "Teste real::sign, resp real::operator== am Beispiel der e-Funktion mit Taylorreihe\n";

int k = 20;
int i,j;
array<real> euler1(0,k+1), euler2(0,k+1);
real faculty=1;
real fraction;
real y = 1;
real one = 1;

i = 1;
cout << "i = ";
overall_time = used_time();
for (i=1; i <= k; i++)
{
  cout << i << " " << flush;
  faculty *= real(i);
  euler1[i] = euler2[i] = one;
  j = i;
  fraction = 1;
  while (j > 0)
  {
    fraction *= real(j);
    euler1[i] = euler1[i]*real(y) + fraction;
    euler2[i] = euler2[i]*real(y)/real(j) + real(1);
    j--;
  }
  euler1[i] = euler1[i] / faculty;
  time = used_time();
  if (euler1[i] != euler2[i])
    error_handler(1,"error in computation of euler number");
  if (i == 20)
  {
    cout << "\nZeit fuer Vergleich gleicher Werte: " << used_time() - time;
    cout << "sec \n";
  }
  time = used_time();
  if (i > 1)
    if (euler1[i] == euler2[i-1])
       error_handler(1,"error in computation of euler number");
  if (i == 20)
  {
    cout << "Zeit fuer Vergleich ungleicher Werte: " << used_time() - time;
    cout << "sec \n";
  }
}
cout << "overall time: " << used_time() - overall_time << "sec \n";


cout << "\nTeste, ob Exaktheit automatisch erkannt wird am Beispiel der e-Funktion mit Taylorreihe\n";

euler1[20] = one;
j = k;
fraction = 1;
time = used_time();
while (j > 0)
{
  fraction *= real(j);
  euler1[20] = euler1[20]*real(y) + fraction;
  j--;
}
cout << "Zeit fuer Taylorauswertung: " << used_time() - time << "sec \n";

time = used_time();
euler1[20].reduce_error_in_approximation_to(1000);
cout << "Zeit fuer Euler-improve: " << used_time() - time << "sec \n";

if (!euler1[20].get_bigfloat_error().isZero()) 
  cout << "Mangel: Exaktheit nicht erkannt";





/*---------------------------------------------------------------------------
teste real::sign an Beispiel der Wurzelberechnung mit Newton
----------------------------------------------------------------------------*/

cout << "Teste real::sign, real::improve, real::compute_with_mantissa_length, real::compute_with_mantissa_length am Beispiel der Wurzelberechnung mit Newton\n";

int prec = 53;

real result;
real it; // iterator in Newton iteration
real& ne = result; // next iteration value in Newton iteration
real diff;

bigfloat sqrt2;
  
cout << "prec = ";
while (prec < 1000)
{
  cout << prec << " " << flush;
  ne = sqrt(2.0);
  time = used_time();
  do {  it=ne; ne = (it*it + 2)/(2*it); }
  while (fabs(ne-it) >= pow2(-prec-1));
  cout << "Zeit fuer Newtoniteration: " << used_time() - time << "sec \n";

  sqrt2 = sqrt(2,prec+1);

  if (fabs(result - sqrt2) >= pow2(-prec))
     error_handler(1,"Fehler in sqrt bei Newtoniteration");

  result.compute_with_mantissa_length(53);
  
  time = used_time();
  result.reduce_error_in_approximation_to(prec+1);
  cout << "Zeit fuer improve: " << used_time() - time << "sec \n";
  if (fabs(result.tobigfloat()-sqrt2) >= pow2(-prec))
    error_handler(1,"Fehler in sqrt bei improve");
  
  result.compute_with_mantissa_length(53);
  time = used_time();
  result.compute_with_mantissa_length(prec+8);
  cout << "Zeit fuer compute_with_mantissa_length: " << used_time() - time << "sec \n";
  if (fabs(result.tobigfloat()-sqrt2) >= pow2(-prec))
    error_handler(1,"Fehler in sqrt bei compute_with_mantissa_length");
  
  result.compute_with_mantissa_length(53);
  time = used_time();
  result.compute_with_mantissa_length(prec+1);
  cout << "Zeit fuer compute_with_mantissa_length: " << used_time() - time << "sec \n";
  if (fabs(result.tobigfloat()-sqrt2) >= pow2(-prec))
    error_handler(1,"Fehler in sqrt bei guarantee_relative_error_two_to_minus");

  prec*=2;
}
cout << "\n";


/*--------------------------------------------------------------------------
interne Funktion detect_degeneracies getestet
---------------------------------------------------------------------------*/

cout << "teste interne Funktion detect_degeneracies\n";
int m;
real sum=0;
time=used_time();
for (m=1;m<=50;m++) 
{ sum = sum + (2*sqrt(real(2))-sqrt(real(8))); 
} 
cout << "Zeit fuer Summenberechnung: " << used_time() - time << "sec \n";
time=used_time();
if (sum.sign() != 0) 
  error_handler(1,"sign of sum wrong");
cout << "Zeit fuer detect_degeneracies: " << used_time() - time << "sec \n";


/*--------------------------------------------------------------------------
degenerierter incircle test
---------------------------------------------------------------------------*/

cout << "mache degenerierten incircle Test\n";

real d = sqrt(2.0);

real a_1=0,b_1= 1,c_1=  d,
     a_2=0,b_2=-1,c_2=  d,
     a_3=1,b_3= 0,c_3= -2*d;

int sign_x=1; int sign_y=1;

real A = a_1*c_2+a_2*c_1; // fr"uher K
real B = b_1*c_2+b_2*c_1; // fr"uher I

real C = 2*c_1*c_2; 

real D = a_1*a_2-b_1*b_2;
real S = a_1*b_2+a_2*b_1;

real RN = sqrt( (a_1*a_1+b_1*b_1)*(a_2*a_2 + b_2*b_2) );

real sqr1 = sqrt(C*(RN + D));
real sqr2 = sqrt(C*(RN - D));

real x_v = A + sign_x*sqr1;
real y_v = B + sign_y*sqr2;
real z_v = RN - (a_1*a_2+b_1*b_2);

real P = a_3*x_v+b_3*y_v+c_3*z_v;

real R = P*P - (a_3*a_3+b_3*b_3)*(x_v*x_v+y_v*y_v);

R.compute_with_mantissa_length(60);
R.reduce_error_in_approximation_to(100);

time = used_time();
if (R.sign() != 0)
  error_handler(1,"incircle Test: Degeneriertheit nicht erkannt");
cout << "Zeit fuer degenerierten sign Test: " << used_time() - time << "sec \n";


/*--------------------------------------------------------------------------
nicht-degenerierter incircle test
---------------------------------------------------------------------------*/

cout << "mache nicht-degenerierten incircle Test\n";

{
real a_1=-12,b_1= -7,c_1=  23,
     a_2=2,b_2=-3,c_2=  9,
     a_3=14,b_3= 10,c_3= 5;

real A = a_1*c_2+a_2*c_1; // fr"uher K
real B = b_1*c_2+b_2*c_1; // fr"uher I

real C = 2*c_1*c_2; 

real D = a_1*a_2-b_1*b_2;

real RN = sqrt( (a_1*a_1+b_1*b_1)*(a_2*a_2 + b_2*b_2) );

real sqr1 = sqrt(C*(RN + D));
real sqr2 = sqrt(C*(RN - D));

real T = fabs(a_1)*b_2+fabs(a_2)*b_1;
real S = a_1*b_2+a_2*b_1;

int sign_x = T.sign();
int sign_y = sign_x*S.sign();

real x_v = A + sign_x*sqr1;
real y_v = B + sign_y*sqr2;
real z_v = RN - (a_1*a_2+b_1*b_2);

real P = (a_3*x_v+b_3*y_v)+c_3*z_v;
real D3_sq = a_3*a_3+b_3*b_3;
real R_sq = x_v*x_v+y_v*y_v;

real R = P*P - D3_sq*R_sq;

R.compute_with_mantissa_length(60);
R.reduce_error_in_approximation_to(100);

time = used_time();
if (R.sign() == 0)
  error_handler(1,"incircle Test: ist nicht degeneriert");
cout << "Zeit fuer degenerierten sign Test: " << used_time() - time << "sec \n";
}



/*--------------------------------------------------------------------------
schwieriger nicht-degenerierter incircle test
---------------------------------------------------------------------------*/

cout << "mache schwierigen nicht-degenerierten incircle Test\n";

{
real a_1=-12,b_1= -7,c_1=  23,
     a_2=2,b_2=-3,c_2=  9,
     a_3=14,b_3= 10,c_3= 5;

real A = a_1*c_2+a_2*c_1; // fr"uher K
real B = b_1*c_2+b_2*c_1; // fr"uher I

real C = 2*c_1*c_2; 

real D = a_1*a_2-b_1*b_2;

real RN = sqrt( (a_1*a_1+b_1*b_1)*(a_2*a_2 + b_2*b_2) );

real sqr1 = sqrt(C*(RN + D));
real sqr2 = sqrt(C*(RN - D));

real T = fabs(a_1)*b_2+fabs(a_2)*b_1;
real S = a_1*b_2+a_2*b_1;

int sign_x = T.sign();
int sign_y = sign_x*S.sign();

real x_v = A + sign_x*sqr1;
real y_v = B + sign_y*sqr2;
real z_v = RN - (a_1*a_2+b_1*b_2);

real P = (a_3*x_v+b_3*y_v)+c_3*z_v;
real D3_sq = a_3*a_3+b_3*b_3;
real R_sq = x_v*x_v+y_v*y_v;

real R = P*P - D3_sq*R_sq;

R.compute_with_mantissa_length(60);
R.reduce_error_in_approximation_to(100);

time = used_time();
if (R.sign() == 0)
  error_handler(1,"incircle Test: ist nicht degeneriert");
cout << "Zeit fuer schwierigen degenerierten sign Test: " << used_time() - time << "sec \n";
}







/*--------------------------------------------------------------------------
 Oliviers Test mit 2x2 Determinante.
 --------------------------------------------------------------------------*/

cout << "Oliviers Test mit der Determinante\n";
 
real a11 =  (double)  6576833001171515.0;
real a21 =  (double)  4645552482121323.0;
real a12 =  (double) -4901643281669312.0;
real a22 =  (double) -3462280570234346.0;
  
if ((a11*a22-a12*a21).sign() != 1)
  error_handler(1,"Oliviers Test versagt");

a11 = 40000000000000.0; a12 = a11+1; a21 = a11+2; a22 = a11+3;
 
if ((a11*a22-a12*a21).sign() != -1)
  error_handler(1,"Oliviers Test versagt");
 

/*--------------------------------------------------------------------------
 Stefan Lutters Test mit Wurzeln
 --------------------------------------------------------------------------*/

cout << "Stefan Lutters Test mit der Determinante\n";

time = used_time();
real H = 3*sqrt(real(2))/sqrt(real(20));
real I = (sqrt(real(8)) + sqrt(real(2)))/(2*sqrt(real(5)));
if ((H-I).sign() != 0)
  error_handler(1,"Stefan Lutters Test versagt");
cout << "Zeit fuer Lutter Test: " << used_time() - time << "sec \n";


/*--------------------------------------------------------------------------
 Kurts Test mit 3. binomischer Formel.
 --------------------------------------------------------------------------*/

cout << "Kurts Test mit der 3. binomischen Formel\n";

integer a = 1;
real b,w,z;

cout << "length ";
while (a.length() <= 50)
{
  a *= 2;
  b = a;
  cout << a.length() << " ";
  w = (sqrt(b+2) - sqrt(b))*(sqrt(b+1) + sqrt(b)) - 1;
  z = (sqrt(b+1) - sqrt(b))*(sqrt(b+1) + sqrt(b)) - 1;
  if (w.sign() == 0) 
    error_handler(1,"Kurts Test gibt Fehler");
  if (z.sign() != 0) 
    error_handler(1,"Kurts Test gibt Fehler");
}
cout << "\n";

/*--------------------------------------------------------------------------
 nochmal Kurts Test mit 3. binomischer Formel, zum Messen der Laufzeit.
 --------------------------------------------------------------------------*/

cout << "Kurts Test mit der 3. binomischen Formel, zur Laufzeitmessung\n";

random_source Source;
int K,L;
int s;
integer Ran;
real sqr_b, sqr_b_inc;
for (K=1;K<=512;K*=2)
{
  cout << "k = " << K << "\n";
  b = pow2(integer(K));
  
  if (K <= 31) 
  { 
    Source.set_precision(K);
    Source >> s;
    b = s;
  }
  else
  {
    L = K;
    Ran = 0;
    Source.set_precision(31);
    while(L >= 0)
    {
      Ran = Ran << 32;
      Source >> s;
      Ran += s;
      L = L-32;
    }
    b = Ran;
  }

  sqr_b = sqrt(b);
  sqr_b_inc = sqrt(b+1);
  w = (sqr_b_inc - sqr_b)*(sqr_b_inc + sqr_b);
  time = used_time();
  s = (w-1).sign();
  time = used_time(time);
  cout << " hard sign braucht Zeit " << time << "\n";
  time = used_time();
  // s = (w-2).sign();
  // time = used_time(time);
  // cout << "easy sign braucht Zeit " << time << "\n";
  // time = used_time();
  // if (b != 0) s = (w/b).sign();
  // time = used_time(time);
  // cout << "average sign braucht Zeit " << time << "\n";
}

/*--------------------------------------------------------------------------
  fertig
----------------------------------------------------------------------------*/

cout << "Alle Tests erfolgreich absolviert\n";

return 0;

} // main
