#include "ged.h"


void make_handler(int i)
{ 
  switch (i) {
   case 0: { list<edge> L = Make_Connected(*Gp);
             edge e;
             forall(e,L) M(source(e),target(e))++;
             break;
            }
   case 1: { list<edge> L = Make_Biconnected(*Gp);
             edge e;
             forall(e,L) M(source(e),target(e))++;
             break;
            }
   case 2: { Make_Simple(*Gp);
             edge e;
             forall_edges(e,*Gp) M(source(e),target(e)) = 1;
             break;
            }
   }

  Wp->clear();
  draw_graph(*Wp,*Gp);
}



void embed_handler(int i)
{ switch (i) {
     case 0: random_embed(*Gp);
             break;
     case 1: grid_embed(*Gp);
             break;
     case 2: circular_embed(*Gp);
             break;
     case 3: planar_embed(*Gp,0);
             break;
     case 4: planar_embed(*Gp,1);
             break;
     case 5: spring_embed(*Gp);
             break;
     case 6: st_embed(*Gp);
             break;
   }
  Wp->clear();
  draw_graph(*Wp,*Gp);
}



void gen_handler(int i)
{ 
  list<node> A,B;
  node_array<double> xcoord(*Gp);
  node_array<double> ycoord(*Gp);

  double w  = x_max - x_min;
  double h  = y_max - y_min;
  double xl = x_min + w/4;
  double xr = x_max - w/4;
  double R  = w/2.5;
  point  M  = point(x_min + w/2, y_min + h/2);

  int n = n_gen;
  int m = m_gen;

  clear_all(0);

  switch(i) {

    case 0: random_graph(*Gp,n,m);
            Make_Simple(*Gp);
            Delete_Loops(*Gp);
            circular_embed(*Gp);
            break;
    case 1: if (directed)
               complete_graph(*Gp,n);
            else
               complete_ugraph(*Gp,n);
            circular_embed(*Gp);
            break;

    case 2: random_bigraph(*Gp,n/2,n/2,m,A,B);
            bi_embed(*Gp,A,B);
            break;

    case 3: complete_bigraph(*Gp,n/2,n/2,A,B);
            bi_embed(*Gp,A,B);
            break;

    case 4: random_planar_graph(*Gp,xcoord,ycoord,n);
            embed(*Gp,xcoord,ycoord);
            break;

    case 5: grid_graph(*Gp,xcoord,ycoord,int(sqrt(double(n))));
            embed(*Gp,xcoord,ycoord);
            break;

    case 6: triangulated_planar_graph(*Gp,xcoord,ycoord,n);
            embed(*Gp,xcoord,ycoord);
            break;
   }

  matrix_init(*Gp);
  Wp->clear();
  draw_graph(*Wp,*Gp);
}



void file_diag(int)
{
  string filename = "graph.ggg";

  panel P("FILE PANEL");
  P.string_item(string("File:"),filename);
  P.button("load",0);
  P.button("read",1);
  P.button("save",2);
  P.button("exit",3);

  switch (P.open(*Wp)) {

      case 0 : // load 
               read_graph(*Wp,*Gp,filename,true);
               break;

      case 1 : // read
               read_graph(*Wp,*Gp,filename,false);
               break;
  
      case 2 : // save
               save_graph(*Wp,*Gp,filename);
               break;
   }
}


void settings_diag(int)
{ panel P("GRAPH EDIT SETTINGS");
  P.int_item("# nodes", n_gen,0,100);
  P.int_item("# edges", m_gen,0,100);
  //P.int_item("x_min",x_min);
  //P.int_item("x_max",x_max);
  //P.int_item("y_min",y_min);
  P.int_item("node width",node_width,1,20);
  P.int_item("edge width",edge_width,1,5);
  P.int_item("anim speed",anim_speed,0,80);
  P.int_item("iterations",sp_iterations,1,500);
  P.bool_item("directed",directed);
  P.color_item("node color",node_color);
  P.color_item("edge color",edge_color);
  P.button("continue");
  P.open(*Wp);
  if (Wp->mono()) node_color = white;
  window_init(*Wp);
  draw_graph(*Wp,*Gp);
}


void help_diag(int)
{ panel P("MOUSE BUTTONS");
  P.text_item("");
  P.text_item("    LEFT BUTTON                                 ");
  P.text_item("creates a new node if clicked on a free position");
  P.text_item("creates a new edge if clicked on a node and held down");
  P.text_item("moves a node if clicked on a node and held down");
  P.text_item("moves a component if clicked on a node and held down");
  P.text_item("");
  P.text_item("    MIDDLE BUTTON                              ");
  P.text_item("shows node info if clicked on a node           ");
  P.text_item("");
  P.text_item("    RIGHT BUTTON                               ");
  P.text_item("deletes a node if clicked on it                ");
  P.text_item("deletes an edge if clicked on it               ");
  P.button("continue");
  P.open(*Wp);
}


void clear_all(int)
{ Wp->clear();
  Wp->set_mode(xor_mode);
  Gp->clear();
  matrix_init(*Gp);
}


void exit_diag(int) { done = true; }

void redraw_diag(int)
{ Wp->clear();
  draw_graph(*Wp,*Gp);
 }


void zoom_handler(int i)   
{ 
  switch(i) {
    case 0: zoom_graph(*Wp,*Gp,1.25); 
            break;
    case 1: zoom_graph(*Wp,*Gp,0.75); 
            break;
   }
}
