#include "ged.h"

void draw_node(window& W, GRAPH<point,int>& G, node v)
{ W.draw_filled_node(G[v],node_color); }


void high_light_node(window& W, GRAPH<point,int>& G, node v) 
{ color high_color = (node_color!=yellow) ? yellow : green; 
  W.draw_filled_node(G[v],node_color);
  W.draw_filled_node(G[v],high_color);
}

void unhigh_light_node(window& W, GRAPH<point,int>& G, node v) 
{ color high_color = (node_color!=yellow) ? yellow : green; 
  W.draw_filled_node(G[v],high_color);
  W.draw_filled_node(G[v],node_color);
}




void draw_edge(window& W, GRAPH<point,int>& G, node v, node w)
{ 
  // M(v,w) = number of edges from v to w (defines thickness of drawn edge)

  if (v == w) return; // still cannot draw self-loops 

  if (M(v,w) == 0) return;  // there is no edge (v,w)

  int lw_save = W.set_line_width(M(v,w)*edge_width);

  segment s(G[v],G[w]);

  if (directed) 
    if (M(w,v) == 0)
       W.draw_edge_arrow(s,edge_color);
    else
       W.draw_arc_edge_arrow(s,0.7*s.length(),edge_color);
  else 
    if (M(w,v) == 0)
       W.draw_edge(s,edge_color); 
    else
       W.draw_arc_edge(s,0.7*s.length(),edge_color);

  W.set_line_width(lw_save);
}

 
void draw_node_with_edges(window& W, GRAPH<point,int>& G, node v)
{ //edge e;
  //forall_in_edges(e,v)  draw_edge(W,G,source(e),v);
  //forall_out_edges(e,v) draw_edge(W,G,v,target(e));
  node u;
  forall_nodes(u,G)
  { draw_edge(W,G,v,u);
    draw_edge(W,G,u,v);
   }
  draw_node(W,G,v);
}

void draw_node_with_edges(window& W, GRAPH<point,int>& G, node v, point p)
{ point save = G[v];
  G[v] = p;
  //edge e;
  //forall_in_edges(e,v)  draw_edge(W,G,source(e),v);
  //forall_out_edges(e,v) draw_edge(W,G,v,target(e));
  node u;
  forall_nodes(u,G)
  { draw_edge(W,G,v,u);
    draw_edge(W,G,u,v);
   }
  draw_node(W,G,v);
  G[v] = save;
}


void draw_graph(window& W, GRAPH<point,int>& G)
{ node v,w;
  forall_nodes(v,G) 
  { draw_node(W,G,v);
    forall_nodes(w,G) draw_edge(W,G,v,w);
   }
}

void draw_graph(window& W, GRAPH<point,int>& G, list<node>& L)
{ list_item i;
  forall_items(i,L) 
  { draw_node(W,G,L[i]);
    list_item j;
    forall_items(j,L) draw_edge(W,G,L[i],L[j]);
   }
}


void redraw_func()  { draw_graph(*Wp, *Gp); }



// moving 


static void draw_graph(const GRAPH<point,int>& G, window& W)
{ node v;
  forall_nodes(v,G) W.draw_filled_node(G[v],node_color);
  edge e;
  forall_edges(e,G)
     W.draw_edge(G[source(e)],G[target(e)],edge_color);
}

static void draw_graph(const GRAPH<point,int>& G, window& W, 
                       const node_array<double>& xc, 
                       const node_array<double>& yc)
{ node v;
  forall_nodes(v,G) W.draw_filled_node(xc[v],yc[v],node_color);

  edge e;
  forall_edges(e,G)
  { node v = source(e);
    node w = target(e);
    W.draw_edge(xc[v],yc[v],xc[w],yc[w],edge_color);
   }
}



void move_graph_vec(window& W, GRAPH<point,int>& G, const list<node>& L, 
                                                    vector vec, int speed) 
{
  int l = W.real_to_pix(vec.length());

  int d = (speed > 0) ? l/speed : 1;

  vec = vec/d;

  for(int i=0; i<d; i++)
  { node v;
    forall(v,L)
    { point p = G[v] + vec;
      draw_node_with_edges(W,G,v,p);
      draw_node_with_edges(W,G,v);
      G[v] = p;
     }
   }
}


void move_graph_rel(window& W, GRAPH<point,int>& G, 
                           node_array<double>& dx,
                           node_array<double>& dy, int speed)
{
  int d = (speed > 0) ? 250/speed : 1;

  node v;

  for(int i=0; i<d; i++)
   forall_nodes(v,G)
   { if (dx[v] == 0 && dy[v] == 0) continue;
     point p = G[v] + vector(dx[v]/d,dy[v]/d);
     draw_node_with_edges(W,G,v,p);
     draw_node_with_edges(W,G,v);
     G[v] = p;
    }
}




void move_graph(window& W, GRAPH<point,int>& G, 
                           node_array<double>& xc,
                           node_array<double>& yc, int speed)
{
  node v;
  node_array<double> dx(G);
  node_array<double> dy(G);

  forall_nodes(v,G) 
  { point p = G[v];
    dx[v] = xc[v] - p.xcoord();
    dy[v] = yc[v] - p.ycoord();
   }

  move_graph_rel(W,G,dx,dy,speed);
}


