#include "ged.h"


void st_embed(GRAPH<point,int>& G) 
{ if (PLANAR(G) == false) 
   { Wp->acknowledge("graph is not planar");
     return;
    }

  int w = x_max - x_min;
  int h = y_max - y_min;

  node_array<double> xc(G);
  node_array<double> yc(G);

  list<edge> bi_edges;
  Make_Biconnected(G,bi_edges);
  Make_Bidirected(G,bi_edges);

  PLANAR(G,true);

  double dx = 0.8*w/(G.number_of_nodes() + 1);
  double dy = 0.8*h/(G.number_of_nodes() + 1);

  list<int> xlist;
  node_array<list_item> it(G,nil);

  node s = G.first_node();

  it[s] = xlist.append(0);

  int count = 0;

  node v;
  forall_nodes(v,G) 
  { count++;
    yc[v] = y_min + 0.1*h + count*dy;
    node u;
    bool first = true;
    forall_adj_nodes(u,v)
      if (it[u] == nil) 
      { it[u] = first ? it[v] : xlist.insert(0,it[v]);
        first = false;
       }
    
   }

  list_item i;
  count = 0;
  forall_items(i,xlist) xlist[i] = ++count; 

  forall_nodes(v,G) 
    xc[v] = x_min + 0.1*w + xlist[it[v]]*dx;

  edge e;
  forall(e,bi_edges)  G.del_edge(e);

  move_graph(*Wp,G,xc,yc,anim_speed);

}

void embed(GRAPH<point,int>& G, node_array<double>& x, node_array<double>& y)
{ int w = x_max - x_min;
  int h = y_max - y_min;
  node_array<double> xc(G);
  node_array<double> yc(G);
  node v;
  forall_nodes(v,G)  
  { xc[v] = x_min + 0.1*w + 0.8*w*x[v];
    yc[v] = y_min + 0.1*h + 0.8*h*y[v];
   }
  move_graph(*Wp,G,xc,yc,anim_speed);
 }


void grid_embed(GRAPH<point,int>& G)
{ 
  double n = G.number_of_nodes();
  int w = x_max - x_min;
  int h = y_max - y_min;
  int l = int(sqrt(n));
  int dx = w/l;
  int dy = h/l;
  int i = 0;

  node_array<double> xc(G);
  node_array<double> yc(G);

  node v;
  forall_nodes(v,G)
  { int row = i/l;
    int col = i%l;
    xc[v] = x_min+(col+0.25)*dx;
    yc[v] = y_min+(row+0.25)*dy;
    i++;
   }
  move_graph(*Wp,G,xc,yc,anim_speed);
}
  

void random_embed(GRAPH<point,int>& G)
{ int h  = y_max - y_min;

  random_source ran(int(y_min+0.1*h),int(y_max-0.1*h));

  node_array<double> xc(G);
  node_array<double> yc(G);

  node v;
  forall_nodes(v,G) 
  { xc[v] = ran();
    yc[v] = ran();
   }

  move_graph(*Wp,G,xc,yc,anim_speed);
}



void circular_embed(GRAPH<point,int>& G)
{ int    w = x_max - x_min;
  int    h = y_max - y_min;
  point  m = point(x_min + w/2, y_min + h/2);
  double r = h/2.5;
  double alpha = 0;
  double step  = 6.2832/G.number_of_nodes();

  node_array<double> xc(G);
  node_array<double> yc(G);

  node v;
  forall_nodes(v,G)  
  { point p = m.translate(alpha,r);
    xc[v] = p.xcoord();
    yc[v] = p.ycoord();
    alpha+=step;
  }
  move_graph(*Wp,G,xc,yc,anim_speed);
}



void bi_embed(GRAPH<point,int>& G, list<node>& A, list<node>& B)
{ int    w = x_max - x_min;
  int    h = y_max - y_min;
  node   v;
  double y,dy;

  node_array<double> xc(G);
  node_array<double> yc(G);

  dy = h/A.length();
  y  = y_min + dy/2;
  forall(v,A) 
  { xc[v] = x_min + w/4;
    yc[v] = y;
    y += dy;
  }

  dy = (y_max-y_min)/B.length();
  y  = y_min + dy/2;
  forall(v,B) 
  { xc[v] = x_max - w/4;
    yc[v] = y;
    y += dy;
  }

  move_graph(*Wp,G,xc,yc,anim_speed);

}




void planar_embed(GRAPH<point,int>& G, int k)
{ 
  if (PLANAR(G) == false) 
      Wp->acknowledge("graph is not planar");
  else
   { int w = x_max - x_min;
     int h = y_max - y_min;
     int n = G.number_of_nodes();

     list<edge> bi_edges;
     Make_Biconnected(G,bi_edges);
     Make_Bidirected(G,bi_edges);

     PLANAR(G,true);

     node_array<int> xcoord(G);
     node_array<int> ycoord(G);

     float fx = w/n;
     float fy = h/n;


     if (k==0)
       { STRAIGHT_LINE_EMBEDDING(G,xcoord,ycoord);
         fx /= 2;
        }
     else
        STRAIGHT_LINE_EMBEDDING2(G,xcoord,ycoord);

     // restore original graph
     edge e;
     forall(e,bi_edges)  G.del_edge(e);

     node_array<double> xc(G);
     node_array<double> yc(G);

     node v;
     forall_nodes(v,G) 
     {  //G[v] = point(x_min + fx*(xcoord[v]+1), y_min + fy*(ycoord[v]+1));
        xc[v] = x_min + fx*(xcoord[v]+1);
        yc[v] = y_min + fy*(ycoord[v]+1);
      }

     move_graph(*Wp,G,xc,yc,anim_speed);
         
   }

}


void zoom_graph(window& W, GRAPH<point,int>& G, double f) 
{ int    w = x_max - x_min;
  int    h = y_max - y_min;

  node_array<double> xc(G);
  node_array<double> yc(G);

  node v;
  forall_nodes(v,G) 
  { xc[v] = f*G[v].xcoord();
    yc[v] = f*G[v].ycoord();
   }

  move_graph(*Wp,G,xc,yc,anim_speed);
}



