#include "ged.h"

//define external variables

window* Wp = 0;
GRAPH<point,int>* Gp = 0;
node_matrix<int>  M;

int n_gen = 16;
int m_gen = 16;

color node_color = red;
color edge_color = black;

int  node_width = 6;
int  edge_width = 1;
bool directed = false;

int x_min = 0;
int x_max = 100; 
int y_min = 0;
int y_max = 100;

double node_radius;

int anim_speed = 40;
int sp_iterations = 250;

bool done = false;


void graph_edit(window& W, GRAPH<point,int>& G, bool dir, bool redraw)
{
  directed = dir;

  Wp = &W;
  Gp = &G;

  matrix_init(G);

  menu sub_menu1;
  sub_menu1.button("menu item 1");
  sub_menu1.button("menu item 2");
  sub_menu1.button("menu item 3");
  sub_menu1.button("menu item 4");
  sub_menu1.button("menu item 5");

  menu sub_menu2;
  sub_menu2.button("menu item 1");
  sub_menu2.button("menu item 2");
  sub_menu2.button("menu item 3");
  sub_menu2.button("menu item 4");
  sub_menu2.button("menu item 5");
  sub_menu2.button("menu item 6");


  menu test_menu;
  test_menu.button("menu item 1");
  test_menu.button("menu item 2");
  test_menu.button("menu item 3",sub_menu1);
  test_menu.button("menu item 4");
  test_menu.button("menu item 5");
  test_menu.button("menu item 6",sub_menu2);
  test_menu.button("menu item 7");
  test_menu.button("menu item 8");


  menu make_menu;
  make_menu.button("connected",   make_handler);
  make_menu.button("biconnected", make_handler);
  make_menu.button("simple",      make_handler);
  make_menu.button("dummy menu",   test_menu);

  menu embed_menu;
/*
  embed_menu.button("random",   0,embed_handler);
  embed_menu.button("grid",     1,embed_handler);
  embed_menu.button("circular", 2,embed_handler);
  embed_menu.button("spring",   5,embed_handler);
  embed_menu.button("planar-1", 3,embed_handler);
  embed_menu.button("planar-2", 4,embed_handler);
*/

  embed_menu.button("random",  100);
  embed_menu.button("grid",    101);
  embed_menu.button("circular",102);
  embed_menu.button("st-embed",106);
  embed_menu.button("spring",  105);
  embed_menu.button("planar-1",103);
  embed_menu.button("planar-2",104);


  menu zoom_menu;
  zoom_menu.button("zoom up",   zoom_handler);
  zoom_menu.button("zoom down", zoom_handler);


  menu gen_menu;
  gen_menu.button("rand",    0,gen_handler);
  gen_menu.button("compl",   1,gen_handler);
  gen_menu.button("bi_rand", 2,gen_handler);
  gen_menu.button("bi_compl",3,gen_handler);
  gen_menu.button("planar",  4,gen_handler);
  gen_menu.button("grid",    5,gen_handler);
  gen_menu.button("triang",  6,gen_handler);


  W.button("Clear",    clear_all);
  W.button("Settings", settings_diag);
  W.button("Redraw",   redraw_diag);
  W.button("Help",     help_diag);
  W.button("Done",     exit_diag);

  W.button("Make",     make_menu);
  W.button("Zoom",     zoom_menu);
  W.button("Embed",    embed_menu);
  W.button("Generate", gen_menu);
  W.button("File",     file_diag);

  W.buttons_per_line(5);

  W.open();
  W.init(0,100,0);

  x_min = (int)W.xmin();
  x_max = (int)W.xmax();
  y_min = (int)W.ymin();
  y_max = (int)W.ymax();

  node_width  = W.get_node_width();
  edge_width  = W.get_line_width();
  node_radius = node_width/W.scale();

  if (W.mono()) node_color = white;

  drawing_mode save = W.set_mode(xor_mode);

  window_init(W);

  if (redraw) draw_graph(W,G);

  done = false;

  while ( ! done )
  {
    double x,y;

    int k = W.read_mouse(x,y);

    unsigned long t0 = W.button_press_time();

    point p = point(x,y);

    if (k >= 100) 
    { int val;
      double x,y;
      //while (W.get_event(val,x,y) == no_event);
      embed_handler(k-100);
      continue;
     }

    switch (k) {

      case MOUSE_BUTTON(1):  { 

                double dv;
                node   v = nearest_node(G,p,dv);


                if (v == nil || dv > node_radius )
                { if (v == nil || dv > 2*node_radius)        // new node
                  { v  = G.new_node(p);
                    draw_node(W,G,v);
                   }
                  break;
                 }
                else               // move node or new edge
                { int val;

                  if (W.shift_key_down())  // move component
                  { double x1,y1;
                    list<node> L;
                    node_array<bool> reached(G,false);
                    dfs(G,v,reached,L);
                    while (W.read_event(val,x1,y1) != button_release_event)
                    { vector trans(x1-x,y1-y);
                      move_graph_vec(*Wp,G,L,trans,0);
                      x = x1;
                      y = y1;
                     }
                     break;
                   }


                 // move single node

                  point  p0 = G[v];
                  double dx = p0.xcoord() - x;
                  double dy = p0.ycoord() - y;

                  p = p0;

                  unsigned long t1;
                  while ( W.read_event(val,x,y,t1) != button_release_event)
                  { if (t1-t0 > 150  || p.distance(p0) >  node_radius/2) 
                    { // node moved certain distance or for certain time 
                      point q = point(x+dx,y+dy);
                      if (q != p)
                      { draw_node_with_edges(W,G,v,q);
                        draw_node_with_edges(W,G,v,p);
                        p = q;
                       }
                     }
                   }


                  if (p != p0)
                  { // assign new position if free
                    node u = find_node(G,p);
                    if (u == nil || u == v)
                       G[v] = p;
                    else
                      { // move node back
                        draw_node_with_edges(W,G,v);
                        draw_node_with_edges(W,G,v,p);
                       }
                    break;
                   }

                  // start new edge at p = p0 if time or distance was to small
                  // do not break; continue with BUTTON(2)-case
                }

/*
             }
      case MOUSE_BUTTON(2):  { // new edge
                
                node v = find_node(G,p);   // start node
*/

                W.message("new edge");
 
                if (v != nil)            
                { 
                  high_light_node(W,G,v);
                  p = G[v];

                  if (W.read_mouse_seg(p.xcoord(),p.ycoord(),x,y) != 3)
                  {
                    point q(x,y);
                    node w = find_node(G,q);
  
                    if (w == nil)       // no hit: create a new node w at q
                    { w  = G.new_node(q);
                      draw_node(W,G,w);
                     }
  
                    draw_edge(W,G,v,w); // delete existing edges between v and w
                    draw_edge(W,G,w,v);
  
                    G.new_edge(v,w);    // insert new edge
                    M(v,w)++;
  
                    draw_edge(W,G,w,v); // reinsert old edges
                    draw_edge(W,G,v,w);
                   }

                  unhigh_light_node(W,G,v);
                 }

                W.del_messages();
                break;
               }


      case MOUSE_BUTTON(2):  {
                node v = find_node(G,p);   // start node
                if (v) display_node_data(W,G,v);
                break;
               }
                               


      case MOUSE_BUTTON(3):  { // delete node or edge

               node v = find_node(G,p);
               if (v != nil) 
               { draw_node_with_edges(W,G,v);
                 edge e;
                 forall_inout_edges(e,v) M(source(e),target(e)) = 0;
                 G.del_node(v);
                 break;
                }

               edge e = find_edge(G,p);
               if (e != nil)
               { node v = source(e);
                 node w = target(e);
                 draw_edge(W,G,v,w);
                 draw_edge(W,G,w,v);
                 G.del_edge(e);
                 M(v,w)--;
                 draw_edge(W,G,w,v);
                 draw_edge(W,G,v,w);
                }
               break;
              }

     } // switch

  } // while ! done



  W.reset_frame_label();
  W.set_mode(save);
  W.set_redraw();
}




 
int main()
{
  window W("The LEDA Graph Editor");
  W.set_line_width(edge_width);
  W.set_node_width(node_width);

  GRAPH<point,int> G;

  graph_edit(W,G,false);

  return 0;
}
