
#include <LEDA/window.h>
#include <LEDA/graph.h>
#include <LEDA/graph_alg.h>
#include <LEDA/node_matrix.h>
#include <math.h>

extern window*           Wp;
extern GRAPH<point,int>* Gp;
extern node_matrix<int>  M;

extern n_gen;
extern m_gen;

extern color node_color;
extern color edge_color;

extern int  node_width;
extern int  edge_width;
extern bool directed;

extern int x_min, x_max, y_min, y_max;
extern double node_radius;

extern bool done;
extern int anim_speed;
extern int sp_iterations;

void draw_node(window& W, GRAPH<point,int>& G, node v);
void high_light_node(window& W, GRAPH<point,int>& G, node v);
void unhigh_light_node(window& W, GRAPH<point,int>& G, node v);
void draw_edge(window& W, GRAPH<point,int>& G, node v, node w);
void draw_node_with_edges(window& W, GRAPH<point,int>& G, node v);
void draw_node_with_edges(window& W, GRAPH<point,int>& G, node v,point p);
void draw_graph(window& W, GRAPH<point,int>& G);
void draw_graph(window& W, GRAPH<point,int>& G, list<node>& L);
void redraw_func();

void move_graph(window& W, GRAPH<point,int>& G, 
                           node_array<double>& xc,
                           node_array<double>& yc, int speed);

void move_graph_rel(window& W, GRAPH<point,int>& G, 
                               node_array<double>& dx,
                               node_array<double>& dy, int speed);

void move_graph_vec(window& W, GRAPH<point,int>& G, const list<node>& L, 
                                                    vector vec, int speed);

void embed(GRAPH<point,int>& G, node_array<double>& x, node_array<double>& y);
void grid_embed(GRAPH<point,int>& G);
void random_embed(GRAPH<point,int>& G);
void circular_embed(GRAPH<point,int>& G);
void bi_embed(GRAPH<point,int>& G, list<node>& A, list<node>& B);
void planar_embed(GRAPH<point,int>& G, int kind);

void spring_embed(GRAPH<point,int>& G);

void st_embed(GRAPH<point,int>& G);


void zoom_graph(window& W, GRAPH<point,int>& G, double f);


void read_graph(window& W, GRAPH<point,int>& G,string s,bool clear=false);
void save_graph(window& W, GRAPH<point,int>& G,string s);

void matrix_init(const graph& g);
void window_init(window& W);
void dfs(graph& G, node v, node_array<bool>& reached, list<node>& L);
void display_node_data(window& W, GRAPH<point,int>& G, node v);

node nearest_node(GRAPH<point,int>& G, point p, double& d) ;
node find_node(GRAPH<point,int>& G, point p) ;
edge find_edge(GRAPH<point,int>& G, point p)  ;

void make_handler(int);
void embed_handler(int);
void zoom_handler(int);
void gen_handler(int);

void file_diag(int);
void settings_diag(int);
void help_diag(int);
void clear_all(int);
void exit_diag(int);
void redraw_diag(int);



void graph_edit(window& W, GRAPH<point,int>& G, bool dir = false, 
                                                bool redraw = true);
