#include "ged.h"
#include <LEDA/array2.h>


#if !defined(__TEMPLATE_FUNCTIONS__)
LEDA_TYPE_PARAMETER(list<node>)
#endif


#define FREPULSE(d) ((k2 > d) ? kk/d : 0)


float log2(int x)
{ float l = 0;
  while (x)
  { l++;
    x >>= 1;
   }
  return l/2;
}


void spring_embed(GRAPH<point,int>& G)
{
  double xleft  = x_min;
  double xright = x_max;
  double ybottom= y_min;
  double ytop   = y_max;

  node_array<double>	xpos(G); 
  node_array<double>	ypos(G); 
  node_array<list_item>	lit(G); 

  node_array<bool>  fixed(G,false);

  node u,v;
  edge e;

  int c_f = 1;

  double width  = xright - xleft;
  double height = ytop - ybottom; 

  double tx_null = width/50;
  double ty_null = height/50;
  double tx = tx_null;
  double ty = ty_null;

  double k = sqrt(width*height / G.number_of_nodes()) / 2;
  double k2 = 2*k;
  double kk = k*k;

  int ki = int(k);

/*
  Wp->message("give list of fixed nodes");

  point p;
  while (*Wp >> p)
  { node v = find_node(G,p);
    if (v)
    { p = G[v];
      fixed[v] = true;
      Wp->draw_filled_node(p,red);
      Wp->draw_filled_node(p,yellow);
     }
   }

  Wp->del_messages();
*/

  //build  matrix of node lists

  int xA = (int) (width / ki + 1);
  int yA = (int) (height / ki + 1);

  array2<list<node> > A(-1,xA,-1,yA);

  forall_nodes(v,G)
  { point p = G[v];
    xpos[v] = p.xcoord();
    ypos[v] = p.ycoord();
    int i = (int) (p.xcoord() - xleft) / ki;
    int j = (int) (p.ycoord() - ybottom) / ki;
    lit[v] = A(i,j).push(v);
  }



  while (c_f < sp_iterations)
  {
    node_array<double>	xdisp(G,0); 
    node_array<double>	ydisp(G,0); 

   // repulsive forces

   forall_nodes(v,G) 
   { 
     int i = (int) ((xpos[v] - xleft) / ki);
     int j = (int) ((ypos[v] - ybottom) / ki);

     double xv = xpos[v];
     double yv = ypos[v];

     for(int m = -1; m < 2; m++)
      for(int n = -1; n < 2; n++)
       forall(u,A(i+m,j+n))
       { if(u == v) continue;
         double xdist = xv - xpos[u];
         double ydist = yv - ypos[u];
         double dist = sqrt(xdist * xdist + ydist * ydist);
         if (dist < 1e-3) dist = 1e-3;
         xdisp[v] += FREPULSE(dist) * xdist / dist;
         ydisp[v] += FREPULSE(dist) * ydist / dist;
        }
      xdisp[v] *=  (double(rand_int(500,1500))/1000.0);
      ydisp[v] *=  (double(rand_int(500,1500))/1000.0);
     }

 

   // attractive forces

   forall_edges(e,G)
   { node u = G.source(e);
     node v = G.target(e);
     double xdist=xpos[v]-xpos[u];
     double ydist=ypos[v]-ypos[u];
     double dist=sqrt(xdist*xdist+ydist*ydist);

     float f = (G.degree(u)+G.degree(v))/6.0;

     dist /= f;

     xdisp[v]-=xdist*dist/k;
     ydisp[v]-=ydist*dist/k;
     xdisp[u]+=xdist*dist/k;
     ydisp[u]+=ydist*dist/k;
    }


   // preventions

   forall_nodes(v,G)
   { 
     if (fixed[v]) continue;

     int i0 = int((xpos[v] - xleft)/ki);
     int j0 = int((ypos[v] - ybottom)/ki);

     double xd= xdisp[v];
     double yd= ydisp[v];
     double dist = sqrt(xd*xd+yd*yd);

     if (dist < 1) dist = 1;

     xd = tx*xd/dist;
     yd = ty*yd/dist;

     double xp = xpos[v] + xd;
     double yp = ypos[v] + yd;
     
     if (xp > xleft && xp < xright) xpos[v] = xp;
     if (yp > ybottom && yp < ytop) ypos[v] = yp;
 
     int i = int((xp - xleft)/ki);
     int j = int((yp - ybottom)/ki);
 
     if(i != i0 || j != j0)
     { A(i0,j0).del_item(lit[v]);
       lit[v] = A(i,j).push(v);
      }
 
    }

   tx = tx_null / log2(c_f); 
   ty = ty_null / log2(c_f);

   c_f++;

   Wp->del_messages();
   Wp->message(string("%3d iterations (click to stop)",c_f));
   if (Wp->get_mouse() != NO_BUTTON) break;

   if (c_f % 16 == 0) move_graph(*Wp,G,xpos,ypos,anim_speed);


  }

}
