#include <LEDA/window.h>
#include <LEDA/stream.h>

#include <unistd.h>

static string prefix;
static string dtype;

static string lpr_cmd  = "lpr -h";
static string xdvi_cmd = "xdvi";

void setup(int)
{ panel P("Lman Setup Panel");
  P.string_item("print cmd",lpr_cmd);
  P.string_item("previewer",xdvi_cmd);
  P.button("ok");
  P.open();
 }

void view(int)
{ string fname = prefix + dtype + ".dvi";
  if (access(fname,F_OK | R_OK) == 0)
     system(xdvi_cmd + " " + fname);
  else
    error_handler(0,"Lman: cannot open " + fname);
 }

void print(int)
{ string fname = prefix + dtype + ".ps";
  if (access(fname,F_OK | R_OK) == 0)
    system(lpr_cmd + " " + fname);
  else
    error_handler(0,"Lman: cannot open " + fname);
}



int main()
{ 
 char* leda_root = getenv("LEDAROOT");

 if (leda_root == 0)
 { cerr << "Environment variable LEDAROOT not defined.";
   exit(1);
  }

 prefix = leda_root;
 prefix += "/Manual/MANUAL/DVI/";

 if (access(prefix,F_OK | R_OK) != 0)
 { cerr << string("Cannot access manual directory %s",~prefix) << endl;
   exit(1);
  }
  

 cmd_istream LS("ls " + prefix + "*.dvi");

 list<string> M;
 M.read(LS);

 if (!M.empty())
 { list_item it;
   forall_items(it,M)
   { string s = M[it];
      s = s.del(prefix);
      s = s.del(".dvi");
      M[it] = s;
    }
  }


  panel P("LEDA Manual Reader");
  P.string_item("",dtype,M);
  P.button("view", 1,view);
  P.button("print",2,print);
  P.button("setup",3,setup);
  P.button("exit", 0);
  P.display(0,0);

  while (P.read() != 0);

  return 0;
}
  
