#include <LEDA/graph.h>
#include <LEDA/graph_alg.h>
#include <LEDA/array.h>
#include <LEDA/stream.h>
#include <LEDA/array.h>
#include <LEDA/window.h>
#include <math.h>



typedef GRAPH<vector,int> polyhedron;

void rotate(float alpha1,float alpha2, vector& p)
{ 
  // rotate 3d-point p about the origin
  // by alpha2 in yz-plane and alpha1 in xy-plane

    double R  = hypot(p[1],p[2]);
    double phi = asin(p[1]/R);
  
    if (p[2] < 0) phi = LEDA_PI - phi;
  
    p[1]  = ((R != 0) ? R*sin(phi+alpha2) : 0);
    p[2]  = ((R != 0) ? R*cos(phi+alpha2) : 0);

    R   = hypot(p[0],p[2]);
    phi = asin(p[0]/R);

    if (p[2] < 0) phi = LEDA_PI - phi;
  
    p[0]  = ((R != 0) ? R*sin(phi+alpha1) : 0);
    p[2]  = ((R != 0) ? R*cos(phi+alpha1) : 0);
}


point project(vector p)   // project p into xy-plane
{
  return point(p[0],p[1]); 
 }



void draw_poly(window& W, polyhedron& poly, node_array<vector>& pos, color col)
{ node v,w;
  node_array<bool> marked(poly,false);
  forall_nodes(v,poly) 
  { forall_adj_nodes(w,v)
      if (!marked[w])
      { point a = project(pos[v]);
        point b = project(pos[w]);
        W.draw_segment(a,b,col);
       }
    marked[v] = true;
   }
 }

/*
void draw_poly(window& W, polyhedron& poly, vector& trans)
{ edge e;
  forall_edges(e,poly) 
  { point a = project(poly[source(e)]+trans);
    point b = project(poly[target(e)]+trans);
    W.draw_segment(a,b,blue);
   }
 }
*/


void make_poly(polyhedron& poly, int N)
{ 
    poly.clear();
  
    node* L = new node[N];
    node* R = new node[N];
  
    float d = 2*LEDA_PI/N;

    node v;

    poly.clear();
  
    int i;
    for(i=0; i<N; i++)
    { point origin(0,0);
      point p = origin.translate(i*d,100);
      L[i] = poly.new_node(vector(p.xcoord(),p.ycoord(), 120));
      R[i] = poly.new_node(vector(p.xcoord(),p.ycoord(),-120));
     }

    node v0 = poly.new_node(vector(0,0,-30));
  
    for(i=1; i<N; i++)
    { poly.new_edge(L[i],L[i-1]);
      poly.new_edge(L[i-1],L[i]);
      poly.new_edge(R[i],R[i-1]);
      poly.new_edge(R[i-1],R[i]);
      poly.new_edge(L[i],R[i]);
      poly.new_edge(R[i],L[i]);
      poly.new_edge(v0,R[i]);
      poly.new_edge(R[i],v0);
     }
  
    poly.new_edge(L[0],L[N-1]);
    poly.new_edge(L[N-1],L[0]);
    poly.new_edge(R[0],R[N-1]);
    poly.new_edge(R[N-1],R[0]);
    poly.new_edge(L[0],R[0]);
    poly.new_edge(R[0],L[0]);
    poly.new_edge(v0,R[0]);
    poly.new_edge(R[0],v0);

    if (!PLANAR(poly,true)) error_handler(1,"graph not planar !");

    // compute an interior point M and move the origin to this point
  
    vector M(3);
  
    forall_nodes(v,poly)
    {  M[0] += poly[v][0];
       M[1] += poly[v][1];
       M[2] += poly[v][2];
     }
  
  
    M[0] /= poly.number_of_nodes();
    M[1] /= poly.number_of_nodes();
    M[2] /= poly.number_of_nodes();
  
  
    forall_nodes(v,poly)
    { poly[v][0] -= M[0];
      poly[v][1] -= M[1];
      poly[v][2] -= M[2];
     }
  
}



void make_sphere(GRAPH<vector,int>& G, int n)
{
  G.clear();

  int m = 2*n;

  array<node>  V(m);

  node north = G.new_node(vector(0,0, 1));

  int i;
  int j;
  double phi1 = 0;
  double phi2 = 0;
  double d1 = LEDA_PI/n;
  double d2 = LEDA_PI/n;

  for(i=0; i<m; i++) V[i] = north;

  for(phi1=d1, j=1; j<n; phi1+=d1, j++)
  { double z = cos(phi1);
    double r = sin(phi1);

    for(phi2=0, i=0; i<m; phi2+=d2, i++)
    {
      double x = r*cos(phi2);
      double y = r*sin(phi2);

      node v = G.new_node(vector(x,y,z));

      if(i==0) 
      { G.new_edge(v,V[i]);
        G.new_edge(V[i],v);
        if (j > 1)  
        { G.new_edge(V[0],V[m-1]);
          G.new_edge(V[m-1],V[0]);
         }
       }
      else
       { G.new_edge(v,V[i-1]);
         G.new_edge(V[i-1],v);
         G.new_edge(v,V[i]);
         G.new_edge(V[i],v);
        }
      V[i] = v;
     }

   }

  node south = G.new_node(vector(0,0,-1));

  G.new_edge(V[m-1],V[0]);
  G.new_edge(V[0],V[m-1]);

  for (i=0;i<m;i++) 
  { G.new_edge(south,V[i]);
    G.new_edge(V[i],south);
   }


  node v;

  forall_nodes(v,G) G[v] = G[v]*100;

  if (!PLANAR(G,true)) error_handler(1," G not planar !");

}


void leda_side(polyhedron& poly, node* L, float z)
{
  L[0]= poly.new_node(vector(-70,-20,z));
  L[1]= poly.new_node(vector(-40,-20,z));
  L[2]= poly.new_node(vector(-40,-10,z));
  L[3]= poly.new_node(vector(-60,-10,z));
  L[4]= poly.new_node(vector(-60,+20,z));
  L[5]= poly.new_node(vector(-70,+20,z));

  poly.new_edge(L[0],L[5]);
  int i;
  for(i = 1; i<=5; i++) poly.new_edge(L[i], L[i-1]);

  L[6] = poly.new_node(vector(-30,-20,z));
  L[7] = poly.new_node(vector( -5,-20,z));
  L[8] = poly.new_node(vector( -5,-10,z));
  L[9] = poly.new_node(vector(-20,-10,z));
  L[10]= poly.new_node(vector(-20, -5,z));
  L[11]= poly.new_node(vector( -5, -5,z));
  L[12]= poly.new_node(vector( -5, +5,z));
  L[13]= poly.new_node(vector(-20, +5,z));
  L[14]= poly.new_node(vector(-20,+10,z));
  L[15]= poly.new_node(vector( -5,+10,z));
  L[16]= poly.new_node(vector( -5,+20,z));
  L[17]= poly.new_node(vector(-30,+20,z));

  poly.new_edge(L[6],L[17]);
  for(i = 7; i<=17; i++) poly.new_edge(L[i], L[i-1]);

  L[18]= poly.new_node(vector( 5,-20,z));
  L[19]= poly.new_node(vector(20,-20,z));
  L[20]= poly.new_node(vector(35,-10,z));
  L[21]= poly.new_node(vector(35,+10,z));
  L[22]= poly.new_node(vector(20,+20,z));
  L[23]= poly.new_node(vector( 5,+20,z));

  poly.new_edge(L[18],L[23]);
  for(i = 19; i<=23; i++) poly.new_edge(L[i], L[i-1]);

  L[24]= poly.new_node(vector(15,-10,z));
  L[25]= poly.new_node(vector(20,-10,z));
  L[26]= poly.new_node(vector(25, -5,z));
  L[27]= poly.new_node(vector(25, +5,z));
  L[28]= poly.new_node(vector(20,+10,z));
  L[29]= poly.new_node(vector(15,+10,z));

  poly.new_edge(L[24],L[29]);
  for(i = 25; i<=29; i++) poly.new_edge(L[i], L[i-1]);

  L[30]= poly.new_node(vector(40,-20,z));
  L[31]= poly.new_node(vector(50,-20,z));
  L[32]= poly.new_node(vector(55, -5,z));
  L[33]= poly.new_node(vector(65, -5,z));
  L[34]= poly.new_node(vector(70,-20,z));
  L[35]= poly.new_node(vector(80,-20,z));
  L[36]= poly.new_node(vector(65, 20,z));
  L[37]= poly.new_node(vector(55, 20,z));

  poly.new_edge(L[30],L[37]);
  for(i = 31; i<=37; i++) poly.new_edge(L[i], L[i-1]);

  L[38]= poly.new_node(vector(55, 0,z));
  L[39]= poly.new_node(vector(65, 0,z));
  L[40]= poly.new_node(vector(60,15,z));
  poly.new_edge(L[38],L[40]);
  poly.new_edge(L[39],L[38]);
  poly.new_edge(L[40],L[39]);

}


void make_leda(polyhedron& poly)
{ 
  poly.clear();

  node L[41];
  node R[41];

  leda_side(poly,L,-5);
  leda_side(poly,R,+5);

  for(int i = 0; i<41; i++) poly.new_edge(L[i],R[i]);

  Make_Bidirected(poly);

  node v;
  forall_nodes(v,poly) poly[v] = poly[v] * 2;
}








void read_poly(polyhedron& poly, string fname)
{
  poly.clear();

  file_istream IN(fname);

  int N;
  int i;

  IN >> N;


  array<node> V(N);

  vector v(3);

  for(i = 0; i<N; i++) 
  { IN >> v;
    V[i] = poly.new_node(v);
   }

/*
  for(i = 0; i<N; i++) 
  { int d;
    IN >> d;
    while(d--)
    { IN >> j;
      poly.new_edge(V[i], V[j])
     }
   }
*/

  for(i = 0; i<N; i++) 
  { int u,v,w;
    IN >> u >> v >> w;
    poly.new_edge(V[u], V[v]);
    poly.new_edge(V[v], V[w]);
    poly.new_edge(V[w], V[u]);
  }

}


static bool but0 = 0;
static int shape = 0;

void redraw(window* wp) 
{ //wp->clear();
  but0 = shape; 
 }



int main()
{ 
 int     N = 4;
 int speed = 8;
 bool  ran = true;
 color col = blue;

 window W;
 W.int_item("speed",speed,0,25);
 W.color_item("color",col);
 //W.bool_item("random",ran);
 //W.int_item("# vertices",N,3,16);
 W.button("poly1",0);
 W.button("poly2",1);
 W.button("leda", 2);
 W.button("quit", 3);
 
 W.set_redraw(redraw);

 W.init(-250,250,-200);
 
 W.set_node_width(4);
 W.set_mode(xor_mode);

 polyhedron poly;
 node_array<vector> pos1;
 vector dir(500,500); 

 double x;
 double y;
 color col0 = col;
 int count = 0;

 int but;

 while ((but = W.get_mouse(x,y)) != 3)
 {
    if (but0 >= 0)
    { but = but0;
      but0 = -1;
     }

    if (but >= 0 && but <= 2)
    { shape = but;
      switch (but) {
       case 0: make_sphere(poly,N);
               break;
       case 1: make_poly(poly,N);
               break;
       case 2: make_leda(poly);
               break;
       }

      pos1.init(poly);
      node v;
      forall_nodes(v,poly) pos1[v] = poly[v];
      forall_nodes(v,poly) rotate(1.5,0.7,pos1[v]);
 
      W.clear();
      W.draw_point(0,0);
      draw_poly(W,poly,pos1,col);
    }

    if (speed == 0) continue;

    if (ran) 
      { if (count-- == 0)
        { dir[0] += rand_int(-100,100);
          dir[1] += rand_int(-100,100);
          count = 10;
          //W.del_messages();
          //W.message(string("dx = %.4f  dy = %.4f",dir[0],dir[1]));
         }
       }
    else
      { dir[0] = x;
        dir[1] = y;
       }

    vector dir1 = dir.norm()*(speed/200.0);
 
    node_array<vector> pos2 = pos1;
    node v;
    forall_nodes(v,poly) rotate(dir1[0],dir1[1],pos1[v]);
    draw_poly(W,poly,pos1,col);  // draw new position
    draw_poly(W,poly,pos2,col0); // erase old position  (xor_mode !!)
    col0 = col;
  }

 return 0;
}
