/*******************************************************************************
+
+  LEDA 3.4
+
+  _g_array.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <LEDA/graph.h>


//------------------------------------------------------------------------------
// graph maps and arrays 
//
// graph_map: base of node/edge/face_map/array
//
// by S. Naeher  (1995,1996)
//------------------------------------------------------------------------------


int graph_map::next_power(int s) const
{ if (s==0) return 0;
  int p = 1;
  while (p < s) p <<= 1;
  return p;
}

void graph_map::init_table(GenPtr* start, GenPtr* stop)
{  for(GenPtr* q=start; q < stop; q++) init_entry(*q); }

void graph_map::clear_table()
{ GenPtr* stop = table+table_size;
  for(GenPtr* q=table; q < stop; q++) clear_entry(*q); 
 }

void graph_map::resize_table(int sz)
{ 
  GenPtr* old_table = table;
  GenPtr* old_stop  = table + table_size;
  
  table_size = sz;
  table = new GenPtr[sz];
  if (table == 0) error_handler(1," graph_map: out of memory");

  GenPtr* p = old_table; 
  GenPtr* q = table;
  while (p < old_stop) *q++ = *p++;

  init_table(q,table+sz);

  if (old_table != old_stop) delete[] old_table;
}


void graph_map::init(const graph* G, int sz) 
{ clear_table();
  if (table_size > 0) delete[] table;
  table = 0;
  g = G;
  table_size = next_power(sz);
  if (table_size > 0) 
  { table = new GenPtr[table_size];
    if (table == 0) error_handler(1," graph_map: out of memory");
   }
}

graph_map::graph_map(const graph* G, int sz) 
{ g = G;
  def_entry = 0;
  table = 0;
  table_size = next_power(sz);
  if (table_size > 0) 
  { table = new GenPtr[table_size];
    if (table == 0) error_handler(1," graph_map: out of memory");
   }
}


graph_map::graph_map(const graph_map& M)
{ g = M.g;
  def_entry = 0;
  table = 0;
  table_size = M.table_size;
  if (table_size > 0)
  { table = new GenPtr[table_size];
    if (table == 0) error_handler(1," graph_map: out of memory");
    GenPtr* p = table;
    GenPtr* stop = M.table+M.table_size;
    for(GenPtr* q=M.table; q < stop; q++) 
    { *p = *q;
      M.copy_entry(*p); 
      p++;
     }
   }
}


graph_map& graph_map::operator=(const graph_map& M)
{ if (&M == this) return *this;
  clear_table();
  if (table_size > 0) delete[] table;
  table = 0;
  g = M.g;
  table_size = M.table_size;
  if (table_size > 0)
  { table = new GenPtr[table_size];
    if (table == 0) error_handler(1," graph_map: out of memory");
    GenPtr* p = table;
    GenPtr* stop = M.table+M.table_size;
    for(GenPtr* q=M.table; q < stop; q++) 
    { *p = *q;
      copy_entry(*p); 
      p++;
     }
   }
  return *this;
}

