/*******************************************************************************
+
+  LEDA 3.4
+
+  _g_inout.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

#include <LEDA/graph.h>
#include <fstream.h>

//------------------------------------------------------------------------------
// graph i/o
//
// S. Naeher  (1995)
//------------------------------------------------------------------------------



/*
static void put_int(filebuf& fb, int n)
{ char* A = (char*)&n;
  char* E = A+sizeof(int);
  while (A<E) fb.sputc(*(A++));
}

static int get_int(istream& from)
{ int n;
  char* A = (char*)&n;
  char* E = A+sizeof(int);
  while (A<E) from.get(*(A++));
  return n;
}
*/

void graph::write(string file_name) const
{ ofstream out(file_name);
  if (out.fail()) error_handler(1,"write: cannot open file");
  write(out);
}


void graph::write(ostream& out) const
{
  int* A = new int[max_n_index+2];

  // nodes get numbers from 1 to |V|

  int count = 1;

  out << "LEDA.GRAPH" << endl;
  out << node_type()  << endl;
  out << edge_type()  << endl;

  out << v_list.length() << endl;

  node v;
  forall_nodes(v,*this)
  { write_node_entry(out,v->data[0]);
    out << endl;
    A[v->name] = count++;
   }

  out << e_list.length() << endl;

  forall_nodes(v,*this)
  { edge e;
    int s = A[v->name];
    forall_adj_edges(e,v)
    { out << s << " " << A[target(e)->name] << " ";
      write_edge_entry(out,e->data[0]);
      out << endl;
     }
   }

 delete[] A;

}

int graph::read(string file_name)
{ ifstream in(file_name);
  if (in.fail())  return 1;
  return read(in);
}

int graph::read(istream& in)
{ int result = 0;

  clear();

  edge e;
  int n,i,v,w;

  string d_type,n_type,e_type;

  string this_n_type = node_type();
  string this_e_type = edge_type();

  while (in && d_type=="") in >> d_type;

  in >> n_type >> e_type >> n;

  if (d_type != "LEDA.GRAPH") return 3;

  read_line(in);

  node* A = new node[n+1];

  if (this_n_type == "void" || n_type != this_n_type)  // do not read node info
    { if (n_type != this_n_type) result = 2;   // incompatible node types
      for (i=1; i<=n; i++)
      { A[i] = new_node();
        read_line(in);
       }
   }
  else
    for (i=1; i<=n; i++)
    { A[i] = new_node(0);
      read_node_entry(in,A[i]->data[0]);
     }
 
  in >> n;       // number of edges

  if (this_e_type == "void" || e_type != this_e_type) // do not read edge info
    { if (e_type != this_e_type) result = 2;   // incompatible edge types
      while (n--) 
      { in >> v >> w;
        e = new_edge(A[v],A[w]);
        read_line(in);
      }
     }
  else
    while (n--) 
    { in >> v >> w;
      e = new_edge(A[v],A[w],GenPtr(0));
      read_edge_entry(in,e->data[0]);
     }

  delete[] A;

  return result;
}


void graph::print_node(node v,ostream& o) const
{ if (super() != 0)
     super()->print_node(node(graph::inf(v)),o);
  else
     { o << "[" << index(v) <<"]" ;
       print_node_entry(o,v->data[0]);
      }
}

void graph::print_edge(edge e,ostream& o) const
{ if (super() != 0)
     super()->print_edge(edge(graph::inf(e)),o);
  else
     { o <<   "[" << source(e)->name << "]";
       o <<  ((undirected) ? "==" : "--");
       print_edge_entry(o,e->data[0]);
       o <<  ((undirected) ? "==" : "-->");
       o << "[" << target(e)->name << "]";
      }
}


void graph::print(string s, ostream& out) const
{ node v;
  edge e;
  out << s << endl;
  forall_nodes(v,*this)
  { print_node(v,out);
    out << " : ";
    forall_adj_edges(e,v) print_edge(e,out);
    out << endl;
   }
  out << endl;
}

