/*******************************************************************************
+
+  LEDA 3.4
+
+  _g_matrix.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <LEDA/graph.h>
#include <LEDA/node_matrix.h>

//------------------------------------------------------------------------------
// node_matrix
//
// by S. Naeher  (1995)
//------------------------------------------------------------------------------


void Node_Matrix::init(const graph& G, int n, GenPtr x) 
{ g = (graph*)&G;
  M.init(G,n,(graph_map*)0);
  for(int i=0;i<M.size();i++) 
  { graph_map* gm = new graph_map(&G,n);
    M.entry(i) = gm;
    for(int j=0;j<gm->size();j++) 
    { copy_entry(x);
      gm->access(j) = x;
     }
   }
}

void Node_Matrix::init(const Node_Matrix& A) 
{ g = A.g;
  int n = A.M.size();
  M.init(*g,n,(graph_map*)0);
  for(int i=0;i<n;i++) 
  { graph_map* gm = new graph_map(g,n);
    M.entry(i) = gm;
    graph_map* gma = A.M.inf(i);
    for(int j=0;j<n;j++) 
    { GenPtr x = gma->read(j);
      copy_entry(x);
      gm->access(j) = x;
     }
   }
}


void Node_Matrix::clear_entries()
{ for(int i=0;i<M.size();i++) 
  { graph_map* gm = M.entry(i);
    for(int j=0;j<gm->size();j++) 
    { GenPtr x = gm->read(j);
      clear_entry(x); 
     }
   }
}


Node_Matrix::~Node_Matrix()
{ for(int i=0; i<M.size();i++) 
    delete M.entry(i); 
 }


