/*******************************************************************************
+
+  LEDA 3.4
+
+  _g_sort.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <LEDA/graph.h>

//--------------------------------------------------------------------------
// sorting
//
// by S. Naeher  (1995)
//------------------------------------------------------------------------------


static const graph_map*  NA;

static graph* GGG;


/*
static int array_cmp_nodes(const node& x, const node& y) 
{ return NA->cmp_entry(NA->array_read(x),NA->array_read(y)); }

static int array_cmp_edges(const edge& x, const edge& y) 
{ return NA->cmp_entry(NA->array_read(x),NA->array_read(y)); }
*/

static int int_array_cmp_nodes(const node& x, const node& y) 
{ return LEDA_COMPARE(int,NA->array_read(x),NA->array_read(y)); }

static int int_array_cmp_edges(const edge& x, const edge& y) 
{ return LEDA_COMPARE(int,NA->array_read(x),NA->array_read(y)); }

static int float_array_cmp_nodes(const node& x, const node& y) 
{ return LEDA_COMPARE(float,NA->array_read(x),NA->array_read(y)); }

static int float_array_cmp_edges(const edge& x, const edge& y) 
{ return LEDA_COMPARE(float,NA->array_read(x),NA->array_read(y)); }

static int double_array_cmp_nodes(const node& x, const node& y) 
{ return LEDA_COMPARE(double,NA->array_read(x),NA->array_read(y)); }

static int double_array_cmp_edges(const edge& x, const edge& y) 
{ return LEDA_COMPARE(double,NA->array_read(x),NA->array_read(y)); }




static int graph_cmp_nodes(const node& x, const node& y)
{ return GGG->cmp_node_entry(x,y); }

static int graph_cmp_edges(const edge& x, const edge& y)
{ return GGG->cmp_edge_entry(x,y); }


void graph::sort_nodes(const list<node>& vl)
{ 
  if (&v_list != &vl) 
  { int  n = 0;
    node v;
    forall(v,vl) 
    { if (v->owner != this) break;
      n++;
     }
    if (n != v_list.length()) 
       error_handler(1,"graph::sort_nodes(list<node>): illegal node list");
   }

  v_list = vl;

  list_item it;
  forall_items(it,v_list) v_list[it]->pos_in_V = it;

 }



void graph::sort_nodes(int (*f)(const node&, const node&))
{ v_list.sort(f); }


void graph::sort_edges(const list<edge>& el)
{ node v;
  edge e;

  if (&e_list != &el) 
  { list_item it;
    if (e_list.length() != el.length())
      error_handler(1,"graph::sort_edges(list<edge>): illegal edge list");
    e_list = el;
    forall_items(it,e_list) 
    { edge e = e_list[it];
      if (e->term[0]->owner != this)
         error_handler(1,"graph::sort_edges(list<edge>): edge not in graph");
      e->pos_in_E = it;
     }
   }

  forall(v,v_list)
    for(int i=0; i<2; i++)
    { v->first_adj_edge[i] = 0;
      v->last_adj_edge[i] = 0;
      v->adj_length[i] = 0;
     }

  forall(e,e_list)
  { source(e)->append_adj_edge(e,0,0);
    if (undirected)
       target(e)->append_adj_edge(e,0,1);
    else
       target(e)->append_adj_edge(e,1,1);
   }
 }


void graph::sort_edges(int (*f)(const edge&, const edge&))
{ e_list.sort(f);
  sort_edges(e_list);
 }


void graph::sort_nodes(const graph_map& A) 
{ NA = &A; 
  const char* type = A.elem_type_name();
  if (strcmp(type, LEDA_TYPE_NAME(int)) == 0)
     sort_nodes(int_array_cmp_nodes); 
  else
     if (strcmp(type, LEDA_TYPE_NAME(float)) == 0)
        sort_nodes(float_array_cmp_nodes); 
     else 
        if (strcmp(type, LEDA_TYPE_NAME(double)) == 0)
           sort_nodes(double_array_cmp_nodes); 
        else 
          error_handler(1,"G.sort_nodes(node_array<T>): T must be numerical.");
 }


void graph::sort_edges(const graph_map& A) 
{ NA = &A; 
  const char* type = A.elem_type_name();
  if (strcmp(type, LEDA_TYPE_NAME(int)) == 0)
     sort_edges(int_array_cmp_edges); 
  else
     if (strcmp(type, LEDA_TYPE_NAME(float)) == 0)
        sort_edges(float_array_cmp_edges); 
     else 
        if (strcmp(type, LEDA_TYPE_NAME(double)) == 0)
           sort_edges(double_array_cmp_edges); 
        else 
          error_handler(1,"G.sort_edges(edge_array<T>): T must be numerical.");
 }




void graph::sort_nodes() 
{ GGG = this; 
  sort_nodes(graph_cmp_nodes); 
 }

void graph::sort_edges() 
{ GGG = this; 
  sort_edges(graph_cmp_edges); 
 }

