/*******************************************************************************
+
+  LEDA 3.4
+
+  calls.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

#include <stdio.h>
#include <LEDA/graphwin.h>

//----------------------------------------------------------------------------

typedef list<node> (*DFS_t)(const graph& G, node s, node_array<bool>& reached);

void gw_call(GraphWin& gw, DFS_t f) {
  const graph &G=gw.get_graph();  
  node s=gw.get_node();
  if (s == nil) return;
  node_array<bool> reached(G,false);
  list<node> n_list;
  n_list=f(G,s,reached);
  bool old_flush=gw.set_flush(true);
  color c=gw.get_hi_node_color();
  while (!n_list.empty()) gw.set_color(n_list.pop(),c);
  gw.set_flush(old_flush);
}

//----------------------------------------------------------------------------

typedef bool (*TOPSORT_t)(const graph& G, node_array<int>& ord);

void gw_call(GraphWin& gw, TOPSORT_t f) {
  const graph &G=gw.get_graph();
  node_array<int> ord(G);
  bool b;
  b=f(G,ord);
  if (!b) {
    gw.acknowledge("Graph isn't acyclic!");
    return;
  }
  bool old_flush=gw.set_flush(true);
  node v;
  forall_nodes(v,G) {
    gw.set_label(v,ord[v]);
    gw.show_label(v);
  }
  gw.set_flush(old_flush);
}

//----------------------------------------------------------------------------

typedef int (*COMPONENTS_t)(const graph& G, node_array<int>& compnum);

void gw_call(GraphWin& gw, COMPONENTS_t f) {
  const graph &G=gw.get_graph();
  node_array<int> compnum(G);
  int i,c;
  c=f(G,compnum);
  node v;
  bool old_flush=gw.set_flush(true);
  forall_nodes(v,G) {
    i=compnum[v];
    gw.set_label(v,i);
    gw.set_color(v,(color)(i%16));
    gw.show_label(v);
  }
  gw.set_flush(old_flush);
}

//----------------------------------------------------------------------------

typedef list<edge> (*DFS_NUM_t)(const graph& G, node_array<int>& dfsnum, node_array<int>& compnum);

void gw_call(GraphWin& gw, DFS_NUM_t f) {
  
  const graph& G=gw.get_graph();
  
  node_array<int> dfsnum(G);
  node_array<int> compnum(G);
  
  list<edge> L;
  
  L=f(G,dfsnum,compnum);
  
  node v;
  
  char s[20];
  
  bool old_flush=gw.set_flush(true);

  forall_nodes(v,G) {
    sprintf(s,"%i|%i",dfsnum[v],compnum[v]);  
    gw.set_label(v,s);
    gw.show_label(v);
  }
  
  edge e;
  
  color c=gw.get_hi_edge_color();

  gw.reset_edge_colors();  
  
  forall(e,L) gw.set_color(e,c);

  gw.set_flush(old_flush);

}

//----------------------------------------------------------------------------

typedef list<edge> (*SPANNING_TREE_t)(const graph& G);

void gw_call(GraphWin& gw, SPANNING_TREE_t f) {

  const graph& G=gw.get_graph();

  list<edge> L;
  
  L=f(G);
  
  color c=gw.get_hi_edge_color();
  
  bool old_flush=gw.set_flush(true); 
 
  edge e;
  
  gw.reset_edge_colors();
  
  forall(e,L) gw.set_color(e,c);
  
  gw.set_flush(old_flush);  
}

//----------------------------------------------------------------------------

typedef void (*do_nothing_t)(void);

void gw_call(GraphWin& gw, do_nothing_t) {


/*
  bool old_flush=gw.set_flush(true);

  node v=gw.get_node();
  
  if (v == nil) return;

  gw.set_color(v,blue);
  
  node w;
  
  forall_adj_nodes(w,v) gw.set_color(w,red);

  gw.set_flush(old_flush);
*/

  double x0,y0,x1,y1;

  gw.get_area(x0,y0,x1,y1,"Flaeche markieren!");
  
  list<node> L;
  
  L=gw.get_area_nodes(x0,y0,x1,y1);
  
  node v;
  
  gw.set_flush(true);
  
  forall(v,L) gw.set_color(v,blue);

}

//----------------------------------------------------------------------------





//----------------------------------------------------------------------------

