/*******************************************************************************
+
+  LEDA 3.4
+
+  draw.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <math.h>

#include <LEDA/window.h>
#include <LEDA/graph.h>
#include <LEDA/list.h>
#include <LEDA/map.h>
#include <LEDA/node_matrix.h>

#include <LEDA/graphwin.h>

//----------------------------------------------------------------------------

void GraphWin::draw_node(node v) {

  node_info &v_inf=n_info[v];
  
  double r1=v_inf.r1,r2=v_inf.r2;

  switch (v_inf.type) {
  
    case circle_node :
     
      r2=r1;

      switch (v_inf.style) {
   
        case filled_node :
          win_p->draw_disc(v_inf.pos,v_inf.r1,v_inf.clr);
          win_p->draw_circle(v_inf.pos,v_inf.r1,black);

        case simple_node :
          win_p->draw_circle(v_inf.pos,v_inf.r1,v_inf.clr);
          break;
    
      }
      break;      

    case rectangle_node :
    
      switch (v_inf.style) {
      
        case filled_node :
          win_p->draw_box(v_inf.pos.xcoord()-v_inf.r1,
                          v_inf.pos.ycoord()-v_inf.r2,
                          v_inf.pos.xcoord()+v_inf.r1,
                          v_inf.pos.ycoord()+v_inf.r2,
                          v_inf.clr);
          win_p->draw_rectangle(v_inf.pos.xcoord()-v_inf.r1,
                                v_inf.pos.ycoord()-v_inf.r2,
                                v_inf.pos.xcoord()+v_inf.r1,
                                v_inf.pos.ycoord()+v_inf.r2,
                                black);

        case simple_node :
          win_p->draw_rectangle(v_inf.pos.xcoord()-v_inf.r1,
                                v_inf.pos.ycoord()-v_inf.r2,
                                v_inf.pos.xcoord()+v_inf.r1,
                                v_inf.pos.ycoord()+v_inf.r2,
                                v_inf.clr);
          break;
      }
      
      break;                      
            

    case ellipse_node :
    
      switch (v_inf.style) {
            
        case filled_node :
          win_p->draw_filled_ellipse(v_inf.pos,v_inf.r1,v_inf.r2,v_inf.clr);
          win_p->draw_ellipse(v_inf.pos,v_inf.r1,v_inf.r2,black);

        case simple_node :
          win_p->draw_ellipse(v_inf.pos,v_inf.r1,v_inf.r2,v_inf.clr);
          break;
          
      }
      break;      
    
  }    

  if (v_inf.show_label) {

    string& label=v_inf.label;
    
    position_t &label_pos=v_inf.label_pos;
    double p_x=v_inf.pos.xcoord(),p_y=v_inf.pos.ycoord();
    double dx=win_p->text_width(label), dy=win_p->text_height(label);
 
    if (label_pos==right || label_pos==right_above || label_pos==right_below)
      p_x+=r1+units_per_pixel;
    else {
      if (label_pos==left || label_pos==left_above || label_pos==left_below) 
        p_x-=r1+dx+units_per_pixel;
      else p_x-=dx/2;
    }

    if (label_pos==above || label_pos==left_above || label_pos==right_above)
      p_y+=r2+dy+units_per_pixel;
    else {
      if (label_pos==below || label_pos==left_below || label_pos==right_below)
        p_y-=r2+units_per_pixel;
      else p_y+=dy/2;
    }

    if ((label_pos == central) && (v_inf.style == filled_node)) 
      win_p->draw_text(p_x,p_y,label,v_inf.clr);
    win_p->draw_text(p_x,p_y,label,v_inf.label_clr);
  }
}

//----------------------------------------------------------------------------

void GraphWin::draw_edge(edge e) {

  if (get_position(source(e)) == get_position(target(e))) return;

  edge_info &e_inf=e_info[e];
 
  list<point>& P=e_inf.p;	// list<node>& get_polygon(edge e)

  if (P.size() < 2) return;

  color c=get_color(e);

  edge_st old_style=win_p->set_line_style(e_inf.style);
  int old_width=win_p->set_line_width(e_inf.width);

  list_item it=P.first();
  
  point e_p0=P[it],e_p1;

  it=P.succ(it);

  for(;;) {
    e_p1=P[it];
    if ((it=P.succ(it)) == nil) break;
    win_p->draw_segment(e_p0,e_p1,c);
    e_p0=e_p1;
  }

  if (directed) win_p->draw_arrow(e_p0,e_p1,c);
  else win_p->draw_segment(e_p0,e_p1,c);

  win_p->set_line_style(old_style);
  win_p->set_line_width(old_width);

}

//----------------------------------------------------------------------------

void GraphWin::draw_node_with_edges(node v) { 
  edge e;
  forall_inout_edges(e,v) draw_edge(e);
  draw_node(v);
}

//----------------------------------------------------------------------------

void GraphWin::draw_nodes() {
  node v;
  forall_nodes(v,*gr_p) draw_node(v);
}

//----------------------------------------------------------------------------

void GraphWin::draw_edges() {
  edge e;
  forall_edges(e,*gr_p) draw_edge(e);
}

//----------------------------------------------------------------------------

void GraphWin::draw_graph() { 
  if (!nodes_embedded) embed_nodes(node_embed);
  if (!edges_embedded) embed_edges();
  draw_nodes();
  draw_edges();
}          
           
//----------------------------------------------------------------------------

void GraphWin::s_redraw(window *w_p) { 
  GraphWin &gw=*GraphWin::win_vec[w_p];
  gw.update_win_bounds();
  gw.draw_graph(); 
}

//----------------------------------------------------------------------------

void GraphWin::redraw() { 
  win_p->clear();
  draw_graph();
}

//----------------------------------------------------------------------------

        
