/*******************************************************************************
+
+  LEDA 3.4
+
+  edit.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <LEDA/graphwin.h>
#include "local.h"

void dfs(graph& G, node v, node_array<bool>& reached, list<node>& L) { 
  reached[v] = true;
  L.append(v);
  edge e;
  forall_inout_edges(e,v) { 
    node u = G.opposite(v,e);
    if ( ! reached[u]) dfs(G,u,reached,L);
  }
}

//----------------------------------------------------------------------------

void dfs(graph& G, node v, 
         node_array<bool>& n_reached, edge_array<bool>& e_reached,
         list<node>& V, list<edge>& E) { 

  n_reached[v] = true;
  V.append(v);
  edge e;
  forall_inout_edges(e,v) { 
    if (!e_reached[e]) {
      e_reached[e]=true;
      E.append(e);
    }
    node u = G.opposite(v,e);
    if (!n_reached[u]) dfs(G,u,n_reached,e_reached,V,E);
  }
}

//----------------------------------------------------------------------------

void GraphWin::move_component(node v, double x, double y) {
  int event,val;
  unsigned long t = win_p->button_press_time(),t1;
  double x1,y1;

  list<node> V;
  list<edge> E;

  node_array<bool> n_reached(*gr_p,false);
  edge_array<bool> e_reached(*gr_p,false);

  dfs(*gr_p,v,n_reached,e_reached,V,E);
  vector vec(2);
  do { 
    event = win_p->read_event(val,x1,y1,t1);
    if (event == motion_event) if (t1 - t < 50) continue;
    t = t1;
    vec=vector(x1-x,y1-y);
    move_nodes(V,vec);
    move_edges(E,vec);
    x=x1;
    y=y1;
  }
  while (event != button_release_event);
}

//----------------------------------------------------------------------------

void GraphWin::scroll_graph(double x, double y) {

  point* p_field= new point [gr_p->number_of_nodes()];
  list<point>*  c_field = new list<point> [gr_p->number_of_edges()];
  
  node v;
  edge e;
  unsigned i;
  
  i=0;
  forall_nodes(v,*gr_p) p_field[i++]=get_position(v);
  i=0;
  forall_edges(e,*gr_p) c_field[i++]=e_info[e].p; 
  
  double x_start=x, y_start=y;

  int event,val;
  unsigned long t = win_p->button_press_time(),t1;
  double x1,y1;

//  double xt=x_min+0.05*(x_max-x_min), yt=y_min+0.05*(y_max-y_min);


  do {   
//    string s("(%f,%f)",x_min-x_start+x,y_min-y_start+y);
//    win_p->draw_text(xt,yt,s);
    event = win_p->read_event(val,x1,y1,t1);
    if (event == motion_event) if (t1 - t < 50) continue;
    t = t1;
    move_nodes(vector(x1-x,y1-y));
//    win_p->draw_text(xt,yt,s);
    x=x1;
    y=y1;
  }      
  while (event != button_release_event);

  double xd=x-x_start;
  double yd=y-y_start;
          
  win_p->init(x_min-xd,x_max-xd,y_min-yd);
  update_win_bounds();
          
  bool old_flush=set_flush(false);
  
  i=0;
  forall_nodes(v,*gr_p) set_position(v,p_field[i++]);
  i=0;
  forall_edges(e,*gr_p) e_info[e].p=c_field[i++];
  
  nodes_embedded=true;
  edges_embedded=true;
  
  draw_graph();

  delete []c_field;
  delete []p_field;
  
  set_flush(old_flush);
}

//----------------------------------------------------------------------------


//----------------------------------------------------------------------------


//----------------------------------------------------------------------------

bool GraphWin::edit() {

  bool old_done=done;
  
  done=false;

  unsigned long t,t0,t1;
  
  int event;
  
  int val;

  double x,y;
  int k;
  node v,w;
  edge e;
  point p,q;

  bool old_flush=get_flush();

  while (!done) {

    set_flush(old_flush);
    
    k=win_p->read_mouse(x,y);

    t = t0 = win_p->button_press_time();

    old_flush=set_flush(true);

    p=point(x,y);
    
    switch (k) {
    
      case NEW_BUTTON:
      
        if (win_p->ctrl_key_down()) {
          scroll_graph(x,y);
          break;
        }
        
        v=find_node(p);
        if (v != nil) {
        
          if (win_p->shift_key_down()) {	// move component
            move_component(v,x,y);
            break;
          }
        
          point &v_pos=n_info[v].pos;
        
          point  p0 = v_pos;
          double dx = p0.xcoord() - x;
          double dy = p0.ycoord() - y;

          bool has_moved=false;
          
          do {

            event = win_p->read_event(val,x,y,t1);
            
            if (event == motion_event) if (t1 - t < 50) continue;
              
            t = t1;
            
            q = point(x+dx,y+dy);

            if (t1 - t0 > 150) {
              if (q != v_pos) { 
                move_node(v,q);
                has_moved=true;
                if (move_node_handler) move_node_handler(*this,v);
              }
            }
          
          }
          while (event != button_release_event);

          node u = find_node(v_pos);
          if (u == v) u=find_node(v_pos,v);

          if (has_moved) {
            if (u != nil) move_node(v,p0);
            break;
          }

          // new edge with start-node v
          
          win_p->message("new edge");
          highlight(v);
          int but=win_p->read_mouse_seg(p0,q);
          unhighlight(v);
          win_p->del_messages();

          if (but != DEL_BUTTON) {
            w=find_node(q);
            if (w == nil) {
              w=new_node(q);
              if (new_node_handler) new_node_handler(*this,w);
            }
            e=new_edge(v,w);
            if (new_edge_handler) new_edge_handler(*this,e);
          }
        }
        else {
          e=find_edge(p);
          if (e != nil) {				// node on edge?
            win_p->message("new node");
            highlight(e);

            string s[]={ "split","leave","cancel" };
            int choice=win_p->read_vpanel("Split edge?",3,s);

            unhighlight(e);
            win_p->del_messages();

            if (choice == 2) break;

            if (choice == 1) {
              node u=new_node(p);
              if (new_node_handler) new_node_handler(*this,u);
              v=source(e);
              w=target(e);
              del_edge(e);
              e=new_edge(v,u);
              if (new_edge_handler) new_edge_handler(*this,e);
              e=new_edge(u,w);
              if (new_edge_handler) new_edge_handler(*this,e);
            }
            else {
              v=new_node(p);
              if (new_node_handler) new_node_handler(*this,v);
            }
            
          }
          else {		//new node or mark area
          
            double x_first=x,y_first=y;

            t1=read_mouse_rect(x_first,y_first,x,y,true);
            
            if ( (t1-t < 150) 
                       || 
                 (p.distance(point(x,y)) < units_per_pixel*10) ) {
              v=new_node(p);
              if (new_node_handler) new_node_handler(*this,v);
            }
            else {    // mark nodes
              list<node> marked;
              marked=get_area_nodes(x_first,y_first,x,y);
              if (!marked.empty()) setup_nodes(marked);
            }                
          }
        }	

        break;
        
      case INFO_BUTTON:
        
        v=find_node(p);
        if (v != nil) node_settings_diag(v);        
        else {
          e=find_edge(p);
          if (e != nil) edge_settings_diag(e);
          else {		// nothing found => general_settings
            set_flush(old_flush);
            settings_diag();
            old_flush=set_flush(true);
          }
        }
        
        break;
        
      case DEL_BUTTON:
      
        v=find_node(p);
        if (v != nil) {
          if (win_p->shift_key_down()) {	// delete component
            list<node> L;
            node_array<bool> reached(*gr_p,false);
            dfs(*gr_p,v,reached,L);
            forall(v,L) {
              if (delete_node_handler) delete_node_handler(*this,v);
              del_node(v);
            }
          }
          else {
            if (delete_node_handler) delete_node_handler(*this,v);
            del_node(v);            // later : diff. between one&more hits
          }
        }
        else {
          e=find_edge(p);
          if (e != nil) {
            if (delete_edge_handler) delete_edge_handler(*this,e);
            del_edge(e);
          }
        }	
    
        break;

    }	// switch

  }	// while !done

  set_flush(old_flush);
  
  done=old_done;
  
  return done_cause;  

}

//----------------------------------------------------------------------------

    
    
