/*******************************************************************************
+
+  LEDA 3.4
+
+  embed.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <math.h>
#include <LEDA/graphwin.h>
#include "local.h"

//----------------------------------------------------------------------------

void GraphWin::zoom_graph(double f) { 

  if (f <= 0) return;

  double xd=(x_max-x_min)*(1/f-1)/2;
  double yd=(y_max-y_min)*(1/f-1)/2;

  win_init(x_min-xd,x_max+xd,y_min-yd);
}  

//----------------------------------------------------------------------------

void GraphWin::zoom_up_area() {

  win_p->message("Determine zoom-area!");
    
  double x0,y0,x1,y1;
  
  int but;
    
  do {

    but=win_p->read_mouse(x0,y0);

    if (but == DEL_BUTTON) {
      win_p->del_messages();
      return;
    }

   
    read_mouse_rect(x0,y0,x1,y1,true);  

/*
    int event;

    double m=(y_max-y_min)/(x_max-x_min);

    double x=x1=x0,y=y1=y0;	// x1,y1 : last  ,  x,y : new

    win_p->draw_rectangle(x0,y0,x,y,black);

    do {
      event = win_p->read_event(but,x,y);
      if (event == motion_event) {
        y=m*(x-x0)*(y>y0?1:-1)+y0;
        win_p->draw_rectangle(x0,y0,x,y,black);
        win_p->draw_rectangle(x0,y0,x1,y1,black);
        x1=x;
        y1=y;
      }
    }
    while(event != button_release_event);

    win_p->draw_rectangle(x0,y0,x1,y1,black);
*/

  }
  while (x0 == x1);

  win_p->del_messages();

  if (y0 > y1) y0=y1;
  if (x0 > x1) { y1=x0; x0=x1; x1=y1; }

  win_init(x0,x1,y0);
}  
      
//----------------------------------------------------------------------------

void GraphWin::fill_in_window() {

  node v=gr_p->first_node();

  if (v == nil) return;
  
  point p=get_position(v);
  
  double x=p.xcoord(), y=p.ycoord();
  
  double r1=get_radius(v), r2= get_type(v)==circle_node ? r1 : get_radius2(v);
  
  double xmin=x-r1,xmax=x+r1,ymin=y-r2,ymax=y+r2;
  
  double r_xmin=r1,r_xmax=r1,r_ymin=r2,r_ymax=r2;

  double xp_min=x,xp_max=x,yp_min=y,yp_max=y;
  
  while ( (v=gr_p->succ_node(v)) != nil) {
    
    p=get_position(v);
    x=p.xcoord();
    y=p.ycoord();
    r1=get_radius(v);
    r2= get_type(v)==circle_node ? r1 : get_radius2(v);
    
    if (x-r1 < xmin) {
      xmin=x-r1;
      r_xmin=r1;
      xp_min=x;
    }
    
    if (x+r1 > xmax) {
      xmax=x+r1;
      r_xmax=r1;
      xp_max=x;
    }
    
    if (y-r2 < ymin) {
      ymin=y-r2;
      r_ymin=r2;
      yp_min=y;
    }

    if (y+r2 > ymax) {
      ymax=y+r2;
      r_ymax=r2;
      yp_max=y;
    }

  }

  double x_d=get_x_max()-get_x_min();
  double y_d=get_y_max()-get_y_min();
 
  bool const_width=get_constant_width();

  double t;

  if (const_width) {
    if (xp_min == xp_max) {
      if (yp_min == yp_max) {
        win_init(xp_min-x_d/2,xp_min+x_d/2,yp_min-y_d/2);
        return;
      }
      t=(y_d-r_ymin-r_ymax)/(yp_max-yp_min);
    }
    else {
      if (yp_min == yp_max) 
        t=(x_d-r_xmin-r_xmax)/(xp_max-xp_min);
      else {
        if ( (ymax-ymin)*x_d < (xmax-xmin)*y_d ) 
          t=(x_d-r_xmin-r_xmax)/(xp_max-xp_min);
        else
          t=(y_d-r_ymin-r_ymax)/(yp_max-yp_min);
      }
    }
  }
  else {
    if ( (ymax-ymin)*x_d < (xmax-xmin)*y_d )
      t=x_d/(xmax-xp_min+r_xmin);
    else 
      t=y_d/(ymax-yp_min+r_ymin); 
  }

  double xd=x_d/t, yd=y_d/t;

  double xmid,ymid;

  if (const_width) {
    xmid=(xp_max+xp_min+(r_xmax-r_xmin)/t)/2;
    ymid=(yp_max+yp_min+(r_ymax-r_ymin)/t)/2;
  }
  else {
    xmid=(xmax+xmin)/2;
    ymid=(ymax+ymin)/2;
  }

  xd*=1.1;
  yd*=1.1;

  win_init(xmid-xd/2,xmid+xd/2,ymid-yd/2);

}

//----------------------------------------------------------------------------

void embed_grid(graph& G, double x_min, double y_min, double x_max,
                double y_max, node_array<point>& dest) { 
  int n = G.number_of_nodes();
  if (n == 0) return;
  int w = (int) (x_max - x_min);
  int h = (int) (y_max - y_min);
  n = (int) (sqrt((float)n));
  if (n*n < G.number_of_nodes()) ++n;
  int dx = w/n;
  int dy = h/n;
  x_min+=dx/2;
  y_min+=dy/2;
  int i = 0;
  node v;
  forall_nodes(v,G) { 
    int row = i/n;
    int col = i%n;
    dest[v]=point(x_min+col*dx,y_min+row*dy);
    i++;
  }
}
  
//----------------------------------------------------------------------------

void embed_random(graph& G, double x_min, double y_min, double x_max,
                  double y_max, node_array<point>& dest) { 
  random_source ran;
  node v;
  double x,y;
  double x_d=x_max-x_min;
  double y_d=y_max-y_min;
  forall_nodes(v,G) {
    ran >> x >> y;
    x*=x_d;
    y*=y_d;
    dest[v]=point(x_min+x,y_min+y);
  }
}

//----------------------------------------------------------------------------

void embed_circular(graph& G, double x_min, double y_min, double x_max,
                    double y_max, node_array<point>& dest) {
  int n=G.number_of_nodes();
  if (n == 0) return;
  point  m = point((x_min+x_max)/2.0, (y_min+y_max)/2.0);
  double x_d=x_max-x_min;
  double y_d=y_max-y_min;
  double r = (x_d < y_d ? x_d : y_d) /2;
  double alpha = 0;
  double step  = 6.2832/n;
  node v;
  forall_nodes(v,G) { 
    dest[v]=m.translate(alpha,r);
    alpha+=step;
  }
}

//----------------------------------------------------------------------------

/*

void bi_embed(GRAPH<point,int>& G, list<node>& A, list<node>& B)
{ int    w = x_max - x_min;
  int    h = y_max - y_min;
  node   v;
  double y,dy;

  node_array<double> xc(G);
  node_array<double> yc(G);

  dy = h/A.length();
  y  = y_min + dy/2;
  forall(v,A) 
  { xc[v] = x_min + w/4;
    yc[v] = y;
    y += dy;
  }

  dy = (y_max-y_min)/B.length();
  y  = y_min + dy/2;
  forall(v,B) 
  { xc[v] = x_max - w/4;
    yc[v] = y;
    y += dy;
  }

  move_graph(*Wp,G,xc,yc,animation);

}

//----------------------------------------------------------------------------

void planar_embed(GRAPH<point,int>& G, int k)
{ 
  if (PLANAR(G) == false) 
      Wp->acknowledge("graph is not planar");
  else
   { int w = x_max - x_min;
     int h = y_max - y_min;
     int n = G.number_of_nodes();

     list<edge> bi_edges = Make_Biconnected(G);
     list<edge> rev_edges = Make_Bidirected(G);

     PLANAR(G,true);

     node_array<int> xcoord(G);
     node_array<int> ycoord(G);

     float fx = w/n;
     float fy = h/n;


     if (k==0)
       { STRAIGHT_LINE_EMBEDDING(G,xcoord,ycoord);
         fx /= 2;
        }
     else
        STRAIGHT_LINE_EMBEDDING2(G,xcoord,ycoord);

     // restore original graph
     edge e;
     forall(e,rev_edges) G.del_edge(e);
     forall(e,bi_edges)  G.del_edge(e);

     node_array<double> xc(G);
     node_array<double> yc(G);

     node v;
     forall_nodes(v,G) 
     {  //G[v] = point(x_min + fx*(xcoord[v]+1), y_min + fy*(ycoord[v]+1));
        xc[v] = x_min + fx*(xcoord[v]+1);
        yc[v] = y_min + fy*(ycoord[v]+1);
      }

     move_graph(*Wp,G,xc,yc,animation);
         
   }

}

*/

//----------------------------------------------------------------------------

void GraphWin::embed_nodes(node_embed_t et) {

  node_array<point> dest(*gr_p);
  
  double dist=(x_max-x_min)/20;

  switch (et) {

    case random_embed : 
      embed_random(*gr_p,x_min+dist,y_min+dist,x_max-dist,y_max-dist,dest);
      break;

    case grid_embed   : 
      embed_grid(*gr_p,x_min+dist,y_min+dist,x_max-dist,y_max-dist,dest);
      break;

    case circle_embed : 
      embed_circular(*gr_p,x_min+dist,y_min+dist,x_max-dist,y_max-dist,dest);
      break;

    default : return;
  }

  set_position(dest);
  node_embed=et;

}

//----------------------------------------------------------------------------

static point leaving_point_of_rectangle_edge( double x0, double y0, 
                                              double x1, double y1,
                                              double r1, double r2  ) {

  double m,k1=r1,k2=r2;
          
  if (x0 == x1) {
    k1=0;
    if (y1 < y0) k2=-k2;
  }
  else {
    if (y0 == y1) {
      k2=0;
      if (x1 < x0) k1=-k1;
    }
    else {
      m=(y1-y0)/(x1-x0);
      if (m > 0) {
        if (m > r2/r1) {
          if (y1 < y0) k2=-k2;
          k1=k2/m;
        }
        else {
          if (y1 < y0) k1=-k1;
          k2=k1*m;
        }
      }
      else {	
        if (-m > r2/r1) {
          if (y1 < y0) k2=-k2;
          k1=k2/m;
        }
        else {
          if (y1 > y0) k1=-k1;
          k2=k1*m;
        }
      }
    }
  }

  return point(x0+k1,y0+k2);
}

//----------------------------------------------------------------------------

static point leaving_point_of_ellipse_edge( double x0, double y0,
                                            double x1, double y1,
                                            double r1, double r2  ) {

  double m,k1=r1,k2=r2,t;
  if (x0 == x1) {
    k1=0;
    if (y1 < y0) k2=-k2;
  }
  else {
    if (y0 == y1) {
      k2=0;
      if (x1 < x0) k1=-k1;
    }
    else {
      m=(y1-y0)/(x1-x0);
      t=r1*m;
      k1=(r1*r2)/sqrt(r2*r2+t*t);
      if (x1 < x0) k1=-k1;
      k2=m*k1;
    }
  }
  return point(x0+k1,y0+k2);
}

//----------------------------------------------------------------------------

static point leaving_point_of_circle_edge( double x0, double y0,
                                           double x1, double y1, double r) {

  double m,k1=r,k2=r;

  if (x0 == x1) {
    k1=0;
    if (y1 < y0) k2=-k2;
  }
  else {
    if (y0 == y1) {
      k2=0;
      if (x1 < x0) k1=-k1;
    }
    else {
      m=(y1-y0)/(x1-x0);
      k1=r/sqrt(1+m*m);
      if (x1 < x0) k1=-k1;
      k2=m*k1;
    }
  }
  return point(x0+k1,y0+k2);

}

//----------------------------------------------------------------------------

static void parallel_tangents_to_ellipse(double rx, double ry,
                                         const line& l, line& l1, line& l2) {

  if (!l.is_vertical()){
    rx*=rx;
    ry*=ry;
    double m=l.slope();
    double n=m*rx;
    double s=sqrt(ry+m*n);
    rx=n/s;ry=-ry/s;
  }

  l1=l.translate(vector(rx,ry));
  l2=l.translate(vector(-rx,-ry));

}

//----------------------------------------------------------------------------

static void parallel_tangents_to_rectangle(double rx, double ry,
                                           const line& l, line& l1, line& l2) {
  
  if (!l.is_vertical()) if (l.slope() > 0) rx=-rx;
  l1=l.translate(vector(rx,ry));
  l2=l.translate(vector(-rx,-ry));
}

//----------------------------------------------------------------------------

static double dist_of_central_two(double x1, double x2, double x3, double x4) {

  // sort arguments by size (x[i1]<x[i2]<x[i3]<x[i4]) and return x[i3]-x[i2] 

  double tmp;

  if (x1 > x2) { tmp=x1; x1=x2; x2=tmp; }
  if (x3 > x4) { tmp=x3; x3=x4; x4=tmp; }
  
  return fabs((x2 < x4 ? x2 : x4) - (x1 < x3 ? x3 : x1));
}

//----------------------------------------------------------------------------

static double distance_of_inmost_parallels(const line& l1, const line& l2, 
                                           const line& l3, const line& l4) {

  if (l1.is_vertical()) return dist_of_central_two(l1.x_proj(0),l2.x_proj(0),l3.x_proj(0),l4.x_proj(0));

  double m=l1.slope();

  return dist_of_central_two(l1.y_proj(0),l2.y_proj(0),l3.y_proj(0),l4.y_proj(0))/sqrt(m*m+1); 
}

//----------------------------------------------------------------------------

double GraphWin::max_edge_distance(node v, node w) {

  point p_v=get_position(v);
  point p_w=get_position(w);

  line l(p_v,p_w);

  if (p_v == p_w) l=line(p_v,vector(0,1));	// embed vertically

  line v_l1,v_l2,w_l1,w_l2;
  
  double rx=get_radius(v);
  double ry=get_radius2(v);

  switch (get_type(v)) {
    case circle_node :
      ry=rx;
    case ellipse_node:
      parallel_tangents_to_ellipse(rx,ry,l,v_l1,v_l2);
      break;
    case rectangle_node:
      parallel_tangents_to_rectangle(rx,ry,l,v_l1,v_l2);
      break;
  }

  rx=get_radius(w);
  ry=get_radius2(w);

  switch (get_type(w)) {
    case circle_node :
      ry=rx;
    case ellipse_node:
      parallel_tangents_to_ellipse(rx,ry,l,w_l1,w_l2);
      break;
    case rectangle_node:
      parallel_tangents_to_rectangle(rx,ry,l,w_l1,w_l2);
      break;
  }

  return 0.9*distance_of_inmost_parallels(v_l1,v_l2,w_l1,w_l2);
}

//----------------------------------------------------------------------------

point GraphWin::first_hit(node v, const line& l, 
                          bool from_left, bool from_above) {

  point p_v=get_position(v);

  double x0=p_v.xcoord();
  double y0=p_v.ycoord();

  double rx=get_radius(v);
  double ry=get_radius2(v);

  switch(get_type(v)) {


    case circle_node:
      ry=rx;
    case ellipse_node: {
      if (l.is_vertical()) {
        double x=l.x_proj(0);
        double dx=x-x0;
        double s=ry/rx*sqrt(rx*rx-dx*dx);
        
        if (from_above) return point(x,y0-s);
        return point(x,y0+s);
      }

      double m=l.slope();
      double t0=y0-l.y_proj(0);
      double t1=m*rx*rx;
      double t2=ry*ry;
      double t3=x0*t2+t0*t1;
      t2=t1*m+t2;
      t1=x0*m-t0;
      t1=rx*ry*sqrt(t2-t1*t1);
      double x;

      if (from_left) x=(t3-t1)/t2;
      else x=(t3+t1)/t2;
      return point(x,l.y_proj(x));
    }
      break;

    case rectangle_node: {

      if (from_left) x0-=rx;
      else x0+=rx;
        
      double y=l.y_proj(x0);

      if (from_above) {
        y0-=ry;
        if (l.is_vertical()) return point(l.x_proj(y0),y0);
        if (y < y0) return point(l.x_proj(y0),y0);
        return point(x0,y);
      }

      y0+=ry;
      if (l.is_vertical()) return point(l.x_proj(y0),y0); 
      if (y > y0) return point(l.x_proj(y0),y0);
      return point(x0,y);
    }
      break;
  }

  error_handler(1,"graphwin: internal error.");

  return point(0,0); // never reached
}  

//----------------------------------------------------------------------------

point GraphWin::compute_leaving_point(node v, const point& p) {

  point p_v=get_position(v);

  double x_v=p_v.xcoord();
  double y_v=p_v.ycoord();
  double x=p.xcoord();
  double y=p.ycoord();

  double rx_v=get_radius(v);
  double ry_v=get_radius2(v);

  switch(get_type(v)) {
    case rectangle_node:
      p_v=leaving_point_of_rectangle_edge(x_v,y_v,x,y,rx_v,ry_v);
      break;
      
    case circle_node:
      p_v=leaving_point_of_circle_edge(x_v,y_v,x,y,rx_v);
      break;
    
    case ellipse_node:
      p_v=leaving_point_of_ellipse_edge(x_v,y_v,x,y,rx_v,ry_v);
      break;
  }
  
  return p_v;  
} 

//----------------------------------------------------------------------------

void GraphWin::compute_leaving_points(node v, node w, point& p_v, point& p_w) {

  p_v=get_position(v);
  p_w=get_position(w);

  double x_v=p_v.xcoord();
  double y_v=p_v.ycoord();
  double x_w=p_w.xcoord();
  double y_w=p_w.ycoord();

  double rx_v=get_radius(v);
  double ry_v=get_radius2(v);
  double rx_w=get_radius(w);
  double ry_w=get_radius2(w);

  switch(get_type(v)) {
    case rectangle_node:
      p_v=leaving_point_of_rectangle_edge(x_v,y_v,x_w,y_w,rx_v,ry_v);
      break;
      
    case circle_node:
      p_v=leaving_point_of_circle_edge(x_v,y_v,x_w,y_w,rx_v);
      break;
    
    case ellipse_node:
      p_v=leaving_point_of_ellipse_edge(x_v,y_v,x_w,y_w,rx_v,ry_v);
      break;
  }
  
  switch(get_type(w)) {
    case rectangle_node:
      p_w=leaving_point_of_rectangle_edge(x_w,y_w,x_v,y_v,rx_w,ry_w);
      break;
      
    case circle_node:
      p_w=leaving_point_of_circle_edge(x_w,y_w,x_v,y_v,rx_w);
      break;
    
    case ellipse_node:
      p_w=leaving_point_of_ellipse_edge(x_w,y_w,x_v,y_v,rx_w,ry_w);
      break;
  }
} 

//----------------------------------------------------------------------------

void GraphWin::embed_edges(node v, node w) {

  list<edge>& L_vw=incidence_list(v,w);
  list<edge>& L_wv=incidence_list(w,v);
  
  edge e; 

  list<list<point>*> auto_list_vw;
  list<list<point>*> auto_list_wv;
  
  forall(e,L_vw) {
  
    edge_info& e_inf=e_info[e];
    
    if (e_inf.auto_edge) auto_list_vw.append(&e_inf.p);
    else {
      list<point>& P=e_inf.p;
      P.pop();
      P.Pop();
      P.push(compute_leaving_point(v,P.head()));
      P.append(compute_leaving_point(w,P.tail()));
    }
    
  }
  
  forall(e,L_wv) {
  
    edge_info& e_inf=e_info[e];
    
    if (e_inf.auto_edge) auto_list_wv.append(&e_inf.p);
    else {	
      list<point>& P=e_inf.p;
      P.pop();
      P.Pop();
      P.push(compute_leaving_point(w,P.head()));
      P.append(compute_leaving_point(v,P.tail()));
    }
    
  }

  unsigned s_vw=auto_list_vw.size();
  unsigned s_wv=auto_list_wv.size();

  if (s_vw + s_wv == 0) return;
  
  unsigned s=s_vw+s_wv;

  if (s == 1) {

    if (s_vw == 1) {
      list<point>& P=*auto_list_vw.pop();
      P.clear();
      point p_v,p_w;
      compute_leaving_points(v,w,p_v,p_w);
      P.push(p_v);
      P.append(p_w);    
      return;
    }

    list<point>& P=*auto_list_wv.pop();
    P.clear();
    point p_v,p_w;
    compute_leaving_points(v,w,p_v,p_w);
    P.push(p_v);
    P.push(p_w);    
    return;
  }

  s--;
  
  double max_dist=max_edge_distance(v,w);
  double h = edge_dist * s > max_dist ? max_dist / s : edge_dist;

  point p_v=get_position(v);
  point p_w=get_position(w);
  
  line l(p_v,p_w);	// embed all edges around l
 
  if (fabs(p_v.xcoord()-p_w.xcoord())*win_p->scale() < 0.5) 
    l=line(p_v,vector(0,1));


  point p_tmp(point(0,0).translate(l.rotate90(p_v).direction(),h));
  vector vec(p_tmp.xcoord(),p_tmp.ycoord());  


//  vector vec=point(0,0).translate(l.rotate90(p_v).direction(),h).to_vector();

  if (s%2) l=l.translate(vec*0.5);

  bool from_left=p_v.xcoord() > p_w.xcoord();
  bool from_above=p_v.ycoord() > p_w.ycoord(); 
  bool sign=true;
  double t=0;

  while(auto_list_vw.size()) {

    if (sign) l=l.translate(vec*t);
    else l=l.translate(vec*(-t));

    list<point>& P=*auto_list_vw.pop();

    P.clear();

    P.push(first_hit(v,l,from_left,from_above));
    P.append(first_hit(w,l,!from_left,!from_above));

    t+=1;
    sign=!sign;

  }

  while(auto_list_wv.size()) {

    if (sign) l=l.translate(vec*t);
    else l=l.translate(vec*(-t));

    list<point>& P=*auto_list_wv.pop();

    P.clear();

    P.push(first_hit(v,l,from_left,from_above));
    P.push(first_hit(w,l,!from_left,!from_above));

    t+=1;
    sign=!sign;

  }

}

//----------------------------------------------------------------------------

void GraphWin::embed_node_with_edges(node u) {

  node_list adj_nodes;
  
  edge e;
  node w;
  
  forall_inout_edges(e,u) {
    
    w=opposite(u,e);
    
    if (adj_nodes.member(w)) continue;
    adj_nodes.append(w);
    
    if (u==w) continue;			// don't know, how to embed self-loops

    embed_edges(source(e),target(e));
  }


}

//----------------------------------------------------------------------------

void GraphWin::embed_edges() {

  node v,w;
  edge e;

  node_list adj_nodes;
  node u;

  forall_nodes(u,*gr_p) {
    if (adj_nodes.member(u)) continue;
    adj_nodes.append(u);
    embed_node_with_edges(u);
  }
  edges_embedded=true;
}

//----------------------------------------------------------------------------
/*
void GraphWin::move_nodes(list<node>& L, vector trans, unsigned anim) {
  
  node v;
  
  vector p_trans(trans * (1.0/(anim+1)));
  
  unsigned i=0;
  
  do {
    forall(v,L) move_node(v,p_trans);
  }
  while (++i <= anim);

}
*/
//----------------------------------------------------------------------------

void GraphWin::move_edges(list<edge>& L, vector trans) {

  bool old_flush=set_flush(false);
  
  edge e;
  list_item it;

  forall(e,L) {
    draw_edge(e);
    list<point>& e_p=e_info[e].p;
    forall_items(it,e_p) e_p[it]=e_p[it].translate(trans);
    draw_edge(e);
  }
  set_flush(old_flush);
}


//----------------------------------------------------------------------------

void GraphWin::move_nodes(list<node>& L, vector trans) {
  
  node v;
  
  bool old_flush=set_flush(false);
  
  forall(v,L) {
    draw_node(v);
    point &v_pos=n_info[v].pos;
    v_pos=v_pos.translate(trans);
    draw_node(v);
  }
  
  set_flush(old_flush);
}

//----------------------------------------------------------------------------

void GraphWin::move_edges(vector trans) {

  edge e;
  
  list_item it;
  
  forall_edges(e,*gr_p) {
    
    draw_edge(e);
    
    list<point>& e_p=e_info[e].p;
    
    forall_items(it,e_p) e_p[it]=e_p[it].translate(trans);
    
    draw_edge(e);

  }

}

//----------------------------------------------------------------------------

void GraphWin::move_nodes(vector trans, unsigned anim) {
  
  node v;
  
  bool old_flush=set_flush(false);
  
  vector p_trans(trans * (1.0/(anim+1)));
  
  unsigned i=0;
  
  do {
    forall_nodes(v,*gr_p) {
      draw_node(v);
      point &v_pos=n_info[v].pos;
      v_pos=v_pos.translate(p_trans);
      draw_node(v);
    }
    move_edges(trans);
  }
  while (++i <= anim);
  
  set_flush(old_flush);

}

//----------------------------------------------------------------------------

void GraphWin::move_nodes(node_array<point>& dest, unsigned anim) {
  node v;
  if (node_move == move_single_node)  {
    forall_nodes(v,*gr_p) move_node(v,dest[v],anim);
  }    
  else {				// move_all_nodes 
    if (anim) {
      node_array<vector> p_trans(*gr_p);
      forall_nodes(v,*gr_p) p_trans[v]=(dest[v]-get_position(v))*(1.0/(anim+1));
      unsigned i=0;
      do forall_nodes(v,*gr_p) move_node(v,p_trans[v]);
      while (++i < anim); 
    }
    forall_nodes(v,*gr_p) move_node(v,dest[v]);
  }
  nodes_embedded=true;
  edges_embedded=true;
}

//----------------------------------------------------------------------------

void GraphWin::move_node(node v, point new_p, unsigned anim) {

  point &p=n_info[v].pos;
  if (p == new_p) return;
  unsigned b=0;				// b in {0,1}
  save_embed(v,b);
  if (anim) {
    vector p_trans((new_p-p)*(1.0/(anim+1)));
    unsigned i=0;
    do {
      p=p+p_trans;
      embed_node_with_edges(v);
      draw_node_with_edges(v);
      save_and_restore_embed(v,1-b,b);
      draw_node_with_edges(v);
      b=1-b;
      restore_embed(v,b);
    }
    while (++i < anim); 
  }
  p=new_p;
  embed_node_with_edges(v);
  draw_node_with_edges(v);
  save_and_restore_embed(v,1-b,b);
  draw_node_with_edges(v);
  restore_embed(v,1-b);
}

//----------------------------------------------------------------------------

void GraphWin::move_node(node v, vector trans, unsigned anim) {
  move_node(v,n_info[v].pos+trans,anim);
}

//----------------------------------------------------------------------------
  
