/*******************************************************************************
+
+  LEDA 3.4
+
+  embed.h
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <math.h>
#include <LEDA/point.h>

//----------------------------------------------------------------------------

inline point leaving_point_of_rectangle_edge( double x0, double y0, 
                                              double x1, double y1,
                                              double r1, double r2  ) {

  double m,k1=r1,k2=r2;
          
  if (x0 == x1) {
    k1=0;
    if (y1 < y0) k2=-k2;
  }
  else {
    if (y0 == y1) {
      k2=0;
      if (x1 < x0) k1=-k1;
    }
    else {
      m=(y1-y0)/(x1-x0);
      if (m > 0) {
        if (m > r2/r1) {
          if (y1 < y0) k2=-k2;
          k1=k2/m;
        }
        else {
          if (y1 < y0) k1=-k1;
          k2=k1*m;
        }
      }
      else {	
        if (-m > r2/r1) {
          if (y1 < y0) k2=-k2;
          k1=k2/m;
        }
        else {
          if (y1 > y0) k1=-k1;
          k2=k1*m;
        }
      }
    }
  }

  return point(x0+k1,y0+k2);
}

//----------------------------------------------------------------------------

inline point leaving_point_of_ellipse_edge( double x0, double y0,
                                            double x1, double y1,
                                            double r1, double r2  ) {

  double m,k1=r1,k2=r2,t;
  if (x0 == x1) {
    k1=0;
    if (y1 < y0) k2=-k2;
  }
  else {
    if (y0 == y1) {
      k2=0;
      if (x1 < x0) k1=-k1;
    }
    else {
      m=(y1-y0)/(x1-x0);
      t=r1*m;
      k1=(r1*r2)/sqrt(r2*r2+t*t);
      if (x1 < x0) k1=-k1;
      k2=m*k1;
    }
  }
  return point(x0+k1,y0+k2);
}

//----------------------------------------------------------------------------

inline point leaving_point_of_circle_edge( double x0, double y0,
                                           double x1, double y1, double r) {

  double m,k1=r,k2=r;

  if (x0 == x1) {
    k1=0;
    if (y1 < y0) k2=-k2;
  }
  else {
    if (y0 == y1) {
      k2=0;
      if (x1 < x0) k1=-k1;
    }
    else {
      m=(y1-y0)/(x1-x0);
      k1=r/sqrt(1+m*m);
      if (x1 < x0) k1=-k1;
      k2=m*k1;
    }
  }
  return point(x0+k1,y0+k2);

}

//----------------------------------------------------------------------------

