/*******************************************************************************
+
+  LEDA 3.4
+
+  get_set.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <LEDA/graphwin.h>
#include "local.h"

/****************************************************************************/
/*                     Settings for existing nodes                          */
/****************************************************************************/

color GraphWin::set_label_color(node v, color c) {
  color& clr=n_info[v].label_clr;
  color old_clr=clr;
  if (old_clr == c) return c;
  if (get_flush()) draw_node(v);
  clr=c;
  if (get_flush()) draw_node(v);
  return old_clr;
}

//----------------------------------------------------------------------------

bool GraphWin::set_show_label(node v, bool show) {
  bool& show_label=n_info[v].show_label;
  bool old_show_label=show_label;
  if (old_show_label == show) return show;
  if (get_flush()) draw_node(v);
  show_label=show;
  if (get_flush()) draw_node(v);
  return old_show_label;
}


//----------------------------------------------------------------------------

double GraphWin::set_radius(node v, double r) {
  double &v_r1=n_info[v].r1;            
  double old_r1=v_r1;
  if (old_r1 == r) return r;
  if (get_flush()) draw_node_with_edges(v);
  if (r>0) v_r1=r;
  if (get_flush()) {
    embed_node_with_edges(v);
    draw_node_with_edges(v);
  }
  return old_r1;
}

//----------------------------------------------------------------------------

double GraphWin::set_radius2(node v, double r) {
  double &v_r2=n_info[v].r2;
  double old_r2=v_r2;
  if (old_r2 == r) return r;
  if (get_flush()) draw_node_with_edges(v);
  if (r>0) v_r2=r;
  if (get_flush()) {
    embed_node_with_edges(v);
    draw_node_with_edges(v);
  }
  return old_r2;
}

//----------------------------------------------------------------------------

color GraphWin::set_color(node v, color c) {
  color &v_clr=n_info[v].clr;
  color old_color=v_clr;
  if (old_color == c) return c;
  if (get_flush()) draw_node(v);
  v_clr=c;
  if (get_flush()) draw_node(v);
  return old_color;
}

//----------------------------------------------------------------------------

node_st GraphWin::set_style(node v, node_st st) {
  node_st &v_style=n_info[v].style;
  node_st old_st=v_style;
  if (old_st == st) return st;
  if (get_flush()) draw_node(v);
  v_style=st;
  if (get_flush()) draw_node(v);
  return old_st;
}

//----------------------------------------------------------------------------

point GraphWin::set_position(node v, point p) {
  point &v_pos=n_info[v].pos;
  point old_p=v_pos;
  if (old_p == p) return p;
  if (get_flush()) move_node(v,p,animation);
  else {
    v_pos=p;
    edges_embedded=false;
  }
  return old_p;
}

//----------------------------------------------------------------------------

string GraphWin::set_label(node v, string label) {
  string &v_label=n_info[v].label;
  string old_label=v_label;
  if (old_label == label) return label;
  if (get_flush()) draw_node(v);
  v_label=label;
  if (get_flush()) draw_node(v);
  return old_label;
}

//----------------------------------------------------------------------------

node_t GraphWin::set_type(node v, node_t new_type) {
  node_t &v_type=n_info[v].type;
  node_t old_type=v_type;
  if (old_type == new_type) return new_type;
  if (get_flush()) draw_node_with_edges(v);
  v_type=new_type;
  if (get_flush()) {
    embed_node_with_edges(v);
    draw_node_with_edges(v);
  }
  return old_type;
}

//----------------------------------------------------------------------------
    
position_t GraphWin::set_label_pos(node v, position_t lp) {
  position_t &v_lp=n_info[v].label_pos;
  position_t old_lp=v_lp;
  if (old_lp == lp) return lp;
  if (get_flush()) draw_node_with_edges(v);
  v_lp=lp;
  if (get_flush()) draw_node_with_edges(v);
  return old_lp;
}

//----------------------------------------------------------------------------
    
/****************************************************************************/
/*                     Settings for existing edges                          */
/****************************************************************************/

string GraphWin::set_label(edge e, string label) {
  string &e_label=e_info[e].label;
  string old_label=e_label;
  if (old_label == label) return label;
  if (get_flush()) draw_edge(e);
  e_label=label;
  if (get_flush()) draw_edge(e);
  return old_label;
}

//----------------------------------------------------------------------------

color GraphWin::set_label_color(edge e, color c) {
  color& clr=e_info[e].label_clr;
  color old_clr=clr;
  if (old_clr == c) return c;
  if (get_flush()) draw_edge(e);
  clr=c;
  if (get_flush()) draw_edge(e);
  return old_clr;
}

//----------------------------------------------------------------------------

bool GraphWin::set_show_label(edge e, bool show) {
  bool& show_label=e_info[e].show_label;
  bool old_show_label=show_label;
  if (old_show_label == show) return show;
  if (get_flush()) draw_edge(e);
  show_label=show;
  if (get_flush()) draw_edge(e);
  return old_show_label;
}


//----------------------------------------------------------------------------

color GraphWin::set_color(edge e, color c) {
  color &e_clr=e_info[e].clr;
  color old_color=e_clr;
  if (old_color == c) return c;
  if (get_flush()) draw_edge(e);
  e_clr=c;
  if (get_flush()) draw_edge(e);
  return old_color;
}

//----------------------------------------------------------------------------

int GraphWin::set_width(edge e, int w) {
  int &e_width=e_info[e].width;
  int old_width=e_width;
  if (old_width == w) return w;
  if (get_flush()) draw_edge(e);
  e_width=w;
  if (get_flush()) draw_edge(e);
  return old_width;
}

//----------------------------------------------------------------------------

edge_st GraphWin::set_style(edge e, edge_st st) {
  edge_st &e_style=e_info[e].style;
  edge_st old_style=e_style;
  if (old_style == st) return st;
  if (get_flush()) draw_edge(e);
  e_style=st;
  if (get_flush()) draw_edge(e);
  return old_style;
}

//----------------------------------------------------------------------------

position_t GraphWin::set_label_pos(edge e, position_t lp) {
  position_t &e_lp=e_info[e].label_pos;
  position_t old_lp=e_lp;
  if (old_lp == lp) return lp;
  if (get_flush()) draw_edge(e);
  e_lp=lp;
  if (get_flush()) draw_edge(e);
  return old_lp;
}

//----------------------------------------------------------------------------

/****************************************************************************/
/*                        Settings for new nodes                            */
/****************************************************************************/

bool GraphWin::set_show_node_label(bool show) {
  int old_show_node_label=n_model.show_label;
  n_model.show_label=show;
  return old_show_node_label;
}

//----------------------------------------------------------------------------

color GraphWin::set_node_color(color c) {
  color old_color=n_model.clr;
  n_model.clr=c;
  return old_color;
}

//----------------------------------------------------------------------------

double GraphWin::set_node_radius(double r) {
  double old_r=n_model.r1;
  if (r>0) n_model.r1=r;
  return old_r;
}

//----------------------------------------------------------------------------

double GraphWin::set_node_radius2(double r) {
  double old_r2=n_model.r2;
  if (r>0) n_model.r2=r;
  return old_r2;
}

//----------------------------------------------------------------------------

node_st GraphWin::set_node_style(node_st st) {
  node_st old_style=n_model.style;
  n_model.style=st;
  return old_style;
}

//----------------------------------------------------------------------------

node_t GraphWin::set_node_type(node_t new_type) {
  node_t old_type=n_model.type;
  n_model.type=new_type;
  return old_type;
}

//----------------------------------------------------------------------------

position_t GraphWin::set_node_label_pos(position_t lp) {
  position_t old_lp=n_model.label_pos;
  n_model.label_pos=lp;
  return old_lp;
}

//----------------------------------------------------------------------------

/****************************************************************************/
/*                        Settings for new edges                            */
/****************************************************************************/

bool GraphWin::set_show_edge_label(bool show) {
  int old_show_edge_label=e_model.show_label;
  e_model.show_label=show;
  return old_show_edge_label;
}

//----------------------------------------------------------------------------

color GraphWin::set_edge_color(color c) {
  color old_color=e_model.clr;
  e_model.clr=c;
  return old_color;
}

//----------------------------------------------------------------------------

edge_st GraphWin::set_edge_style(edge_st st) {
  edge_st old_style=e_model.style;
  e_model.style=st;
  return old_style;
}

//----------------------------------------------------------------------------

int GraphWin::set_edge_width(int w) {
  int old_width=e_model.width;
  e_model.width=w;
  return old_width;
}

//----------------------------------------------------------------------------

position_t GraphWin::set_edge_label_pos(position_t lp) {
  position_t old_lp=e_model.label_pos;
  e_model.label_pos=lp;
  return old_lp;
}

//----------------------------------------------------------------------------

/****************************************************************************/
/*                     Settings for list of nodes                           */
/****************************************************************************/

#define NL_SET(f,arg_t)	void GraphWin::f(const list<node>& L, arg_t a) { node v; forall(v,L) f(v,a); }

NL_SET(set_color,color);
NL_SET(set_type,node_t);
NL_SET(set_style,node_st);
NL_SET(set_radius,double);
NL_SET(set_radius2,double);

NL_SET(set_label,string);
NL_SET(set_label_pos,position_t);
NL_SET(set_label_color,color);
NL_SET(set_show_label,bool);

NL_SET(set_position,point);
NL_SET(set_width,int);
NL_SET(set_height,int);

//----------------------------------------------------------------------------

/****************************************************************************/
/*                     Settings for list of edges                           */
/****************************************************************************/

#define EL_SET(f,arg_t)	void GraphWin::f(const list<edge>& L, arg_t a) { edge e; forall(e,L) f(e,a); }

EL_SET(set_color,color);
EL_SET(set_style,edge_st);

EL_SET(set_label,string);
EL_SET(set_label_pos,position_t);
EL_SET(set_label_color,color);
EL_SET(set_show_label,bool);

EL_SET(set_width,int);

//----------------------------------------------------------------------------

/****************************************************************************/
/*                      Settings for arrays of nodes                        */
/****************************************************************************/

void GraphWin::set_position(node_array<point>& dest) {
  if (get_flush()) {
    move_nodes(dest,animation);
  }
  else {
    node v;
    forall_nodes(v,*gr_p) n_info[v].pos=dest[v]; 
    nodes_embedded=true;
    edges_embedded=false;
  }
}  

//----------------------------------------------------------------------------

void GraphWin::set_position( node_array<double>& destx, 
                              node_array<double>& desty  ) {
  node_array<point> dest(*gr_p);
  node v;
  forall_nodes(v,*gr_p) dest[v]=point(destx[v],desty[v]);
  set_position(dest);
}  

//----------------------------------------------------------------------------

/****************************************************************************/
/*                                                                          */
/****************************************************************************/

node GraphWin::get_node() {
  if (gr_p->number_of_nodes() == 0) return nil;
  double x,y;
  node v;
  win_p->message("Determine a node!");
  do {
    if (win_p->read_mouse(x,y)==DEL_BUTTON) {
      v=nil;
      break;
    }
    v=find_node(point(x,y));
  }
  while (v == nil);
  win_p->del_messages();
  return v;
}

//----------------------------------------------------------------------------

edge GraphWin::get_edge() {
  if (gr_p->number_of_edges() == 0) return nil;
  double x,y;
  edge e;
  win_p->message("Determine an edge!");
  do {
    if (win_p->read_mouse(x,y)==DEL_BUTTON) {
      e=nil;
      break;
    }
    e=find_edge(point(x,y));
  }
  while (e == nil);
  win_p->del_messages();
  return e;
}

//----------------------------------------------------------------------------

/****************************************************************************/
/*                          global Settings                                 */
/****************************************************************************/

bool GraphWin::set_directed(bool b) {
  bool old_directed=directed;
  if (old_directed == b) return b;
  if (get_flush()) draw_edges();   
  directed=b;
  if (get_flush()) draw_edges();   
  return old_directed;
}

//----------------------------------------------------------------------------

void GraphWin::make_directed() {
  if (is_directed()) return;
  gr_p->make_directed();
  directed=true;
  if (get_flush()) redraw();
}

//----------------------------------------------------------------------------

void GraphWin::make_undirected() {
  if (!is_directed()) return;
  gr_p->make_undirected();
  directed=false;
  if (get_flush()) redraw();
}

//----------------------------------------------------------------------------

int GraphWin::set_grid_mode(int d) {
  int old_grid=grid;
  if (old_grid == d) return d;
  win_p->set_grid_mode(grid=d);
  return old_grid;
}

//----------------------------------------------------------------------------

bool GraphWin::set_show_coordinates(bool b) {
  bool old_show_crd=show_crd;
  if (old_show_crd == b) return b;
  win_p->set_show_coordinates(show_crd=b);
  return old_show_crd;
}

//----------------------------------------------------------------------------

double GraphWin::set_zoom_factor(double f) {
  double old_zoom_factor=zoom_factor;
  zoom_factor=f;
  return old_zoom_factor;
}

//----------------------------------------------------------------------------

unsigned GraphWin::set_animation(unsigned new_animation) {
  unsigned old_animation=animation;
  animation=new_animation;
  return old_animation;
}

//----------------------------------------------------------------------------

node_move_t GraphWin::set_node_move(node_move_t nm) {
  node_move_t old_node_move=node_move;
  node_move=nm;
  return old_node_move;
}

//----------------------------------------------------------------------------

int GraphWin::set_gen_nodes(int n) {
  if (n <= 0) return gen_nodes;
  int old_gen_nodes=gen_nodes;
  gen_nodes=n;
  return old_gen_nodes;
}

//----------------------------------------------------------------------------

int GraphWin::set_gen_edges(int m) {
  int old_gen_edges=gen_edges;
  gen_edges=m;
  return old_gen_edges;
}

//----------------------------------------------------------------------------

int GraphWin::set_buttons_per_line(int c) {
  int old_buttons_per_line=buttons_per_line;
  win_p->buttons_per_line(c);
  buttons_per_line=c;
  return old_buttons_per_line;
}  

//----------------------------------------------------------------------------

double GraphWin::set_edge_dist(double d) {
  if (d == edge_dist) return d;
  double old_dist=edge_dist;
  edge_dist=d;
  if (get_flush()) {
    embed_edges();
    redraw();
  }
  else edges_embedded=false;
  return old_dist;
}

//----------------------------------------------------------------------------

bool GraphWin::set_flush(bool b) {
  bool old_flush=flush;
  flush=b;
  return old_flush;
}

//----------------------------------------------------------------------------

bool GraphWin::set_constant_width(bool b) {
  bool old_const=constant_width;
  constant_width=b;
  return old_const;
}

//----------------------------------------------------------------------------

color GraphWin::set_hi_edge_color(color c) {
  color old_color=hi_edge_color;
  hi_edge_color=c;
  return old_color;
}

//----------------------------------------------------------------------------

color GraphWin::set_hi_node_color(color c) {
  color old_color=hi_node_color;
  hi_node_color=c;
  return old_color;
}

//----------------------------------------------------------------------------

/****************************************************************************/
/*                           reset - functions                              */
/****************************************************************************/

void GraphWin::reset_node_colors() {
  node v;
  forall_nodes(v,*gr_p) set_color(v,DefNodeColor);
}

//----------------------------------------------------------------------------

void GraphWin::reset_node_styles() {
  node v;
  forall_nodes(v,*gr_p) set_style(v,DefNodeStyle);
}

//----------------------------------------------------------------------------

void GraphWin::reset_node_types() {
  node v;
  forall_nodes(v,*gr_p) set_type(v,DefNodeType);
}

//----------------------------------------------------------------------------

void GraphWin::reset_node_radii() {
  node v;
  forall_nodes(v,*gr_p) {
    set_width(v,DefNodeWidth);
    set_height(v,DefNodeHeight);
  }
}

//----------------------------------------------------------------------------

void GraphWin::reset_edge_colors() {
  edge e;
  forall_edges(e,*gr_p) set_color(e,DefEdgeColor);
}

//----------------------------------------------------------------------------

void GraphWin::reset_edge_styles() {
  edge e;
  forall_edges(e,*gr_p) set_style(e,DefEdgeStyle);
}

//----------------------------------------------------------------------------

void GraphWin::reset_edge_widths() {
  edge e;
  forall_edges(e,*gr_p) set_width(e,DefEdgeWidth);
}

//----------------------------------------------------------------------------

void GraphWin::reset_nodes() {
  reset_node_types();
  reset_node_radii();
  reset_node_colors();
  reset_node_styles();
}

//----------------------------------------------------------------------------

void GraphWin::reset_edges() {
  reset_edge_colors();
  reset_edge_styles();
  reset_edge_widths();
}

//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

