/*******************************************************************************
+
+  LEDA 3.4
+
+  init.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

#include <LEDA/graphwin.h>
#include "local.h"

//----------------------------------------------------------------------------

void GraphWin::init_menu() {

  set_buttons_per_line(buttons_per_line);

  menu *m_setup=new menu;
  m_setup->button("global",mcode(setup_global),gw_settings_diag);
  m_setup->button("new items",mcode(setup_new),gw_settings_diag);
  m_setup->button("animation",mcode(setup_anim),gw_settings_diag);
  m_setup->button("generation",mcode(setup_gen),gw_settings_diag);
  
  menu *m_zoom=new menu;
  m_zoom->button("Zoom Up",     mcode(zoom_up),   gw_zoom_handler);
  m_zoom->button("Zoom Down",   mcode(zoom_down), gw_zoom_handler);
  m_zoom->button("Zoom Area",   mcode(zoom_area), gw_zoom_handler);
  m_zoom->button("Fill Window", mcode(zoom_fill), gw_zoom_handler);

  menu *m_embed_nodes=new menu;
  m_embed_nodes->button("random",   mcode(random_embed), gw_node_embed_handler);
  m_embed_nodes->button("grid",     mcode(grid_embed),   gw_node_embed_handler);
  m_embed_nodes->button("circular", mcode(circle_embed), gw_node_embed_handler);

  menu *m_reset=new menu;
  menu *m_reset_nodes=new menu;
  menu *m_reset_edges=new menu;

  m_reset_nodes->button("Types" , mcode(node_type_reset), gw_reset_handler);
  m_reset_nodes->button("Styles", mcode(node_style_reset), gw_reset_handler);
  m_reset_nodes->button("Colors", mcode(node_color_reset), gw_reset_handler);
  m_reset_nodes->button("Radii",  mcode(node_radii_reset), gw_reset_handler);
  m_reset_edges->button("Styles", mcode(edge_style_reset), gw_reset_handler);
  m_reset_edges->button("Colors", mcode(edge_color_reset), gw_reset_handler);
  m_reset_edges->button("Widths", mcode(edge_width_reset), gw_reset_handler);

  m_reset->button("Redraw" , mcode(0), gw_redraw_diag);
  m_reset->button("Node - ",           *m_reset_nodes);
  m_reset->button("Edge - ",           *m_reset_edges);

  m_reset->button("Nodes", mcode(nodes_reset), gw_reset_handler);
  m_reset->button("Edges", mcode(edges_reset), gw_reset_handler);
  m_reset->button("All", mcode(both_reset), gw_reset_handler);
  m_reset->button("Clear Graph", mcode(clear_graph_reset), gw_reset_handler);
  
  menu *m_gen=new menu;
  m_gen->button("complete graph",mcode(g_complete), gw_generation_handler);
  m_gen->button("complete ugraph",mcode(ug_complete), gw_generation_handler);
  m_gen->button("random graph",mcode(g_random), gw_generation_handler);
  m_gen->button("complete bigraph",mcode(bi_complete), gw_generation_handler);
  m_gen->button("random bigraph",mcode(bi_random), gw_generation_handler);
  m_gen->button("planar graph",mcode(pl_random), gw_generation_handler);
  m_gen->button("triangulated",mcode(tri_planar), gw_generation_handler);
  m_gen->button("grid graph",mcode(g_grid), gw_generation_handler);

  win_p->button("File",     mcode(0), gw_file_handler);
  win_p->button("Reset",               *m_reset);
  win_p->button("Embed",               *m_embed_nodes);
  win_p->button("Generate",            *m_gen);
  win_p->button("Zoom",                *m_zoom);
  win_p->button("Setup",               *m_setup);
  win_p->button("Continue", mcode(0), gw_continue_diag);
  win_p->button("Quit",     mcode(1), gw_continue_diag);
  
}

//----------------------------------------------------------------------------

void GraphWin::init_call_menu() {

#define MAX_CALL_ENTRIES MAX_ITEM_NUM

  sub_menus=new menu* [MAX_CALL_ENTRIES];
  menu_functions=new base_function* [MAX_CALL_ENTRIES];
  sub_menu_counter=1;
  call_entry_counter=0;
}

//----------------------------------------------------------------------------

void GraphWin::init_once() {

  create_flag=cr_none;
  id_vec[gw_id=++max_gw_id]=this;

  menu_functions=NULL;
  call_entry_counter=0;
  
  sub_menus=NULL;

  new_node_handler=NULL;
  new_edge_handler=NULL;
  move_node_handler=NULL;
  delete_node_handler=NULL;
  delete_edge_handler=NULL;

  window_is_mapped=false;

  init_default();

}

//----------------------------------------------------------------------------

void GraphWin::init_default() {

  n_model.clr=DefNodeColor;
  n_model.type=DefNodeType;
  n_model.style=DefNodeStyle;
  n_model.r1=DefNodeRadius;
  n_model.r2=DefNodeRadius2;
  n_model.show_label=DefShowNodeLabel;
  n_model.label=DefNodeLabel;
  n_model.label_clr=DefNodeLabelClr;
  n_model.label_pos=DefNodeLabelPos;
  n_model.marked=DefNodeMarked;

  e_model.clr=DefEdgeColor;
  e_model.style=DefEdgeStyle;
  e_model.width=DefEdgeWidth;
  e_model.show_label=DefShowEdgeLabel;
  e_model.label=DefEdgeLabel;
  e_model.label_clr=DefEdgeLabelClr;
  e_model.label_pos=DefEdgeLabelPos;
  e_model.auto_edge=DefAutoEdge;
  e_model.marked=DefEdgeMarked;

  hi_node_color=DefHiNodeColor;
  hi_edge_color=DefHiEdgeColor;
  edge_dist=DefEdgeDist;
  node_embed=DefNodeEmbed;
  node_move=DefNodeMove;
  flush=DefFlush;
  zoom_factor=DefZoomFactor;
  animation=DefAnimation;
  grid=DefGrid;
  show_crd=DefShowCrd;
  constant_width=DefConstWidth;
  directed=DefDirected;

  x_min=DefXMin;
  x_max=DefXMax;
  y_min=DefYMin;
  
  filename=DefFileName;
  f_ext=DefFileExt;
  ps_ext=DefPSExt;
  gw_ext=DefGWExt;


  units_per_pixel=1.0;
  
  buttons_per_line=DefButtonsPerLine;
 
  gen_nodes=DefGenNodes;
  gen_edges=DefGenEdges;

}

//----------------------------------------------------------------------------

void GraphWin::create(long c_flag) {
  create_flag|=c_flag;
  if (c_flag & cr_graph) gr_p=new graph;
  if (c_flag & cr_win) win_p=new window; 
  win_vec[win_p]=this;
}


//----------------------------------------------------------------------------

void GraphWin::init_matrix() {
  node v,w;
  forall_nodes(v,*gr_p) 
    forall_nodes(w,*gr_p) 
      del_incident_edges(v,w);
      
  edge e;    
  forall_edges(e,*gr_p) 
    new_incident_edge(e);
}

//----------------------------------------------------------------------------

void GraphWin::init_node(node v) {
    node_info &n_inf=n_info[v]; 
    n_inf.id=id_queue.empty()?++max_node_id:id_queue.pop();
    n_inf.clr=n_model.clr;
    n_inf.type=n_model.type;
    n_inf.style=n_model.style;
    n_inf.r1=n_model.r1;
    n_inf.r2=n_model.r2;
    n_inf.show_label=n_model.show_label;
    n_inf.label=n_model.label;
    n_inf.label_clr=n_model.label_clr;
    n_inf.label_pos=n_model.label_pos;
    n_inf.marked=n_model.marked;
}

//----------------------------------------------------------------------------

void GraphWin::init_edge(edge e) {
    edge_info &e_inf=e_info[e];
    e_inf.clr=e_model.clr;
    e_inf.style=e_model.style;
    e_inf.width=e_model.width;
    e_inf.show_label=e_model.show_label;
    e_inf.label=e_model.label;
    e_inf.label_clr=e_model.label_clr;
    e_inf.label_pos=e_model.label_pos;
    e_inf.p.clear();
    e_inf.p.push(get_position(source(e)));
    e_inf.p.push(get_position(target(e))); 
    e_inf.auto_edge=e_model.auto_edge;
    e_inf.marked=e_model.marked;
}

//----------------------------------------------------------------------------

void GraphWin::init_graph() {

  n_info.init(*gr_p);
  e_info.init(*gr_p);

  id_queue.clear();
  max_node_id=0;
  
  node v;
  forall_nodes(v,*gr_p) init_node(v);

  edge e;
  forall_edges(e,*gr_p) init_edge(e);
  
  init_matrix();

  nodes_embedded = (gr_p->number_of_nodes()?false:true);
  edges_embedded = (gr_p->number_of_edges()?false:true);
}

//----------------------------------------------------------------------------

void GraphWin::win_init(double xmin, double xmax, double ymin) {

  if (constant_width) {

    bool old_flush=set_flush(false);

    double scale=(xmax-xmin)/(x_max-x_min);

    node v;

    set_node_radius(get_node_radius()*scale);
    set_node_radius2(get_node_radius2()*scale);

    forall_nodes(v,*gr_p) {
    	set_radius(v,get_radius(v)*scale);  
        set_radius2(v,get_radius2(v)*scale);
    }
    
    embed_edges();
    
    set_flush(old_flush);
  
  }

  win_p->init(xmin, xmax, ymin);
  win_p->set_grid_mode(grid);
 
  units_per_pixel=1.0/win_p->scale();

  update_win_bounds();

  redraw();
}

//----------------------------------------------------------------------------

void GraphWin::init_window() {

  init_menu();

  if (!window_is_mapped) {
    win_p->open();
    window_is_mapped=win_p->is_open();;
  }

  win_p->init(x_min,x_max,y_min,grid);
  win_p->set_redraw(GraphWin::s_redraw);

  win_p->set_line_width(e_model.width);

  win_p->clear();

  win_p->set_mode(DefDrawMode);

  win_p->set_show_coordinates(show_crd);

  units_per_pixel=1.0/win_p->scale();

  update_win_bounds();

}

