/*******************************************************************************
+
+  LEDA 3.4
+
+  misc.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <LEDA/graphwin.h>
#include "local.h"

//----------------------------------------------------------------------------

unsigned long GraphWin::read_mouse_rect(double& x0, double& y0, double& x1, double& y1, bool pressed) {
 
  double x_first,y_first;
  double x_last,y_last;
  double x,y;
  
  unsigned long t0,t1;
  
  int val,event;
  
  if (pressed) {
    x_first=x0;
    y_first=y0;
  }
  else {
    win_p->read_mouse(x_first,y_first);
    t0 = win_p->button_press_time();
  }
  
  x=x_last=x_first;
  y=y_last=y_first;
  
  win_p->draw_rectangle(x_first,y_first,x,y,black);

  do {
    event = win_p->read_event(val,x,y,t1);
    if (event == motion_event) {
      win_p->draw_rectangle(x_first,y_first,x,y,black);
      win_p->draw_rectangle(x_first,y_first,x_last,y_last,black);
      x_last=x;
      y_last=y;
    }
  }
  while(event != button_release_event);
  
  win_p->draw_rectangle(x_first,y_first,x,y,black);

  x1=x;
  y1=y;
  
  if (pressed) return t1;
  
  return t1-t0; 

}

//----------------------------------------------------------------------------

bool GraphWin::get_area(double& x0, double& y0, double& x1, double& y1, string msg) {

  win_p->message(msg);
    
  point p,q;
  
  int but;
    
  but=win_p->read_mouse(x0,y0);

  if (but == DEL_BUTTON) {
    win_p->del_messages();
    return false;
  }

  read_mouse_rect(x0,y0,x1,y1,true);  

  win_p->del_messages();

  return true;
}

//----------------------------------------------------------------------------

list<node> GraphWin::get_area_nodes(double x0, double y0, double x1, double y1) {

  if (x0 > x1) { double tmp=x0; x0=x1; x1=tmp; }
  if (y0 > y1) { double tmp=y0; y0=y1; y1=tmp; }
  
  node v;

  double x,y;
  
  point p;

  list<node> L;

  forall_nodes(v,*gr_p) {
  
    p=get_position(v);
    
    x=p.xcoord();
    y=p.ycoord();
  
    if ((x >= x0) && (x <= x1) && (y >= y0) && (y <= y1)) L.append(v);

  }  

  return L;

}

//----------------------------------------------------------------------------

void GraphWin::highlight(node v) {
  bool old_flush=set_flush(true);
  color old_color=set_color(v,get_hi_node_color());
  set_flush(false);
  set_color(v,old_color);
  set_flush(old_flush);
}

//----------------------------------------------------------------------------

void GraphWin::unhighlight(node v) {
  bool old_flush=set_flush(false);
  color old_color=set_color(v,get_hi_node_color());
  set_flush(true);
  set_color(v,old_color);
  set_flush(old_flush);
}

//----------------------------------------------------------------------------

void GraphWin::highlight(edge e) {
  bool old_flush=set_flush(true);
  color old_color=set_color(e,get_hi_edge_color());
  set_flush(false);
  set_color(e,old_color);
  set_flush(old_flush);
}
  
//----------------------------------------------------------------------------

void GraphWin::unhighlight(edge e) {
  bool old_flush=set_flush(false);
  color old_color=set_color(e,get_hi_edge_color());
  set_flush(true);
  set_color(e,old_color);
  set_flush(old_flush);
}
  
//----------------------------------------------------------------------------

list<point> _gw_cc_[2][500];

point _gw_pp_[2];

//----------------------------------------------------------------------------

void GraphWin::save_embed(node v, unsigned s) {

  unsigned k=0;
  edge e;

  _gw_pp_[s]=n_info[v].pos;
  
  list<point> *c=_gw_cc_[s];

  forall_inout_edges(e,v) c[k++]=e_info[e].p;
}

//----------------------------------------------------------------------------

void GraphWin::restore_embed(node v, unsigned r) {

  unsigned k=0;
  edge e;

  n_info[v].pos=_gw_pp_[r];

  list<point> *c=_gw_cc_[r];

  forall_inout_edges(e,v) e_info[e].p=c[k++];
}

//----------------------------------------------------------------------------

void GraphWin::save_and_restore_embed(node v, unsigned s, unsigned r) {

  unsigned k=0;
  edge e;
  
  point &p=n_info[v].pos;

  _gw_pp_[s]=p;
  p=_gw_pp_[r];
  
  list<point> *c_s=_gw_cc_[s];
  list<point> *c_r=_gw_cc_[r];

  forall_inout_edges(e,v) {
    edge_info &e_inf=e_info[e];
    c_s[k]=e_inf.p;
    e_inf.p=c_r[k++];
  }
}

//----------------------------------------------------------------------------

void GraphWin::acknowledge(string s) {
  win_p->acknowledge(s);
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

