#include <LEDA/graph_alg.h>
#include "graphwin.h"
#include <LEDA/graph_alg.h>

static void dfs(const graph& G, node v, edge_array<bool>& marked,
                                        node_array<int>&  deg)
{ edge e;
  forall_inout_edges(e,v)
    if (marked[e])
    { marked[e] = false;
      node w = G.opposite(v,e);
      if (deg[w] == 2) dfs(G,w,marked,deg);
      else deg[w] = -3;
     }
}


int KURATOWSKI(graph& G, list<node>& V, list<edge>& E, node_array<int>& deg)
{ 
  // deg[v] = 0     not in Kuratowski Subdivison
  //          2     subdivison point
  //          3(-3) sides L(R) of K33
  //          4     nodes of K5

  if (PLANAR(G,E,false)) return 0;

  edge_array<bool> marked(G,false);
  deg.init(G,0);

  edge e;
  forall(e,E) 
  { marked[e] = true;
    if (++deg[source(e)] == 1) V.append(source(e));
    if (++deg[target(e)] == 1) V.append(target(e));
   }

  node v;
  forall(v,V)
    if (deg[v] == 3) 
    { dfs(G,v,marked,deg);
      return 1;
     }

  return 2;
}



void highlight(GraphWin& GW, list<node> V, list<edge> E, node_array<int>& kind) {

  node v;
  edge e;

  forall(v,V) 
    switch (kind[v]) {
        
      case  2: GW.set_color(v,black);
               GW.set_width(v,4);
               break;

      case  4:
      case  3: GW.set_type(v,rectangle_node);
               GW.set_color(v,red);
               break;

      case -3: GW.set_type(v,rectangle_node);
               GW.set_color(v,blue2);
               break;
    }

    const graph& G=GW.get_graph();

    forall_edges(e,G) GW.set_style(e,dashed);
        
    forall(e,E) {
      GW.set_style(e,solid);
      GW.set_color(e,black);
      GW.set_width(e,2);
    }

    GW.redraw();
}


main() {

  graph G;

  GraphWin GW(G);

  GW.set_flush(false);
  GW.display();

  while (GW.edit()) {
    if (PLANAR(G)) { 
        
        list<edge> b_edges = Make_Biconnected(G);
        list<edge> r_edges = Make_Bidirected(G);
        node_array<double> xcoord(G);
        node_array<double> ycoord(G);
        PLANAR(G,true);
        STRAIGHT_LINE_EMBEDDING(G,xcoord,ycoord);
        node v;
        forall_nodes(v,G) {  
           xcoord[v] = 10 + 10*xcoord[v];
           ycoord[v] = 10 + 10*ycoord[v];
         }
        edge e;
        forall(e,b_edges) G.del_edge(e);
        forall(e,r_edges) G.del_edge(e);
        
        GW.set_flush(true);
        GW.set_position(xcoord,ycoord);
        GW.set_flush(false);
      
    }
    else {
        list<node> V;
        list<edge> E;
        node_array<int> kind(G);

        KURATOWSKI(G,V,E,kind);

        highlight(GW,V,E,kind);

        GW.wait("Press continue!");  

        GW.reset();
        GW.redraw();
       }

  }



  return 0;
}
