/*******************************************************************************
+
+  LEDA 3.4
+
+  ps.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

#include <iostream.h>
#include <LEDA/window.h>
#include <LEDA/stream.h>
#include <LEDA/graph.h>
#include <LEDA/color.h>

#include "ps.h"

// ---------------------------------------------------------------------------

#define DefPSBufSize	1000

#define FontSizeName	"fsz"
#define MaxScaleName	"scl"

// ---------------------------------------------------------------------------

char ps_buf[DefPSBufSize];

// ---------------------------------------------------------------------------

void PostScript::add(const char* str) {
  s << str;
}

// ---------------------------------------------------------------------------

void PostScript::add(const string& str) {
  s << str;
}

// ---------------------------------------------------------------------------

void PostScript::add(color c) {
  s << color_name[c];
}

// ---------------------------------------------------------------------------

PostScript& operator << (PostScript& ps, const char* str) {
  ps.add(str);
  return ps;
}

// ---------------------------------------------------------------------------

PostScript& operator << (PostScript& ps, const string& str) {
  ps.add(str);
  return ps;
}

// ---------------------------------------------------------------------------

PostScript& operator << (PostScript& ps, color c) {
  ps.add(c);
  ps << " ";
  return ps;
}

// ---------------------------------------------------------------------------

PostScript& operator << (PostScript& ps, double d) {
  ps << string("%f ",d);
  return ps;
}

// ---------------------------------------------------------------------------

PostScript& operator << (PostScript& ps, int i) {
  ps << string("%d ",i);
  return ps;
}

// ---------------------------------------------------------------------------

char* PostScript::str() { 
  s << ends;
  return s.str(); 
}

// ---------------------------------------------------------------------------

bool PostScript::save(string fname) {
  ofstream f(fname,ios::out);
  if (!f) return false;
  f << *this;
  if (f.fail()) return false;
  f.close();
  return true;
}
  
// ---------------------------------------------------------------------------

bool PostScript::save() { 
  return save(f); 
}

// ---------------------------------------------------------------------------

void PostScript::init_color_names() {
  color_name[black]  = "black";
  color_name[white]  = "white";
  color_name[red]    = "red";
  color_name[green]  = "green";
  color_name[blue]   = "blue";
  color_name[yellow] = "yellow";
  color_name[violet] = "violet";
  color_name[orange] = "orange";
  color_name[cyan]   = "cyan";
  color_name[brown]  = "brown";
  color_name[pink]   = "pink";
  color_name[green2] = "green2";
  color_name[blue2]  = "blue2";
  color_name[grey1]  = "grey1";
  color_name[grey2]  = "grey2";
  color_name[grey3]  = "grey3";
}

// ---------------------------------------------------------------------------

void PostScript::header(ostream& o) {
  o << "%!PS-Adobe-1.0" << endl;
  o << "%%Creator: LEDA-PS" << endl;
  o << "%%EndComments" << endl;
}

// ---------------------------------------------------------------------------

void PostScript::footer(ostream& o) {
  o << "showpage" << endl;
  o << "%%EOF" << endl;
}

// ---------------------------------------------------------------------------

void PostScript::define_colors(ostream& out) {
  out << '/' << color_name[black] << "  { 0     0     0     } def" << endl;
  out << '/' << color_name[white] << "  { 1     1     1     } def" << endl;
  out << '/' << color_name[red] << "    { 1     0     0     } def" << endl;
  out << '/' << color_name[green] << "  { 0     1     0     } def" << endl;
  out << '/' << color_name[blue] << "   { 0     0     1     } def" << endl;
  out << '/' << color_name[yellow] << " { 1     1     0     } def" << endl;
  out << '/' << color_name[violet] << " { 0.933 0.510 0.933 } def" << endl;
  out << '/' << color_name[orange] << " { 1     0.647 0     } def" << endl;
  out << '/' << color_name[cyan] << "   { 0     1     1     } def" << endl;
  out << '/' << color_name[brown] << "  { 1     0.251 0.251 } def" << endl;
  out << '/' << color_name[pink] << "   { 1     0.753 0.796 } def" << endl;
  out << '/' << color_name[green2] << " { 0     0.804 0     } def" << endl;
  out << '/' << color_name[blue2] << "  { 0     0     0.804 } def" << endl;
  out << '/' << color_name[grey1] << "  { 0.25  0.25  0.25  } def" << endl;
  out << '/' << color_name[grey2] << "  { 0.5   0.5   0.5   } def" << endl;
  out << '/' << color_name[grey3] << "  { 0.75  0.75  0.75  } def" << endl;
}

// ---------------------------------------------------------------------------

void PostScript::prolog(ostream& o) {
  define_colors(o);
  o << "/set_color { setrgbcolor } def" << endl;
  o << "/set_line_width { " << MaxScaleName << " div setlinewidth } def" << endl;
  o << "/set_font { dup 3 -1 roll cvn exch selectfont /" << FontSizeName
    << " exch def } def" << endl;
  o << "/draw_segment { set_color 4 2 roll moveto lineto stroke } def" << endl;
  o << "/draw_rectangle { set_color 2 index sub 4 1 roll 2 index sub 4 -1 roll "
    << "rectstroke } def" << endl;
  o << "/draw_filled_rectangle { set_color 2 index sub 4 1 roll 2 index sub "
    << "4 -1 roll rectfill } def" << endl;
  o << "/draw_circle { set_color 0 360 arc stroke } def" << endl;
  o << "/draw_disc { set_color 0 360 arc fill stroke } def" << endl;
  o << "/draw_ellipse { set_color 1 index 4 1 roll 2 copy div dup 7 1 roll "
    << "3 1 roll exch div 1 exch scale mul exch 0 360 arc stroke 1 exch scale "
    << "} def" << endl; 
  o << "/draw_filled_ellipse { set_color 1 index 4 1 roll 2 copy div dup 7 1 "
    << "roll 3 1 roll exch div 1 exch scale mul exch 0 360 arc fill stroke 1 "
    << "exch scale } def" << endl; 
  o << "/draw_circle_node { 6 3 roll 3 copy 9 -3 roll draw_disc black "
    << "draw_circle } def" << endl; 
  o << "/draw_ellipse_node { 7 3 roll 4 copy 11 -3 roll draw_filled_ellipse "
    << "black draw_ellipse } def" << endl;
  o << "/draw_rectangle_node { 7 3 roll 4 copy 11 -3 roll draw_filled_rectangle "
    << "black draw_rectangle } def" << endl;
  o << "/draw_text { set_color matrix currentmatrix exch 4 2 roll moveto "
    << "initmatrix show stroke setmatrix } def" << endl;
  o << "/draw_ctext { set_color matrix currentmatrix 4 2 roll moveto "
    << "initmatrix exch dup stringwidth pop " << FontSizeName
    << " -3 div exch -2 div exch rmoveto show stroke setmatrix } def" << endl;
  o << "/draw_polygon { setcolor 3 1 roll newpath moveto 1 sub { lineto } "
    << "repeat closepath stroke } def" << endl;
  o << "/draw_filled_polygon { setcolor 3 1 roll newpath moveto 1 sub "
    << "{ lineto } repeat closepath fill stroke } def" << endl;
  o << "/draw_polygon_edge { setcolor 3 1 roll moveto 1 sub { lineto } repeat "
    << "stroke } def" << endl;
  o << "/set_scale { 2 copy lt {dup} {1 index} ifelse /" << MaxScaleName
    << " exch def scale currentlinewidth " << MaxScaleName
    << " div setlinewidth } def" << endl;
  o << "%%EndProlog" << endl;
}

// ---------------------------------------------------------------------------

void PostScript::init() {
    init_color_names();
    x_width=21 * PSPixelPerCM;
    y_width=29 * PSPixelPerCM;
}

// ---------------------------------------------------------------------------

void PostScript::set_color(color c) {
  *this << c << " set_color\n";
}

// ---------------------------------------------------------------------------

void PostScript::set_origin(double x, double y) {
  *this << x << y << "translate\n";
}

// ---------------------------------------------------------------------------

void PostScript::scale_x(double f) {
  *this << f << 1 << "scale\n";
}

// ---------------------------------------------------------------------------

void PostScript::scale_y(double f) {
  *this << 1 << f << "scale\n";
}

// ---------------------------------------------------------------------------

void PostScript::scale(double f_x, double f_y) {
  *this << f_x << f_y << "set_scale\n";
}

// ---------------------------------------------------------------------------

void PostScript::set_line_width(int w) {
//  *this << 1 << f << "div setlinewidth\n";
  *this << w << "set_line_width\n";
}

// ---------------------------------------------------------------------------

double PostScript::get_x_width() {
  return x_width;
}

// ---------------------------------------------------------------------------

double PostScript::set_x_width(double new_x_width) {
  double old_x=x_width;
  x_width=new_x_width;
  return old_x;
}

// ---------------------------------------------------------------------------

double PostScript::get_y_width() {
  return y_width;
}

// ---------------------------------------------------------------------------

double PostScript::set_y_width(double new_y_width) {
  double old_y=y_width;
  y_width=new_y_width;
  return old_y;
}

// ---------------------------------------------------------------------------

void PostScript::set_page_size(double new_x_width, double new_y_width) {
  set_x_width(new_x_width);
  set_y_width(new_y_width);
}

// ---------------------------------------------------------------------------

void PostScript::set_variable(string name, int value) {
  *this << "/" << name << " " << value << "def\n";
}

// ---------------------------------------------------------------------------

void PostScript::set_font(string name, int size) {
  *this << "(" << name << ") " << size << "set_font\n";
}   

// ---------------------------------------------------------------------------

void PostScript::draw_text(double x, double y, string s, color c) {
  *this << x << y << "(" << s << ") " << c << "draw_text\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_text(point p, string s, color c) {
  draw_text(p.xcoord(),p.ycoord(),s,c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_ctext(double x, double y,string s, color c) {
  *this << x << y << "(" << s << ") " << c << "draw_ctext\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_ctext(point p, string s, color c) {
  draw_ctext(p.xcoord(),p.ycoord(),s,c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_circle(double x, double y, double r, color c) {
  *this << x << y << r << c << "draw_circle\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_circle(point p, double r, color c) {
  draw_circle(p.xcoord(),p.ycoord(),r,c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_circle(const circle& c, color clr) {
  draw_circle(c.center(),c.radius(),clr);
}                        

// ---------------------------------------------------------------------------

void PostScript::draw_disc(double x, double y, double r, color c) {
  *this << x << y << r << c << "draw_disc\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_disc(point p, double r, color c) {
  draw_disc(p.xcoord(),p.ycoord(),r,c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_disc(const circle& c, color clr) {
  draw_disc(c.center(),c.radius(),clr);
}                        

// ---------------------------------------------------------------------------

void PostScript::draw_segment(double x1, double y1, double x2, double y2, color c) {
  *this << x1 << y1 << x2 << y2 << c << "draw_segment\n";  
}

// ---------------------------------------------------------------------------

void PostScript::draw_segment(point p, point q, color c) {
  draw_segment(p.xcoord(),p.ycoord(),q.xcoord(),q.ycoord(),c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_segment(const segment& s, color c) {
  draw_segment(s.source(),s.target(),c);
}    

// ---------------------------------------------------------------------------

void PostScript::draw_rectangle(double x0, double y0, double x1, double y1, color c) {
  *this << x0  << y0 << x1 << y1 << c << "draw_rectangle\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_filled_rectangle(double x0, double y0, double x1, double y1, color c) {
  *this << x0  << y0 << x1 << y1 << c << "draw_filled_rectangle\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_ellipse(double x, double y, double r1, double r2, color c) {
  *this << x << y << r1 << r2 << c << "draw_ellipse\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_ellipse(point p, double r1, double r2, color c) {
  draw_ellipse(p.xcoord(),p.ycoord(),r1,r2,c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_filled_ellipse(double x, double y, double r1, double r2, color c) {
  *this << x << y << r1 << r2 << c << "draw_filled_ellipse\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_filled_ellipse(point p, double r1, double r2, color c) {
  draw_ellipse(p.xcoord(),p.ycoord(),r1,r2,c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_circle_node(double x, double y, double r, color c) {
  *this << x << y << r << c << "draw_circle_node\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_circle_node(point p, double r, color c) {
  draw_circle_node(p.xcoord(),p.ycoord(),r,c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_circle_node(const circle& c, color clr) {
  draw_circle_node(c.center(),c.radius(),clr);
}                        

// ---------------------------------------------------------------------------

void PostScript::draw_rectangle_node(double x0, double y0, double x1, double y1, color c) {
  *this << x0 << y0 << x1 << y1 << c << "draw_rectangle_node\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_ellipse_node(double x, double y, double r1, double r2, color c) {
  *this << x << y << r1 << r2 << c << "draw_ellipse_node\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_ellipse_node(point p, double r1, double r2, color c) {
  draw_ellipse_node(p.xcoord(),p.ycoord(),r1,r2,c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_polygon(const list<point>& P, color c) {
  point p;
  forall(p,P) *this << p.xcoord() << p.ycoord(); 
  *this << P.size() << c << "draw_polygon\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_polygon(const polygon& P, color c) {
  draw_polygon(P.vertices(),c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_filled_polygon(const list<point>& P, color c) {
  point p;
  forall(p,P) *this << p.xcoord() << p.ycoord(); 
  *this << P.size() << c << "draw_filled_polygon\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_filled_polygon(const polygon& P, color c) {
  draw_filled_polygon(P.vertices(),c);
}

// ---------------------------------------------------------------------------

void PostScript::draw_polygon_edge(const list<point>& P, color c) {
  point p;
  forall(p,P) *this << p.xcoord() << p.ycoord();
  *this << P.size() << c << "draw_polygon_edge\n";
}

// ---------------------------------------------------------------------------

void PostScript::draw_polygon_edge(const polygon& P, color c) {
  draw_polygon(P.vertices(),c);
}

// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------
// ----------------------------  Operators  ----------------------------------
// ---------------------------------------------------------------------------

PostScript& operator << (PostScript& ps, const circle& c) {
  ps.draw_circle(c);
  return ps;
}

// ---------------------------------------------------------------------------

PostScript& operator << (PostScript& ps, const segment& s) {
  ps.draw_segment(s);
  return ps;
}

// ---------------------------------------------------------------------------

ostream& operator << (ostream& o, PostScript& ps) {
  char *ps_str=ps.str();
  ps.header(o);
  ps.prolog(o);
  o << ps_str;
  ps.footer(o);
  delete(ps_str);
  return o;
} 

// ---------------------------------------------------------------------------

