/*******************************************************************************
+
+  LEDA 3.4
+
+  _gdos.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
#include <LEDA/impl/x_window.h>
#include "gbase.h"

// RGB values    wi bl  re  gr  bl  ye  vi  or  cy  br  pi  gr  bl  g1  g2  g3
int _R_[16] = {255, 0,255,  0,  0,224,160,255,  0,192,255,  0,128,212,184, 96};
int _G_[16] = {255, 0,  0,192,  0,224,  0,160,192,112,  0,160,128,212,184, 96};
int _B_[16] = {255, 0,  0,  0,255,  0,208,  0,192, 56,255,128,192,212,184, 96};

#if defined(__EMX__)
#include "dos/emx.c"
#elif defined(__WATCOMC__)
#include "dos/wcc.c"
#else
#include "dos/vga.c"
#endif


// event handling

#include <dos.h>
#include <conio.h>
#include <stdlib.h>

#if defined(__EMX__)
union  REGISTERS { struct  _wordregs x; struct  _byteregs h; };
#else
union  REGISTERS { struct  WORDREGS x; struct  BYTEREGS h; };
#endif

#if defined(__WATCOMC__)
inline void int_86(int i, REGISTERS* p, REGISTERS* q)
{ int386(i,(REGS*)p, (REGS*)q); }

#elif defined(__EMX__)
inline void int_86(int i, REGISTERS* p, REGISTERS* q)
{ _int86(i,(REGS*)p, (REGS*)q); }

#else
inline void int_86(int i, REGISTERS* p, REGISTERS* q)
{ int86(i,(REGS*)p, (REGS*)q); }

#endif



struct event {
  int ev;
  int val;
  int x;
  int y;
  event() { ev = -1; val = x = y = 0; }
};


static event event_buf;
static event last_event;


static int mouse_x;
static int mouse_y;
static int mouse_installed;
static int mouse_two_buttons;

void dos_mouse_init(int DISP_MAX_X, int DISP_MAX_Y)
{ 
  mouse_x = 0;
  mouse_y = 0;
  event_buf.ev = -1;


  union REGISTERS regs;

  //INT 33 - MS MOUSE - RESET DRIVER AND READ STATUS
  //    AX = 0000h
  //Return: AX = status
  //	    0000h hardware/driver not installed
  //	    FFFFh hardware/driver installed
  //	    BX = number of buttons
  //	    FFFFh two buttons
  //	    0000h other than two
  //	    0003h Mouse Systems/Logitech mouse
  regs.x.ax=0x00;
  int_86(0x33,&regs,&regs);
  mouse_installed = (regs.x.ax == 0x0ffff);
  mouse_two_buttons = (regs.x.bx == 0x0ffff);

  //INT 33 - MS MOUSE - DEFINE HORIZONTAL CURSOR RANGE = 0..DISP_MAX_X
  regs.x.ax=0x07;
  regs.x.cx=0;
  regs.x.dx=8*DISP_MAX_X;
  int_86(0x33,&regs,&regs);

  //INT 33 - MS MOUSE - DEFINE VERTICAL CURSOR RANGE = 0..DISP_MAX_Y
  regs.x.ax=0x08;
  regs.x.cx=0;
  regs.x.dx=8*DISP_MAX_Y;
  int_86(0x33,&regs,&regs);

  //INT 33 - MS MOUSE - DEFINE MICKEY/PIXEL RATIO = 1
  regs.x.ax=0x0F;
  regs.x.cx=1;
  regs.x.dx=1;
  int_86(0x33,&regs,&regs);

  //INT 33 - MS MOUSE - SET CURSOR POSITION
  regs.x.cx = 8*(DISP_MAX_X/2);
  regs.x.dx = 8*(DISP_MAX_Y/2);
  regs.x.ax=04;
  int_86(0x33,&regs,&regs);

}


void dos_mouse_exit()
{ union REGISTERS regs;
  regs.h.ah=0x00;
  regs.h.al=0x02;
  int_86(0x10,&regs,&regs);
 }






#define UP  72
#define DO  80
#define LE  75
#define RI  77

static char read_kbd()
{
#if defined(__EMX__)
   return _read_kbd(0,0,0); 
#else
   return (kbhit()) ? getch() : 0; 
#endif
 }


void dos_back_event() 
{ event_buf.ev  = last_event.ev;  
  event_buf.val = last_event.val; 
  event_buf.x = last_event.x; 
  event_buf.y = last_event.y; 
}


static int check_event(int* val, int* x, int* y)
{
  int  e =  no_event;
  int  i;

  union REGISTERS key_regs;
  union REGISTERS mouse_regs;

  if (event_buf.ev != -1)
  { *val = event_buf.val;
    *x = event_buf.x;
    *y = event_buf.y;
    last_event.ev  = event_buf.ev;
    last_event.val = event_buf.val;
    last_event.x = event_buf.x;
    last_event.y = event_buf.y;
    event_buf.ev = -1;
    return last_event.ev;
   }

  mouse_regs.x.cx = 8*mouse_x;
  mouse_regs.x.dx = 8*mouse_y;

  char c;

  if ((c = read_kbd()) > 0)
  { 
    int step = 16;

    if (c==27) 
    { vga_init(0);
      dos_mouse_exit();
      exit(0);
     }

    if (c != 0)   /* ascii char */
       { *val = c;
         e = key_press_event;
        }
    else          /* cursor or function key  */
       { 
         c = getch();

         switch(c) {

         case  LE: mouse_regs.x.cx -= step;
                   e = motion_event;
                   break;

         case  RI: mouse_regs.x.cx += step;
                   e = motion_event;
                   break;

         case  UP: mouse_regs.x.dx -= step;
                   e = motion_event;
                   break;

         case  DO: mouse_regs.x.dx += step;
                   e = motion_event;
                   break;

        }

        /* set cursor position */
        mouse_regs.x.ax=04;
        int_86(0x33,&mouse_regs,&mouse_regs);
      }

   } /* kbhit */

 else

  if (mouse_installed)
  { 
    int but;

    /* check for button press and release events */

//INT 33 - MS MOUSE - RETURN BUTTON PRESS DATA
//	AX = 0005h
//	BX = button
//	    0000h left
//	    0001h right
//	    0002h middle (Mouse Systems/Logitech mouse)
//Return: AX = 1 if button pressed
//	  BX = number of times specified button has been pressed since last call
//	  CX = column at time specified button was last pressed
//	  DX = row at time specified button was last pressed

//INT 33 - MS MOUSE - RETURN BUTTON RELEASE DATA
//	AX = 0006h
//	BX = button
//	    0000h left
//	    0001h right
//	    0002h middle (Mouse Systems/Logitech mouse)
//Return: AX = 1 if button released
//	  BX = # of times specified button has been released since last call
//	  CX = column at time specified button was last released
//	  DX = row at time specified button was last released

    for(but=0; but < 3; but++)
    { 
      mouse_regs.x.ax=5;
      mouse_regs.x.bx=but;
      int_86(0x33,&mouse_regs,&mouse_regs);

      if (mouse_regs.x.bx)
      { e = button_press_event;
        break;
       }

      mouse_regs.x.ax=6;
      mouse_regs.x.bx=but;
      int_86(0x33,&mouse_regs,&mouse_regs);

      if (mouse_regs.x.bx)
      { e = button_release_event;
        break;
       }

     }


    if (but < 3) 
    { 
      if (but == 0) *val = 1;   // left
      if (but == 1) *val = 3;   // right
      if (but == 2) *val = 2;   // middle

      //INT 16 - GET KEYBOARD SHIFT KEY STATES
      //          AH = 0002h
      //  OUTPUT: AL  bit 0    right shift
      //              bit 1    left shift
      //              bit 2    ctrl
      //              bit 3    alt

      key_regs.h.ah=0x02;
      int_86(0x16,&key_regs,&key_regs);

      // ALT key simulates middle button  (two-button mouse)
      if (key_regs.h.al & 8) *val = 2;       // alt
      if (key_regs.h.al & 3) *val |= 256;    // shift
      if (key_regs.h.al & 4) *val |= 512;    // ctrl
     }

   // INT 33 MS-MOUSE GET CURSOR POSITION
   mouse_regs.x.ax=03;
   int_86(0x33,&mouse_regs,&mouse_regs);
  }

  if ((mouse_regs.x.cx/8 != mouse_x || mouse_regs.x.dx/8 != mouse_y)
       && e==no_event) e = motion_event;

  mouse_x = mouse_regs.x.cx/8;
  mouse_y = mouse_regs.x.dx/8;

  if (e != no_event)
  { last_event.ev  = e;
    last_event.val = *val;
    last_event.x = mouse_x;
    last_event.y = mouse_y;
   }

  *x = mouse_x;
  *y = mouse_y;

  return e;
}



int dos_next_event(int* val, int* x, int* y, int block)
{ int e = check_event(val,x,y);
  while (block && e == no_event) e = check_event(val,x,y);
  return e;
 }
  
  



//------------------------------------------------------------------------------
// mouse cursor
//------------------------------------------------------------------------------

static unsigned char c_map[4][28] = {

{0xc0,0x00, 0xf0,0x00, 0x7c,0x00, 0x7f,0x00, 0x3f,0xc0, 0x3f,0xc0, 0x1f,0x00, 
 0x1f,0x80, 0x0d,0xc0, 0x0c,0xe0, 0x00,0x70, 0x00,0x38, 0x00,0x1c, 0x00,0x0c},

{0x00,0x03, 0x00,0x0f, 0x00,0x3e, 0x00,0xfe, 0x03,0xfc, 0x03,0xfc, 0x00,0xf8,
 0x01,0xf8, 0x03,0xb0, 0x07,0x30, 0x0e,0x00, 0x1c,0x00, 0x38,0x00, 0x30,0x00},

{0x00,0x0c, 0x00,0x1c, 0x00,0x38, 0x00,0x70, 0x0c,0xe0, 0x0d,0xc0, 0x1f,0x80, 
 0x1f,0x00, 0x3f,0xc0, 0x3f,0xc0, 0x7f,0x00, 0x7c,0x00, 0xf0,0x00, 0xc0,0x00}, 

{0x30,0x00, 0x38,0x00, 0x1c,0x00, 0x0e,0x00, 0x07,0x30, 0x03,0xb0, 0x01,0xf8, 
 0x00,0xf8, 0x03,0xfc, 0x03,0xfc, 0x00,0xfe, 0x00,0x3e, 0x00,0x0f, 0x00,0x03},

};


void dos_draw_pointer(int x, int y)
{ int k = 0;
  if (x+16 > vga_width())  { x -= 15; k += 1; }
  if (y+14 > vga_height()) { y -= 13; k += 2; }
  vga_setmode(3);
  vga_setcolor(black);
  vga_bitmap(x,y,c_map[k],2,14);
}

