/*******************************************************************************
+
+  LEDA 3.4
+
+  _gwin.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

#include <LEDA/impl/x_window.h>
#include "gbase.h"

#include <windows.h>
#include <stdlib.h>

extern int LEDA_WINMAIN();

struct event {
int kind;
int val;
int x,y;
int t;
};

static event cur_event;
static event last_event;
static int   putback=0;

static HWND hwnd = NULL;
static HDC  hdc;
static PAINTSTRUCT ps;
static COLORREF color_table[16];
static COLORREF COLOR;
static int MODE;

static LOGPEN pen_data = { PS_SOLID,1,1,0 };

// RGB values    wi bl  re  gr  bl  ye  vi  or  cy  br  pi  gr  bl  g1  g2  g3
char _R_[16] = {255, 0,255,  0,  0,255,192,255,  0,255,255,  0, 64,192,128, 64};
char _G_[16] = {255, 0,  0,255,  0,255,  0,128,255,128,  0,192,  0,192,128, 64};
char _B_[16] = {255, 0,  0,  0,255,  0,192,  0,255, 64,255, 64,192,192,128, 64};

static int rop2_mode[] = { R2_COPYPEN, R2_MASKPEN, R2_MERGEPEN, R2_NOTXORPEN };

static unsigned long dw_mode[] = { SRCAND, SRCPAINT, SRCPAINT, SRCINVERT };

int  vga_width() 
{ RECT rect;
  GetClientRect(hwnd,&rect);
  return (rect.right < 640) ? 640 : rect.right; 
}

int  vga_height() 
{ RECT rect;
  GetClientRect(hwnd,&rect);
  return (rect.bottom < 480) ? 480 : rect.bottom; 
}

int  vga_depth() { return 4; }


void vga_clear(int col) { }

void vga_setpal(int col, int r, int g, int b) 
{ color_table[col] = RGB(r,g,b); }

void vga_getpal(int col, int* r, int* g, int* b) 
{  *r = GetRValue(color_table[col]);
   *g = GetGValue(color_table[col]);
   *b = GetBValue(color_table[col]);
 }

void vga_setcolor(int col) 
{ COLOR = color_table[col]; 
  pen_data.lopnColor = COLOR;
  DeleteObject(SelectObject(hdc,CreatePenIndirect(&pen_data)));
}

void vga_setmode(int m) 
{ MODE = m;
  SetROP2(hdc,rop2_mode[m]); 
 }

void vga_pixel(int x, int y) { SetPixel(hdc, x, y, COLOR); }

void vga_hline(int x0, int x1, int y)
{ MoveToEx(hdc,x0,y,NULL);
  LineTo(hdc,x1+1,y);
 }

void vga_vline(int x, int y0, int y1)
{ MoveToEx(hdc,x,y0,NULL);
  LineTo(hdc,x,y1);
 }

void vga_box(int x0, int y0, int x1, int y1)
{ while (y0 <= y1) vga_hline(x0,x1,y0++); }


/*
void vga_box(int x0, int y0, int x1, int y1)
{ HBRUSH hBrush = CreateSolidBrush(COLOR);
  RECT r;
  r.left = x0;
  r.right = x1 + 1;
  r.top = y0;
  r.bottom = y1 + 1;
  FillRect(hdc,&r,hBrush);
  DeleteObject (hBrush);
 }

void vga_hline(int x0, int x1, int y) { vga_box(x0,y,x1,y); }

void vga_bitmap(int x, int y, unsigned char* lp, int width,int height) 
{ 
  if (MODE == 0) // invert
  { unsigned char* stop = lp + width*height;
    for(unsigned char* p = lp; p< stop; p++) *p = ~*p;
   }

  width *= 8;

  HDC     hdcMem = CreateCompatibleDC(hdc);
  HBRUSH  hBrush = CreateSolidBrush(COLOR);
  SelectObject(hdcMem,hbm);
  HBRUSH save_brush = SelectObject(hdcMem,hBrush);
  BitBlt(hdc,x,y,width,height,hdcMem,0,0,dw_mode[MODE]);
  SelectObject(hdcMem,save_brush);
  DeleteDC(hdcMem);
  DeleteObject(hbm);
  DeleteObject(hBrush);

}
*/


static void vga_bytes(int x, int y, unsigned char* ptr, int len)
{ unsigned char* stop = ptr+len;
  while (ptr < stop)
  { int d = 0;
    for (d=0; ptr <stop && *ptr==0xFF; ptr++) d += 8;
    if (d > 0)
      { MoveToEx(hdc,x,y,NULL);
        LineTo(hdc,x+d,y);
        x += d;
       }
    else
      { unsigned char c = *ptr++;
        if (c & 128) SetPixel(hdc,x,  y,COLOR);
        if (c &  64) SetPixel(hdc,x+1,y,COLOR);
        if (c &  32) SetPixel(hdc,x+2,y,COLOR);
        if (c &  16) SetPixel(hdc,x+3,y,COLOR);
        if (c &   8) SetPixel(hdc,x+4,y,COLOR);
        if (c &   4) SetPixel(hdc,x+5,y,COLOR);
        if (c &   2) SetPixel(hdc,x+6,y,COLOR);
        if (c &   1) SetPixel(hdc,x+7,y,COLOR);
        x += 8;
       }
   }
}


void vga_bitmap(int x, int y, unsigned char* pm, int width, int height)
{ while (height--)
  { vga_bytes(x,y++,pm,width);
    pm += width;
   }
 }

struct Image {
 short w;
 short h;
 HBITMAP map;
};


char* vga_getimage(int x1, int y1, int x2, int y2) 
{ HDC hdcMem  = CreateCompatibleDC (hdc) ;
  Image* im = new Image;
  im->w = x2-x1+1;
  im->h = y2-y1+1;
  im->map = CreateCompatibleBitmap (hdc,im->w,im->h) ;
  SelectObject (hdcMem,im->map);
  BitBlt (hdcMem,0,0,im->w,im->h,hdc,x1,y1,SRCCOPY) ;
  DeleteDC (hdcMem) ;
  return (char*)im;
}

void  vga_putimage(int x, int y,  char* image) 
{ Image* im = (Image*)image;
  HDC hdcMem  = CreateCompatibleDC (hdc) ;
  SelectObject (hdcMem,im->map);
  BitBlt (hdc,x,y,im->w,im->h,hdcMem,0,0,SRCCOPY) ;
  DeleteDC (hdcMem) ;
}

void  vga_delimage(char* image) 
{ Image* im = (Image*)image;
  DeleteObject(im->map);
 }

void  vga_copyimage(int x1,int y1,int x2,int y2,int x,int y) 
{ HDC hdcMem  = CreateCompatibleDC (hdc) ;
  Image* im = new Image;
  int w = x2-x1+1;
  int h = y2-y1+1;
  HBITMAP map = CreateCompatibleBitmap (hdc,w,h) ;
  SelectObject (hdcMem,map);
  BitBlt (hdcMem,0,0,w,h,hdc,x1,y1,SRCCOPY) ;
  BitBlt (hdc,x,y,w,h,hdcMem,0,0,SRCCOPY) ;
  DeleteDC (hdcMem) ;
}


long FAR PASCAL _export WndProc (HWND hwnd, UINT message, 
                         UINT wParam, LONG lParam)
{
  switch (message) { 

      case WM_PAINT :
           { PAINTSTRUCT ps;
             HDC save = hdc;
             hdc = BeginPaint (hwnd, &ps);
             //redraw();
             EndPaint (hdc, &ps);
             hdc = save;
             return 0;
            }

        case WM_MOUSEMOVE:
             cur_event.kind = motion_event;
             cur_event.x = LOWORD (lParam);
             cur_event.y = HIWORD (lParam);
             cur_event.t = 0;
             return 0;

        case WM_LBUTTONDOWN :
        case WM_LBUTTONDBLCLK :
             cur_event.kind = button_press_event;
             cur_event.x = LOWORD (lParam);
             cur_event.y = HIWORD (lParam);
             cur_event.t = 0;
             cur_event.val = 1;
             if (wParam & MK_CONTROL) cur_event.val = 2;
             if (wParam & MK_SHIFT) cur_event.val |= 256;
             return 0;

        case WM_MBUTTONDOWN :
        case WM_MBUTTONDBLCLK :
             cur_event.kind = button_press_event;
             cur_event.x = LOWORD (lParam);
             cur_event.y = HIWORD (lParam);
             cur_event.val = 2;
             cur_event.t = 0;
             if (wParam & MK_SHIFT) cur_event.val |= 256;
             return 0;


        case WM_RBUTTONDOWN :
        case WM_RBUTTONDBLCLK :
             cur_event.kind = button_press_event;
             cur_event.x = LOWORD (lParam);
             cur_event.y = HIWORD (lParam);
             cur_event.val = 3;
             cur_event.t = 0;
             if (wParam & MK_CONTROL) cur_event.val = 2;
             if (wParam & MK_SHIFT) cur_event.val |= 256;
             return 0;

        case WM_LBUTTONUP :
             cur_event.kind = button_release_event;
             cur_event.x = LOWORD (lParam);
             cur_event.y = HIWORD (lParam);
             cur_event.val = 1;
             cur_event.t = 0;
             //if (wParam & MK_SHIFT) cur_event.val |= 256;
             if (wParam & MK_SHIFT) cur_event.val = 2;
             return 0;

        case WM_MBUTTONUP :
             cur_event.kind = button_release_event;
             cur_event.x = LOWORD (lParam);
             cur_event.y = HIWORD (lParam);
             cur_event.val = 2;
             cur_event.t = 0;
             if (wParam & MK_SHIFT) cur_event.val |= 256;
             return 0;

        case WM_RBUTTONUP :
             cur_event.kind = button_release_event;
             cur_event.x = LOWORD (lParam);
             cur_event.y = HIWORD (lParam);
             cur_event.val = 3;
             cur_event.t = 0;
             if (wParam & MK_SHIFT) cur_event.val |= 256;
             return 0;

        case WM_CHAR:
             cur_event.kind = key_press_event;
             cur_event.val = (char)wParam;
             cur_event.t = 0;
             if (cur_event.val == 27) exit(0);
             return 0;

        case WM_QUIT:
             cur_event.kind = destroy_event;
             break;

        case WM_DESTROY :
             PostQuitMessage (0);
             return 0;

      }

   return DefWindowProc (hwnd, message, wParam, lParam);
}


void vga_init(int mode) {}


int PASCAL WinMain (HANDLE this_inst, HANDLE prev_inst, LPSTR params, int flag)
{ 
     static char szAppName[] = "LEDA MS-Windows Application";
     WNDCLASS    wndclass;

     if (!prev_inst)
     { wndclass.style         = CS_HREDRAW | CS_VREDRAW;
       wndclass.lpfnWndProc   = (WNDPROC)WndProc;
       wndclass.cbClsExtra    = 0;
       wndclass.cbWndExtra    = 0;
       wndclass.hInstance     = this_inst;
       wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
       wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
       wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
       wndclass.lpszMenuName  = NULL;
       wndclass.lpszClassName = szAppName;
       RegisterClass (&wndclass);
      }

     short  cxSize  = GetSystemMetrics (SM_CXSCREEN);
     short  cySize  = GetSystemMetrics (SM_CYSCREEN);

     hwnd = CreateWindow (szAppName,         // Fensterklassenname
                    "LEDA Windows Application", // Titelleiste
                    WS_OVERLAPPEDWINDOW,     // Fensterstil
                    CW_USEDEFAULT,           // Fensterposition, x
                    CW_USEDEFAULT,           // Fensterposition, y 
                    cxSize, //CW_USEDEFAULT,    // horizontale Gre
                    cySize, //CW_USEDEFAULT,    // vertikale Gre
                    NULL,                    // Handle Parent-Window 
                    NULL,                    // Handle Men
                    this_inst,               // Handle Programmkopie 
                    NULL);                   // Spezialparameter

     ShowWindow (hwnd, flag);

     hdc = BeginPaint (hwnd, &ps);

     for (int i=0; i<16; i++)
        color_table[i] = RGB(_R_[i],_G_[i],_B_[i]);

     COLOR = color_table[1];
     MODE = 0;

     pen_data.lopnColor = COLOR;
     SelectObject(hdc,CreatePenIndirect(&pen_data));

     return LEDA_WINMAIN();
}


// events


void dos_mouse_init(int disp_max_x, int disp_max_y) {}

void dos_mouse_exit() {}

int  dos_next_event(int* val, int* x, int* y, int block) 
{ MSG msg;
  if (putback) 
   { cur_event = last_event;
     putback = 0;
    }
  else
  { cur_event.kind = no_event;
    if (block)
      if (GetMessage (&msg, NULL, 0, 0))
      { TranslateMessage (&msg);
        DispatchMessage (&msg);
       }
      else exit(0);
    else
      if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
      { if (msg.message == WM_QUIT) exit(0);
        TranslateMessage (&msg);
        DispatchMessage (&msg);
       }
    last_event = cur_event;
   }
  *val = cur_event.val;
  *x   = cur_event.x;
  *y   = cur_event.y;
  return cur_event.kind;
}

void dos_back_event() {  putback = 1; }

void dos_draw_pointer(int,int) {}

