/*******************************************************************************
+
+  LEDA 3.4
+
+  _mswin.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
//-----------------------------------------------------------------------------
// Basic Graphics for MsWindows (nt, win95, win32s)
//
// (c) LEDA Sofware GmbH 1996
//-----------------------------------------------------------------------------


#include <LEDA/system.h>
#include <LEDA/impl/x_basic.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#if defined(__GNUC__)
#include <wingnuc.h>
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>



#define MAX_WIN 32
#define FONT_SIZE 15
#define FONT_TYPE (FF_ROMAN | FIXED_PITCH)


typedef int Window;

struct win_struct 
{
  HWND         hwnd;
  LOGPEN       pen_data;
  HFONT        hfont;

  int          width;
  int          height;
  int          bg_col;
  int          COLOR;
  int          LWIDTH;
  line_style   LSTYLE;
  text_mode    TMODE;
  drawing_mode MODE;

  int          save_co;
  int          save_lw;
  line_style   save_ls;
  text_mode    save_tm;
  drawing_mode save_mo;

  char*        header;
  void*        inf;
  
};




struct event {
  int win;
  int kind;
  int val;
  int x;
  int y;
  unsigned long t;
};


// global data

static event cur_event;
static event last_event;
static int putback;

static int wm_paint_count;

static win_struct wlist[MAX_WIN+1];
static int wcount = 0;

static COLORREF rgb_table[16];
static HFONT    std_font;
static LOGFONT  lf;

static char szAppName1[] = "LEDA Windows Application";
static char szAppName2[] = "LEDA Subwindow";


static HANDLE this_instance;


void x_open_display(void)  
{ 
  if (std_font) return;

  WinMain(0,0,0,0);

  lf.lfHeight = FONT_SIZE;
  lf.lfPitchAndFamily = FONT_TYPE;
  std_font = CreateFontIndirect(&lf);

  // initialize rgb table

  rgb_table[white]  = RGB(255,255,255);
  rgb_table[black]  = RGB(  0,  0,  0);
  rgb_table[red]    = RGB(255,  0,  0);
  rgb_table[green]  = RGB(  0,255,  0);
  rgb_table[blue]   = RGB(  0,  0,128);
  rgb_table[yellow] = RGB(255,255,  0);
  rgb_table[violet] = RGB(128,  0,255);
  rgb_table[orange] = RGB(255,128,  0);
  rgb_table[cyan]   = RGB(  0,255,255);
  rgb_table[brown]  = RGB(128,  0,  0);
  rgb_table[pink]   = RGB(255,  0,255);
  rgb_table[green2] = RGB(  0,128,128);
  rgb_table[blue2]  = RGB(  0,  0,255);
  rgb_table[grey1]  = RGB(216,216,216);
  rgb_table[grey2]  = RGB(184,184,184);
  rgb_table[grey3]  = RGB(128,128,128);

 }

void x_close_display(void) {}

void x_flush_display(void) {}



int x_display_width(void)  { return GetSystemMetrics(SM_CXSCREEN); }

int x_display_height(void) { return GetSystemMetrics(SM_CYSCREEN); }

int x_display_depth(void)
{ HDC hdc = CreateIC("DISPLAY",NULL,NULL,NULL); 
  int c = GetDeviceCaps(hdc,NUMCOLORS);
  DeleteDC(hdc);
  return (c == 2) ? 1 : 2;
}

int x_display_bits_saved(void) { return 1; }

void x_start_batch(Window) {}
void x_end_batch(Window)   {}
   


/* windows */

static long WINAPI WndProc (HWND hwnd, UINT message, UINT wParam, LONG lParam)
{
  int w = 1;
  while(w <= wcount && wlist[w].hwnd != hwnd) w++;
  cur_event.win = w;

  switch (message) { 

      case WM_PAINT :
         { 
           RECT rect;
           GetUpdateRect(hwnd,&rect,TRUE);
           cur_event.kind = exposure_event;
           cur_event.x =   rect.left; 
           cur_event.y =   rect.top;
           cur_event.val = rect.right - rect.left + 1;
           cur_event.t =   (unsigned long)(rect.bottom - rect.top + 1);
           ValidateRect(hwnd,&rect);
           wm_paint_count++;
           return 0;
          }

      case WM_MOUSEMOVE:
           cur_event.kind = motion_event;
           cur_event.x = LOWORD (lParam);
           cur_event.y = HIWORD (lParam);
           cur_event.t = 0;
           return 0;

      case WM_LBUTTONDOWN :
      case WM_LBUTTONDBLCLK :
           cur_event.kind = button_press_event;
           cur_event.x = LOWORD (lParam);
           cur_event.y = HIWORD (lParam);
           cur_event.t = 0;
           cur_event.val = 1;
           if (wParam & MK_CONTROL) cur_event.val = 2;
           if (wParam & MK_SHIFT) cur_event.val |= 256;
           return 0;

      case WM_MBUTTONDOWN :
      case WM_MBUTTONDBLCLK :
           cur_event.kind = button_press_event;
           cur_event.x = LOWORD (lParam);
           cur_event.y = HIWORD (lParam);
           cur_event.val = 2;
           cur_event.t = 0;
           if (wParam & MK_SHIFT) cur_event.val |= 256;
           return 0;


      case WM_RBUTTONDOWN :
      case WM_RBUTTONDBLCLK :
           cur_event.kind = button_press_event;
           cur_event.x = LOWORD (lParam);
           cur_event.y = HIWORD (lParam);
           cur_event.val = 3;
           cur_event.t = 0;
           if (wParam & MK_CONTROL) cur_event.val = 2;
           if (wParam & MK_SHIFT) cur_event.val |= 256;
           return 0;

      case WM_LBUTTONUP :
           cur_event.kind = button_release_event;
           cur_event.x = LOWORD (lParam);
           cur_event.y = HIWORD (lParam);
           cur_event.val = 1;
           cur_event.t = 0;
           if (wParam & MK_SHIFT) cur_event.val |= 256;
           return 0;

      case WM_MBUTTONUP :
           cur_event.kind = button_release_event;
           cur_event.x = LOWORD (lParam);
           cur_event.y = HIWORD (lParam);
           cur_event.val = 2;
           cur_event.t = 0;
           if (wParam & MK_SHIFT) cur_event.val |= 256;
           return 0;

      case WM_RBUTTONUP :
           cur_event.kind = button_release_event;
           cur_event.x = LOWORD (lParam);
           cur_event.y = HIWORD (lParam);
           cur_event.val = 3;
           cur_event.t = 0;
           if (wParam & MK_SHIFT) cur_event.val |= 256;
           return 0;

      case WM_CHAR:
           cur_event.kind = key_press_event;
           cur_event.val = (char)wParam;
           cur_event.t = 0;
           if (cur_event.val == 27) exit(0);
           return 0;

      case WM_QUIT:
           cur_event.kind = destroy_event;
           break;

      case WM_DESTROY :
           PostQuitMessage (0);
           return 0;

     }

  return DefWindowProc (hwnd, message, wParam, lParam);
}




Window x_create_window(void* inf, int width,int height,int bg_col, 
                       const char* header, const char* label, 
                       int pwin, void (*func)(void*))
{
   if (wcount++ >= MAX_WIN) return 0; 

   win_struct* wp = &wlist[wcount];

   wp->hwnd = NULL;

   wp->pen_data.lopnStyle   = PS_SOLID;
   wp->pen_data.lopnWidth.x = 1;
   wp->pen_data.lopnWidth.y = 1;
   wp->pen_data.lopnColor   = 0;


   lf.lfHeight = FONT_SIZE;
   lf.lfPitchAndFamily = FONT_TYPE;
   wp->hfont = CreateFontIndirect(&lf);

   wp->width  = width;
   wp->height = height;
   wp->bg_col = bg_col;
   wp->COLOR  = black;
   wp->LSTYLE = solid;
   wp->LWIDTH = 1;
   wp->MODE   = src_mode;
   wp->TMODE  = transparent;

   wp->save_co = black;
   wp->save_ls = solid;
   wp->save_lw = 1;
   wp->save_mo = src_mode;
   wp->save_tm = transparent;

   wp->inf    = inf;

   wp->header = new char[strlen(header) + 1];
   strcpy(wp->header,header);

   return wcount;
}


void x_set_clip_rect(int win, int x0, int y0, int w, int h) 
{ HWND hwnd = wlist[win].hwnd;
  if (hwnd)
  { HDC hdc = GetDC(hwnd);
    HRGN hrgn = CreateRectRgn(x0,y0,x0+w,y0+h);
    SelectClipRgn(hdc,hrgn);
    DeleteObject(hrgn);
    ReleaseDC(hwnd,hdc);
   }
 }


int  x_window_opened(int win) 
{ HWND hwnd = wlist[win].hwnd;
  return hwnd && IsWindowVisible(hwnd); 
 }


void x_open_window(Window win, int x, int y, int width,int height,int pwin) 
{
  HWND hwnd = wlist[win].hwnd;
  HWND hwndparent = NULL;

  DWORD win_style = WS_OVERLAPPEDWINDOW;

  char* class_name = szAppName1;


  if (pwin == 0)
    { width  += 2*GetSystemMetrics(SM_CXFRAME);
      height += 2*GetSystemMetrics(SM_CYFRAME);
      height += GetSystemMetrics(SM_CYCAPTION);
     }
  else
    { width  += 2*GetSystemMetrics(SM_CXBORDER);
      height += 2*GetSystemMetrics(SM_CYBORDER);
      hwndparent = wlist[pwin].hwnd;
      win_style = WS_POPUPWINDOW;
      RECT r;
      GetWindowRect(hwndparent,&r);
      x += r.left;
      x += GetSystemMetrics(SM_CXFRAME);
      y += r.top; 
      y += GetSystemMetrics(SM_CYCAPTION);
      y += GetSystemMetrics(SM_CYFRAME);
      class_name = szAppName2;
     }


  // restrict window size to display size

  int disp_width  = x_display_width();
  int disp_height = x_display_height();
  int dx = disp_width - width;
  int dy = disp_height - height;

  if (dx < 0) width = disp_width;
  if (dy < 0) height = disp_height;

  if ( x < 0 ) x = 0;
  if ( y < 0 ) y = 0;

  dx = disp_width - width;
  dy = disp_height - height;

  if ( x > dx ) x = dx;
  if ( y > dy ) y = dy;


  if (!hwnd)
  { 
    win_style |= WS_VISIBLE;

    hwnd = CreateWindow (class_name,           // window class name
                         wlist[win].header,    // frame label
                         win_style,            // window style
                         x,y,                  // window position
                         width,height,         // window size
                         hwndparent,           // handle parent window 
                         NULL,                 // handle menu
                         this_instance,        // handle program copy
                         NULL);                // special parameter
  
    wlist[win].hwnd = hwnd;
   }
  else
   { MoveWindow(hwnd,x,y,width,height,FALSE);
     ShowWindow (hwnd, SW_SHOWNORMAL);
    }


  // wait for first WM_PAINT message

  wm_paint_count = 0;

  MSG msg;
  while (wm_paint_count == 0)
  { GetMessage (&msg, NULL, 0, 0);
    TranslateMessage (&msg);
    DispatchMessage (&msg);
   }


  // empty message queue

  while (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
  { TranslateMessage (&msg);
    DispatchMessage (&msg);
   }

  x_set_color(win,wlist[win].COLOR);
  x_set_mode(win,wlist[win].MODE);
  x_set_text_mode(win,wlist[win].TMODE);
  x_set_line_width(win,wlist[win].LWIDTH);
  x_set_line_style(win,wlist[win].LSTYLE);

  HDC hdc = GetDC(hwnd);
  SelectObject(hdc, wlist[win].hfont);
  ReleaseDC(hwnd,hdc);
}


void x_close_window(Window win) 
{ HWND hwnd = wlist[win].hwnd;
  if (hwnd) ShowWindow(hwnd,SW_HIDE);
 }


void x_destroy_window(Window win) 
{ HWND hwnd = wlist[win].hwnd;
  if (hwnd) 
  { DestroyWindow(hwnd);
    DeleteObject(wlist[win].hfont);
   }
  delete[] wlist[win].header;
 }


void x_set_header(Window win, const char *s)
{ HWND hwnd = wlist[win].hwnd;
  SetWindowText(hwnd,s);
 }


void* x_window_inf(Window win) { return wlist[win].inf; }

int x_window_width(Window win)
{ 
  HWND hwnd = wlist[win].hwnd;
  if (hwnd)
     { RECT r;
       GetClientRect(wlist[win].hwnd,&r);
       return r.right; 
      }
   else
     return wlist[win].width;
 }

int x_window_height(Window win)
{ 
  HWND hwnd = wlist[win].hwnd;
  if (hwnd)
     { RECT r;
       GetClientRect(wlist[win].hwnd,&r);
       return r.bottom; 
      }
   else
     return wlist[win].height;
 }


void x_window_position(Window win, int* x, int* y)
{ HWND hwnd = wlist[win].hwnd;
  if (hwnd)
  { RECT r;
    GetWindowRect(hwnd,&r);
    *x = r.left;
    *y = r.right;
   }
  else
    *x = *y = 0;
 }

void x_clear_window(Window) {}



//------------------------------------------------------------------------------
// drawing
//------------------------------------------------------------------------------

void x_line(Window win, int x1, int y1, int x2, int y2)
{ HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  MoveToEx(hdc,x1,y1,NULL);
  LineTo(hdc,x2,y2);
  SetPixel(hdc,x2,y2,wlist[win].pen_data.lopnColor);
  ReleaseDC(hwnd,hdc);
 }



/*
void x_rect(Window win, int x1, int y1, int x2, int y2)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  HBRUSH save_brush = SelectObject(hdc,GetStockObject(NULL_BRUSH));
  Rectangle(hdc,x1,y1,x2,y2);
  SelectObject(hdc,save_brush);
  ReleaseDC(hwnd,hdc);
 }
*/


void x_rect(Window win, int x1, int y1, int x2, int y2)
{ HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  POINT  p[5];
  p[0].x = x1; p[0].y = y1;
  p[1].x = x1; p[1].y = y2;
  p[2].x = x2; p[2].y = y2;
  p[3].x = x2; p[3].y = y1;
  p[4].x = x1; p[4].y = y1;
  Polyline(hdc,p,5);
  ReleaseDC(hwnd,hdc);
}


/*
void x_box(Window win, int x1, int y1, int x2, int y2)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  COLORREF col = wlist[win].pen_data.lopnColor;
  HBRUSH hBrush = CreateSolidBrush(col);
  if (x1 <= x2) x2++; else x1++;
  if (y1 <= y2) y2++; else y1++;
  RECT r;
  SetRect(&r,x1,y1,x2,y2);
  FillRect(hdc,&r,hBrush);
  DeleteObject(hBrush);
  ReleaseDC(hwnd,hdc);
}
*/

void x_box(Window win, int x1, int y1, int x2, int y2)
{ int xcoord[4];
  int ycoord[4];
  xcoord[0] = x1; ycoord[0] = y1;
  xcoord[1] = x2; ycoord[1] = y1;
  xcoord[2] = x2; ycoord[2] = y2;
  xcoord[3] = x1; ycoord[3] = y2;
  x_fill_polygon(win,4,xcoord,ycoord);
}


void x_polygon(Window win, int n, int* xcoord, int* ycoord)
{ HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  POINT*  p = new POINT[n+1];
  for(int i=0; i < n; i++) 
  { p[i].x = xcoord[i];
    p[i].y = ycoord[i];
   }
  p[n] = p[0];
  Polyline(hdc,p,n+1);
  delete[] p;
  ReleaseDC(hwnd,hdc);
}


void x_fill_polygon(Window win, int n, int* xcoord, int* ycoord)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  COLORREF col = wlist[win].pen_data.lopnColor;
  POINT* p = new POINT[n];
  for(int i=0; i < n; i++) 
  { p[i].x = xcoord[i];
    p[i].y = ycoord[i];
   }
  HBRUSH save_brush = SelectObject(hdc,CreateSolidBrush(col));
  SetPolyFillMode(hdc,ALTERNATE);
  Polygon(hdc,p,n);
  DeleteObject(SelectObject(hdc,save_brush));
  delete[] p;
  ReleaseDC(hwnd,hdc);
}



/*
void x_circle(Window win, int x, int y, int r)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  HBRUSH save_brush = SelectObject(hdc,GetStockObject(NULL_BRUSH));
  Ellipse(hdc,x-r,y-r,x+r,y+r);
  SelectObject(hdc,save_brush);
  ReleaseDC(hwnd,hdc);
}


void x_fill_circle(Window win, int x, int y, int r)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  COLORREF col = wlist[win].pen_data.lopnColor;
  SetBkMode(hdc,OPAQUE);
  HBRUSH save_brush = SelectObject(hdc,CreateSolidBrush(col));
  Ellipse(hdc,x-r,y-r,x+r,y+r);
  SetBkMode(hdc,TRANSPARENT);
  DeleteObject(SelectObject(hdc,save_brush));
  ReleaseDC(hwnd,hdc);
}
*/


void x_circle(Window win, int x0,int y0,int r0)
{ HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  int col = wlist[win].pen_data.lopnColor;
  int lw  = wlist[win].LWIDTH;

  for (int r = r0-lw/2; r <= r0+lw/2; r++)
  { int y = r;
    int x = 0;
    int e = 3-2*y;

    SetPixel(hdc,x0,y0+r,col);
    SetPixel(hdc,x0,y0-r,col);
    SetPixel(hdc,x0+r,y0,col);
    SetPixel(hdc,x0-r,y0,col);

    for (x=1;x<y;)
      { SetPixel(hdc,x0+x,y0+y,col);
        SetPixel(hdc,x0+x,y0-y,col);
        SetPixel(hdc,x0-x,y0+y,col);
        SetPixel(hdc,x0-x,y0-y,col);
        SetPixel(hdc,x0+y,y0+x,col);
        SetPixel(hdc,x0+y,y0-x,col);
        SetPixel(hdc,x0-y,y0+x,col);
        SetPixel(hdc,x0-y,y0-x,col);
        x++;
        if (e>=0) { y--; e = e - 4*y; }
        e = e + 4*x + 2;
       }

    if (x == y)
    { SetPixel(hdc,x0+x,y0+y,col);
      SetPixel(hdc,x0+x,y0-y,col);
      SetPixel(hdc,x0-x,y0+y,col);
      SetPixel(hdc,x0-x,y0-y,col);
     }
  }
 ReleaseDC(hwnd,hdc);
}


inline void h_line(HDC hdc, int x1, int x2, int y) 
{ MoveToEx(hdc,x1,y,NULL); 
  LineTo(hdc,x2+1,y); 
 }

void x_fill_circle(Window win, int x0, int y0, int r)
{ 
  int save_lw = x_set_line_width(win,1); 

  HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  
  int y = 1;
  int x = r;
  int e = 3-2*r;

  h_line(hdc,x0-x,x0+x,y0);

  while (y<=x)
  { h_line(hdc,x0-x,x0+x,y0+y);
    h_line(hdc,x0-x,x0+x,y0-y);
    if (y<x && e>=0)
    { h_line(hdc,x0-y,x0+y,y0+x);
      h_line(hdc,x0-y,x0+y,y0-x);
      x--;
      e = e - 4*x;
     }
    y++;
    e = e + 4*y + 2;
   }
  ReleaseDC(hwnd,hdc);
  x_set_line_width(win,save_lw); 
}


void x_pixel(Window win, int x, int y)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  COLORREF col = wlist[win].pen_data.lopnColor;
  SetPixel(hdc,x,y,col); 
  ReleaseDC(hwnd,hdc);
}


void x_pixels(Window win, int n, int* x, int* y)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  COLORREF col = wlist[win].pen_data.lopnColor;
  while (n--) SetPixel(hdc,x[n],y[n],col); 
  ReleaseDC(hwnd,hdc);
}


void x_point(Window win, int x, int y)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  COLORREF col = wlist[win].pen_data.lopnColor;
  SetPixel(hdc,x-2,y-2,col);
  SetPixel(hdc,x-1,y-1,col);
  SetPixel(hdc,x-0,y-0,col);
  SetPixel(hdc,x+1,y+1,col);
  SetPixel(hdc,x+2,y+2,col);
  SetPixel(hdc,x-2,y+2,col);
  SetPixel(hdc,x-1,y+1,col);
  SetPixel(hdc,x+1,y-1,col);
  SetPixel(hdc,x+2,y-2,col);
  ReleaseDC(hwnd,hdc);
}



void x_arc(Window win,int mx,int my,int r1,int r2,double start,double angle)
{ HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  int R  = r1+r2;
  int x0 = int(mx + R*cos(start+angle));
  int y0 = int(my - R*sin(start+angle));
  int x1 = int(mx + R*cos(start));
  int y1 = int(my - R*sin(start));
  Arc(hdc,mx-r1,my-r2,mx+r1,my+r2,x0,y0,x1,y1);
  ReleaseDC(hwnd,hdc);
}


void x_fill_arc(Window, int, int, int, int, double, double) {}


void x_ellipse(Window win, int x, int y, int r1, int r2)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  HBRUSH save_brush = SelectObject(hdc,GetStockObject(NULL_BRUSH));
  Ellipse(hdc,x-r1,y-r2,x+r1,y+r2);
  SelectObject(hdc,save_brush);
  ReleaseDC(hwnd,hdc);
}


void x_fill_ellipse(Window win, int x, int y, int r1, int r2)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  COLORREF col = wlist[win].pen_data.lopnColor;
  HBRUSH save_brush = SelectObject(hdc,CreateSolidBrush(col));
  SetBkMode(hdc,OPAQUE);
  Ellipse(hdc,x-r1,y-r2,x+r1,y+r2);
  DeleteObject(SelectObject(hdc,save_brush));
  ReleaseDC(hwnd,hdc);
}


void x_text(Window win, int x, int y, const char* s0, int l)
{ HWND hwnd = wlist[win].hwnd;
  HDC  hdc = GetDC(hwnd);
  COLORREF col = wlist[win].pen_data.lopnColor;

  if (l > strlen(s0)) l = strlen(s0);

  char* s = new char[l+1];
  for(int i=0; i<l; i++) 
     s[i] =  (s0[i] != '0') ? s0[i] : 'O';

  s[l] = '\0';

  if (wlist[win].TMODE == opaque)
  { SetBkMode(hdc,OPAQUE);
    SetTextColor(hdc,col);
    TextOut(hdc,x,y,s,l);
    ReleaseDC(hwnd,hdc);
    delete[] s;
    return;
   }

  SIZE sz;

  GetTextExtentPoint(hdc,s,l,&sz);
  int w = sz.cx;
  int h = sz.cy;

  HDC     hdcMem    = CreateCompatibleDC(hdc) ;
  HBITMAP save_hbm  = SelectObject(hdcMem,CreateCompatibleBitmap(hdc,w,h));

  SelectObject(hdcMem,wlist[win].hfont);
  SetBkMode(hdcMem,OPAQUE);
  SetBkColor(hdcMem,RGB(0,0,0));
  SetTextColor(hdcMem,RGB(255,255,255));
  TextOut(hdcMem,0,0,s,l);


  HBRUSH  save_brush = SelectObject(hdc,CreateSolidBrush(col));

  if (wlist[win].MODE == xor_mode)
     BitBlt(hdc,x,y,w,h,hdcMem,0,0,0x00A60706);
  else
     BitBlt(hdc,x,y,w,h,hdcMem,0,0,0x00E20746);

  DeleteObject(SelectObject(hdcMem,save_hbm));
  DeleteDC(hdcMem);

  DeleteObject(SelectObject(hdc,save_brush));
  ReleaseDC(hwnd,hdc);
  delete[] s;
}

  
void x_text(Window win, int x, int y, const char* s)
{ x_text(win, x, y, s, strlen(s)); }


void x_ctext(Window win, int x, int y, const char* s)
{ int h = x_text_height(win,s);
  int w = x_text_width(win,s);
  x_text(win,x-w/2,y-h/2,s);
}



//------------------------------------------------------------------------------
// pixrects and bitmaps 
//------------------------------------------------------------------------------

inline void SWAP(int& x1, int& x2)
{ int t = x1; x1 = x2; x2 = t; }

struct Image {
 short w;
 short h;
 HBITMAP map;
};


static char rev_byte(char c)
{ char c1 = 0x00;
  if (c & 0x01) c1 |= 0x80;
  if (c & 0x02) c1 |= 0x40;
  if (c & 0x04) c1 |= 0x20;
  if (c & 0x08) c1 |= 0x10;
  if (c & 0x10) c1 |= 0x08;
  if (c & 0x20) c1 |= 0x04;
  if (c & 0x40) c1 |= 0x02;
  if (c & 0x80) c1 |= 0x01;
  return c1;
 }


char* x_create_bitmap(Window win, int w, int h, char* data)
{ HWND hwnd = wlist[win].hwnd;
  HDC   hdc = GetDC(hwnd);
  int   bw0 = (w+7)/8;
  int   bw1 = 2*((bw0+1)/2);
  char* buf = new char[bw1*h];
  char* p   = buf;
  char* q   = data;

  for(int i=0; i<h; i++)
    for(int j=0; j<bw1; j++) 
       *p++ = (j >= bw0) ? 0 : rev_byte(*q++);

  BITMAP bm;
  bm.bmType = 0;
  bm.bmWidth = w;
  bm.bmHeight = h;
  bm.bmWidthBytes = bw1;
  bm.bmPlanes = 1;
  bm.bmBitsPixel = 1;
  bm.bmBits = buf;
  Image* im = new Image;
  im->w = w;
  im->h = h;
  im->map = CreateBitmapIndirect(&bm);
  delete[] buf;

  ReleaseDC(hwnd,hdc);
  return (char*)im;
}


char* x_create_pixrect(Window win, int w, int h, char* data, int fc, int bc)
{ return x_create_bitmap(win,w,h,data); }

char* x_create_pixrect(Window win, int x1, int y1, int x2, int y2)
{ HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  HDC hdcMem  = CreateCompatibleDC (hdc) ;
  Image* im = new Image;
  if (x1 > x2) SWAP(x1,x2);
  if (y1 > y2) SWAP(y1,y2);
  im->w = x2-x1+1;
  im->h = y2-y1+1;
  im->map = CreateCompatibleBitmap (hdc,im->w,im->h) ;
  SelectObject (hdcMem,im->map);
  BitBlt (hdcMem,0,0,im->w,im->h,hdc,x1,y1,SRCCOPY) ;
  DeleteDC (hdcMem) ;
  ReleaseDC(hwnd,hdc);
  return (char*)im;
}


void x_insert_pixrect(Window win, int x, int y, char* rect)
{ // (x,y) lower left corner !
  HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  HDC hdcMem  = CreateCompatibleDC (hdc) ;
  Image* im = (Image*)rect;
  SelectObject (hdcMem,im->map);

  BitBlt (hdc,x,y-im->h+1,im->w,im->h,hdcMem,0,0,SRCCOPY) ;

  DeleteDC (hdcMem) ;
  ReleaseDC(hwnd,hdc);
}


void x_insert_bitmap(Window win, int x, int y, char* rect)
{ // (x,y) lower left corner !
  HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  HDC hdcMem  = CreateCompatibleDC (hdc) ;
  Image* im = (Image*)rect;
  SelectObject (hdcMem,im->map);

  COLORREF col = wlist[win].pen_data.lopnColor;
  HBRUSH  save_brush = SelectObject(hdc,CreateSolidBrush(col));

  if (wlist[win].MODE == xor_mode)
     BitBlt (hdc,x,y-im->h+1,im->w,im->h,hdcMem,0,0,0x00A60706);
  else
     BitBlt (hdc,x,y-im->h+1,im->w,im->h,hdcMem,0,0,0x00E20746);

  DeleteDC (hdcMem);

  DeleteObject(SelectObject(hdc,save_brush));
  ReleaseDC(hwnd,hdc);
}



void x_delete_bitmap(char* rect)
{ Image* im = (Image*)rect;
  DeleteObject(im->map);
 }

void x_delete_pixrect(char* rect) { x_delete_bitmap(rect); }



void x_copy_pixrect(Window win, int x1, int y1, int x2, int y2, int x, int y)
{ HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  HDC hdcMem  = CreateCompatibleDC (hdc) ;
  Image* im = new Image;
  int w = x2-x1+1;
  int h = y2-y1+1;
  HBITMAP map = CreateCompatibleBitmap (hdc,w,h) ;
  SelectObject (hdcMem,map);
  BitBlt (hdcMem,0,0,w,h,hdc,x1,y1,SRCCOPY) ;
  BitBlt (hdc,x,y,w,h,hdcMem,0,0,SRCCOPY) ;
  DeleteDC (hdcMem) ;
  ReleaseDC(hwnd,hdc);
}



/* fonts and text */

int x_load_text_font(const char* font_name) 
{ for(int w=1; w<=wcount; w++) x_set_font(w,font_name);
  return 1; 
 }

int x_load_bold_font(const char* font_name)    { return 0; }
int x_load_message_font(const char* font_name) { return 0; }

int x_set_font(Window win, const char* fname)
{ 
  int scaling = 1;
  if (fname[0] >= '1' && fname[0] <= '9') 
     scaling = fname[0] - '0';

  lf.lfHeight =  FONT_SIZE*scaling;
  lf.lfPitchAndFamily = FONT_TYPE;
  wlist[win].hfont = CreateFontIndirect(&lf);

  HWND hwnd = wlist[win].hwnd;
  HDC hdc = GetDC(hwnd);
  DeleteObject(SelectObject(hdc, wlist[win].hfont));
  ReleaseDC(hwnd,hdc);

  return 1;
}


void x_set_text_font(Window)    {}
void x_set_bold_font(Window)    {}
void x_set_message_font(Window) {}


int x_text_width(Window win,const char* s)
{ HWND hwnd = wlist[win].hwnd;
  SIZE sz;
  if (hwnd == NULL) 
    { x_open_display();
      HDC hdc = CreateDC("DISPLAY",NULL,NULL,NULL); 
      SelectObject(hdc, std_font);
      GetTextExtentPoint(hdc,s,strlen(s),&sz);
      DeleteDC(hdc);
     }
  else 
  { HDC hdc = GetDC(hwnd);
    GetTextExtentPoint(hdc,s,strlen(s),&sz);
    ReleaseDC(hwnd,hdc);
   }
    
  return sz.cx;
 }

int x_text_height(Window win,const char* s) 
{ HWND hwnd = wlist[win].hwnd;
  SIZE sz;
  if (hwnd == NULL) 
    { x_open_display();
      HDC hdc = CreateDC("DISPLAY",NULL,NULL,NULL); 
      SelectObject(hdc, std_font);
      GetTextExtentPoint(hdc,s,strlen(s),&sz);
      DeleteDC(hdc);
     }
  else 
  { HDC hdc = GetDC(hwnd);
    GetTextExtentPoint(hdc,s,strlen(s),&sz);
    ReleaseDC(hwnd,hdc);
   }
  return sz.cy;
 }


/* drawing parameters */

int x_set_color(Window win, int col)
{ win_struct* wp = &wlist[win];
  HWND hwnd = wp->hwnd;
  int old_col = wp->COLOR;
  if (col == old_col) return old_col;
  wp->COLOR = col;
  if (hwnd) 
  { HDC hdc = GetDC(hwnd);
    wp->pen_data.lopnColor = GetNearestColor(hdc,rgb_table[col]);
    DeleteObject(SelectObject(hdc,CreatePenIndirect(&wp->pen_data)));
    ReleaseDC(hwnd,hdc);
   }
  return old_col;
 }


drawing_mode x_set_mode(Window win, drawing_mode mod) 
{ win_struct* wp = &wlist[win];
  HWND hwnd = wp->hwnd;
  drawing_mode old_mod = wp->MODE;
  if (mod == old_mod) return old_mod;
  wp->MODE = mod;
  if (hwnd)
  { HDC hdc = GetDC(hwnd);
    switch (mod) {
    case src_mode: SetROP2(hdc, R2_COPYPEN);
                   break;
    case and_mode: SetROP2(hdc, R2_MASKPEN);
                   break;
    case or_mode:  SetROP2(hdc, R2_MERGEPEN);
                   break;
    case xor_mode: SetROP2(hdc, R2_NOTXORPEN);
                   break;
    }
    ReleaseDC(hwnd,hdc);
   }

  return old_mod;
 }


int x_set_line_width(Window win, int lw) 
{ win_struct* wp = &wlist[win];
  HWND hwnd = wp->hwnd;
  int old_lw = wp->LWIDTH;
  if (lw == old_lw) return old_lw;
  wp->LWIDTH = lw;
  if (hwnd)
  { HDC hdc = GetDC(hwnd);
    wp->pen_data.lopnWidth.x = lw;
    wp->pen_data.lopnWidth.y = lw;
    DeleteObject(SelectObject(hdc,CreatePenIndirect(&wp->pen_data)));
    ReleaseDC(hwnd,hdc);
   }
  return old_lw;
}


line_style x_set_line_style(Window win, line_style ls)
{ win_struct* wp = &wlist[win];
  HWND hwnd = wp->hwnd;
  line_style old_ls = wp->LSTYLE;
  if (ls == old_ls) return old_ls;
  wp->LSTYLE = ls;
  if (hwnd)
  { HDC hdc = GetDC(hwnd);
    switch (ls) {
    case solid:  wp->pen_data.lopnStyle = PS_SOLID;
                 break;
    case dashed: wp->pen_data.lopnStyle = PS_DASH;
                 break;
    case dotted: wp->pen_data.lopnStyle = PS_DOT;
                 break;
    }
    DeleteObject(SelectObject(hdc,CreatePenIndirect(&wp->pen_data)));
    ReleaseDC(hwnd,hdc);
   }
  return old_ls;
 }


text_mode x_set_text_mode(Window win, text_mode tm) 
{ text_mode save = wlist[win].TMODE;
  wlist[win].TMODE = tm;  
  return save;
 }


int           x_get_color(Window win)      { return wlist[win].COLOR; }
drawing_mode  x_get_mode(Window win)       { return wlist[win].MODE;   }
int           x_get_line_width(Window win) { return wlist[win].LWIDTH; }
line_style    x_get_line_style(Window win) { return wlist[win].LSTYLE; }
text_mode     x_get_text_mode(Window win)  { return wlist[win].TMODE;  }



void x_set_read_gc(Window win)
{ win_struct* wp = &wlist[win];
  wp->save_mo = x_set_mode(win,xor_mode);
  wp->save_ls = x_set_line_style(win,solid);
  wp->save_lw = x_set_line_width(win,1);
  x_set_color(win,black);
 }


void x_reset_gc(Window win)
{ win_struct* wp = &wlist[win];
  x_set_mode(win,wp->save_mo);
  x_set_line_style(win,wp->save_ls);
  x_set_line_width(win,wp->save_lw);
 }


/* colors */

void x_set_palette(int i, int r, int g, int b)
{ rgb_table[i]  = RGB(r,g,b); }

void x_get_palette(int i, int* red, int* green, int* blue)
{ *red   = GetRValue(rgb_table[i]);
  *green = GetGValue(rgb_table[i]);
  *blue  = GetBValue(rgb_table[i]);
 }


int x_new_color(const char*) { return black; }



/* events */

int x_get_next_event(Window* win, int* val, int* x, int* y, unsigned long *t)
{ 
  if (putback) 
   { cur_event = last_event;
     putback = 0;
    }
  else
  { MSG msg;
    cur_event.kind = no_event;
    if (GetMessage (&msg, NULL, 0, 0))
      { TranslateMessage (&msg);
        DispatchMessage (&msg);
       }
    else 
       exit(0);

    if (cur_event.kind != exposure_event)
        cur_event.t = (unsigned long)msg.time;

    if (cur_event.kind != no_event) 
       last_event = cur_event;
   }

  *win = cur_event.win;
  *val = cur_event.val;
  *x   = cur_event.x;
  *y   = cur_event.y;
  *t   = cur_event.t;
  return cur_event.kind;
 }


int x_check_next_event(Window* win, int* val, int* x, int* y, unsigned long *t)
{ MSG msg;
  if (putback || PeekMessage(&msg,NULL,0,0,PM_NOREMOVE))
     return x_get_next_event(win,val,x,y,t); 
  else
     return no_event;
 }

void x_put_back_event(void) {  putback = 1; }


int  WINAPI  WinMain(HANDLE this_inst, HANDLE prev_inst, LPSTR, int)
{ 
     WNDCLASS    wndclass;

     this_instance = this_inst;

     if (!prev_inst)
     { wndclass.lpszClassName = szAppName1;
       wndclass.style         = CS_OWNDC |CS_HREDRAW | CS_VREDRAW;
       wndclass.lpfnWndProc   = (WNDPROC)WndProc;
       wndclass.cbClsExtra    = 0;
       wndclass.cbWndExtra    = 0;
       wndclass.hInstance     = this_inst;
       wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
       wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
       wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
       wndclass.lpszMenuName  = NULL;
       RegisterClass (&wndclass);

       wndclass.lpszClassName = szAppName2;
       wndclass.style         = CS_SAVEBITS | CS_OWNDC |CS_HREDRAW | CS_VREDRAW;
       wndclass.lpfnWndProc   = (WNDPROC)WndProc;
       wndclass.cbClsExtra    = 0;
       wndclass.cbWndExtra    = 0;
       wndclass.hInstance     = this_inst;
       wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
       wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
       wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
       wndclass.lpszMenuName  = NULL;
       RegisterClass (&wndclass);
      }

  return 0;
}


