/*******************************************************************************
+
+  LEDA 3.4
+
+  _bigfloat.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

/* This file has been automatically generated from "_bigfloat.w"
   by CTANGLE (Version 3.1 [km2c]) */




#include <LEDA/bigfloat.h>

#include <math.h>
/*
#include <stdio.h>
#include <string.h>
*/



const bigfloat NaN(NAN_VALUE);
const bigfloat pInf(PINF_VALUE);
const bigfloat nInf(NINF_VALUE);
const bigfloat pZero(PZERO_VALUE);
const bigfloat nZero(NZERO_VALUE);


rounding_modes round_mode = TO_NEAREST;
long global_prec = 53;
bool dbool = true;



/* since we need the |pow2| and |compose_parts| functions we have to use
   prototyping */

double compose_parts(long, long, long, long);
//double pow2(long);

const double double_min   = compose_parts(0, 1, 0, 0); // pow2(-1022);
const double NaN_double   = compose_parts(0, 2047, 0, 1);
const double pInf_double  = compose_parts(0, 2047, 0, 0);
const double nInf_double  = -pInf_double;
const double pZero_double = compose_parts(0, 0, 0, 0);
const double nZero_double = compose_parts(1, 0, 0, 0);



const integer integer_1  = integer(1);
const integer integer_52 = (integer_1 << 52) - integer_1;
const integer integer_32 = (integer_1 << 32) - integer_1;
const integer integer_20 = (integer_1 << 20) - integer_1;



output_modes output_mode = DEC_OUT;
const long bin_maxlen = 10000;



long max(long l1, long l2)
{
  return (l1 > l2 ? l1 : l2);
}
integer sq(const integer & op)
{
  return op * op;
}
long sign(int i)
{
  if (i == 0)
    return 0;
  else
    return (i > 0 ? 1 : -1);
}
long sign(long i)
{
  if (i == 0)
    return 0;
  else
    return (i > 0 ? 1 : -1);
}



bigfloat Powl(const bigfloat & x, long n, long prec = 1,
   rounding_modes mode = EXACT)
{
  bigfloat z = 1, y = x;
  long n_prefix = n;

  while (n_prefix > 0) {
    if (n_prefix % 2)
      z = mul(z, y, prec, mode);
    n_prefix = n_prefix / 2;
    y = mul(y, y, prec, mode);
  }
  return z;
}


bool isnum(char ch)
{
  if ((ch >= '0') && (ch <= '9'))
    return true;
  else
    return false;
}



long bigfloat::non_zero_sign(void) const
{
  long s = 0;

  switch (special) {
      case NOT_VALUE:
	s = ::sign(significant);
	break;
      case PINF_VALUE:
      case PZERO_VALUE:
	s = 1;
	break;
      case NINF_VALUE:
      case NZERO_VALUE:
	s = -1;
	break;
      case NAN_VALUE:
	error_handler(1, "sign: NaN has no sign");
  }
  return s;
}



void cut(integer & b, long prec)
{
  b = (b >> (b.length() - prec));
}



double compose_parts(long sign_1, long exp_11, long most_sig_20, long least_sig_32)
{
  double a;
  long high32 = 0;


  if (sign_1)
    high32 = high32 | 0x80000000;
  exp_11 = exp_11 << 20;
  high32 = high32 | exp_11;
  high32 = high32 | most_sig_20;




  long *p;
  p = (long *) &a;
#ifndef LITTLE_ENDIAN
  (*p) = high32;
  p++;
  (*p) = least_sig_32;
#else
  (*p) = least_sig_32;
  p++;
  (*p) = high32;
#endif




  return a;
}


double pow2(long exp)
{
  return compose_parts(0, exp + 1023, 0, 0);
}


void binout(ostream & os, integer b)
{
  char temp[bin_maxlen];
  long flag = 0, count = 0;

  if (b < 0)
    b *= -1;

  do {
    temp[count++] = (char) (b % 2).tolong() + '0';
    if (b <= 1)
      flag = 1;
    else
      b /= 2;
  } while (!flag);
  for (long i = count - 1; i >= 0; i--)
    os << temp[i];
}


void decimal_output(ostream & os, bigfloat b, long prec)
{
  if (!b.get_exponent().islong())
    error_handler(1, "decimal_output: not implemented for large exponents");


  integer dd = 10;

  if (b.sign() < 0) {
    b = -b;
    os << "-";
  }



  long log2_b = b.get_precision() + b.get_exponent().tolong();
  bigfloat b_rem(b.get_significant(), -b.get_precision());
  long log10_b = (long) ceil(log2_b * log10(2.0) + log10(b_rem.to_double()));



  long digits = b.get_precision() + 32;
  long diff = prec - log10_b;
  bigfloat multiplicator, divisor;
  bigfloat b_shift;
  integer significant;
  if (diff >= 0) {
    multiplicator = Powl(dd, diff, digits, TO_NEAREST);
    b_shift = mul(b, multiplicator, digits, TO_NEAREST);
  }
  else {
    divisor = Powl(dd, -diff, digits, TO_NEAREST);
    b_shift = div(b, divisor, digits, TO_NEAREST);
  }
  significant = b_shift.to_integer(TO_NEAREST);


  //ostrstream oss;		/* string stream needed for output */
  //oss << significant;
  //char *str = oss.str();

  string str = significant.tostring();

  if (prec > 1)
     os << str.insert(1,".");
  else
     os << str;

  if (log10_b != 1) os << "E" << log10_b - 1;


}


bigfloat outofchar(char *rep, long prec)
{
  integer significant = 0, exponent = 0;
  bigfloat result, pow;
  integer dd = 10;
  long int_length = 0, frac_length = 0;

  int s;

  s = 1;
  if (rep[0] == '-') {
    s = -1;
    rep++;
  }
  else if (rep[0] == '+')
    rep++;


  int Sign = s;

  while (isnum(*rep)) {
    int_length++;
    significant = significant * dd + (*(rep++) - '0');
  }



  if (*rep == '.') {
    rep++;
    while (isnum(*rep)) {
      significant = significant * dd + (*(rep++) - '0');
      frac_length++;
    }
  }




  if (*rep == 'E') {
    rep++;

    s = 1;
    if (rep[0] == '-') {
      s = -1;
      rep++;
    }
    else if (rep[0] == '+')
      rep++;


    while (isnum(*rep))
      exponent = exponent * dd + (*(rep++) - '0');
    if (s == -1)
      exponent = -exponent;
  }


  if (prec == 0) {
    pow = Powl(dd, int_length + frac_length);
    prec = pow.get_precision() + pow.get_exponent().tolong();
  }
  long exp = exponent.tolong() - frac_length;
  if (exp > 0) {
    pow = Powl(dd, exp, prec + 10, TO_NEAREST);
    result = mul(significant, pow, prec, TO_NEAREST);
  }
  else {
    pow = Powl(dd, -exp, prec + 10, TO_NEAREST);
    result = div(significant, pow, prec, TO_NEAREST);
  }
  if (Sign == 1)
    return result;
  else
    return -result;
}





void bigfloat::normalize(void)
{
  int signum = ::sign(significant);
  if ((signum == 0) && (special == NOT_VALUE))
    special = PZERO_VALUE;

  long z = significant.zeros();
/* |z| is the number of final zeros in the significant */
  if (z > 0)
    significant = significant >> z;
  precision = significant.length();
  exponent += z;
}





bigfloat::bigfloat()
{
  special = PZERO_VALUE;
  exponent = significant = precision = 0;
}
bigfloat::bigfloat(const integer & s, const integer & e)
{
  significant = s;
  exponent = e;
  special = NOT_VALUE;
  precision = s.length();
}
bigfloat::bigfloat(special_values spec)
{
  special = spec;
}
bigfloat::bigfloat(const integer & a)
{
  special = NOT_VALUE;
  significant = a;
  exponent = 0;
  precision = significant.length();

  if (significant == 0)
    special = PZERO_VALUE;



}
bigfloat::bigfloat(long a)
{
  special = NOT_VALUE;
  significant = a;
  exponent = 0;
  precision = significant.length();

  if (significant == 0)
    special = PZERO_VALUE;



}
bigfloat::bigfloat(int a)
{
  special = NOT_VALUE;
  significant = a;
  exponent = 0;
  precision = significant.length();

  if (significant == 0)
    special = PZERO_VALUE;



}



bigfloat::bigfloat(double d)
{
  int sign;
  long flag = 0;
  unsigned long mh = 0, ml = 0;
  long *p;


  if (fabs(d) < double_min) {
    //d = d * pow2(52);
    d *= compose_parts(0, 52 + 1023, 0, 0);
    flag = 1;
  }



  p = (long *) &d;
#ifndef LITTLE_ENDIAN
  mh = *p;
  p++;
  ml = *p;
#else
  ml = *p;
  p++;
  mh = *p;
#endif



  if (mh & 0x80000000)
    sign = -1;
  else
    sign = 1;




  long sig = 0;
/* get the significant bits out of the highword */
  sig = mh & 0x000fffff;
/* we have ensured that |d| is normalized $\Rightarrow$ add the leading one
       bit */
  sig = sig | 0x00100000;
/* compose significant */
  significant = integer(sig);
  significant = significant << 32;
  significant = significant + integer(ml);
  if (sign == -1)
    significant = -significant;



  long e = 0;
/* get the 11 bits of the exponent */
  e = mh & 0x7ff00000;
/* shift the result to get the right value */
  e = e >> 20;
/* subtract bias and 52*/
  e -= 1075;
/* subtract 52 if |d| was denormalized */
  if (flag)
    e -= 52;
  exponent = e;




  special = NOT_VALUE;
  if (e == 972) {		/* Inf or NaN-Case */

    sig = mh & 0x000fffff;

    if ((mh & 0x000fffff) == 0 && ml == 0)
      special = (sign > 0) ? PINF_VALUE : NINF_VALUE;
    else
      special = NAN_VALUE;
  }
  if (d == 0)
    special = (sign > 0) ? PZERO_VALUE : NZERO_VALUE;



  normalize();
}



long bigfloat::sign(void) const
{
  long s = 0;

  switch (special) {
      case NOT_VALUE:
	s = ::sign(significant);
	break;
      case PZERO_VALUE:
      case NZERO_VALUE:
	s = 0;
	break;
      case PINF_VALUE:
	s = 1;
	break;
      case NINF_VALUE:
	s = -1;
	break;
      case NAN_VALUE:
	error_handler(1, "sign: NaN has no sign");
  }

  return s;
}


bigfloat bigfloat::round(long digits, rounding_modes mode, bool &is_exact,
   long bias)
{
  normalize();
  if (isSpecial() || (mode == EXACT))
    return *this;
  int test = 0;
  long shift;

  if (digits < precision) {
    switch (mode) {
	case TO_NEAREST:

	  cut(significant, digits + 1);
	  test =
	    ((significant & integer(1)) == 0);

	  cut(significant, digits);
	  if (!test) {
	    if ((precision > digits + 1)
	      || (::sign(bias) == ::sign(significant))
	      || ((bias == 0) && !
		((significant & integer(1)) == 0)
	      )
	      )
	      if (::sign(significant) > 0)
		significant++;
	      else
		significant--;
	  }



	  break;
	case TO_ZERO:

	  cut(significant, digits);


	  break;
	case TO_P_INF:

	  cut(significant, digits);
	  if (::sign(significant) > 0)
	    significant++;


	  break;
	case TO_N_INF:

	  cut(significant, digits);
	  if (::sign(significant) < 0)
	    significant--;


	  break;
	case TO_INF:

	  cut(significant, digits);
	  if (::sign(significant) > 0)
	    significant++;
	  else
	    significant--;


	  break;
	case EXACT:;
    }
    if (mode != EXACT)
      exponent += (precision - digits);
  }
  else if (bias != 0) {
    shift = digits - precision;
    long bf_sign = ::sign(significant);

    switch (mode) {
	case TO_ZERO:
	  if (::sign(bias) != bf_sign) {


	    if (shift > 0) {
	      significant = significant << shift;
	      exponent -= shift;
	    }


	    if (bf_sign == 1)
	      significant--;
	    else
	      significant++;
	  }
	  break;
	case TO_INF:
	  if (::sign(bias) == bf_sign) {


	    if (shift > 0) {
	      significant = significant << shift;
	      exponent -= shift;
	    }


	    if (bf_sign == 1)
	      significant++;
	    else
	      significant--;
	  }
	  break;
	case TO_P_INF:
	  if (::sign(bias) == 1) {


	    if (shift > 0) {
	      significant = significant << shift;
	      exponent -= shift;
	    }


	    significant++;
	  }
	  break;
	case TO_N_INF:
	  if (::sign(bias) == -1) {


	    if (shift > 0) {
	      significant = significant << shift;
	      exponent -= shift;
	    }


	    significant--;
	  }
	  break;
/* in the cases |TO_NEAREST| and |EXACT| we have to do nothing */
	case TO_NEAREST:
	case EXACT:;
    }
  }




  is_exact = (digits >= precision) && (bias == 0);
  normalize();
  return *this;
}



bigfloat bigfloat::round_with_error(long digits, rounding_modes mode,
   bool &is_exact, long bias)
{
  bigfloat temp = (*this);
  round(digits, mode, is_exact, bias);
  return (sub((*this), temp, temp.precision));
}


integer bigfloat::to_integer(rounding_modes rmode) const
{
  if ((isNaN()) || (isInf()))
    error_handler(1, "bigfloat::to_integer : special values cannot be converted to integer");
  if (!(exponent + (integer) significant.length()).islong())
    error_handler(1, "bigfloat::to_integer : (exp+sig len) has to be in long range");

  bigfloat result = (*this);
  long pow2 = precision + exponent.tolong();
  long length = max(1, pow2);
  result.round(length, rmode);
  if (result.isZero() || (pow2 <= 0))
    return integer(0);
  else
    return (result.significant << result.exponent.tolong());
}


double bigfloat::to_double(void) const
{
  long s = ::sign(significant);
  long t_exp = 0;
  integer t_sig = 0;

  bigfloat rounded_value = fabs(*this);


  if (isSpecial()) {
    if (ispZero())
      return pZero_double;
    if (isnZero())
      return nZero_double;
    if (isNaN())
      return NaN_double;
    if (ispInf())
      return pInf_double;
    if (isnInf())
      return nInf_double;
  }



  integer log2 = exponent + precision;
  if (log2 > 1024)
    return non_zero_sign() * pInf_double;
  if (log2 < -1073)
    return non_zero_sign() * pZero_double;
  if (log2 > -1021)
    rounded_value.round(53, TO_NEAREST);
  else
    rounded_value.round(1074 + log2.tolong(), TO_NEAREST);




  long normal = 1;
  log2 = rounded_value.exponent + rounded_value.precision;

  if (log2 == 1025)
    return non_zero_sign() * pInf_double;
  if (log2 < -1021)
    normal = 0;



  if (normal) {
    t_exp = rounded_value.exponent.tolong() + (rounded_value.precision - 1) + 1023;
/* if the length of the rounded |significant| is lower than 53 we
     have to shift it to the left */
    t_sig = rounded_value.significant << (53 - rounded_value.significant.length());
    t_sig = t_sig & integer_52;
  }


  else {
    long l_shift = rounded_value.exponent.tolong() + 1074;
    if (l_shift > 0)
      t_sig = rounded_value.significant << l_shift;
    else
      t_sig = rounded_value.significant >> (-l_shift);
  }



  double a;

  unsigned long sign, h_sig, l_sig;
  sign = (s == (-1));
  h_sig = ((t_sig >> 32) & integer_20).tolong();
  l_sig = (t_sig & integer_32).tolong();
  a = compose_parts(sign, t_exp, h_sig, l_sig);















  return a;
}



void bigfloat::print_contents(void) const
{
  integer s = significant, e = exponent;
  cout << "Significant: ";
  if (::sign(significant) == -1)
    cout << "-";


  binout(cout, abs(s));
  cout << "\n";
  cout << "Exponent: ";
  if (e < 0) {
    cout << "-";
    e = e * integer(-1);
  }
  binout(cout, e);
  cout << "\n";
  cout << "Precision: " << precision << "\n";

  cout << "special flag: ";
  switch (special) {
      case NOT_VALUE:
	cout << "NOT_VALUE\n";
	break;
      case PZERO_VALUE:
	cout << "PZERO_VALUE\n";
	break;
      case NZERO_VALUE:
	cout << "NZERO_VALUE\n";
	break;
      case PINF_VALUE:
	cout << "PINF_VALUE\n";
	break;
      case NINF_VALUE:
	cout << "NINF_VALUE\n";
	break;
      case NAN_VALUE:
	cout << "NAN_VALUE\n";
	break;
  }
}









bigfloat add(const bigfloat & x, const bigfloat & y, long prec, rounding_modes mode,
   bool &is_exact)
{
  bigfloat a, b;
  is_exact = true;


  if (x.isSpecial() || y.isSpecial()) {
    if (x.isNaN() || y.isNaN())
      return NaN;
    if (x.isZero())
      return y;
    if (y.isZero())
      return x;

    if (x.isInf() && y.isInf()) {
      if (x.non_zero_sign() == y.non_zero_sign())
	return x;
      else
	return NaN;
    }
    if (x.isInf())
      return x;

/* it is obvious that |y| has to be $\infty$ */
    return y;
  }




  bigfloat *a_ptr, *b_ptr;
  integer log_x = x.exponent + (integer) x.precision;
  integer log_y = y.exponent + (integer) y.precision;
  integer diff = log_x - log_y;

  if (diff >= 0) {
    a_ptr = (bigfloat *) & x;
    b_ptr = (bigfloat *) & y;
  }
  else {
    a_ptr = (bigfloat *) & y;
    b_ptr = (bigfloat *) & x;
  }

  a = *a_ptr;
  b = *b_ptr;

  diff.absolute();
  integer exp_diff = a.exponent - b.exponent;


/* now $\lceil \log_2{a} \rceil \geq \lceil \log_2{b} \rceil$  */

  bigfloat sum, error;

  if ((mode != EXACT) && (diff > max(prec + 1, a.precision))) {
    sum = a;
    error = b;
  }
  else {


    if (!exp_diff.islong())
      error_handler(1, "bigfloat::add() : exponential difference has to be in long range");

    if (exp_diff > 0) {
      a.significant = a.significant << exp_diff.tolong();
      a.exponent = b.exponent;
    }
    else {
      b.significant = b.significant << (-exp_diff).tolong();
      b.exponent = a.exponent;
    }

    sum = bigfloat(a.significant + b.significant, a.exponent);





  }



  sum.round(prec, mode, is_exact, error.sign());
  return sum;
}




bigfloat sub(const bigfloat & a, const bigfloat & b, long prec,
   rounding_modes mode, bool &is_exact)
{
  return add(a, -b, prec, mode, is_exact);
}



bigfloat mul(const bigfloat & a, const bigfloat & b, long prec,
   rounding_modes mode, bool &is_exact)
{
  bigfloat result;
  is_exact = true;



  long sign_result;

  if ((a.isSpecial()) || (b.isSpecial())) {
    if ((a.isNaN()) || (b.isNaN()))
      return NaN;
    if ((a.isZero() && b.isInf()) || (a.isInf() && b.isZero()))
      return NaN;

    sign_result = a.non_zero_sign() * b.non_zero_sign();

    if (a.isZero() || b.isZero()) {
      if (sign_result == 1)
	return pZero;
      else
	return nZero;
    }
    if (a.isInf() || b.isInf()) {
      if (sign_result == 1)
	return pInf;
      else
	return nInf;
    }
  }




  result.special = NOT_VALUE;
  result.significant = a.significant * b.significant;
  result.exponent = a.exponent + b.exponent;
  return result.round(prec, mode, is_exact);
}



bigfloat div(const bigfloat & a, const bigfloat & b, long prec,
   rounding_modes mode, bool &is_exact)
{
  is_exact = true;
  bigfloat result;
  long bias;



  if ((a.isSpecial()) || (b.isSpecial())) {
    long sign_result;

    if ((a.isNaN()) || (b.isNaN()))
      return NaN;
    if (((a.isZero()) && (b.isZero())) || ((a.isInf()) && (b.isInf())))
      return NaN;
    sign_result = a.non_zero_sign() * b.non_zero_sign();
    if ((a.isInf()) || (b.isZero())) {
      if (sign_result == 1)
	return pInf;
      else
	return nInf;
    }

/* it is clear that |isZero(a)||(isInf(b)| */

    if (sign_result == 1)
      return pZero;
    else
      return nZero;
  }




  bigfloat aa = a;
  long d = prec + b.significant.length() - a.significant.length() + 1;
  if (d > 0) {
    aa.significant = aa.significant << d;
    aa.exponent -= d;
  }



  result.special = NOT_VALUE;
  result.significant = aa.significant / b.significant;

  result.exponent = aa.exponent - b.exponent;
  result.precision = result.significant.length();

  integer R = aa.significant - b.significant * result.significant;
  if (R != 0)
    is_exact = false;
  if (mode == EXACT)
    mode = TO_NEAREST;

  bias = sign(R) * sign(b.significant);



  result.round(prec, mode, is_exact, bias);

  return result;
}



bigfloat sqrt(const bigfloat & a, const bigfloat & start, long prec,
   rounding_modes mode, bool &is_exact)
{
  bigfloat result;
  long t_prec, p_bound = prec + 1;
  bool exact = true;



  if (a.isZero())
    return a;
  if (a.sign() < 0) {
    is_exact = false;
    return NaN;
  }
  if (a.isSpecial()) {
    is_exact = false;
    return a;
  }



  long prec_a = a.get_precision();
  double d;
  if (start.isSpecial() || start <= 0) {
    result = bigfloat(a.significant, -prec_a);
    integer log_a = a.exponent + prec_a;
    d = sqrt(result.to_double());
    if (!(log_a % 2 == 0))
      d *= 2;
    result = d;
    result.exponent += log_a >> 1;
    t_prec = 52;
  }
  else
    result = start;



  bigfloat it;			/* iterator */
  bigfloat & ne = result;	/* next iteration value */

  t_prec = ne.precision;
  bigfloat square, sum;
  while (1 == 1) {
    exact = true;
    t_prec *= 2;
    if (t_prec > p_bound)
      t_prec = p_bound;
    it = ne;
    square = mul(it, it, t_prec, EXACT);
    sum = add(square, a, t_prec, EXACT);
    ne = div(sum, it, t_prec, TO_P_INF, exact);
    ne.exponent--;
    if ((t_prec == p_bound) || exact)
      if (ne == it)
	break;
  }
  if (!exact)
    is_exact = false;



  if (exact)
    result.round(prec, mode, is_exact);
  else {
    switch (mode) {
	case TO_NEAREST:
	  result.round(prec, TO_ZERO, is_exact);
	  break;
	case EXACT:
	  error_handler(1, "bigfloat::sqrt : the EXACT-rounding mode is not provided!");
	case TO_P_INF:
	case TO_N_INF:
	case TO_INF:
	case TO_ZERO:
	  result.round(prec, mode, is_exact);
    }
    is_exact = false;
  }



  return result;
}



bool operator == (const bigfloat & a, const bigfloat & b) {

  if (a.isSpecial() || b.isSpecial()) {
    if (a.isZero() && b.isZero())
      return true;
    if (a.isNaN() || b.isNaN())
      error_handler(1, "bigfloat::operator == : NaN case occurred");
    return (a.special == b.special);
  }




  ((bigfloat &) a).normalize();
  ((bigfloat &) b).normalize();
  return ((a.significant == b.significant) && (a.exponent == b.exponent));
}


bool operator > (const bigfloat & a, const bigfloat & b) {

  if (a.isSpecial() || b.isSpecial()) {
    if (a.isNaN() || b.isNaN())
      error_handler(1, "bigfloat::operator > : NaN case occurred!");
    if (a.isZero() && b.isZero())
      return false;
    if (a.ispInf())
      return !b.ispInf();
    if (b.isnInf())
      return !a.isnInf();
    return (a.sign() > b.sign());
  }






  int sign_a = ::sign(a.significant);
  int sign_b = ::sign(b.significant);
  if (sign_a != sign_b)
    return (sign_a > sign_b);

  integer sig, diff = a.exponent - b.exponent;
  if (!diff.islong())
    error_handler(1, "bigfloat operator > : exponential difference not in long range");
  if (::sign(diff) >= 0) {
    sig = a.significant << (diff.tolong());
    return (sig > b.significant);
  }
  else {
    sig = b.significant << ((-diff).tolong());
    return (a.significant > sig);
  }
}



ostream & operator << (ostream & os, const bigfloat & b) {
  if (b.isSpecial()) {
    if (b.isNaN())
      return os << "NaN";
    if (b.ispInf())
      return os << "+Inf";
    if (b.isnInf())
      return os << "-Inf";
    if (b.ispZero())
      return os << "+0";
    if (b.isnZero())
      return os << "-0";
  }
  int sign_b = sign(b.significant);
  if (output_mode == BIN_OUT) {
    if (sign_b < 0)
      os << "-";
    os << "0.";
    if (sign_b >= 0)
      binout(os, b.significant);
    else
      binout(os, -b.significant);
    os << "E";
    if (b.exponent < 0)
      os << "-";
    else
      os << "+";
    binout(os, b.exponent);
  }
  if (output_mode == DEC_OUT)
    decimal_output(os, b, (long) ceil(log10(2.0) * b.precision));
  return os;
}


istream & operator >> (istream & is, bigfloat & b) {
  char temp[bin_maxlen];
  is >> temp;
  b = outofchar(temp);
  return is;
}




bigfloat operator + (const bigfloat & a, const bigfloat & b) {
#ifdef DEBUG_BF
  long operation = 0;
  double a_d = a.to_double();
  double b_d = b.to_double();

  char *out;
  switch (round_mode) {
      case TO_ZERO:
	ieee_flags("set", "direction", "tozero", &out);
	break;
      case TO_P_INF:
	ieee_flags("set", "direction", "positive", &out);
	break;
      case TO_N_INF:
	ieee_flags("set", "direction", "negative", &out);
	break;
      case TO_NEAREST:
      case EXACT:
      case TO_INF:
	break;
  }


  double result_d = a_d + b_d;

  ieee_flags("set", "direction", "nearest", &out);


  bigfloat result_bf = add(a, b);

  if ((a != bigfloat(a_d)) || (b != bigfloat(b_d))) {
    cout << "a : \n";
    a.print_contents();
    cout << "Kontrolle : \n";
    bigfloat(a_d).print_contents();
    cout << "b : \n";
    b.print_contents();
    cout << "Kontrolle : \n";
    bigfloat(a_d).print_contents();
    error_handler(1, "error: arithmetic operation: operands not correct");
  }

  if (result_bf != bigfloat(result_d)) {
    cout << "----------------------------------------------------\n";
    cout << "bigfloat result: \n";
    result_bf.print_contents();
    cout << "double result: \n";
    bigfloat(result_d).print_contents();
    switch (operation) {
	case 0:
	  result_bf = add(a, b);
	  break;
	case 1:
	  result_bf = sub(a, b);
	  break;
	case 2:
	  result_bf = mul(a, b);
	  break;
	case 3:
	  result_bf = div(a, b);
    }
    error_handler(1, "error: double result != bigfloat result");
  }

#endif
  return add(a, b);
}

bigfloat operator - (const bigfloat & a, const bigfloat & b) {
#ifdef DEBUG_BF
  long operation = 1;
  double a_d = a.to_double();
  double b_d = b.to_double();

  char *out;
  switch (round_mode) {
      case TO_ZERO:
	ieee_flags("set", "direction", "tozero", &out);
	break;
      case TO_P_INF:
	ieee_flags("set", "direction", "positive", &out);
	break;
      case TO_N_INF:
	ieee_flags("set", "direction", "negative", &out);
	break;
      case TO_NEAREST:
      case EXACT:
      case TO_INF:
	break;
  }


  double result_d = a_d - b_d;

  ieee_flags("set", "direction", "nearest", &out);


  bigfloat result_bf = sub(a, b);

  if ((a != bigfloat(a_d)) || (b != bigfloat(b_d))) {
    cout << "a : \n";
    a.print_contents();
    cout << "Kontrolle : \n";
    bigfloat(a_d).print_contents();
    cout << "b : \n";
    b.print_contents();
    cout << "Kontrolle : \n";
    bigfloat(a_d).print_contents();
    error_handler(1, "error: arithmetic operation: operands not correct");
  }

  if (result_bf != bigfloat(result_d)) {
    cout << "----------------------------------------------------\n";
    cout << "bigfloat result: \n";
    result_bf.print_contents();
    cout << "double result: \n";
    bigfloat(result_d).print_contents();
    switch (operation) {
	case 0:
	  result_bf = add(a, b);
	  break;
	case 1:
	  result_bf = sub(a, b);
	  break;
	case 2:
	  result_bf = mul(a, b);
	  break;
	case 3:
	  result_bf = div(a, b);
    }
    error_handler(1, "error: double result != bigfloat result");
  }

#endif
  return sub(a, b);
}

bigfloat operator *(const bigfloat & a, const bigfloat & b) {
#ifdef DEBUG_BF
  long operation = 2;
  double a_d = a.to_double();
  double b_d = b.to_double();

  char *out;
  switch (round_mode) {
      case TO_ZERO:
	ieee_flags("set", "direction", "tozero", &out);
	break;
      case TO_P_INF:
	ieee_flags("set", "direction", "positive", &out);
	break;
      case TO_N_INF:
	ieee_flags("set", "direction", "negative", &out);
	break;
      case TO_NEAREST:
      case EXACT:
      case TO_INF:
	break;
  }


  double result_d = a_d * b_d;

  ieee_flags("set", "direction", "nearest", &out);


  bigfloat result_bf = mul(a, b);

  if ((a != bigfloat(a_d)) || (b != bigfloat(b_d))) {
    cout << "a : \n";
    a.print_contents();
    cout << "Kontrolle : \n";
    bigfloat(a_d).print_contents();
    cout << "b : \n";
    b.print_contents();
    cout << "Kontrolle : \n";
    bigfloat(a_d).print_contents();
    error_handler(1, "error: arithmetic operation: operands not correct");
  }

  if (result_bf != bigfloat(result_d)) {
    cout << "----------------------------------------------------\n";
    cout << "bigfloat result: \n";
    result_bf.print_contents();
    cout << "double result: \n";
    bigfloat(result_d).print_contents();
    switch (operation) {
	case 0:
	  result_bf = add(a, b);
	  break;
	case 1:
	  result_bf = sub(a, b);
	  break;
	case 2:
	  result_bf = mul(a, b);
	  break;
	case 3:
	  result_bf = div(a, b);
    }
    error_handler(1, "error: double result != bigfloat result");
  }

#endif
  return mul(a, b);
}

bigfloat operator / (const bigfloat & a, const bigfloat & b) {
#ifdef DEBUG_BF
  long operation = 3;
  double a_d = a.to_double();
  double b_d = b.to_double();

  char *out;
  switch (round_mode) {
      case TO_ZERO:
	ieee_flags("set", "direction", "tozero", &out);
	break;
      case TO_P_INF:
	ieee_flags("set", "direction", "positive", &out);
	break;
      case TO_N_INF:
	ieee_flags("set", "direction", "negative", &out);
	break;
      case TO_NEAREST:
      case EXACT:
      case TO_INF:
	break;
  }


  double result_d = a_d / b_d;

  ieee_flags("set", "direction", "nearest", &out);


  bigfloat result_bf = div(a, b);

  if ((a != bigfloat(a_d)) || (b != bigfloat(b_d))) {
    cout << "a : \n";
    a.print_contents();
    cout << "Kontrolle : \n";
    bigfloat(a_d).print_contents();
    cout << "b : \n";
    b.print_contents();
    cout << "Kontrolle : \n";
    bigfloat(a_d).print_contents();
    error_handler(1, "error: arithmetic operation: operands not correct");
  }

  if (result_bf != bigfloat(result_d)) {
    cout << "----------------------------------------------------\n";
    cout << "bigfloat result: \n";
    result_bf.print_contents();
    cout << "double result: \n";
    bigfloat(result_d).print_contents();
    switch (operation) {
	case 0:
	  result_bf = add(a, b);
	  break;
	case 1:
	  result_bf = sub(a, b);
	  break;
	case 2:
	  result_bf = mul(a, b);
	  break;
	case 3:
	  result_bf = div(a, b);
    }
    error_handler(1, "error: double result != bigfloat result");
  }

#endif
  return div(a, b);
}

bigfloat operator - (const bigfloat & a) {
  if (a.isSpecial()) {
    if (a.isZero())
      return pZero;
    if (a.isInf())
      return pInf;
    return NaN;
  }
  return bigfloat(-a.significant, a.exponent);
}
