/*******************************************************************************
+
+  LEDA 3.4
+
+  _int_vector.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

/* This file has been automatically generated from "integer_vector.w"
   by CTANGLE (Version 3.1 [km2c]) */


#include <LEDA/integer_vector.h>


#define LEDA_SMALL (std_memory.max_size()/sizeof(integer))


void integer_vector::allocate_small(integer*&vi,int di)
/* we use this procedure to allocate memory for small arrays. We first
   get an appropriate piece of memory from the LEDA memory manager and
   then initialize each cell by an inplace new.  */
{
vi= (integer*)std_memory.allocate_bytes(d*sizeof(integer));
integer*p= vi+di-1;
while(p>=vi){new(p)integer;p--;}
}

void integer_vector::deallocate_small(integer*vi,int di)
/* we use this procedure to deallocate memory for small arrays. We
   first call the destructor for type |integer| for each cell of the
   array and then return the piece of memory to the LEDA memory manager. */

{
integer*p= vi+di-1;
while(p>=vi){p->~integer();p--;}
std_memory.deallocate_bytes(vi,d*sizeof(integer));
}

integer_vector::integer_vector()
{d= 0;v= nil;}

integer_vector::integer_vector(int n)
{
if(n<0)error_handler(1,"integer_vector::constructor: negative dimension.");
d= n;
v= nil;
if(d>0)
{if(d<LEDA_SMALL)
integer_vector::allocate_small(v,d);
else
v= new integer[d];

integer zero= 0;
while(n--)v[n]= zero;
}
}


integer_vector::~integer_vector()
{
if(v)
if(d<LEDA_SMALL)
integer_vector::deallocate_small(v,d);
else
delete[]v;
}


integer_vector::integer_vector(const integer_vector&p)
{
d= p.d;
v= nil;
if(d>0)
{if(d<LEDA_SMALL)
integer_vector::allocate_small(v,d);
else
v= new integer[d];
for(int i= 0;i<d;i++)v[i]= p.v[i];
}
}


integer_vector&integer_vector::operator= (const integer_vector&vec)
{
register int n= vec.d;
if(n!=d)
{
if(v)
{if(d<LEDA_SMALL)
integer_vector::deallocate_small(v,d);
else
delete[]v;
}

if(n<LEDA_SMALL)
integer_vector::allocate_small(v,n);
else
v= new integer[n];

d= n;
}

while(n--)v[n]= vec.v[n];

return*this;
}


integer_vector::integer_vector(integer x,integer y)
{
integer_vector::allocate_small(v,2);
d= 2;
v[0]= x;
v[1]= y;
}

integer_vector::integer_vector(integer x,integer y,integer z)
{
integer_vector::allocate_small(v,3);
d= 3;
v[0]= x;
v[1]= y;
v[2]= z;
}

;


void integer_vector::check_dimensions(const integer_vector&vec)const
{
if(d!=vec.d)
error_handler(1,"integer_vector::check_dimensions: different dimensions.");
}



integer_vector&integer_vector::operator+= (const integer_vector&vec)
{
check_dimensions(vec);
register int n= d;
while(n--)v[n]+= vec.v[n];
return*this;
}

integer_vector&integer_vector::operator-= (const integer_vector&vec)
{
check_dimensions(vec);
register int n= d;
while(n--)v[n]-= vec.v[n];
return*this;
}

integer_vector integer_vector::operator+(const integer_vector&vec)const
{
check_dimensions(vec);
register int n= d;
integer_vector result(n);
while(n--)result.v[n]= v[n]+vec.v[n];
return result;
}

integer_vector integer_vector::operator-(const integer_vector&vec)const
{
check_dimensions(vec);
register int n= d;
integer_vector result(n);
while(n--)result.v[n]= v[n]-vec.v[n];
return result;
}

integer_vector integer_vector::operator-()const/* unary minus */
{
register int n= d;
integer_vector result(n);
while(n--)result.v[n]= -v[n];
return result;
}


integer_vector integer_vector::compmul(integer x)const
{
int n= d;
integer_vector result(n);
while(n--)result.v[n]= v[n]*x;
return result;
}


integer_vector operator*(const integer& f,const integer_vector&v)
{return v.compmul(f);}

integer_vector operator*(const integer_vector&v, const integer& f)
{return v.compmul(f);}

integer integer_vector::operator*(const integer_vector&vec)const
{
check_dimensions(vec);
integer result= 0;
register int n= d;
while(n--)result= result+v[n]*vec.v[n];
return result;
}


bool integer_vector::operator==(const integer_vector&vec)const
{
if(vec.d!=d)return false;
int i= 0;
while((i<d)&&(v[i]==vec.v[i]))i++;
return(i==d);
}


ostream&operator<<(ostream&s,const integer_vector&v)
{
for(int i= 0;i<v.d;i++)s<<v[i]<<" ";
return s;
}

istream&operator>>(istream&s,integer_vector&x)
{
int i= 0;
while(i<x.d&&s>>x.v[i++]);
return s;
}


