/*******************************************************************************
+
+  LEDA 3.4
+
+  _real.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/

/* This file has been automatically generated from "_real.w"
   by CTANGLE (Version 3.1 [km2c]) */

#include <LEDA/real.h>
#include <math.h>

enum constructortype {
  DOUBLE = 0, BIGFLOAT = 1, NEGATION = 2,
  SQUAREROOT = 3, ADDITION = 4, SUBTRACTION = 5, MULTIPLICATION = 6,
DIVISION = 7};

struct app_bf {
  bigfloat NUM_BF;
  bigfloat ERROR_BF;
   app_bf() {
  }
   app_bf(const bigfloat & x, const bigfloat & error_x = pZero) {
    NUM_BF = x;
    ERROR_BF = error_x;
  }
};


struct sep_bound {
  integer M, deg;
   sep_bound() {
  }
};


enum statustype {
EMPTY = 0, CLEARED = 1, VISITED = 2};



//double pow2(long);

const double eps = ldexp(1.0,-53);
const double correction = 1 + 8 * eps;
const double MaxError = ldexp(1.0,915);
const double MinDbl = ldexp(1.0,-1022);
const double twoMinDbl = ldexp(1.0,-1021);
const double MaxDbl = (2 - ldexp(1.0,-52)) * ldexp(1.0,1023);

//const double Infinity = ldexp(1.0,1024);

const double Infinity = 2*HUGE_VAL;


int isdouble(const integer & x)
{
  return (x == bigfloat(bigfloat(x).to_double()));
}
int isdouble(const bigfloat & x)
{
  return (x == bigfloat(x.to_double()));
}



inline bigfloat ldexp_bf(const bigfloat & x, const integer & k)
{
  return bigfloat(x.get_significant(), x.get_exponent() + k);
}
integer Maximum(integer x, integer y)
{
  return (x > y ? x : y);
}
int sign(double x)
{
  if (x == 0)
    return 0;
  else
    return (x > 0 ? 1 : -1);
}




struct sep_bound;
class real_rep {
  friend class real;

  double NUM;
  double ERROR;
  app_bf *APP_BF_PTR;
  constructortype CON;
  real_rep *OP1, *OP2;
  int count;
  long status;
  sep_bound *sep_bound_ptr;

   real_rep();
   real_rep(double);
   real_rep(const bigfloat &);

   ~real_rep();

   LEDA_MEMORY(real_rep)
  void init_app_bf();
  void adjust_dbl();
  void compute_op(long);
  void compute_approximation(long);
  int sign_with_separation_bound(const integer &, bool = false);
  void guarantee_bound_two_to(const integer &);
  inline bigfloat & num_bf() {
    return APP_BF_PTR->NUM_BF;
  }
  inline bigfloat & error_bf() {
    return APP_BF_PTR->ERROR_BF;
  }
  inline bool exact() {
    return error_bf().isZero();
  }
  inline integer num_exp() {
    return log2(num_bf());
  }
  inline integer err_exp() {
    return log2(error_bf());
  }


  void compute_separation_bound();


  void initialize_separation_bound();


  void improve_add(const integer &);
  void improve_sub(const integer &);
  void improve_mul(const integer &);
  void improve_div(const integer &);
  void improve_sqrt(const integer &);


  void compute_mul_error(bigfloat &);
  void compute_div_error(bigfloat &);
  void compute_sqrt_error(bigfloat &);



  void clear_visited_marks();



  friend ostream & operator << (ostream &, const real &);

  friend real operator + (const real &, const real &);
  friend real operator - (const real &, const real &);
  friend real operator *(const real &, const real &);
  friend real operator / (const real &, const real &);
  friend real operator - (const real &);
  friend real sqrt(const real &);



  friend int operator < (const real &, const real &);
  friend int operator <= (const real &, const real &);
  friend int operator > (const real &, const real &);
  friend int operator >= (const real &, const real &);
  friend int operator == (const real &, const real &);
  friend int operator != (const real &, const real &);



};




real::real()
{
  is_double = true;
  value = 0;
  PTR = nil;
}
real::real(int x)
{
  is_double = true;
  value = x;
  PTR = nil;
}
real::real(double x)
{
  is_double = true;
  value = x;
  PTR = nil;
}
real::real(const bigfloat & x)
{
  if (is_double = isdouble(x)) {
    value = x.to_double();
    PTR = nil;
  }
  else
    PTR = new real_rep(x);
}
real::real(const integer & x)
{
  if (is_double = isdouble(x)) {
    value = x.to_double();
    PTR = nil;
  }
  else
    PTR = new real_rep(bigfloat(x));
}
real::real(const rational & x)
{
  real OP1 = *new real_rep(x.numerator());
  real OP2 = *new real_rep(x.denominator());
  *this = OP1 / OP2;
}
real::real(real_rep & x_rep)
{
  is_double = false;
  PTR = &x_rep;
}


real::real(const real & x)
{
  if (is_double = x.is_double) {
    value = x.value;
    PTR = nil;
  }
  else {
    x.PTR->count++;
    PTR = x.PTR;
  }
}
real & real::operator = (const real & x) {

/* we avoid problems by ruling out the assignment |x = x;| */
  if (this == &x)
    return (*this);

/* first the actions that must anyway be performed, in the respective cases
        of |is_double| */
  if (is_double = x.is_double)
    value = x.value;
  else
    x.PTR->count++;

/* second, an existing reference in PTR must always be deleted;
        if |x.PTR == PTR != nil| and |is_double == false|, this leaves the
        number of references to |*PTR| unchanged, because we increased
        |x.PTR| before, and for |is_double == true| we in fact lose one
        reference because |PTR| is set to |nil| at the end
     */
  if (PTR != nil)
    if (--PTR->count == 0)
      delete PTR;

/* last, we set |this->PTR| */
  if (is_double)
    PTR = nil;
  else
    PTR = x.PTR;

  return (*this);
}


real_rep::real_rep()
{
  APP_BF_PTR = nil;
  sep_bound_ptr = nil;
  count = 1;
  status = 0;
}
real_rep::real_rep(double x)
{
  APP_BF_PTR = nil;
  sep_bound_ptr = nil;
  OP1 = OP2 = nil;
  CON = DOUBLE;
  NUM = x;
  ERROR = 0;
  count = 1;
  status = 0;
}
real_rep::real_rep(const bigfloat & x)
{
  APP_BF_PTR = new app_bf(x, 0);
  sep_bound_ptr = nil;
  OP1 = OP2 = nil;
  CON = BIGFLOAT;
  adjust_dbl();
  count = 1;
  status = 0;
}


real::~real()
{
  if (PTR != nil)
    if (--PTR->count == 0)
      delete PTR;
}
real_rep::~real_rep()
{
  if (APP_BF_PTR != nil)
    delete APP_BF_PTR;
  if (sep_bound_ptr != nil)
    delete sep_bound_ptr;
  if ((OP1 != nil) && (--OP1->count == 0))
    delete OP1;
  if ((OP2 != nil) && (--OP2->count == 0))
    delete OP2;
#ifdef DEBUG_REAL
  APP_BF_PTR = nil;
  OP1 = OP2 = nil;
#endif
}




void real_rep::init_app_bf()
{
  if (APP_BF_PTR != nil)
    return;
  if (OP1 != nil)
    OP1->init_app_bf();
  if (OP2 != nil)
    OP2->init_app_bf();
  if (ERROR < Infinity) {
    APP_BF_PTR =
      new app_bf(bigfloat(NUM), fabs(bigfloat(ERROR * NUM * correction)));
    return;
  }
  APP_BF_PTR = new app_bf();
  compute_op(53);
}


void real_rep::adjust_dbl()
{
  double n_head = num_bf().to_double();
  double e_head = error_bf().to_double();
  if (n_head != 0) {
    NUM = n_head;
    ERROR = eps + Max(MinDbl, e_head) / fabs(n_head);
  }
  else {
    NUM = e_head + 2 * MinDbl;
    ERROR = 2;
  }
  ERROR = ERROR * correction;
  if ((ERROR >= MaxError) || (fabs(n_head) == Infinity)) {
    NUM = 1;
    ERROR = Infinity;
  }
}


void real::adjust() const
{
  if (is_double)
    return;
  if (PTR->exact() && isdouble(PTR->num_bf())) {
    ((real &) * this).is_double = true;
    ((real &) * this).value = PTR->num_bf().to_double();
    return;
  }
  PTR->adjust_dbl();
}



ostream & operator << (ostream & out, const real & x) {
  bigfloat low_bound, upp_bound;
  long prec;
  if (x.is_double)
    low_bound = upp_bound = x.value;
  else {
    x.PTR->init_app_bf();
    prec = (log2(x.PTR->num_bf()) - log2(x.PTR->error_bf())).tolong();
    if (prec < 53)
      prec = 53;
    low_bound =
      sub(x.to_bigfloat(), x.PTR->error_bf(), prec, TO_N_INF);
    upp_bound =
      add(x.to_bigfloat(), x.PTR->error_bf(), prec, TO_P_INF);
  }
  out << "[" << low_bound << ",";
  out << upp_bound << "]";
  return out;
}
istream & operator >> (istream & in, real & x) {
  double x_num;
  in >> x_num;
  x = real(x_num);
  return in;
}


double real::to_double() const
{
  if (is_double)
    return value;
  if (PTR->APP_BF_PTR != nil)
    PTR->adjust_dbl();
  return PTR->NUM;
}
double real::get_double_error() const
{
  if (is_double)
    return 0;
  if (PTR->APP_BF_PTR != nil)
    PTR->adjust_dbl();
  return PTR->ERROR;
}
bigfloat real::to_bigfloat() const
{
  if (is_double)
    return value;
  else {
    PTR->init_app_bf();
    return PTR->num_bf();
  }
}
bigfloat real::get_bigfloat_error() const
{
  if (is_double)
    return 0;
  else {
    PTR->init_app_bf();
    return PTR->error_bf();
  }
}





real operator + (const real & x, const real & y) {

  double NUM_result;
  if (x.is_double && y.is_double) {
    NUM_result = x.value + y.value;
    if ((NUM_result - x.value == y.value)
      && ((NUM_result - y.value) == x.value)
      )
      return NUM_result;
  }



  if (x.PTR == nil)
    ((real &) x).PTR = new real_rep(x.value);
  if (y.PTR == nil)
    ((real &) y).PTR = new real_rep(y.value);
  double &x_num = x.PTR->NUM;
  double &y_num = y.PTR->NUM;


  real_rep & z_rep = *new real_rep();

  z_rep.OP1 = x.PTR;
  z_rep.OP2 = y.PTR;
  z_rep.OP1->count++;
  z_rep.OP2->count++;
  double &NUM = z_rep.NUM;


  z_rep.CON = ADDITION;
  z_rep.NUM = x.PTR->NUM + y.PTR->NUM;

  if (fabs(NUM) >= MinDbl) {
    z_rep.ERROR = fabs(x_num / NUM) * x.PTR->ERROR
      + fabs(y_num / NUM) * y.PTR->ERROR
      + eps;
  }
  else {

    NUM = fabs(x_num) * x.PTR->ERROR +
      fabs(y_num) * y.PTR->ERROR + 4 * MinDbl;
    z_rep.ERROR = 2;
  }
  z_rep.ERROR *= correction;



  if (!((fabs(NUM) + x.PTR->ERROR + y.PTR->ERROR < Infinity)
      && (z_rep.ERROR < MaxError)
    )
    ) {
    z_rep.ERROR = Infinity;
    NUM = 1;
  }


  return z_rep;
}
real operator - (const real & x, const real & y) {

  double NUM_result;
  if (x.is_double && y.is_double) {
    NUM_result = x.value - y.value;
    if ((NUM_result + y.value == x.value)
      && (x.value - NUM_result == y.value)
      )
      return NUM_result;
  }




  if (x.PTR == nil)
    ((real &) x).PTR = new real_rep(x.value);
  if (y.PTR == nil)
    ((real &) y).PTR = new real_rep(y.value);
  double &x_num = x.PTR->NUM;
  double &y_num = y.PTR->NUM;


  real_rep & z_rep = *new real_rep();

  z_rep.OP1 = x.PTR;
  z_rep.OP2 = y.PTR;
  z_rep.OP1->count++;
  z_rep.OP2->count++;
  double &NUM = z_rep.NUM;


  z_rep.CON = SUBTRACTION;
  z_rep.NUM = x.PTR->NUM - y.PTR->NUM;

  if (fabs(NUM) >= MinDbl) {
    z_rep.ERROR = fabs(x_num / NUM) * x.PTR->ERROR
      + fabs(y_num / NUM) * y.PTR->ERROR
      + eps;
  }
  else {

    NUM = fabs(x_num) * x.PTR->ERROR +
      fabs(y_num) * y.PTR->ERROR + 4 * MinDbl;
    z_rep.ERROR = 2;
  }
  z_rep.ERROR *= correction;



  if (!((fabs(NUM) + x.PTR->ERROR + y.PTR->ERROR < Infinity)
      && (z_rep.ERROR < MaxError)
    )
    ) {
    z_rep.ERROR = Infinity;
    NUM = 1;
  }


  return z_rep;
}
real operator *(const real & x, const real & y) {

  double NUM_result;
  long *x_ptr, *y_ptr;
  if (x.is_double && y.is_double) {
    NUM_result = x.value * y.value;
    x_ptr = (long *) &(x.value);
    y_ptr = (long *) &(y.value);
#ifndef LITTLE_ENDIAN
    x_ptr++;
    y_ptr++;
#endif
    if (!(*x_ptr & 0x03ffffff) && !(*y_ptr & 0x03ffffff))
      return NUM_result;
  }




  if (x.PTR == nil)
    ((real &) x).PTR = new real_rep(x.value);
  if (y.PTR == nil)
    ((real &) y).PTR = new real_rep(y.value);
  double &x_num = x.PTR->NUM;
  double &y_num = y.PTR->NUM;


  real_rep & z_rep = *new real_rep();

  z_rep.OP1 = x.PTR;
  z_rep.OP2 = y.PTR;
  z_rep.OP1->count++;
  z_rep.OP2->count++;
  double &NUM = z_rep.NUM;


  z_rep.CON = MULTIPLICATION;
  z_rep.NUM = x.PTR->NUM * y.PTR->NUM;

  double &qx = x.PTR->ERROR;
  double &qy = y.PTR->ERROR;
  if (fabs(NUM) >= MinDbl) {
    z_rep.ERROR = qx + qy + qx * qy + eps;
  }
  else {
    if ((x_num != 0) && (y_num != 0)) {
      NUM = MinDbl * (1 + qx) * (1 + qy);
      z_rep.ERROR = 2;
    }
    else
      return 0;
  }
  z_rep.ERROR *= correction;



  if (!((fabs(NUM) + x.PTR->ERROR + y.PTR->ERROR < Infinity)
      && (z_rep.ERROR < MaxError)
    )
    ) {
    z_rep.ERROR = Infinity;
    NUM = 1;
  }


  return z_rep;
}


real operator / (const real & x, const real & y) {

  if (y.is_double) {
    if (y.value == 0)
      error_handler(1, "real::operator/:Division by zero");
  }
  else if ((y.PTR->NUM == 0) && (y.PTR->ERROR == Infinity))
    error_handler(1, "real::operator/:Division by zero");




  double NUM_result;
  long *num_ptr, *y_ptr;
  if (x.is_double && y.is_double) {
    NUM_result = x.value * y.value;
    num_ptr = (long *) &NUM_result;
    y_ptr = (long *) &y.value;
#ifndef LITTLE_ENDIAN
    num_ptr++;
    y_ptr++;
#endif
    if (!(*num_ptr & 0x03ffffff) && !(*y_ptr & 0x03ffffff))
      if (NUM_result * y.value == x.value)
	return NUM_result;
  }



  if (x.PTR == nil)
    ((real &) x).PTR = new real_rep(x.value);
  if (y.PTR == nil)
    ((real &) y).PTR = new real_rep(y.value);
  double &x_num = x.PTR->NUM;
  double &y_num = y.PTR->NUM;


  real_rep & z_rep = *new real_rep();

  z_rep.OP1 = x.PTR;
  z_rep.OP2 = y.PTR;
  z_rep.OP1->count++;
  z_rep.OP2->count++;
  double &NUM = z_rep.NUM;


  z_rep.CON = DIVISION;
  NUM = x_num / y_num;

  if (y.PTR->ERROR >= 1) {
    z_rep.ERROR = Infinity;
    NUM = 1;
    return z_rep;
  }
  if (fabs(NUM) >= MinDbl)
    z_rep.ERROR = (x.PTR->ERROR + y.PTR->ERROR) / (1 - y.PTR->ERROR)
      + eps;
  else {
    if (x_num != 0) {
      NUM = MinDbl * (1 + x.PTR->ERROR) / (1 - y.PTR->ERROR);
      z_rep.ERROR = 2;
    }
    else
      return 0;
  }
  z_rep.ERROR *= correction;




  if (!((fabs(NUM) + x.PTR->ERROR + y.PTR->ERROR < Infinity)
      && (z_rep.ERROR < MaxError)
    )
    ) {
    z_rep.ERROR = Infinity;
    NUM = 1;
  }


  return z_rep;
}


real sqrt(const real & x)
{

  if ((x.is_double) && (x.value < 0))
    error_handler(1, "real::operator sqrt:negative argument");




  double NUM_result;
  long *num_ptr;
  if (x.is_double) {
    NUM_result = sqrt(x.value);
    num_ptr = (long *) &NUM_result;
#ifndef LITTLE_ENDIAN
    num_ptr++;
#endif
    if (!(*num_ptr & 0x03ffffff))
      return NUM_result;
  }



  if (x.PTR == nil)
    ((real &) x).PTR = new real_rep(x.value);
  real_rep & z_rep = *new real_rep();
  z_rep.OP1 = x.PTR;
  z_rep.OP1->count++;
  z_rep.OP2 = nil;
  z_rep.CON = SQUAREROOT;
  if (x.PTR->ERROR >= 1) {
    z_rep.ERROR = Infinity;
    z_rep.NUM = 1;
    return real(z_rep);
  }
  if (x.PTR->NUM < 0)
    error_handler(1, "real::operator sqrt:negative argument");
  z_rep.NUM = sqrt(x.PTR->NUM);

  z_rep.ERROR = x.PTR->ERROR + eps;
  z_rep.ERROR *= correction;


  return z_rep;
}


real operator - (const real & x) {
  if (x.is_double)
    return -x.value;
  real_rep & z_rep = *new real_rep();
  z_rep.CON = NEGATION;
  z_rep.OP1 = x.PTR;
  z_rep.OP1->count++;
  z_rep.OP2 = nil;
  z_rep.NUM = -x.PTR->NUM;
  z_rep.ERROR = x.PTR->ERROR;
  return z_rep;
}





void real::reduce_error_in_approximation_to(const integer & p) const
{
  if (is_double)
    return;
  PTR->init_app_bf();
  PTR->guarantee_bound_two_to(-p);
  adjust();
}


void real_rep::guarantee_bound_two_to(const integer & e)
{
  bool current_precision_is_sufficient =
  error_bf().isZero() || (err_exp() <= e);
  if (current_precision_is_sufficient)
    return;

  switch (CON) {
      case DOUBLE:
      case BIGFLOAT:
	return;
      case NEGATION:
	OP1->guarantee_bound_two_to(e);
	num_bf() = -OP1->num_bf();
	error_bf() = OP1->error_bf();
	break;
      case ADDITION:
	improve_add(e);
	break;
      case SUBTRACTION:
	improve_sub(e);
	break;
      case MULTIPLICATION:
	improve_mul(e);
	break;
      case DIVISION:
	improve_div(e);
	break;
      case SQUAREROOT:
	improve_sqrt(e);
	break;
  }

  if (exact()) {
    if (OP1 != nil)
      if (--OP1->count == 0)
	delete OP1;
    if (OP2 != nil)
      if (--OP2->count == 0)
	delete OP2;
    OP1 = OP2 = nil;
    CON = BIGFLOAT;
  }



  if (sep_bound_ptr) {
    compute_separation_bound();
    if (error_bf() + fabs(num_bf()) <= pow2(-sep_bound_ptr->M)) {

      {
	if (OP1 != nil)
	  if (--OP1->count == 0)
	    delete OP1;
	if (OP2 != nil)
	  if (--OP2->count == 0)
	    delete OP2;
	OP1 = OP2 = nil;
	CON = BIGFLOAT;
      }


      num_bf() = error_bf() = pZero;
      sep_bound_ptr->M = integer(0);
      sep_bound_ptr->deg = 1;
    }
  }

  adjust_dbl();
}


void real_rep::compute_op(long p)
{
  bool isexact;
  bigfloat operation_error, operand_error;
  switch (CON) {
      case DOUBLE:
      case BIGFLOAT:
	return;
      case NEGATION:
	num_bf() = -OP1->num_bf();
	error_bf() = OP1->error_bf();
	break;
      case ADDITION:
	num_bf() = add(OP1->num_bf(), OP2->num_bf(), p, TO_NEAREST, isexact);

	operand_error = add(OP1->error_bf(), OP2->error_bf(), 16, TO_INF);



	break;
      case SUBTRACTION:
	num_bf() = sub(OP1->num_bf(), OP2->num_bf(), p, TO_NEAREST, isexact);

	operand_error = add(OP1->error_bf(), OP2->error_bf(), 16, TO_INF);



	break;
      case MULTIPLICATION:
	num_bf() = mul(OP1->num_bf(), OP2->num_bf(), p, TO_NEAREST, isexact);
	compute_mul_error(operand_error);
	break;
      case DIVISION:
	if (OP2->sign_with_separation_bound(integer(0)) == 0)
	  error_handler(1, "compute_op:division by zero");
	num_bf() = div(OP1->num_bf(), OP2->num_bf(), p, TO_NEAREST, isexact);
	compute_div_error(operand_error);
	break;
      case SQUAREROOT:
	if (OP1->sign_with_separation_bound(integer(0)) < 0)
	  error_handler(1, "compute_op:sqrt of negative number");
	num_bf() = sqrt(OP1->num_bf(), pZero, p, TO_NEAREST, isexact);
	compute_sqrt_error(operand_error);
  }
  if (!isexact)
    operation_error = ldexp_bf(num_bf(), -p);
  error_bf() = operation_error + operand_error;
  if (exact()) {
    if (OP1 != nil)
      if (--OP1->count == 0)
	delete OP1;
    if (OP2 != nil)
      if (--OP2->count == 0)
	delete OP2;
    OP1 = OP2 = nil;
    CON = BIGFLOAT;
  }


}


void real_rep::improve_add(const integer & e)
{
  long p;

  OP1->guarantee_bound_two_to(e - 2);
  OP2->guarantee_bound_two_to(e - 2);
  integer log_epsilon = -1;
  if (!OP1->num_bf().isZero() && !OP2->num_bf().isZero())
    log_epsilon = e - Maximum(OP1->num_exp(), OP2->num_exp()) - 2;

  if (!log_epsilon.islong())
    error_handler(1, "sorry:improve:computation to expensive");
  else
    p = -log_epsilon.tolong();




  bool isexact;
  num_bf() = add(OP1->num_bf(), OP2->num_bf(), p, TO_NEAREST, isexact);

  if (isexact && OP1->exact() && OP2->exact())
    error_bf() = pZero;
  else
    error_bf() = pow2(e);



}

void real_rep::improve_sub(const integer & e)
{
  long p;

  OP1->guarantee_bound_two_to(e - 2);
  OP2->guarantee_bound_two_to(e - 2);
  integer log_epsilon = -1;
  if (!OP1->num_bf().isZero() && !OP2->num_bf().isZero())
    log_epsilon = e - Maximum(OP1->num_exp(), OP2->num_exp()) - 2;

  if (!log_epsilon.islong())
    error_handler(1, "sorry:improve:computation to expensive");
  else
    p = -log_epsilon.tolong();




  bool isexact;
  num_bf() = sub(OP1->num_bf(), OP2->num_bf(), p, TO_NEAREST, isexact);

  if (isexact && OP1->exact() && OP2->exact())
    error_bf() = pZero;
  else
    error_bf() = pow2(e);



}


void real_rep::compute_mul_error(bigfloat & operand_error)
{
  bigfloat x = OP1->num_bf();
  bigfloat y = OP2->num_bf();
  bigfloat error_x = mul(fabs(x.round(16, TO_INF)), OP2->error_bf(), 32, TO_INF);
  bigfloat error_y = mul(fabs(y.round(16, TO_INF)), OP1->error_bf(), 32, TO_INF);
  operand_error = add(error_x, error_y, 16, TO_INF);
}


void real_rep::improve_mul(const integer & e)
{
  bigfloat y_high = add(fabs(OP2->num_bf()), OP2->error_bf(), 32, TO_P_INF);
  integer ex = e - log2(y_high) - 2;
  OP1->guarantee_bound_two_to(ex);
  integer log_epsilon = -1;
  if (!OP1->num_bf().isZero()) {
    integer ey = e - OP1->num_exp() - 2;
    OP2->guarantee_bound_two_to(ey);
    log_epsilon = e - (OP1->num_exp() + OP2->num_exp()) - 1;
  }
  bool isexact;
  long p;

  if (!log_epsilon.islong())
    error_handler(1, "sorry:improve:computation to expensive");
  else
    p = -log_epsilon.tolong();


  num_bf() = mul(OP1->num_bf(), OP2->num_bf(), p, TO_NEAREST, isexact);

  if (isexact && OP1->exact() && OP2->exact())
    error_bf() = pZero;
  else
    error_bf() = pow2(e);



}



void real_rep::compute_div_error(bigfloat & operand_error)
{
  bigfloat y_low = sub(fabs(OP2->num_bf()), OP2->error_bf(), 32, TO_N_INF);
  bigfloat x = OP1->num_bf();
  bigfloat z = mul(fabs(x.round(16)), OP2->error_bf(), 32, TO_INF);
  z = ldexp_bf(z, integer(1) - OP2->num_exp());
  operand_error = add(z, fabs(OP1->error_bf()), 16, TO_INF);
  operand_error = ldexp_bf(operand_error, integer(1) - log2(y_low));
}


void real_rep::improve_div(const integer & e)
{
  bigfloat y_low = sub(fabs(OP2->num_bf()), OP2->error_bf(), 32, TO_N_INF);
  integer ex = e + log2(y_low) - 3;
  OP1->guarantee_bound_two_to(ex);
  integer log_epsilon = -1;
  if (!OP1->num_bf().isZero()) {
    integer ey = ex + OP2->num_exp() - OP1->num_exp() - 3;
    OP2->guarantee_bound_two_to(ey);
    log_epsilon = e + OP2->num_exp() - OP1->num_exp() - 2;
  }
  bool isexact;
  long p;

  if (!log_epsilon.islong())
    error_handler(1, "sorry:improve:computation to expensive");
  else
    p = -log_epsilon.tolong();


  num_bf() = div(OP1->num_bf(), OP2->num_bf(), p, TO_NEAREST, isexact);

  if (isexact && OP1->exact() && OP2->exact())
    error_bf() = pZero;
  else
    error_bf() = pow2(e);



}


void real_rep::compute_sqrt_error(bigfloat & operand_error)
{
  integer log_sqrt = (num_exp() + 1) >> 1;
  if (!OP1->exact())
    operand_error = ldexp_bf(OP1->error_bf(), -log_sqrt);
}



void real_rep::improve_sqrt(const integer & e)
{
  bool isexact;
  bigfloat x_low = sub(OP1->num_bf(), OP1->error_bf(), 32, TO_N_INF);
  integer ex = e + (log2(x_low) - 1) / 2 - 1;
  OP1->guarantee_bound_two_to(ex);
  bigfloat x_high = add(OP1->num_bf(), OP1->error_bf(), 32, TO_P_INF);
  integer log_epsilon = e - (log2(x_high) + 1) / 2 - 1;
  long p;

  if (!log_epsilon.islong())
    error_handler(1, "sorry:improve:computation to expensive");
  else
    p = -log_epsilon.tolong();


  num_bf() = sqrt(OP1->num_bf(), num_bf(), p, TO_NEAREST, isexact);
  if (isexact && OP1->exact())
    error_bf() = pZero;
  else
    error_bf() = pow2(e);
}


void real::guarantee_relative_error_two_to_minus(long p) const
{
  if (is_double)
    return;
  if (PTR->sign_with_separation_bound(integer(0)) == 0)
    return;
  PTR->init_app_bf();
  bigfloat num_low = fabs(PTR->num_bf()) - PTR->error_bf();
  PTR->guarantee_bound_two_to(log2(num_low) - p);
  adjust();
}


void real::compute_with_mantissa_length(long p) const
{
  if (is_double)
    return;
  PTR->init_app_bf();
  PTR->clear_visited_marks();
  PTR->compute_approximation(p);
  adjust();
}



void real_rep::compute_approximation(long p)
{
  if (status == VISITED)
    return;
  if (OP1 != nil)
    OP1->compute_approximation(p);
  if (OP2 != nil)
    OP2->compute_approximation(p);
  compute_op(p);
  status = VISITED;
}





int real::sign() const
{
  if (is_double)
    return::sign(value);
  else
    return PTR->sign_with_separation_bound(integer(0));
}

int real::sign(const integer & q) const
{
  if (is_double)
    return::sign(value);
  else
    return PTR->sign_with_separation_bound(q, true);
}
int real::sign(long p) const
{
  if (is_double)
    return::sign(value);
  else
    return PTR->sign_with_separation_bound(p, true);
}


void real_rep::compute_separation_bound()
{
  sep_bound *ptr = sep_bound_ptr;

  switch (CON) {
      case DOUBLE:
      case BIGFLOAT:
	{
	  bigfloat & num = APP_BF_PTR->NUM_BF;
	  ptr->deg = 1;
	  if (num.isZero())
	    ptr->M = integer(0);
	  else {
	    if (sign(num.get_exponent()) >= 0)
	      ptr->M = log2(num);
	    else
	      ptr->M = integer(1) + Maximum(num.get_precision(), -num.get_exponent());
	  }
	  break;
	}
      case NEGATION:
	ptr->deg = OP1->sep_bound_ptr->deg;
	ptr->M = OP1->sep_bound_ptr->M;
	break;
      case MULTIPLICATION:
      case DIVISION:
	ptr->M = OP1->sep_bound_ptr->M * OP2->sep_bound_ptr->deg
	  + OP2->sep_bound_ptr->M * OP1->sep_bound_ptr->deg;
	ptr->deg = OP1->sep_bound_ptr->deg * OP2->sep_bound_ptr->deg;
	break;
      case ADDITION:
      case SUBTRACTION:
	ptr->deg = OP1->sep_bound_ptr->deg * OP2->sep_bound_ptr->deg;
	ptr->M = ptr->deg
	  + OP1->sep_bound_ptr->M * OP2->sep_bound_ptr->deg
	  + OP2->sep_bound_ptr->M * OP1->sep_bound_ptr->deg;
	break;
      case SQUAREROOT:
	ptr->M = OP1->sep_bound_ptr->M;
	ptr->deg = OP1->sep_bound_ptr->deg << 1;
	break;
  }
}


void real_rep::initialize_separation_bound()
{
  if (sep_bound_ptr != nil)
    return;
  if (OP1 != nil)
    OP1->initialize_separation_bound();
  if (OP2 != nil)
    OP2->initialize_separation_bound();
  sep_bound_ptr = new sep_bound();
  compute_separation_bound();
}


int real_rep::sign_with_separation_bound(const integer & q, bool supported)
{

  if ((ERROR != Infinity) && ((ERROR < 1) || (NUM == 0)))
    return::sign(NUM);
  init_app_bf();
  if
    (exact() || (!num_bf().isZero() && (num_exp() > err_exp())))
    return num_bf().sign();


  initialize_separation_bound();

  bool user_bound_is_reached = false;
  integer current_precision = 26;
  if (err_exp() <= 0)
    current_precision = (integer(53) - err_exp()) / 2;
  do {
    current_precision = current_precision * 2;
    guarantee_bound_two_to(-current_precision);
    user_bound_is_reached = supported && (current_precision >= q);
  }
  while (!(user_bound_is_reached ||
      (exact() || (!num_bf().isZero() && (num_exp() > err_exp())))
    ));


  if
    (exact() || (!num_bf().isZero() && (num_exp() > err_exp())))
    return num_bf().sign();
  NUM = ERROR = 0;
  num_bf() = error_bf() = pZero;

  {
    if (OP1 != nil)
      if (--OP1->count == 0)
	delete OP1;
    if (OP2 != nil)
      if (--OP2->count == 0)
	delete OP2;
    OP1 = OP2 = nil;
    CON = BIGFLOAT;
  }


  return 0;
}


int operator == (const real & x, const real & y) {
  double x_error, y_error;
  bool is_easy;
  if (x.is_double && y.is_double)
    is_easy = true;
  else {
    if (x.is_double)
      x_error = 0;
    else {
      ((real &) x).value = x.PTR->NUM;
      x_error = fabs(x.value) * x.PTR->ERROR;
    }
    if (y.is_double)
      y_error = 0;
    else {
      ((real &) y).value = y.PTR->NUM;
      y_error = fabs(y.value) * y.PTR->ERROR;
    }
    is_easy =
      (fabs(x.value - y.value) > (x_error + y_error + twoMinDbl) * correction);
  }


  if (is_easy)
    return (x.value == y.value);
  return ((y - x).sign() == 0);
}

int operator != (const real & x, const real & y) {
  double x_error, y_error;
  bool is_easy;
  if (x.is_double && y.is_double)
    is_easy = true;
  else {
    if (x.is_double)
      x_error = 0;
    else {
      ((real &) x).value = x.PTR->NUM;
      x_error = fabs(x.value) * x.PTR->ERROR;
    }
    if (y.is_double)
      y_error = 0;
    else {
      ((real &) y).value = y.PTR->NUM;
      y_error = fabs(y.value) * y.PTR->ERROR;
    }
    is_easy =
      (fabs(x.value - y.value) > (x_error + y_error + twoMinDbl) * correction);
  }

  ;
  if (is_easy)
    return (x.value != y.value);
  return ((y - x).sign() != 0);
}

int operator < (const real & x, const real & y) {
  double x_error, y_error;
  bool is_easy;
  if (x.is_double && y.is_double)
    is_easy = true;
  else {
    if (x.is_double)
      x_error = 0;
    else {
      ((real &) x).value = x.PTR->NUM;
      x_error = fabs(x.value) * x.PTR->ERROR;
    }
    if (y.is_double)
      y_error = 0;
    else {
      ((real &) y).value = y.PTR->NUM;
      y_error = fabs(y.value) * y.PTR->ERROR;
    }
    is_easy =
      (fabs(x.value - y.value) > (x_error + y_error + twoMinDbl) * correction);
  }

  ;
  if (is_easy)
    return (x.value < y.value);
  return ((y - x).sign() > 0);
}

int operator > (const real & x, const real & y) {
  double x_error, y_error;
  bool is_easy;
  if (x.is_double && y.is_double)
    is_easy = true;
  else {
    if (x.is_double)
      x_error = 0;
    else {
      ((real &) x).value = x.PTR->NUM;
      x_error = fabs(x.value) * x.PTR->ERROR;
    }
    if (y.is_double)
      y_error = 0;
    else {
      ((real &) y).value = y.PTR->NUM;
      y_error = fabs(y.value) * y.PTR->ERROR;
    }
    is_easy =
      (fabs(x.value - y.value) > (x_error + y_error + twoMinDbl) * correction);
  }

  ;
  if (is_easy)
    return (x.value > y.value);
  return ((y - x).sign() < 0);
}

int operator <= (const real & x, const real & y) {
  double x_error, y_error;
  bool is_easy;
  if (x.is_double && y.is_double)
    is_easy = true;
  else {
    if (x.is_double)
      x_error = 0;
    else {
      ((real &) x).value = x.PTR->NUM;
      x_error = fabs(x.value) * x.PTR->ERROR;
    }
    if (y.is_double)
      y_error = 0;
    else {
      ((real &) y).value = y.PTR->NUM;
      y_error = fabs(y.value) * y.PTR->ERROR;
    }
    is_easy =
      (fabs(x.value - y.value) > (x_error + y_error + twoMinDbl) * correction);
  }

  ;
  if (is_easy)
    return (x.value <= y.value);
  return ((y - x).sign() >= 0);
}

int operator >= (const real & x, const real & y) {
  double x_error, y_error;
  bool is_easy;
  if (x.is_double && y.is_double)
    is_easy = true;
  else {
    if (x.is_double)
      x_error = 0;
    else {
      ((real &) x).value = x.PTR->NUM;
      x_error = fabs(x.value) * x.PTR->ERROR;
    }
    if (y.is_double)
      y_error = 0;
    else {
      ((real &) y).value = y.PTR->NUM;
      y_error = fabs(y.value) * y.PTR->ERROR;
    }
    is_easy =
      (fabs(x.value - y.value) > (x_error + y_error + twoMinDbl) * correction);
  }

  ;
  if (is_easy)
    return (x.value >= y.value);
  return ((y - x).sign() <= 0);
}




void real_rep::clear_visited_marks()
{
  if (status == CLEARED)
    return;
  if (OP1 != nil)
    OP1->clear_visited_marks();
  if (OP2 != nil)
    OP2->clear_visited_marks();
  status = CLEARED;
}




real fabs(const real & x)
{
  if (x.sign() < 0)
    return -x;
  else
    return x;
}

real sq(const real & x)
{
  return x * x;
}

real dist(const real & x, const real & y)
{
  real a = x * x;
  real b = y * y;
  return sqrt(a + b);
}

real powi(const real & x, int n)
{
  real y = x, z = 1;
  int n_prefix = n;
  while (n_prefix > 0) {
    if (n_prefix % 2)
      z = z * y;
    n_prefix = n_prefix / 2;
    y = y * y;
  }
  return z;
}
