/*******************************************************************************
+
+  LEDA 3.4
+
+  voronoi.c
+
+  This file is part of the LEDA research version (LEDA-R) that can be 
+  used free of charge in academic research and teaching. Any commercial
+  use of this software requires a license which is distributed by the
+  LEDA Software GmbH, Postfach 151101, 66041 Saarbruecken, FRG
+  (fax +49 681 31104).
+
+  Copyright (c) 1991-1996  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 66123 Saarbruecken, Germany     
+  All rights reserved.
+ 
*******************************************************************************/
//------------------------------------------------------------------------------
// VORONOI DIAGRAMS
//
// DELAUNAY_TO_VORONOI(DT,VD)  : compute VD from Delaunay Triangulation DT
// VORONOI(L,VD)   : compute VD for a list L of points
//------------------------------------------------------------------------------


void DELAUNAY_TO_VORONOI(const GRAPH<POINT,edge>& DT, GRAPH<CIRCLE,POINT>& VD)
{
  VD.clear();

  if (DT.number_of_nodes() < 2) return;


  // create Voronoi nodes

  edge_array<node> vnode(DT,nil);


  // for outer face

  edge  e = DT.last_edge();
  POINT a = DT[source(e)];
  edge  x = e;
  do { POINT b = DT[target(x)];
       vnode[x] =  VD.new_node(CIRCLE(a,center(a,b),b));
       a = b;
       x = DT.next_face_edge(x);
     } while ( x != e);


  // for all other faces

  forall_edges(e,DT)
  { 
    if (vnode[e]) continue;

    edge  x = DT.next_face_edge(e);
    POINT a = DT[source(e)];
    POINT b = DT[target(e)];
    POINT c = DT[target(x)];
    node  v = VD.new_node(CIRCLE(a,b,c));

    vnode[e] = v;

    do { vnode[x] = v;
         x = DT.next_face_edge(x);
        } while( x != e);

   }



  // construct Voronoi edges

  // for outer face

  x = e = DT.last_edge();
  x = e;
  do { edge r = DT.reversal(x); 
       POINT p = DT[target(x)];
       VD.new_edge(vnode[x],vnode[r],p);
       x = DT.cyclic_adj_pred(r);
     } while ( x != e);


  // for all other faces

  forall_edges(e,DT)
  { node v = vnode[e]; 
    if (VD.outdeg(v) == 0)
    { POINT p = DT[target(e)];
      edge x = e;
      do { edge r = DT.reversal(x); 
           VD.new_edge(v,vnode[r],p);
           x = DT.cyclic_adj_pred(r);
         } while ( x != e);
     }
   }

}



void VORONOI(const list<POINT>& L, GRAPH<CIRCLE,POINT>& VD)
{ GRAPH<POINT,edge> DT;
  DELAUNAY_DIAGRAM(L,DT);
  DELAUNAY_TO_VORONOI(DT,VD);
}
